#!/bin/bash

source fcSourceScript.sh >/dev/null 2>&1

usage () {
    echo ""
    echo "====================================== Usage ======================================"
    echo "fcRunStress.sh <BDF | HostName | PortNumber | wwn> <StressType> <StopOnError> [Pattern]"
    echo ""
    echo "    BDF:"
    echo "        Bus:Device.Function (eg. 02:00.0)"
    echo ""
    echo "    HostName:"
    echo "        FC Host Name (eg. host7)"
    echo ""
    echo "    PortNumber:"
    echo "        FC Port Number (eg. 3)"
    echo ""
    echo "    wwn:"
    echo "        FC Port WWN (eg. 10:00:00:90:fa:c7:71:1a)"
    echo ""
    echo "    StressType:"
    echo "        HBA Tests"
    echo "            0:  PCI LoopBack Tests"
    echo "            1:  Internal LoopBack Tests"
    echo "        CNA Tests"
    echo "            2:  External LoechoBack Test (requires loopback plug)"
    echo "            3:  Host to DDR DMA (OneConnect adapters) loopback Test"
    echo "            4:  PHY Loopback Test"
    echo "            5:  MAC Loopback Test"
    echo ""
    echo "    StopOnError:"
    echo "        0: Do Not stop on error"
    echo "        1: Stop test on error"
    echo ""
    echo "    Pattern (optional): Up to 8 hexadecimal characters (example: 1a2b3c4d)"
    echo "==================================================================================="
    echo ""
}

if [[ "$*" == "" ]]; then
    echo "Error: parameter is NULL"
    usage
    exit 1
fi

if [ $# -ne 3 ] && [ $# -ne 4 ]; then
    echo "Error: parameter amounts error !!"
    usage
    exit 1
fi

mkdir -p /tmp/log/

FC_STRESS_LOG_PREFIX=/tmp/log/fcstress_FC-PORT
FC_STRESS_STOP_FLAG=/tmp/fc_stress_stop_flag
GLOBAL_STRESS_STOP_FLAG=/tmp/stop_flag
GLOBAL_STRESS_DURATION_FLAG=/tmp/duration_flag

# remove old temp file
rm "${FC_STRESS_LOG_PREFIX}"*.log >/dev/null 2>&1
rm ${FC_STRESS_STOP_FLAG} >/dev/null 2>&1

RESULT_PORT_NUMBER=""
for BDF in ${BDF_ARRAY[*]}
do
    if [[ "$1" == "$BDF" ]]; then
	RESULT_PORT_NUMBER=`fcGetHost.sh | grep "$1" | awk '{print $3}'`
	WWN_PARM_OK=1
    fi
done

for HOST in ${HOST_NAME_ARRAY[*]}
do
    if [[ "$1" == "$HOST" ]]; then
	RESULT_PORT_NUMBER=`fcGetHost.sh | grep "$1" | awk '{print $3}'`
	WWN_PARM_OK=1
    fi
done

for PORT_NUMBER in ${PORT_NUMBER_ARRAY[*]}
do
    if [[ "$1" == "$PORT_NUMBER" ]]; then
	RESULT_PORT_NUMBER=`fcGetHost.sh | sed -n $(($1 + 3))p | grep "$1" | awk '{print $3}'`
	WWN_PARM_OK=1
    fi
done

for WWN in ${WWN_ARRAY[*]}
do
    if [[ "$1" == "$WWN" ]]; then
	RESULT_PORT_NUMBER=`fcGetHost.sh | grep "$1" | awk '{print $3}'`
	WWN_PARM_OK=1
    fi
done

echo `date`": start" > "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log

if [ $WWN_PARM_OK -eq 0 ] && [[ "$1" != "" ]]; then
    echo ""
    echo "Error: <BDF | HostName | PortNumber | wwn> parameter error, your input is $1 !!!" | tee -a "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
    usage | tee -a "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
    exit 1
fi

echo "clearing statistics..." | tee -a "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
fcRunUtility.sh ResetStatistics 0 | tee -a "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
if [ $? != 0 ]; then
    echo "Error clearing statistics !!!" | tee -a "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
fi

if [ $# -eq 3 ]; then
    fcRunTest.sh loopbacktest $1 $2 0 $3 | tee -a "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log &
else
    fcRunTest.sh loopbacktest $1 $2 0 $3 $4 | tee -a "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log &
fi

while [[ 1 ]]; do
    if [ -f "$FC_STRESS_STOP_FLAG" ] || [ -f "$GLOBAL_STRESS_STOP_FLAG" ] || [ -f "$GLOBAL_STRESS_DURATION_FLAG" ]; then
	killall tee
	killall -2 hbacmd
	echo "" >> "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
	echo "" >> "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
	echo "" >> "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
	echo `date`": stop" >> "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
	echo "" >> "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log

	ERROR_STATISTIC=`fcGetPortStatistic.sh "$RESULT_PORT_NUMBER" | grep Err`
	INVALID_STATISTIC=`fcGetPortStatistic.sh "$RESULT_PORT_NUMBER" | grep -i invalid`

	ERROR_STATISTIC_ZERO_AMOUNT=`echo "$ERROR_STATISTIC" | grep ":  0" | wc -l`
	INVALID_STATISTIC_ZERO_AMOUNT=`echo "$INVALID_STATISTIC" | grep ":  0" | wc -l`

	echo "Statistics:" >> "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
	echo "$ERROR_STATISTIC" >> "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log
	echo "$INVALID_STATISTIC" >> "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log

	HAS_ERROR=`cat "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log | grep -i error`
	HAS_ERROR2=`cat "$FC_STRESS_LOG_PREFIX""$RESULT_PORT_NUMBER".log | grep -i "not detected"`

	# need adjust
	if [[ "$HAS_ERROR" != "" ]] || [[ "$HAS_ERROR2" != "" ]] || [ "$ERROR_STATISTIC_ZERO_AMOUNT" != 2 ] || [ "$INVALID_STATISTIC_ZERO_AMOUNT" != 2 ]; then
	    exit 1
	fi
	exit 0
    fi
    sleep 2
done
