/// @file       CommonFunctions.cpp
/// @brief      Common functions for all modules in CLI layer
/// @details    Provide miscellaneous functions for all modules in CLI layer
/// @author     KenHu
/// @copyright  Copyright 2015-2016 Hon Hai Technology Group. All Rights Reserved.

#include <iostream>
#include "CommonCLIFunctions.hpp"

using namespace std;

DEBUG_LEVEL g_eDebugLevel = DEBUG_LEVEL_2;
RESULT_MESSAGE g_eResultMessage = RESULT_MESSAGE_OFF;

enum RESULT_MESSAGE getResultMessageConfig(void)
{
    return g_eResultMessage;
}

void printError(ErrorMsg *in_pErrorMsg)
{
    printError(g_eDebugLevel, in_pErrorMsg);
}

void printError(DEBUG_LEVEL in_eLevel, ErrorMsg *in_pErrorMsg)
{
    string strErrorMsg, strDebugMsg;

    if (in_pErrorMsg)
    {
        switch (in_eLevel)
        {
            case DEBUG_LEVEL_1:
                getLastErrorMessage(in_pErrorMsg, &strErrorMsg);
                cout << "Error Message: " << strErrorMsg << endl;
                break;
            case DEBUG_LEVEL_2:
                getErrorMessage(in_pErrorMsg, &strErrorMsg);
                strDebugMsg = strErrorMsg.substr(0, strErrorMsg.rfind(":"));
                strErrorMsg = strErrorMsg.substr(strErrorMsg.rfind(":") + 3, strErrorMsg.size() - strErrorMsg.rfind(":") - 4);

                cout << "Debug Message: " << strDebugMsg << endl;
                cout << "Error Message: " << strErrorMsg << endl;
                break;
            case DEBUG_LEVEL_0:
            default:
                deleteErrorMsg(in_pErrorMsg);
                break;
        }
    }
}

void setResultMessageConfig(enum RESULT_MESSAGE in_eConfig)
{
    g_eResultMessage = in_eConfig;
}
