#include "I2COperation.hpp"



I2COperation::I2COperation(OPERATION_TYPE oprationType, string helpContent) : OperationBaseClass(oprationType, helpContent)
{

}



I2COperation::~I2COperation()
{
  
}



int
I2COperation::executeOperation(StringParser command_line)
{
	  string parameter_1;
	  int    err = 0;


    command_line.getParameter(&parameter_1, 0);

    //If the first parameter is "help" or space, show help message         
    if (parameter_1 == HELP || parameter_1 == "")
    {
        Help_parameter(command_line);
        err = -1;
        return err;
    }

    //Execute operation
    switch (getOperationType())
    {
        case OPERATION_READ:
            err = read(command_line);
            break;
        case OPERATION_WRITE:
            err = write(command_line);
            break;
        case OPERATION_DUMP:
            err = dump(command_line);
            break;
        case OPERATION_MUXTest:
            err = testMUX(command_line);
            break;
        case OPERATION_PreventionStuck:
            err = stuckPrevention(command_line);
            break;
        case OPERATION_WRC:
           err = writeReadCompare(command_line);
           break;
        case OPERATION_PATH:
           err = path(command_line);
           break;
        default:
            cout << "Unsupport operation" << endl;
            err = -1; 
            break;
    }

    return err;
}



int
I2COperation::Help_parameter(StringParser command_line)
{
    string module, command, operation;

    command_line.getModule(&module);
    command_line.getCommand(&command);
    getName(&operation);

    cout << "Usage:	lemur " << module << " " << command << " " << operation << " ";
    //Show parameter help message depend on operation type
    switch (getOperationType())
    {
        case OPERATION_READ:
            cout << "P1 P2 P3" << endl;
            cout << "Read I2C device data" << endl;
            cout << "Syntax: read P1 P2 P3" << endl;
            cout << " P1: Start address" << endl;
            cout << " P2: Length" << endl;
            cout << " P3: Device name" << endl;
            break;
        case OPERATION_WRITE:
            cout << "P1 P2 P3" << endl;
            cout << "Write I2C device data" << endl;
            cout << "Syntax: write P1 P2 P3" << endl;
            cout << " P1: reg. address" << endl;
            cout << " P2: write data" << endl;
            cout << " P3: Device name" << endl;
            break;
        case OPERATION_DUMP:
            cout << "P1 P2 P3 P4" << endl;
            cout << "Dump I2C device data in specific data size" << endl;
            cout << "Syntax: dump P1 P2 P3 p4" << endl;
            cout << " P1: Start address" << endl;
            cout << " P2: Unit(1/2/4)" << endl;
            cout << " P3: Length" << endl;
            cout << " P4: Device Name" << endl;
            break;
        case OPERATION_MUXTest:
            cout << "P1" << endl;
            cout << "MUX read/write tes" << endl;
            cout << "Syntax: mux P1" << endl;
            cout << " P1: MUX device name" << endl;
            break;
        case OPERATION_PreventionStuck:
            cout << "P1" << endl;
            cout << "Bus prevention stuck test" << endl;
            cout << "Syntax: preventionstuck p1" << endl;
            cout << " P1: Bus number" << endl;
            break;
        case OPERATION_WRC:
            cout << "P1" << endl;
            cout << "Write ,read and compare test" << endl;
            cout << "Syntax: wrc p1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_PATH:
            cout << "P1" << endl;
            cout << "Show device path" << endl;
            cout << "Syntax: path p1" << endl;
            cout << " P1: Device name" << endl;
            break;
       default:
            cout << "Not define parameter help message" << endl;
            break;
    }
    
    return 0;
}



int
I2COperation::read(StringParser command_line)
{
    int      err = 0;
    int      StartAddress, Length, EndAddress, i;
    int      data[8192], count;
    I2CDAPI  *pI2CDAPI = NULL;
    ErrorMsg *error_code;
    string   WellKnownName, temp_string, error_massage;


    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 2);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get Start address
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &StartAddress);
        //cout << "Start: " << StartAddress << endl;

        //Get Length address
        command_line.getParameter(&temp_string, 1);
        command_line.convertStringToHex(temp_string, &Length);
        //Calculate the end address
        EndAddress = StartAddress + (Length - 1);
        //cout << "EndAddress: " << EndAddress << endl;

        //Creat I2C DAPI object 
        pI2CDAPI = new I2CDAPI("I2CDAPI");

        //Get I2C device data by API
        error_code = pI2CDAPI->Read(WellKnownName, StartAddress, EndAddress, data, &count);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }

        if (error_code == NO_ERROR_MSG)
        {
            for (i=0; i<count ; i++)
            {
                if (i == 0 || (i % 16) == 0 )
                {
                    printf("%04x: ", StartAddress+i);
                }
                printf("%02x ", data[i]);
                if (((i+1) % 16) == 0  && i != (count-1))
                {
                    printf("\n");
                }
            }
            printf("\n");
        }

        delete pI2CDAPI;
    }

    return err;
}



int
I2COperation::write(StringParser command_line)
{
    int      err = 0;
    int      StartAddress, i;
    int      data_buffer[8192], count;
    int      numberRealWrite = 0;
    I2CDAPI  *pI2CDAPI = NULL;
    ErrorMsg *error_code;
    string   WellKnownName, temp_string, error_massage;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 2);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get Start address
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &StartAddress);
        //cout << "Start: " << StartAddress << endl;

        //Get write data
        command_line.getParameter(&temp_string, 1);
        command_line.convertHexStringToArrary(temp_string, data_buffer, &count);

        //Creat I2C DAPI object
        pI2CDAPI = new I2CDAPI("I2CDAPI");

        //Write data to I2C device by API
        //error_code = pI2CDAPI->Write(WellKnownName, StartAddress, data_buffer, count);
        error_code = pI2CDAPI->Write(WellKnownName, StartAddress, data_buffer, count, &numberRealWrite);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }
        else
        {
            //Write successful and print the write data
            //Get I2C device data by API
            error_code = pI2CDAPI->Read(WellKnownName, StartAddress, StartAddress + (numberRealWrite - 1), data_buffer, &count);

            if (error_code != NO_ERROR_MSG)
            {
                printError(error_code);
                err = -1;
            }
            else
            {
                for (i=0; i<count ; i++)
                {
                    if (i == 0 || (i % 16) == 0 )
                    {
                        printf("%04x: ", StartAddress+i);
                    }
                    printf("%02x ", data_buffer[i]);
                    if (((i+1) % 16) == 0  && i != (count-1))
                    {
                        printf("\n");
                    }
                }
                printf("\n");
            }
        }
        delete pI2CDAPI;
    }

    return err;
}



int
I2COperation::dump(StringParser command_line)
{
    int      err = 0;
    int      StartAddress, Length, EndAddress, Unit, i;
    int      data[8192], count;
    I2CDAPI  *pI2CDAPI = NULL;
    ErrorMsg *error_code;
    string   WellKnownName, temp_string, error_massage;


    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 3);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get Start address
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &StartAddress);
        //cout << "Start: " << StartAddress << endl;

        //Get Unit size
        command_line.getParameter(&temp_string, 1);
        command_line.convertStringToHex(temp_string, &Unit);
        //cout << "Unit: " << Unit << endl;

        //Get Length address
        command_line.getParameter(&temp_string, 2);
        command_line.convertStringToHex(temp_string, &Length);
        //Calculate the end address
        EndAddress = StartAddress + (Length - 1);
        //cout << "EndAddress: " << EndAddress << endl;

        //Creat I2C DAPI object
        pI2CDAPI = new I2CDAPI("I2CDAPI");

        //Get I2C device data by API
        error_code = pI2CDAPI->Read(WellKnownName, StartAddress, EndAddress, data, &count);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }
        else
        {
            for (i=0; i<count ; i++)
            {
                if (i == 0 || (i % 16) == 0 )
                {
                    printf("%04x: ", StartAddress+i);
                }

                printf("%02x", data[i]);
                if (((i+1) % Unit) == 0)
                {
                    printf(" ");
                }

                if (((i+1) % 16) == 0  && i != (count-1))
                {
                    printf("\n");
                }
            }
            printf("\n");
        }
        delete pI2CDAPI;
    }

    return err;
}



int
I2COperation::testMUX(StringParser command_line)
{
    int      err = 0;
    string   WellKnownName, error_massage;
    I2CDAPI  *pI2CDAPI = NULL;
    ErrorMsg *error_code;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;

        //Creat I2C DAPI object
        pI2CDAPI = new I2CDAPI("I2CDAPI");

        //Excute I2C MUX test by API
        error_code = pI2CDAPI->MUX_Test(WellKnownName);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }
    }

    return err;
}



int
I2COperation::stuckPrevention(StringParser command_line)
{
    int      bus_id, err = 0;
    string   temp_string, error_massage;
    I2CDAPI  *pI2CDAPI = NULL;
    ErrorMsg *error_code;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C bus number
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &bus_id);
        //cout << "Bus: " << bus_id << endl;

        //Creat I2C DAPI object
        pI2CDAPI = new I2CDAPI("I2CDAPI");

        //Excute I2C bus stuck prevention test by API
        error_code = pI2CDAPI->StuckPrevention(bus_id);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }
        else
        {
            cout << "Pass" << endl;
            err = 0;
        }
    }

    return err;
}



int
I2COperation::writeReadCompare(StringParser command_line)
{
    int      err = 0;
    string   WellKnownName, error_massage;
    I2CDAPI  *pI2CDAPI = NULL;
    ErrorMsg *error_code;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;

        //Creat I2C DAPI object
        pI2CDAPI = new I2CDAPI("I2CDAPI");

        //Excute I2C MUX test by API
        error_code = pI2CDAPI->WriteReadTest(WellKnownName);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }
        else
        {
            cout << "Pass" << endl;
            err = 0;
        }
    }

    return err;
}




int
I2COperation::path(StringParser command_line)
{
    int      err = 0;
    string   WellKnownName, error_massage, path;
    I2CDAPI  *pI2CDAPI = NULL;
    ErrorMsg *error_code;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;

        //Creat I2C DAPI object
        pI2CDAPI = new I2CDAPI("I2CDAPI");

        //Excute I2C get device PATH by API
        error_code = pI2CDAPI->ShowRoutePath(WellKnownName, &path);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }
        else
        {
            cout << path << endl;
            err = 0;
        }
    }

    return err;
}



int
I2COperation::Syntax_check(StringParser command_line)
{
    int    i, parameter_number = 0;
    string parameter[7];
    int    err = 0;
    string operation_name;

    getName(&operation_name);

    command_line.getParameterNumber(&parameter_number);

    //Get parameter
    for (i=0; i<parameter_number; i++)
    {
        command_line.getParameter(&parameter[i], i);
    }

    switch (getOperationType())
    {
        case OPERATION_READ:
        case OPERATION_WRITE:

            //Check parameter number
            if (parameter_number != 3)
            {
                cout << "Parameter numbers isn't 3" << endl;
                return -1;
            }

            //Check parameter 1 is numeric
            err = command_line.isHEXString(parameter[0]);
            if (err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                return -1;
            }

            //Check parameter 2 is numeric
            err = command_line.isHEXString(parameter[1]);
            if (err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                return -1;
            }

            //For debug, show input value
            //cout << "P1: " << parameter[0] << endl;
            //cout << "P2: " << hex[0] << endl;
            //cout << "P3: " << hex[1] << endl;
            break;
        case OPERATION_DUMP:

            //Check parameter number
            if (parameter_number != 4)
            {
                cout << "Parameter numbers isn't 4" << endl;
                return -1;
            }

            //Check parameter 1 is numeric
            err = command_line.isHEXString(parameter[0]);
            if (err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                return -1;
            }

            //Check parameter 2 is numeric
            err = command_line.isHEXString(parameter[1]);
            if (err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                return -1;
            }

            if (parameter[1] != "1" && parameter[1] != "2" && parameter[1] != "4")
            {
                cout << "parameter 2 numeric shoule be only 1/2/4 ." << endl;
                return -1;
            }

            //Check parameter 3 is numeric
            err = command_line.isHEXString(parameter[2]);
            if (err != 0)
            {
                cout << "parameter 3 isn't numeric." << endl;
                return -1;
            }

            break;
        case OPERATION_MUXTest:
        case OPERATION_PATH:
        case OPERATION_WRC:
            //Check parameter number
            if (parameter_number != 1)
            {
                cout << "Parameter numbers isn't 1" << endl;
                return -1;
            }
            break;
        case OPERATION_PreventionStuck:
            //Check parameter number
            if (parameter_number != 1)
            {
                cout << "Parameter numbers isn't 1" << endl;
                return -1;
            }

            //Check parameter 2 is numeric
            err = command_line.isHEXString(parameter[0]);
            if (err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                return -1;
            }

            break;

       default:
            cout << "Not define syntax_check for " << operation_name << endl;
            err = -1;
            break;
    }

    return err;
}
