#!/bin/bash
#1.0 
#	Add help and version to command options 
#	fixed iblog creation: sn had trailing space like "0 "
VERSION=1.0
mode=0
run=0
myip=""
himip=""
timeout=30
size=800000
iters=2000
port=0
trytime=0
trytimeout=10
path="/etc/opensm/opensm.conf"
GPIO="/usr/local/Foxconn/CLI/pchgpiotool"
TEM="/usr/local/Foxconn/CLI/sptest"
ibw="/usr/bin/ib_write_bw"
#sn="`dmidecode -s baseboard-serial-number`"
sn="`$GPIO -r 22 | sed -n 1p`"
#Need the following command to get rid of trailing space
sn=`printf %d $sn`
# set log file
iblog="/usr/local/Foxconn/CLI/tmp/IB-$sn.log"
ibinfo="/usr/local/Foxconn/CLI/tmp/IB-PORT"

declare FAS=`dmidecode -s baseboard-product-name`
readonly PROGRAM="ibtest"
readonly VERSION="1.1"
readonly RELEASEDATE="10/10/2013 16:45:06"
readonly AUTHOR="Ian Fang"
readonly COPYRIGHT="Copyright(c)2005-2011 Foxconn IND., Group."

# identify server or client
id=`$GPIO -r 22 | sed -n 1p`

PORT=$1

function usage()
{
    echo "Usage: $0 [option]"
    echo "[option]: "
    echo "    -h : help"
    echo "    -V : version"
    echo "    -S [PORT1|PORT2]: server mode"
    echo "    -C [PORT1|PORT2]: client mode"
    echo "    -E : Transmit/Receive Quantity Test"
}

function showpf() 
{
    echo
    printf "$TITLE "
    tlen=`expr length "$TITLE"`
    while (($tlen < 59)); do
        printf "-"
        let tlen+=1
    done
    if [ $1 -eq 0 ]; then
        printf " [ PASS ]\n"
    else
        printf " [ FAIL ]\n"
    fi
    echo
}

function version_show()
{
    echo
    echo "Program Name:   $PROGRAM"
    echo "version:        $VERSION"
    echo "Release date:   $RELEASEDATE"
    echo "Author:         $AUTHOR"
    echo "$COPYRIGHT"
    echo
}

function showdash()
{
    i=0
    while (($i < 66)); do
        printf "-"
        let i+=1
    done
    echo
}

# show time in sync and in/out flow temperature
function showpt()
{
    showdash
    tmp="/opt/tmp/$sn"
    touch $tmp
    Tm="`ls --full-time $tmp | awk '{print $6,$7}'`"
    Ti="`$TEM -g 8 0x00 | sed -n 1p`"
    To="`$TEM -g 8 0x01 | sed -n 1p`"
    printf "%-13s %-18s %s\n" "S/N" "I/O Temperature" "start-time"
    printf "%-13s %-18s %s\n" "$sn" "$Ti/$To" "$Tm"
    rm -fr $tmp
}

function showdmesg()
{
    echo "Debug-message: "
    dmesg | tail -n 30|grep ib0
}

function chklogs()
{
    str="`grep $size $iblog`"
    #printf "str /: $str\n"
    if [ -z "$str" ]; then
        return 1
    fi

    tmp="`echo $str | grep -ic $iters`"
    #printf "tmp: $tmp\n"
    if [ $tmp -eq 0 ]; then
        return 2
    fi

    tmp="`echo $str | awk '{print $4}'`"
    if [ $(echo "$tmp < 1300" | bc) -eq 1 ]; then
        return 3
    fi

    #tmp="`echo $str | awk '{print $5}'`"
    #printf "tmp: $tmp\n"
    #if [ $(echo "$tmp < 1300" | bc) -eq 1 ]; then
    #    return 4
    #fi

    return 0
}

function overtime()
{
    i=0
    while (($i < $timeout)); do
        pid="`pgrep ib_write_bw`"
        if [ -z "$pid" ]; then
            return 0
        fi
        sleep 1
        let i+=1
    done

    if [ ! -z "$pid" ]; then
        kill -9 $pid 2>&1 > /dev/null
    fi
    return 5
}

function run_client()
{
    sleep 5
    retry=0
    while (($retry < 3)); do
        #        showpt
        #echo "PORT=$PORT"
        $ibw -i $PORT -s $size -b -n $iters $himip -p $port -F | tee -a $iblog
        showdmesg
        chklogs
        err=$?
        if [ $err -eq 0 ]; then
            break
        fi
        let retry+=1
        echo "-=-=-=-=-=-=-=- Retry $retry -=-=-=-=-=-=-=-"
        sleep 20
    done
    #    showpt
    showdash

    return $err
}

function run_server()
{
    #    showpt
    #    echo "PORT=$PORT"
    $ibw -i $PORT -s $size -b -n $iters -p $port -F| tee -a $iblog & > IB.log
    overtime
    err=$?
    if [ $err -eq 0 ]; then
        chklogs
        err=$?
    fi

    printf "err : $err"
    showdmesg
    showdash

    return $err
}

function initialize()
{
    err=0

    if [  $FAS = FAS8080 ]; then
        port=18700
        err=0
        ib_idx=$PORT
        let "ib_idx-=1"
        let "ib_idx%=2"
        if [ $PORT -eq 1 ]; then
            myip="30.0.0.1"
            himip="30.0.0.2"
        elif [ $PORT -eq 2 ]; then 
            myip="30.0.0.2"
            himip="30.0.0.1"
        else 
            echo "port $PORT not valid valid ports: 1,2"
            return 7
        fi
        tmp="`ifconfig ib$ib_idx | grep 'inet addr' | awk '{gsub(":"," ");print $3}'`"
        if [ "$tmp" != "$myip" ]; then
            ifconfig ib$ib_idx $myip/24
            sleep 2
        fi
        showdash
        printf "%-13s %-18s %-18s %s\n" "Unit-ID" "Local" "Partner" "Port"
        printf "%-13s %-18s %-18s %s\n" "$id" "$myip" "$himip" "$port"

    else
        while [ -z "$id" ] && [ $err -lt 3 ]; do
            sleep 2
            id="`$GPIO -r 22 | sed -n 1p`"
            let err+=1
        done
        if [ -z "$id" ]; then
            echo
            echo "Fail in getting unit's id when read gpio 47 register!!!"
            return 7
        fi
        err=$run$id
        if [ $err -eq 10 ] || [ $err -eq 21 ]; then
            port=18800
        elif [ $err -eq 11 ] || [ $err -eq 20 ]; then
            port=18700
        else 
            # if $run != 1 OR 2, then program can not get here;
            # if $id != 0 OR 1, then $err will be a random string, like: "2xxxx" OR "1xxxx".
            echo
            echo "The unit's ID is incorrect!!!"
            return 8
        fi

        err=0
        if [ $PORT -eq 1 ]; then
            ip=`expr $id + 1`
            myip="30.0.0.$ip"
            tmp="`ifconfig ib0 | grep 'inet addr' | awk '{gsub(":"," ");print $3}'`"
            if [ "$tmp" != "$myip" ]; then
                ifconfig ib0 $myip/24
                sleep 2
            fi

            fid=$(echo "$id == 0"|bc)
            ip=`expr $fid + 1`
            himip="30.0.0.$ip"
            showdash
            printf "%-13s %-18s %-18s %s\n" "Unit-ID" "Local" "Partner" "Port"
            printf "%-13s %-18s %-18s %s\n" "$id" "$myip" "$himip" "$port"
        elif [ $PORT -eq 2 ]; then
            ip=`expr $id + 1`
            myip="20.0.0.$ip"
            tmp="`ifconfig ib1 | grep 'inet addr' | awk '{gsub(":"," ");print $3}'`"
            if [ "$tmp" != "$myip" ]; then
                ifconfig ib1 $myip/24
                sleep 2
            fi

            fid=$(echo "$id == 0"|bc)
            ip=`expr $fid + 1`
            himip="20.0.0.$ip"
            showdash
            printf "%-13s %-18s %-18s %s\n" "Unit-ID" "Local" "Partner" "Port"
            printf "%-13s %-18s %-18s %s\n" "$id" "$myip" "$himip" "$port"
        else
            echo "Error Port !"
        fi
    fi
    return 0
}

function checkservice()
{
    isstart="`service openibd status | grep -ic 'ib0'`"
    linenum=0
    if [ $isstart -eq 0 ]; then
        /etc/init.d/openibd start
        sleep 2 
    fi
    #stand by IB link
    echo -ne "Linking..."
    trytime=0
    while [[ $linenum -ne 2 && $trytime -lt $trytimeout ]]; do
        linenum="`ibstatus | grep -ic 'LinkUp'`"
        echo -ne "."
        sleep 1
        let trytime+=1
        if [ $trytime -eq $trytimeout ]; then
            printf "\nThe IB Link FAILED!!\n" 
            exit -1    
        fi
    done

    echo ""
    isstart="`service opensmd status | grep -ic 'running'`"
    linenum="`ibstatus | grep -ic 'ACTIVE'`"
    if [ $isstart -eq 1 ] && [ $linenum -ne 2 ]; then
        /etc/init.d/opensmd stop
    fi
    #getguid
    sleep 2

    # /etc/init.d/openibd restart > /dev/null
    isstart="`service opensmd status | grep -ic 'stopped'`"
    if [ $isstart -ne 0 ]; then
        if [ $id -eq 0 ]; then
            PCMA="`ibstat -p | awk NR==1`"
            opensm -g $PCMA -B
        else
            PCMB="`ibstat -p | awk NR==2`"
            opensm -g $PCMB -B
        fi
        #/etc/init.d/opensmd start
        sleep 2
    fi
    service openibd status
    service opensmd status
}

function stopTransmit()
{
    #echo "Stop Transmit!!"
    if [ `lsmod | grep ib_mthca -c` -ne 0 ]; then
        /sbin/rmmod ib_mthca > /dev/null
        sleep 2
    fi
}

function saveFlowInfo()
{
    #echo "Save Port Information"
    PortNum=`expr $id + 1`
    #echo "Port Number : $PortNum"
    #echo "ibinfo : $ibinfo$PortNum"
    perfquery -P $PortNum > $ibinfo$PortNum  
}

function standbyEachOther()
{
    echo "standby function"
    tic=0
    timer=20
    partner=`expr $(echo "$id == 0"|bc) + 1`
    #echo "partnet : $partner"
    echo -n "Wait Partner Create File..."
    while [ $tic -lt $timer ] && [ ! -e $ibinfo$partner ]; do
        #test -e $ibinfo$partner && echo "exist" || echo "Not exist"
        #echo "Path : $ibinfo$partner"
        sleep 1
        let tic+=1
        echo -n "."
        ls tmp > /dev/null
    done
}

function cmpFlowInfo()
{
    ret=0
    if  cmpXmitData; then
        ret=1
    fi
    if cmpRcvData; then
        ret=2
    fi
    if cmpXmitPkt; then
        ret=3
    fi
    if cmpRcvPkt; then
        ret=4
    fi

    return $ret
}

function deleteFile()
{
    rm $ibinfo$partner
}
function cmpXmitData()
{
    partner=`expr $(echo "$id == 0"|bc) + 1`
    #echo "partner : $partner"
    Exe=`/usr/sbin/perfquery -P $partner | grep PortXmitData | cut -c 34- `
    #echo "Program get PORT:$partner XmitData : $Exe"
    file=`cat $ibinfo$partner | grep PortRcvData | cut -c 34- `
    #echo "Get PORT:$partner RcvData from File : $file"

    if [ $Exe -eq $file ]; then
        return 1 
    else
        echo "Port $partnet Transmit Data Quantity not Match"
        return 0
    fi
}

function cmpRcvData()
{
    Exe=`/usr/sbin/perfquery -P $partner | grep PortRcvData | cut -c 34- `
    #echo "Program get PORT:$partner RcvData : $Exe"
    file=`cat $ibinfo$partner | grep PortXmitData | cut -c 34- `
    #echo "Get PORT:$partner XmitData from File : $file"

    if [ $Exe -eq $file ]; then
        return 1
    else
        echo "Port $partnet Receive Data Quantity not Match"
        return 0
    fi
}

function cmpXmitPkt()
{
    #partner=`expr $(echo "$id == 0"|bc) + 1`
    #echo "partnet : $partner"
    Exe=`/usr/sbin/perfquery -P $partner | grep PortXmitPkts | cut -c 34- `
    #echo "Program get PORT:$partner XmitPkts : $Exe"
    file=`cat $ibinfo$partner | grep PortRcvPkts | cut -c 34- `
    #echo "Get PORT:$partner RcvPkts from File : $file"

    if [ $Exe -eq $file ]; then
        return 1 
    else
        echo "Port $partner Transmit Packet Quantity not Match"
        return 0
    fi
}

function cmpRcvPkt()
{
    Exe=`/usr/sbin/perfquery -P $partner | grep PortRcvPkts | cut -c 34- `
    #echo "Program get PORT:$partner RcvPkts : $Exe"
    file=`cat $ibinfo$partner | grep PortXmitPkts | cut -c 34- `
    #echo "Get PORT:$partner XmitPkts from File : $file"

    if [ $Exe -eq $file ]; then
        return 1
    else
        echo "Port $partnet Receive Packet Quantity not Match"
        return 0 
    fi
}



while getopts EC:S:s:n:p:hV OPTIONS 2>/dev/null
do
    case $OPTIONS in
        h)
            usage
            exit 0
            ;;
        V)
            version_show 
            exit 0
            ;;
        E)
            mode=1
            ;;
        C)
            run=1
            PORT=$OPTARG
            ;;
        S)
            run=2
            PORT=$OPTARG
            ;;
        s)
            size=$OPTARG
            ;;
        n)
            iters=$OPTARG
            ;;
        p)
            port=$OPTARG
            ;;
        \?)
            run=0
            ;;
    esac
done
shift

if [ $mode -eq 0 ]; then
    TITLE="IB-chip stress test"
    if (($run != 1 && $run != 2)); then
        usage
        exit -1
    fi

    #start mst
    if [ ! -d /dev/mst ]; then
        mst start
    fi

    # check services
    checkservice

    # init: setting ip, define comm-port, etc.
    initialize
    err=$?
    if [ $err -ne 0 ]; then
        showpf "$err"
        exit $err
    fi

    rm -fr $iblog

    if [ $run -eq 1 ]; then
        TITLE="Client : IB-chip stress test"
        run_client
    else
        TITLE="Server : IB-chip stress test"
        run_server
    fi

    err=$?
    showpf "$err"
    #rm -fr $iblog

    printf "\n"
    #/etc/init.d/opensmd stop
    #/etc/init.d/openibd stop

    exit $err
fi
if [ $mode -eq 1 ]; then
    TITLE="IB Xmit/Rcv Quantity Test"

    #stop flow transmit
    stopTransmit

    #save flow information
    saveFlowInfo

    #wait each other
    standbyEachOther

    #compare flow information
    cmpFlowInfo
    err=$?
    #echo "err : $err"
    showpf "$err"

    #delete /usr/local/Foxconn/CLI/tmp  IB-PORT1 and IB-PORT2
    deleteFile
    /etc/init.d/opensmd stop > /dev/null
    /etc/init.d/openibd stop > /dev/null
fi




