/// @file       MemoryTestInfo.hpp
/// @brief      MemoryTestInfo class
/// @details    Provide testing information for memory test item
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.
#ifndef MEMORY_TEST_INFO
#define MEMORY_TEST_INFO

#include "MemoryDAPI.hpp"

/// @class      MemoryTestInfo MemoryTestInfo.hpp "MemoryTestInfo.hpp"
/// @brief      Memory testing informaton
/// @param      m_pMemoryDAPI is a pointer to memory DAPI
/// @param      m_strTestItem is a string for test item
/// @param      m_testRange is a memory range for testing
/// @param      m_s32DataSize is a 32 bits value for size of data
/// @param      m_u64TestPattern is a 64 bits value for test pattern
/// @param      m_u32TestTime is a 32 bits value for testing time
/// @param      m_strErrorMsg is string for error messages
class MemoryTestInfo
{
    public:
                        MemoryTestInfo(void) 
                        {
                            m_pMemoryDAPI = NULL;
                            m_pErrorMsg = NO_ERROR_MSG;
                            m_bIsTestDone = FALSE;
                        }
                        
                        MemoryTestInfo(string in_strItemName) 
                        {
                            m_strTestItem = in_strItemName;
                            m_pMemoryDAPI = NULL;
                            m_pErrorMsg = NO_ERROR_MSG;
                            m_bIsTestDone = FALSE;
                        }
                        
        virtual         ~MemoryTestInfo(void) 
                        {
                            if (m_pMemoryDAPI)
                            {
                                delete m_pMemoryDAPI;
                            }
                        }

        MemoryDAPI*     m_pMemoryDAPI;
        
        string          m_strTestItem;
        MemoryRange     m_testRange;
        INT32           m_s32TargetController;
        INT32           m_s32TargetChannel;
        INT32           m_s32TargetRank;
        UINT64          m_u64TargetTestSize;
        INT32           m_s32DataSize;
        UINT64          m_u64TestPattern;
        UINT32          m_u32TestTime;

        ErrorMsg*       m_pErrorMsg;

        BOOL            m_bIsTestDone;
    protected:
    private:
};

#endif

