#ifndef PCIOperation_H
#define PCIOperation_H

#include "PCIDAPI.hpp"
#include "MemoryDAPI.hpp"
#include "OperationBaseClass.hpp"

#include <sstream>
#include <string>
using namespace std;

//#define OPERATION_PCI_STRING            "pci"
//#define OPERATION_EEPROM_STRING         "eeprom"
//#define OPERATION_HBA_STRING            "hba"
#define OPERATION_RETRAIN_LINK_STRING   "retrainlink"
#define OPERATION_RETRAIN_TEST_STRING   "retrain"

#define PCIE_SWITCH_EEPROM_IMAGE_SIZE   512*1024

//#define PCI_DEBUG_LEVEL     DEBUG_LEVEL_1
#define PCI_DEBUG_LEVEL     DEBUG_LEVEL_2 //debug mode

class PCIOperation : public OperationBaseClass
{
    public:
                PCIOperation(OPERATION_TYPE, string);
        virtual ~PCIOperation();

        int executeOperation(StringParser);

    private:
        int             m_s32Result;
        int             m_s32TotalSteps;
        int             m_s32ParmCount;
        string          m_comman_name;
        string          m_strParameter[16];
        ErrorMsg*       m_pErrorMsg;
        MemoryDAPI*     m_memoryDAPI;
        PCIDAPI*        m_pciDAPI;
        OPERATION_TYPE  m_operation_type;

        void    showHelp();
        int     showPciInfo(StringParser);
        int     showSpecificPciInfo(PFA, OPERATION_TYPE);
        bool    isHex(STRING);
        bool    isBDF(string);
        
        int     checkBDF(string);
        int     convertStringToBDF(string, PFA*);
        int     convertStringToBytes(string   in_string,
                                     UINT8*   out_u8pBuffer);
        int     convertStringToDWords(string      in_string,
                                      UINT32*     out_u32pBuffer);

        int     checkSyntax(StringParser);
        int     checkEepromName(string strEepromName);
        int     checkEepromReadSyntax(string  *strParameters);
        int     checkEepromWriteSyntax(string  *strParameters);
        int     checkSyntaxForImageFunctions(string  *strParameters);

        int showPciErrorStatistics();
        //int showSpecificErrorStatistics(PFA,
        //                                OPERATION_TYPE,
        //                                bool  in_showSummary);

        int showPlxNtbErrorStatistics(PFA               in_bdf,
                                      OPERATION_TYPE    in_op,
                                      UINT8             in_u8Interface);

        int clearErrors();

        void showConfig(UINT8* in_u8pBuffer, 
                        UINT32 in_u32StartOffset,
                        UINT32 in_u32Length);

        int readConfig(StringParser in_classCommandLine);
        int writeConfig(StringParser in_classCommandLine);


        int dumpEeprom(StringParser in_classCommandLine);
        int writeEeprom(StringParser in_classCommandLine);
        int readEeprom(StringParser in_classCommandLine);
        int testEeprom(StringParser in_classCommandLine);

        int readEepromRun(string   in_strEepromName,
                          UINT32   in_u32Unit,
                          UINT32   in_u32Address,
                          UINT32   in_u32BufferSize,
                          UINT32  *out_u32pBuffer);

        int programEeprom(StringParser in_classCommandLine);
        int saveEeprom(StringParser in_classCommandLine);

        int showEepromContent(UINT32  in_u32Unit,
                              UINT32  in_u32StartAddress,
                              UINT32  in_u32BufferSize, //Dword aligned
                              UINT32* in_u32pBuffer);

        void showEepromContentInWord(UINT32  in_u32StartAddress,
                                     UINT32  in_u32BufferSize, //Dword aligned 
                                     UINT32* in_u32pBuffer);

        void showEepromContentInByte(UINT32  in_u32StartAddress,
                                     UINT32  in_u32BufferSize, //Dword aligned 
                                     UINT32* in_u32pBuffer);

        int testRetrain(StringParser);
        void showTestRetrainResult(vector<TEST_RESULT>);

        void progressHandler();

        int searchHiddenDevices(StringParser);

        int listDevices(StringParser);
        int listPciDevices(StringParser);
        int showPciDevicesByClass(string in_strClass);
        void showPciDevices();

        void showRetrainInfo(StringParser);

        int showEepromFW(StringParser);

        int listPCIeSwitch();
};


#endif
