
#include "PSUCommand.hpp"

PSUCommand::PSUCommand(COMMAND_TYPE   in_eCommandType,
                       string         in_strHelpContent) : CommandBaseClass(in_eCommandType, in_strHelpContent)
{
    switch (in_eCommandType)
    {
        case CMD_LIST:
            addOperation(new PSUOperation(OPERATION_PSU, "List supported PSUs"));
            addOperation(new PSUOperation(OPERATION_FAN, "List supported FANs of PSUs"));
            addOperation(new PSUOperation(OPERATION_LED, "List supported LEDs of PSUs"));
            addOperation(new PSUOperation(OPERATION_TEMP, "List supported temperature sensors of PSUs"));
            addOperation(new PSUOperation(OPERATION_THRESHOLD, "List supported thresholds of PSU temperatures"));
            break;
        case CMD_SHOW:
            addOperation(new PSUOperation(OPERATION_ALL, "Show all information of PSU"));
            addOperation(new PSUOperation(OPERATION_PROTOCOL_TYPE, "Show type of PSU interface"));
            addOperation(new PSUOperation(OPERATION_VENDOR, "Show vendor name of PSU"));
            addOperation(new PSUOperation(OPERATION_FW, "Show FW version of PSU"));
            addOperation(new PSUOperation(OPERATION_PN, "Show part number of PSU"));
            addOperation(new PSUOperation(OPERATION_SN, "Show serial number of PSU"));
            addOperation(new PSUOperation(OPERATION_MAX_POWER, "Show maximum power rating of PSU"));
            break;
        case CMD_FAN:
            addOperation(new PSUOperation(CMD_FAN, OPERATION_GET, "Display the current speed of a FAN"));
            addOperation(new PSUOperation(CMD_FAN, OPERATION_SET, "Set speed to a FAN"));
            addOperation(new PSUOperation(OPERATION_RANGE, "Display the supported speed range of a FAN"));
            break;
        case CMD_LED:
            addOperation(new PSUOperation(CMD_LED, OPERATION_GET, "Display the current status of an LED"));
            addOperation(new PSUOperation(CMD_LED, OPERATION_SET, "Turn on/off LED"));
            break;
        case CMD_TEMP:
            addOperation(new PSUOperation(CMD_TEMP, OPERATION_GET, "Display the current status of a temperature sensor"));
            addOperation(new PSUOperation(CMD_TEMP, OPERATION_SET, "Set threshold to a temperature sensor"));
            break;
        case CMD_FW:
            addOperation(new PSUOperation(OPERATION_PROGRAM, "Program PSU's FW"));
            break;
        default:
            break;
    }
}

PSUCommand::~PSUCommand()
{

}

int
PSUCommand::executeCommand(StringParser in_parser)
{
    int             error = 0;
	COMMAND_TYPE    eCommand = NO_COMMAND;
	string          strOperation;
	string          strTemp;
	int             i = 0;

	in_parser.getCommandType(&eCommand);

    switch (eCommand)
    {
        case CMD_LIST:
        case CMD_SHOW:
        case CMD_FAN:
        case CMD_LED:
        case CMD_TEMP:
        case CMD_FW:
            strOperation.clear();

            in_parser.getOperation(&strOperation);

            for (i = 0; i< m_operation_count; i++)
            {
                if (m_operation[i] == NULL)
                    break;

                m_operation[i]->getName(&strTemp);
                if (strOperation == strTemp)
                {
                    return m_operation[i]->executeOperation(in_parser);
                }
            }

            // Show supported operations because operation can not be found
            listOperation(in_parser);

            error = -1;
            break;
        default:
            cout << "Not supported command" << endl;
            error = -1;
            break;
    }

    return error;
}
