#ifndef SAS_OPERATION_H
#define SAS_OPERATION_H

#include <stdio.h>
#include <iostream>
#include <string>
#include <thread>
#include "OperationBaseClass.hpp"
#include "PCIDAPI.hpp"
#include "SASDAPI.hpp"
#include "StorageDAPI.hpp"

class SASOperation: public OperationBaseClass
{
    public:
        SASOperation(OPERATION_TYPE, string);
        virtual ~SASOperation();

        //Decide which operation to be executed
        int executeOperation(StringParser);

        //List SAS device node
        int executeListIOC(StringParser);

        //List SAS device node
        int executeListDrive(StringParser);

        //Show SAS controller characteristics
        int executeShowController(StringParser);

        //Show SAS controller characteristics
        int executeShowPHY(StringParser);

        //Show SAS controller characteristics
        int executeShowDrive(StringParser);

        //Execute Error command
        INT32 executeError(StringParser);

        //Execute clear Error command
        INT32 executeClearError(StringParser);

        //Execute Utility command
        INT32 executeUtility(StringParser);

        //Execute test command
        int executeTest(StringParser);

        //Execute program command
        int executeProgram(StringParser);

        //Show parameter message here
        int helpParameter();

        //Check parameter syntax here
        int checkSyntax(StringParser);

    private:
        OPERATION_TYPE operationType;
        PCIDAPI* pciDAPI;
        SASDAPI* sasDAPI;
        StorageDAPI* storageDAPI;
        thread* progressHandler;
        void handleTestProgress(UINT32 in_u32TestSizeMB);
        // Format integer with thousands separator
        const char *format_with_thousands_sep(char * str, int strsize, uint64_t val, const char * thousands_sep = 0);
        // Format capacity with SI prefixes
        const char *format_capacity(char * str, int strsize, uint64_t val, const char * decimal_point = 0);


};

#endif
