#include "TemperatureOperation.hpp"



TemperatureOperation::TemperatureOperation(OPERATION_TYPE OprationType, string helpContent) : OperationBaseClass(OprationType, helpContent)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    //m_pTemperatureDAPI = NULL;
    switch (OprationType)
    {
        case OPERATION_ALL:
            setName("all");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_TEMPERATURE:
            setName("temp");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_TEMPERATURE_THRESHOLD_HI0:
            setName("threshold_hi0");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_TEMPERATURE_THRESHOLD_HI1:
            setName("threshold_hi1");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_TEMPERATURE_THRESHOLD_LOW0:
            setName("threshold_low0");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_TEMPERATURE_THRESHOLD_LOW1:
            setName("threshold_low1");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_READ:
            setName("read");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_WRITE:
            setName("write");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        default:
            break;
    }
}



TemperatureOperation::~TemperatureOperation()
{
    /*
  if (m_pTemperatureDAPI != NULL)
      delete m_pTemperatureDAPI;
      */
}



int
TemperatureOperation::executeOperation(StringParser CommandLine)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    string parameter1;
    COMMAND_TYPE CommandType;
    int    err = 0;

    CommandLine.getParameter(&parameter1, 0);
    CommandLine.getCommandType(&CommandType);

    //If the first parameter is "help" or space, show help message
    if (CommandType == CMD_LIST)
    {
        parameter1 = NO_OPRATION;

    }
    else
    {
        if (parameter1 == HELP || parameter1 == "")
        {
            helpParameter();
            err = -1;
            return err;
        }
    }

    //Execute operation
    switch (m_OperationType)
    {
        case NO_OPRATION:
            err = executeList(CommandLine);
            break;
        case OPERATION_ALL:
        case OPERATION_TEMPERATURE:
        case OPERATION_TEMPERATURE_THRESHOLD_HI0:
        case OPERATION_TEMPERATURE_THRESHOLD_HI1:
        case OPERATION_TEMPERATURE_THRESHOLD_LOW0:
        case OPERATION_TEMPERATURE_THRESHOLD_LOW1:
            err = executeShow(CommandLine);
            break;

        case OPERATION_READ:
        case OPERATION_WRITE:
            err = executeUtility(CommandLine);
            break;
        default:
            cout << "Unsupport operation" << endl;
            err = -1; 
            break;
    }

    return err;
}



int
TemperatureOperation::helpParameter()
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    //Show parameter help message depend on operation type
    switch (m_OperationType)
    {
        case OPERATION_ALL:
        case OPERATION_TEMPERATURE:
        case OPERATION_TEMPERATURE_THRESHOLD_HI0:
        case OPERATION_TEMPERATURE_THRESHOLD_HI1:
        case OPERATION_TEMPERATURE_THRESHOLD_LOW0:
        case OPERATION_TEMPERATURE_THRESHOLD_LOW1:
            cout << "Show Temperature sensor characteristic" << endl;
            cout << "Syntax: show P1 P2" << endl;
            cout << "Operation parameter:" << endl;
            cout << "P1: Threshold number" << endl;
            cout << "P2: Sensor Name" << endl;
            break;
/*
        case READ:
            cout << "Read Temperature device data" << endl;
            cout << "Syntax: read P1 P2 P3" << endl;
            cout << "P1: Device name" << endl;
            cout << "P2: Start address" << endl;
            cout << "P3: End address" << endl;
            break;
        case WRITE:
            cout << "Write Temperature device data" << endl;
            cout << "Syntax: write P1 P2 P3" << endl;
            cout << "P1: Device name" << endl;
            cout << "P2: Start address" << endl;
            cout << "P3: Data" << endl;
            break;
*/
        case OPERATION_READ:
            cout << "Temperature sensor Read" << endl;
            cout << "Syntax: read P1 P2" << endl;
            cout << "P1: Threshold number" << endl;
            cout << "P2: Sensor Name" << endl;
            break;
        case OPERATION_WRITE:
            cout << "Temperature Sensor Write" << endl;
            cout << "Syntax: writep P1 P2 P3" << endl;
            cout << "P1: Threshold number" << endl;
            cout << "P2: Data (Decimal)" << endl;
            cout << "P3: Sensor Name" << endl;
            break;
       default:
            break;
    }
    
    return 0;
}


int TemperatureOperation::executeList(StringParser CommandLine)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    INT32 err = 0;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    list<string>  TemperatureDevicelist;
    TemperatureDevicelist.clear();

    string  strErrorMessage;
    strErrorMessage.clear();

    m_pTemperatureDAPI = new TemperatureDAPI("TemperatureDAPI");

    err = checkSyntax(CommandLine);
    if (err == 0)
    {
        pErrorMsg = m_pTemperatureDAPI-> displayTemperatureNameList(&TemperatureDevicelist);

        //pErrorMsg = m_pTemperatureDAPI->displayTemperatureDevicesList(&TemperatureDevicelist);
        if( pErrorMsg == NO_ERROR_MSG)
        {
            while (!TemperatureDevicelist.empty())
            {
                cout << TemperatureDevicelist.front() << endl;
                TemperatureDevicelist.pop_front();
            }
        }
        else
        {
            getErrorMessage(pErrorMsg, &strErrorMessage);
            //cout << "Error:" << strErrorMessage.c_str() << endl;
            //getErrorMsg();
        }

    }
    TemperatureDevicelist.clear();




    return err;
}


int
TemperatureOperation::executeShow(StringParser CommandLine)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    int err = 0;
    string TargetName;
    TargetName.clear();

    TEMPERATURE_DATA_ELEMENT TemperatureDataElement;
    list<string>  Infolist;

//    UINT32 intLogicalID = -1;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string  strErrorMessage;
    strErrorMessage.clear();


    m_pTemperatureDAPI = new TemperatureDAPI("TemperatureDAPI");

    err = checkSyntax(CommandLine);
    if (err == 0)
    {
        CommandLine.getParameter(&TargetName, 0);
        //printf("2nd Parameter:%s\n", TargetName.c_str());


        switch(m_OperationType)
        {
            case OPERATION_ALL:
                TemperatureDataElement=TEMPERATURE_DATA_ALL_INFO;
                break;
            case OPERATION_TEMPERATURE:
                TemperatureDataElement=TEMPERATURE_DATA_CURRENT_TEMPERATURE;
                break;
            case OPERATION_TEMPERATURE_THRESHOLD_HI0:
                TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_HIGH0;
                break;
            case OPERATION_TEMPERATURE_THRESHOLD_HI1:
                TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_HIGH1;
                break;
            case OPERATION_TEMPERATURE_THRESHOLD_LOW0:
                TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_LOW0;
                break;
            case OPERATION_TEMPERATURE_THRESHOLD_LOW1:
                TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_LOW1;
                break;
            default:
                return -1;
                break;
        }

        //printf("Operation-executeShow-LogicalID=%x\n",intLogicalID );
        //transform(TargetName.begin(), TargetName.end(), TargetName.begin(),(int (*)(int))tolower);
        pErrorMsg = m_pTemperatureDAPI->getDataByName(TargetName.c_str(), TemperatureDataElement, &Infolist);
        if( pErrorMsg == NO_ERROR_MSG)
        {
            while (!Infolist.empty())
            {
				cout << Infolist.front() << endl;
                Infolist.pop_front();
            }
        }
        else
        {
            getErrorMessage(pErrorMsg, &strErrorMessage);
            cout <<  strErrorMessage.c_str() << endl;
            return -1;
        }

        Infolist.clear();
    }

    return err;
}

int
TemperatureOperation::executeUtility(StringParser CommandLine)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    int err = 0;


    string TargetName;
    TargetName.clear();

    string Temp;
    Temp.clear();

    list<string>  Infolist;

    string SetValue;
    SetValue.clear();

    CHAR    TempString[256];

    //BOOL    Status = TRUE;
    //UINT32 intLogicalID = -1;
    //ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

    TEMPERATURE_DATA_ELEMENT TemperatureDataElement;

    string  strErrorMessage;
    strErrorMessage.clear();

    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

    m_pTemperatureDAPI = new TemperatureDAPI("TemperatureDAPI");

    //m_pGPIODAPI = new GPIODAPI("GPIODAPI");


    err = checkSyntax(CommandLine);
    if (err == 0)
    {
        switch(m_OperationType)
        {
            case OPERATION_READ:
                CommandLine.getParameter(&Temp, 0);
                sprintf(TempString, "%s", Temp.c_str());
                ConvertStringLowerCase(TempString);
                Temp = TempString;
                if(strcmp(Temp.c_str(),"all") == 0)
                {
                	TemperatureDataElement=TEMPERATURE_DATA_ALL_INFO;
                }
                else if(strcmp(Temp.c_str(), "temp") == 0)
        		{
       				TemperatureDataElement=TEMPERATURE_DATA_CURRENT_TEMPERATURE;
        		}
        		else if(strcmp(Temp.c_str(), "threshold_hi0") == 0)
        		{
        			TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_HIGH0;
        		}
        		else if(strcmp(Temp.c_str(), "threshold_hi1") == 0)
        		{
        			TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_HIGH1;
        		}
        		else if(strcmp(Temp.c_str(), "threshold_low0") == 0)
        		{
        			TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_LOW0;
        		}
        		else if(strcmp(Temp.c_str(), "threshold_low1") == 0)
        		{
        			TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_LOW1;
        		}
        		else
        		{
					return -1;
					break;
        		}
                //printf("TargetName=%s\n", TargetName.c_str());
                //printf("TemperatureDataElement=%d\n", TemperatureDataElement);
        		CommandLine.getParameter(&TargetName, 1);

                pErrorMsg = m_pTemperatureDAPI->getDataByName(TargetName.c_str(), TemperatureDataElement, &Infolist);
                if(pErrorMsg != NO_ERROR_MSG)
                {
                    getErrorMessage(pErrorMsg, &strErrorMessage);
                    cout <<  strErrorMessage.c_str() << endl;
                    return -1;
                }

                if( pErrorMsg == NO_ERROR_MSG)
                {
                    while (!Infolist.empty())
                    {
        				cout << Infolist.front() << endl;
                        Infolist.pop_front();
                    }
                }
                else
                {
                    getErrorMessage(pErrorMsg, &strErrorMessage);
                    //cout << "Error:" << strErrorMessage.c_str() << endl;
                    //getErrorMsg();
                }
                Infolist.clear();
                break;
            case OPERATION_WRITE:
                CommandLine.getParameter(&Temp, 0);
                CommandLine.getParameter(&SetValue, 1);
                CommandLine.getParameter(&TargetName, 2);

                sprintf(TempString, "%s", Temp.c_str());
                ConvertStringLowerCase(TempString);
                Temp = TempString;

        		if(strcmp(Temp.c_str(), "temp") == 0)
        		{
       				TemperatureDataElement=TEMPERATURE_DATA_CURRENT_TEMPERATURE;
        		}
        		else if(strcmp(Temp.c_str(), "threshold_hi0") == 0)
        		{
        			TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_HIGH0;
        		}
        		else if(strcmp(Temp.c_str(), "threshold_hi1") == 0)
        		{
        			TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_HIGH1;
        		}
        		else if(strcmp(Temp.c_str(), "threshold_low0") == 0)
        		{
        			TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_LOW0;
        		}
        		else if(strcmp(Temp.c_str(), "threshold_low1") == 0)
        		{
        			TemperatureDataElement=TEMPERATURE_DATA_THRESHOLD_LOW1;
        		}
        		else
        		{
					return -1;
					break;
        		}

                pErrorMsg = m_pTemperatureDAPI->setDataByName(TargetName.c_str(), TemperatureDataElement, SetValue);
                if(pErrorMsg != NO_ERROR_MSG)
                {
                    getErrorMessage(pErrorMsg, &strErrorMessage);
                    cout <<  strErrorMessage.c_str() << endl;
                    return -1;
                }

                pErrorMsg = m_pTemperatureDAPI->getDataByName(TargetName.c_str(), TemperatureDataElement, &Infolist);
                if(pErrorMsg != NO_ERROR_MSG)
                {
                    getErrorMessage(pErrorMsg, &strErrorMessage);
                    cout <<  strErrorMessage.c_str() << endl;
                    return -1;
                }

                if( pErrorMsg == NO_ERROR_MSG)
                {
                    while (!Infolist.empty())
                    {
        				cout << Infolist.front() << endl;
                        Infolist.pop_front();
                    }
                }
                else
                {
                    getErrorMessage(pErrorMsg, &strErrorMessage);
                    //cout << "Error:" << strErrorMessage.c_str() << endl;
                    //getErrorMsg();
                }

                Infolist.clear();

                break;
            default:
                return -1;
                break;
        }

        Temp.clear();
        TargetName.clear();
        SetValue.clear();
    }


    return err;
}


int
TemperatureOperation::checkSyntax(StringParser CommandLine)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    int    i, hex[2],intParameterNumber = 0;
    string parameter[7];
    int    err = 0;

    CommandLine.getParameterNumber(&intParameterNumber);
    //printf("intParameterNumber=%d\n", intParameterNumber);

    switch (m_OperationType)
    {
        case NO_OPRATION:
            if (intParameterNumber > 0)
            {
                cout << "Not Necessary Operation Parameter" << endl;
                helpParameter();
                return -1;
            }
            break;
        case OPERATION_ALL:
        case OPERATION_TEMPERATURE:
        case OPERATION_TEMPERATURE_THRESHOLD_HI0:
        case OPERATION_TEMPERATURE_THRESHOLD_HI1:
        case OPERATION_TEMPERATURE_THRESHOLD_LOW0:
        case OPERATION_TEMPERATURE_THRESHOLD_LOW1:
             //Check parameter number
            if (intParameterNumber != 1)
            {
                cout << "Parameter numbers isn't 1" << endl;
                helpParameter();
                return -1;
            }
            break;
        case OPERATION_READ:
             //Check parameter number
            if (intParameterNumber != 2)
            {
                cout << "Parameter numbers isn't 2" << endl;
                helpParameter();
                return -1;
            }
            break;
        case OPERATION_WRITE:
            if (intParameterNumber != 3)
            {
                cout << "Parameter numbers isn't 3" << endl;
                helpParameter();
                return -1;
            }

            //Get parameter
            for (i=0; i<intParameterNumber; i++)
            {
                CommandLine.getParameter(&parameter[i], i);
            }

            //Check parameter 1 is numeric

            err = CommandLine.convertStringToHex(parameter[1], &hex[0]);
            if (err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                helpParameter();
                return -1;
            }

            break;
       default:
            break;
    }

    return 0;
}
