#include "USBOperation.hpp"



USBOperation::USBOperation(OPERATION_TYPE OprationType, string helpContent) : OperationBaseClass(OprationType, helpContent)
{
    m_pUSBDAPI = NULL;
    for(UINT32 u32CNT=0; u32CNT<MAX_USB_DEVICE_AMOUNT; u32CNT++)
    {
	  m_pProgressHandler[u32CNT] = NULL;
    }
    switch (OprationType)
    {
        case OPERATION_ALL:
            setName("all");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_NODE:
            setName("node");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_SN:
            setName("sn");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_VENDOR:
            setName("vendor");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_CAPACITY:
            setName("capacity");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_LINK:
            setName("link");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_SPEED:
            setName("speed");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_PHYSICAL_BLOCK_SIZE:
            setName("physical");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_LOGICAL_BLOCK_SIZE:
            setName("logical");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_SG_NODE:
        	setName("sg");
        	setHelp(helpContent);
        	m_OperationType=OprationType;
            break;
        case OPERATION_READPFMC:
            setName("readpfmc");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_WRITEPFMC:
            setName("writepfmc");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        case OPERATION_WRC:
            setName("wrc");
            setHelp(helpContent);
            m_OperationType = OprationType;
            break;
        default:
            break;
    }
}



USBOperation::~USBOperation()
{
  if (m_pUSBDAPI != NULL)
      delete m_pUSBDAPI;
  for(UINT32 u32CNT=0; u32CNT<MAX_USB_DEVICE_AMOUNT; u32CNT++)
  {
	  if(m_pProgressHandler[u32CNT] != NULL)
	  {
	       delete m_pProgressHandler[u32CNT];
	       m_pProgressHandler[u32CNT] = NULL;
	  }
  }
}



int
USBOperation::executeOperation(StringParser CommandLine)
{
    string parameter1;
    COMMAND_TYPE CommandType;
    int    err = 0;

    CommandLine.getParameter(&parameter1, 0);
    CommandLine.getCommandType(&CommandType);

    //If the first parameter is "help" or space, show help message
    if (CommandType == CMD_LIST)
    {
        parameter1 = NO_OPRATION;

    }
    else
    {
        if (parameter1 == HELP || parameter1 == "")
        {
            helpParameter();
            err = -1;
            return err;
        }
    }

    //Execute operation
    switch (m_OperationType)
    {
        case NO_OPRATION:
            err = executeList(CommandLine);
            break;
        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_SN:
        case OPERATION_VENDOR:
        case OPERATION_CAPACITY:
        case OPERATION_LINK:
        case OPERATION_SPEED:
        case OPERATION_PHYSICAL_BLOCK_SIZE:
        case OPERATION_LOGICAL_BLOCK_SIZE:
        case OPERATION_SG_NODE:
            err = executeShow(CommandLine);
            break;

        case OPERATION_READPFMC:
        case OPERATION_WRITEPFMC:
        case OPERATION_WRC:
            err = executeTest(CommandLine);
            break;
        default:
            cout << "Unsupport operation" << endl;
            err = -1; 
            break;
    }

    return err;
}



int
USBOperation::helpParameter()
{
    //Show parameter help message depend on operation type
    switch (m_OperationType)
    {
        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_SN:
        case OPERATION_VENDOR:
        case OPERATION_CAPACITY:
        case OPERATION_LINK:
        case OPERATION_SPEED:
        case OPERATION_PHYSICAL_BLOCK_SIZE:
        case OPERATION_LOGICAL_BLOCK_SIZE:
        case OPERATION_SG_NODE:
            cout << "Show USB device characteristic" << endl;
            cout << "Syntax: P1" << endl;
            cout << "Operation parameter:" << endl;
            cout << "P1: Device Name" << endl;
            break;
/*
        case READ:
            cout << "Read USB device data" << endl;
            cout << "Syntax: read P1 P2 P3" << endl;
            cout << "P1: Device name" << endl;
            cout << "P2: Start address" << endl;
            cout << "P3: End address" << endl;
            break;
        case WRITE:
            cout << "Write USB device data" << endl;
            cout << "Syntax: write P1 P2 P3" << endl;
            cout << "P1: Device name" << endl;
            cout << "P2: Start address" << endl;
            cout << "P3: Data" << endl;
            break;
*/
        case OPERATION_READPFMC:
            cout << "USB Device Read Performance" << endl;
            cout << "Syntax: readpfc P1 P2 P3" << endl;
            cout << "P1: Start Sector(Hex)" << endl;
            cout << "P2: Test Size/MB(Dec)" << endl;
            cout << "P3: Device Name" << endl;
            break;
        case OPERATION_WRITEPFMC:
            cout << "USB Device Write Performance" << endl;
            cout << "Syntax: writepfmc P1 P2 P3" << endl;
            cout << "P1: Start Sector(Hex)" << endl;
            cout << "P2: Test Size/MB(Dec)" << endl;
            cout << "P3: Device Name" << endl;
            break;
        case OPERATION_WRC:
            cout << "USB Device Read/Write Test" << endl;
            cout << "Syntax: wrc P1 P2 P3" << endl;
            cout << "P1: Start Sector(Hex)" << endl;
            cout << "P2: Test Size/MB(Dec)" << endl;
            cout << "P3: Device Name" << endl;
            break;
       default:
            break;
    }
    
    return 0;
}


int USBOperation::executeList(StringParser CommandLine)
{
    INT32 err = 0;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    list<string>  USBDevicelist;
    USBDevicelist.clear();
    string  strErrorMessage;
    strErrorMessage.clear();

    m_pUSBDAPI = new USBDAPI("USBDAPI");

    err = checkSyntax(CommandLine);
    if (err == 0)
    {
        pErrorMsg = m_pUSBDAPI->displayUSBDevicesList(&USBDevicelist);
        if( pErrorMsg == NO_ERROR_MSG)
        {
            while (!USBDevicelist.empty())
            {
                cout << USBDevicelist.front().c_str() << endl;
                USBDevicelist.pop_front();
            }
        }
        else
        {
            getErrorMessage(pErrorMsg, &strErrorMessage);
            //cout << "Error:" << strErrorMessage.c_str() << endl;
            //getErrorMsg();
        }

    }




    return err;
}


int
USBOperation::executeShow(StringParser CommandLine)
{
    int err = 0;
    string TargetName;
    TargetName.clear();

    USB_DATA_ELEMENT USBDataElement;
    list<string>  Infolist;

    UINT32 intLogicalID = -1;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string  strErrorMessage;
    strErrorMessage.clear();


    m_pUSBDAPI = new USBDAPI("USBDAPI");

    err = checkSyntax(CommandLine);
    if (err == 0)
    {
        CommandLine.getParameter(&TargetName, 0);
        //printf("2nd Parameter:%s\n", TargetName.c_str());
        switch(m_OperationType)
        {
            case OPERATION_ALL:
                USBDataElement=USB_DATA_ALL_INFO;
                break;
            case OPERATION_NODE:
                USBDataElement=USB_DATA_NODE_INFO;
                break;
            case OPERATION_SN:
                USBDataElement=USB_DATA_SN_INFO;
                break;
            case OPERATION_VENDOR:
                USBDataElement=USB_DATA_VENDOR_INFO;
                break;
            case OPERATION_CAPACITY:
                USBDataElement=USB_DATA_CAPACITY_INFO;
                break;
            case OPERATION_LINK:
                USBDataElement=USB_DATA_LINK_INFO;
                break;
            case OPERATION_SPEED:
                USBDataElement=USB_DATA_SPEED_INFO;
                break;
            case OPERATION_PHYSICAL_BLOCK_SIZE:
            	USBDataElement=USB_DATA_PHYSICAL_BLOCK_SIZE_INFO;
            	break;
            case OPERATION_LOGICAL_BLOCK_SIZE:
            	USBDataElement=USB_DATA_LOGICAL_BLOCK_SIZE_INFO;
                break;
            case OPERATION_SG_NODE:
            	USBDataElement=USB_DATA_SG_NODE_INFO;
            	break;
            default:
                return -1;
                break;
        }
        pErrorMsg = m_pUSBDAPI->getIndexByName(TargetName.c_str(), &intLogicalID);
        if(pErrorMsg != NO_ERROR_MSG)
        {
            getErrorMessage(pErrorMsg, &strErrorMessage);
            cout <<  strErrorMessage.c_str() << endl;
            return -1;
        }
        //printf("Operation-executeShow-LogicalID=%x\n",intLogicalID );
        pErrorMsg = m_pUSBDAPI->getElementByIndex(intLogicalID, USBDataElement, &Infolist);
        if(pErrorMsg != NO_ERROR_MSG)
        {
            getErrorMessage(pErrorMsg, &strErrorMessage);
            cout <<  strErrorMessage.c_str() << endl;
            return -1;
        }
        TargetName.clear();

        if(Infolist.size() != 1)
        {
            for(list<string>::iterator index = Infolist.begin(); index != Infolist.end(); ++index)
            {
                switch(distance(Infolist.begin(), index))
                {
                    case 0:
                        TargetName.append("Node:                ");
                        TargetName.append(*index);
                        break;
                    case 1:
                        TargetName.append("SN:                  ");
                        TargetName.append(*index);
                        break;
                    case 2:
                        TargetName.append("Vendor:              ");
                        TargetName.append(*index);
                        break;
                    case 3:
                        TargetName.append("Capacity:            ");
                        TargetName.append(*index);
                        TargetName.append(" MB");
                        break;
                    case 4:
                        TargetName.append("Link:                ");
                        TargetName.append(*index);
                        break;
                    case 5:
                        TargetName.append("Speed:               ");
                        TargetName.append(*index);
                        TargetName.append(" M/s");
                        break;
                    case 6:
                    	TargetName.append("Physical block size: ");
                    	TargetName.append(*index);
                    	break;
                    case 7:
                    	TargetName.append("Logical block size:  ");
                    	TargetName.append(*index);
                        break;
                    case 8:
						TargetName.append("SG Node:             ");
						TargetName.append(*index);
						break;
                    default:
                        break;
                }
                cout << TargetName.c_str() << endl;
                TargetName.clear();

            }

        }
        else
        {
            switch(m_OperationType)
            {
                case OPERATION_NODE:
                    TargetName.append("Node:                 ");
                    TargetName.append(Infolist.front().c_str());
                    break;
                case OPERATION_SN:
                    TargetName.append("SN:                   ");
                    TargetName.append(Infolist.front().c_str());
                    break;
                case OPERATION_VENDOR:
                    TargetName.append("Vendor:               ");
                    TargetName.append(Infolist.front().c_str());
                    break;
                case OPERATION_LINK:
                    TargetName.append("Link:                 ");
                    TargetName.append(Infolist.front().c_str());
                    break;
                case OPERATION_CAPACITY:
                    TargetName.append("Capacity:             ");
                    TargetName.append(Infolist.front().c_str());
                    TargetName.append(" MB");
                    break;

                case OPERATION_SPEED:
                    TargetName.append("Speed:                ");
                    TargetName.append(Infolist.front().c_str() );
                    TargetName.append(" M/s");
                    break;
                case OPERATION_PHYSICAL_BLOCK_SIZE:
                	TargetName.append("Physical block size:  ");
                	TargetName.append(Infolist.front().c_str());
                	break;
                case OPERATION_LOGICAL_BLOCK_SIZE:
                	TargetName.append("Logical block size:  ");
                	TargetName.append(Infolist.front().c_str());
                    break;
                case OPERATION_SG_NODE:
                	TargetName.append("SG Node:             ");
                	TargetName.append(Infolist.front().c_str());
                	break;
                default:
                    break;
            }
            cout << TargetName.c_str() << endl;
            //cout << Infolist.front().c_str() << endl;
            //Infolist.pop_front();
        }
        Infolist.clear();
        TargetName.clear();
    }

    return err;
}


int
USBOperation::executeTest(StringParser CommandLine)
{
    INT32 err = 0;
    string TempString;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;


    INT32 StartSector = 0;
    INT32 TestSize = 0;
    string TargetName;

    TempString.clear();
    TargetName.clear();

    string  strErrorMessage;
    strErrorMessage.clear();

    UINT32 intLogicalID = 0;
    list<string>  Infolist;

    m_pUSBDAPI = new USBDAPI("USBDAPI");

    err = checkSyntax(CommandLine);
    if (err == 0)
    {
        CommandLine.getParameter(&TempString, 0);
        //printf("1st Parameter:%s\n", TempString.c_str());
        CommandLine.convertStringToHex(TempString, &StartSector);
        //printf("1st ParameterToHex:%d\n", StartSector);
        TempString.clear();

        CommandLine.getParameter(&TempString, 1);
        //printf("2nd Parameter:%s\n", TempString.c_str());
        CommandLine.convertStringToHex(TempString, &TestSize);
        //printf("2nd ParameterToHex:%d\n", TestSize);
        TempString.clear();

        CommandLine.getParameter(&TargetName, 2);
        //printf("3rd Parameter:%s\n", TargetName.c_str());

        m_pUSBDAPI->getIndexByName(TargetName.c_str(), &intLogicalID);
        //printf("Operation-executeShow-LogicalID=%d\n",intLogicalID );
        //m_pUSBDAPI->getDataByID(intLogicalID, USBDataElement, &Infolist);

        m_pProgressHandler[0] = new thread(&USBOperation::handleTestProgress, this, (UINT32)TestSize);

        switch(m_OperationType)
        {
            case OPERATION_READPFMC:
                pErrorMsg = m_pUSBDAPI->readPFMCTest(intLogicalID, &Infolist, (UINT32)StartSector, (UINT32)TestSize);
                break;
            case OPERATION_WRITEPFMC:
                pErrorMsg = m_pUSBDAPI->writePFMCTest(intLogicalID, &Infolist, (UINT32)StartSector, (UINT32)TestSize);
                break;
            case OPERATION_WRC:
                pErrorMsg = m_pUSBDAPI->readNwriteTest(intLogicalID, &Infolist, (UINT32)StartSector, (UINT32)TestSize);
                break;
            default:
                cout << "Unsupport operation" << endl;
                err = -1;
                break;
        }
        m_pProgressHandler[0]->join();
        if(pErrorMsg != NO_ERROR_MSG)
        {
            getErrorMessage(pErrorMsg, &strErrorMessage);
            cout <<  strErrorMessage.c_str() << endl;
            return -1;
        }
        else
        {
            while(!Infolist.empty())
            {
                cout << Infolist.front().c_str() << endl;
                Infolist.pop_front();
            }
        }

        //cout << "Execute Test operation." << endl;
    }

    return err;
}



int
USBOperation::checkSyntax(StringParser CommandLine)
{
    int    i, hex[2],intParameterNumber = 0;
    string parameter[7];
    int    err = 0;

    CommandLine.getParameterNumber(&intParameterNumber);
    //printf("intParameterNumber=%d\n", intParameterNumber);

    switch (m_OperationType)
    {
        case NO_OPRATION:
            if (intParameterNumber > 0)
            {
                cout << "Not Necessary Operation Parameter" << endl;
                helpParameter();
                return -1;
            }
            break;
        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_SN:
        case OPERATION_VENDOR:
        case OPERATION_CAPACITY:
        case OPERATION_LINK:
        case OPERATION_SPEED:
        case OPERATION_PHYSICAL_BLOCK_SIZE:
        case OPERATION_LOGICAL_BLOCK_SIZE:
        case OPERATION_SG_NODE:
             //Check parameter number
            if (intParameterNumber != 1)
            {
                cout << "Parameter numbers isn't 1" << endl;
                helpParameter();
                return -1;
            }
            break;
        case OPERATION_READPFMC:
        case OPERATION_WRITEPFMC:
        case OPERATION_WRC:
            if (intParameterNumber != 3)
            {
                cout << "Parameter numbers isn't 3" << endl;
                helpParameter();
                return -1;
            }

            //Get parameter
            for (i=0; i<intParameterNumber; i++)
            {
                CommandLine.getParameter(&parameter[i], i);
            }

            //Check parameter 1 is numeric
            err = CommandLine.convertStringToHex(parameter[0], &hex[0]);
            if (err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                helpParameter();
                return -1;
            }

            //Check parameter 1 is numeric
            err = CommandLine.convertStringToHex(parameter[1], &hex[1]);
            if (err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                helpParameter();
                return -1;
            }
            break;
       default:
            break;
    }

    return 0;
}

void USBOperation::handleTestProgress(UINT32 in_u32TestSizeMB)
{
     INT32 s32Progress = 0;

     sleep(1);
     while(s32Progress < 100)// <= 100%
     {
	  sleep(1);
	  s32Progress = m_pUSBDAPI->getProgress();
	  printf("Progressing: %3u%% \r", s32Progress);
	  fflush(stdout);
     }
     printf("\n");
}
