/// @file       ResourceLock.hpp
/// @brief      Mutex lock for resources
/// @details    Provide basic mutex lock for resources user want to protect
/// @author     KenHu
/// @copyright  Copyright 2016 Hon Hai Technology Group. All Rights Reserved.

#ifndef RESOURCE_LOCK_HPP
#define RESOURCE_LOCK_HPP

#include <sys/sem.h>
#include <errno.h>
#include "common.hpp"

using namespace std;

class ResourceLock
{
public:
            ResourceLock() {}
           ~ResourceLock() {}

    BOOL    lock(INT32 in_s32Resource)
    {
        INT32 s32SemaphoreID = getSemaphore(in_s32Resource);

        if (s32SemaphoreID == -1)
            return FALSE;

        struct sembuf op = {0, -1, SEM_UNDO};

        if (semop(s32SemaphoreID, &op, 1) == -1)
            return FALSE;
        else
            return TRUE;
    }

    void    release(INT32 in_s32Resource)
    {
        INT32 s32SemaphoreID = getSemaphore(in_s32Resource);

        if (s32SemaphoreID == -1)
            return;

        struct sembuf op = {0, 1, SEM_UNDO};

        semop(s32SemaphoreID, &op, 1);
    }

protected:

private:
    union semun {
        int val;
        struct semid_ds *buf;
        unsigned short *array;
    };

    INT32   getSemaphore(INT32 in_s32Resource)
    {
        INT32 s32SemaphoreID = semget((key_t)in_s32Resource, 0, 0666);
        // Create a new semaphore because it does not exist
        if (s32SemaphoreID == -1)
        {
            s32SemaphoreID = semget((key_t)in_s32Resource, 1, (0666 | IPC_CREAT));
            if (s32SemaphoreID == -1)
                return -1;
            // Set semaphore default value to 1
            union semun argument;
            argument.val = 1;
            if (semctl(s32SemaphoreID, 0, SETVAL, argument) == -1)
                return -1;

            return s32SemaphoreID;
        }

        return s32SemaphoreID;
    }
};

#endif
