#include "ProcessorDAPI.hpp"


ProcessorDAPI::ProcessorDAPI(STRING className) : BaseClassDAPI(className)
{
    m_ErrorMsg = NO_ERROR_MSG;

    if ((m_cpuDriver = new ProcessorDDAPI("CPU")) == NULL)
    {
        addErrorMsg( (ERRORCODE)(ERROR_PROCESSOR_DAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR), "Create Processor_DDAPI fail");
    }
}


ProcessorDAPI::~ProcessorDAPI(void)
{
    if (m_cpuDriver != NULL)
    {
        delete m_cpuDriver;
    }
}


ErrorMsg*
ProcessorDAPI::GetInformation(string   WellKnownName,
                              CPU_ITEM option,
                              string   *result)
{
    list<CPUInformation> CPU_data;
    list<CPUInformation>::iterator it;
    char string_temp[4069], Frequency_char_string[20];
    int  id = 0, cpu_thread = 0;
    string CPU, Vendor, Model, Family, Core, Frequency, Stepping, Cache, Temperature;

    m_ErrorMsg = m_cpuDriver->GetCPUInformation(&CPU_data);

    //Get CPU ID by well-known name
    m_ErrorMsg = GetCPUIDByWellKnowName(WellKnownName, &id);
    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_INFORMATIOM, NULL, m_ErrorMsg);
    }

    for (it = CPU_data.begin(); it != CPU_data.end(); it++)
    {
        if (id == (*it).m_physical_id)
        {
            //sprintf(string_temp, "%d\n", (*it).m_processor_id);
            //*result = *result + "ID: "  + string_temp;

            if (option == CPU_ITEM_ALL || option == CPU_ITEM_CPU)
            {
                //*result = *result + "CPU: " + (*it).m_model_name + "\n";
                if ((*it).m_core_id == 0)
                {
                   cpu_thread++;
                }
                sprintf(string_temp, "Thread per core: %d\n", cpu_thread);
                CPU = string_temp;
            }
            if (option == CPU_ITEM_ALL || option == CPU_ITEM_VENDOR)
            {
                //*result = *result + "Vendor: " +(*it). m_vendor_id + "\n";
                Vendor = "Vendor: " +(*it). m_vendor_id + "\n";
            }

            if (option == CPU_ITEM_ALL || option == CPU_ITEM_MODEL)
            {
                sprintf(string_temp, "Model: %d\n", (*it).m_model);
                //*result = *result + "Model: " + string_temp;
                Model = string_temp;
            }

            if (option == CPU_ITEM_ALL || option == CPU_ITEM_FAMILY)
            {
                sprintf(string_temp, "Family: %d\n", (*it).m_cpu_family);
                //*result = *result + "Family: " + string_temp;
                Family = string_temp;
            }

            if (option == CPU_ITEM_ALL || option == CPU_ITEM_CORE)
            {
                sprintf(string_temp, "Core: %d\n", (*it).m_cpu_cores);
                //*result = *result + "Core: " + string_temp;
                Core = string_temp;
            }

            if (option == CPU_ITEM_ALL || option ==CPU_ITEM_FREQUENCY )
            {
                //sprintf(string_temp, "Frequency: %f\n", (*it).m_cpu_frequency);
                //*result = *result + "Frequency: " + string_temp;
                sscanf((*it).m_model_name.c_str(), "%*[^@]@%s", Frequency_char_string);
                sprintf(string_temp, "Frequency: %s\n", Frequency_char_string);
                Frequency = string_temp;
            }

            if (option == CPU_ITEM_ALL || option == CPU_ITEM_STEPPING)
            {
                sprintf(string_temp, "Stepping: %d\n", (*it).m_stepping);
                //*result = *result + "Stepping: " + string_temp;
                Stepping = string_temp;
            }

            if (option == CPU_ITEM_ALL || option == CPU_ITEM_CACHE)
            {
                sprintf(string_temp, "Cache: %d KB\n", (*it).m_cache_size);
                //*result = *result + "Cache: " + string_temp +" KB\n";
                Cache = string_temp;
            }

            if (option == CPU_ITEM_ALL || option == CPU_ITEM_TEMPERATURE)
            {
                string Temperature_result;
                m_cpuDriver->getTemperatureDataByID(WellKnownName, CPU_CURRENT_TEMPERATURE, &Temperature_result);
                //*result = *result + "Temperature: " + "\n";
                Temperature = "Temperature: " + Temperature_result;
            }

            //Remove the last newline
            //result->erase(result->length() - 1, 1);
        }
        /* //For debug show
        printf("processor_id %d\n", (*it).m_processor_id);
        printf("vendor_id %s", (*it).m_vendor_id.c_str());
        printf("cpu_family %d\n", (*it).m_cpu_family);
        printf("model_name %s", (*it).m_model_name.c_str());
        printf("model %d\n", (*it).m_model);
        printf("stepping %d\n", (*it).m_stepping);
        printf("frequency %f\n", (*it).m_cpu_frequency);
        printf("cache_size %d\n", (*it).m_cache_size);
        printf("physical_id %d\n", (*it).m_physical_id);
        printf("siblings %d\n", (*it).m_siblings);
        printf("core id %d\n", (*it).m_core_id);
        printf("cpu_cores %d\n", (*it).m_cpu_cores);
        printf("apic id %d\n", (*it).m_apicid);
        printf("initial_apicid %d\n", (*it).m_initial_apicid);
        printf("fdiv_bug %d\n", (*it).m_fdiv_bug);
        printf("hlt_bug %d\n", (*it).m_hlt_bug);
        printf("f00f_bug %d\n", (*it).m_f00f_bug);
        printf("coma_bug %d\n", (*it).m_coma_bug);
        printf("fpu_exception %d\n", (*it).m_fpu_exception);
        printf("fpu %d\n", (*it).m_fpu);
        printf("cpuid_level %d\n", (*it).m_cpuid_level);
        printf("wp %d\n", (*it).m_wp);
        printf("flags %s\n", (*it).m_flags.c_str());
        printf("bogomips %f\n", (*it).m_bogomips);
        printf("clflush_size %d\n", (*it).m_clflush_size);
        printf("cache_alignment %d\n", (*it).m_cache_alignment);
        printf("address sizes: %d bits physical, %d bits virtual\n", (*it).m_address_sizes_bits_physical, (*it).m_address_sizes_bits_virtual);
        printf("power_management  %s\n", (*it).m_power_management.c_str());

        printf("\n");*/
    }

    *result = CPU + Vendor + Model + Family + Core + Frequency + Stepping + Cache + Temperature;

    return NO_ERROR_MSG;
}



ErrorMsg*
ProcessorDAPI::GetCPUIDByWellKnowName(string wellKnownName, int *ID)
{
    int    i, cpu_number;
    char   cpuID[15];

    //Get CPU number
    m_cpuDriver->GetCPUNumber(&cpu_number);

    for (i = 0; i < cpu_number; i++)
    {
        sprintf(cpuID, "processor%d", i);
        if (wellKnownName == cpuID)
        {
            *ID = i;
            break;
        }
    }

    if (i == cpu_number)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_INFORMATIOM, "CPU ID out of range", NULL);
    }

    return NO_ERROR_MSG;
}



/*
ErrorMsg*
ProcessorDAPI::GetInformation(XML_CPU* CPU_data)
{
    m_ErrorMsg = m_cpuDriver->GetCPUInformation(CPU_data);
    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_INFORMATIOM, NULL, m_ErrorMsg);
    }

    return NO_ERROR_MSG;
}*/


/*
ErrorMsg*
ProcessorDAPI::StabilityTest(INT32 *Test_result, int display, int loop)
{
    StabilityTestObject ST;

    m_ErrorMsg = ST.PiCalculate(Test_result, display, loop);

    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_STABILITY_TEST, NULL, m_ErrorMsg);
    }

    return NO_ERROR_MSG;
}*/



ErrorMsg*
ProcessorDAPI::Get_CPU_number(int *CPU_count)
{
    m_ErrorMsg = m_cpuDriver->GetCPUNumber(CPU_count);
    
    return m_ErrorMsg;
}



ErrorMsg*
ProcessorDAPI::Get_CPU_physical_number(int *CPU_count)
{
    m_ErrorMsg = m_cpuDriver->GetCPUPhysicalNumber(CPU_count);

    return m_ErrorMsg;
}



ErrorMsg*
ProcessorDAPI::GetErrorRegisters(INT32 *Test_result)
{
    m_ErrorMsg = m_cpuDriver->GetErrorRegistersExecute(Test_result);

    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_ERROR_REGISTER, NULL, m_ErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg*
ProcessorDAPI::GetMCE(string *Test_result)
{
    GetMCEDAPI getMCEobject("MCE");
    
    Test_result->clear();
    m_ErrorMsg = getMCEobject.Execute(Test_result);

    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_MCE, NULL, m_ErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg*
ProcessorDAPI::GetMCE(string wellKnownName, string *MCE_message)
{
    GetMCEDAPI getMCEobject("MCE");
    int        id;

    MCE_message->clear();

    m_ErrorMsg = GetCPUIDByWellKnowName(wellKnownName, &id);
    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_INFORMATIOM, NULL, m_ErrorMsg);
    }

    m_ErrorMsg = getMCEobject.Execute(id, MCE_message);

    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_MCE, NULL, m_ErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg*
ProcessorDAPI::SetExecutionCPUCore(string wellKnownName)
{
    list<CPUInformation>           CPU_data;
    list<CPUInformation>::iterator it;
    list<int>           CPU_list;
    list<int>::iterator CPU_list_it;
    cpu_set_t  mask;
    int        id = 0;

    m_ErrorMsg = GetCPUIDByWellKnowName(wellKnownName, &id);
    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_INFORMATIOM, NULL, m_ErrorMsg);
    }

    //Get CPU information
    m_ErrorMsg = m_cpuDriver->GetCPUInformation(&CPU_data);
    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_ERROR_REGISTER, NULL, m_ErrorMsg);
    }

    //Search CPU physical id
    for (it = CPU_data.begin(); it != CPU_data.end(); it++)
    {
        if (id == (*it).m_physical_id)
        {
            CPU_list.push_back((*it).m_processor_id);
        }
    }

    //Setting execution CPU core
    CPU_ZERO(&mask);
    for (CPU_list_it = CPU_list.begin(); CPU_list_it != CPU_list.end(); CPU_list_it++)
    {
        CPU_SET(*CPU_list_it, &mask);
    }
    sched_setaffinity(0, sizeof(mask), &mask);

    return NO_ERROR_MSG;
}



ErrorMsg*
ProcessorDAPI::Read(string wellKnownName, int CPU_register, int count, list<unsigned long long> *val)
{
    m_ErrorMsg = m_cpuDriver->ReadCPURegister(wellKnownName, CPU_register, count, val);
    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_ERROR_REGISTER, NULL, m_ErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg*
ProcessorDAPI::Write(string wellKnownName, int CPU_register, unsigned long long Data)
{
    m_ErrorMsg = m_cpuDriver->WriteCPURegister(wellKnownName, CPU_register, Data);
    if (m_ErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PROCESSOR_DAPI_GET_ERROR_REGISTER, NULL, m_ErrorMsg);
    }

    return NO_ERROR_MSG;
}
