#ifndef Ethernet_DAPI_H
#define Ethernet_DAPI_H

#include "BaseClassDAPI.hpp"
#include "EthernetDDAPI.hpp"
#include <linux/ethtool.h>
#include "PCIDAPI.hpp"

#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))

/* Indicates what features are advertised by the interface. */
#define ADVERTISED_10baseT_Half         (1 << 0)
#define ADVERTISED_10baseT_Full         (1 << 1)
#define ADVERTISED_100baseT_Half        (1 << 2)
#define ADVERTISED_100baseT_Full        (1 << 3)
#define ADVERTISED_1000baseT_Half       (1 << 4)
#define ADVERTISED_1000baseT_Full       (1 << 5)
#define ADVERTISED_Autoneg              (1 << 6)
#define ADVERTISED_TP                   (1 << 7)
#define ADVERTISED_AUI                  (1 << 8)
#define ADVERTISED_MII                  (1 << 9)
#define ADVERTISED_FIBRE                (1 << 10)
#define ADVERTISED_BNC                  (1 << 11)
#define ADVERTISED_10000baseT_Full      (1 << 12)
#define ADVERTISED_Pause                (1 << 13)
#define ADVERTISED_Asym_Pause           (1 << 14)
#define ADVERTISED_2500baseX_Full       (1 << 15)
#define ADVERTISED_Backplane            (1 << 16)
#define ADVERTISED_1000baseKX_Full      (1 << 17)
#define ADVERTISED_10000baseKX4_Full    (1 << 18)
#define ADVERTISED_10000baseKR_Full     (1 << 19)
#define ADVERTISED_10000baseR_FEC       (1 << 20)
#define ADVERTISED_20000baseMLD2_Full   (1 << 21)
#define ADVERTISED_20000baseKR2_Full    (1 << 22)
#define ADVERTISED_40000baseKR4_Full    (1 << 23)
#define ADVERTISED_40000baseCR4_Full    (1 << 24)
#define ADVERTISED_40000baseSR4_Full    (1 << 25)
#define ADVERTISED_40000baseLR4_Full    (1 << 26)


struct Ethetnet_statistics{
   string rx_packets;
   string rx_errors;
   string rx_dropped;
   string rx_frame;
   string tx_packets;
   string tx_errors;
   string tx_dropped;
   string tx_carrier;
};

class EthernetDAPI : public BaseClassDAPI
{
    public: 
                        EthernetDAPI(STRING className);
    virtual             ~EthernetDAPI(void);

          ErrorMsg*     GetDeviceNames(CHAR* Ethernet_list);
/*        ERRORCODE     PciRegisterReadTest
          ERRORCODE     PciRegisterAddressLineTest
          ERRORCODE     PciRegisterDataLineTest*/
          ErrorMsg*     EepromValidationTest(CHAR *ethernet_name, INT32 *test_result);
          ErrorMsg*     EepromValidationTest(string ethernet_name, INT32 *test_result);
          ErrorMsg*     MDIRegisterTest(string ethernet_name, INT32 *test_result);
          ErrorMsg*     InterruptTest(string ethernet_name, INT32 *test_result);
          ErrorMsg*     InternalLoopbackTest(string ethernet_name, INT32 *test_result);
          ErrorMsg*     ReadMacAddress(string ethernet_name, string *mac_address);
          ErrorMsg*     WriteMacAddress(string ethernet_name, string *mac_address);
          ErrorMsg*     GetDeviceNames(list<struct ethernet_data>* ethernet_name_list);
          ErrorMsg*     ReadIPAddress(CHAR* ethernet_name, UCHAR* ip);
          ErrorMsg*     ReadIPMask(CHAR* ethernet_name, UCHAR* mask);
          ErrorMsg*     ReadMTU(CHAR* ethernet_name, INT32 *mtu);
          ErrorMsg*     ReadEEPROM(CHAR *ethernet_name, UCHAR *data, INT32 data_counter);
          ErrorMsg*     ReadEEPROM(string ethernet_name, INT32 StartAddress, INT32 Length, INT32 *data);
          ErrorMsg*     ReadEEPROMSize(CHAR *ethernet_name, INT32 *data_counter);
          ErrorMsg*     WriteEEPROM(CHAR *ethernet_name, INT32 offset, INT32 length, UCHAR *data);
          ErrorMsg*     WriteEEPROM(string ethernet_name, INT32 StartAddress, INT32 *data, INT32 Length);
          ErrorMsg*     Extrenal_loopback(string nic, int speed, float *throughput, string *statistics);                    //Singal port
          ErrorMsg*     Extrenal_loopback(string nic_1, string nic_2, int speed, float *throughput, string *statistics);    //Dual port
          ErrorMsg*     Setup_speed(string nic, int speed, int wait_link_up);
          ErrorMsg*     GetSupportedLinkModes(string nic, string *modes, int *mode_count);
          ErrorMsg*     GetSpeedDuplex(string nic, string *speed_result);
          ErrorMsg*     GetLinkStatus(string nic, string *status_result);
          ErrorMsg*     GetStatistics(string nic, struct Ethetnet_statistics *eth_statis);
          ErrorMsg*     GetStatistics(string nic, string *statistic);
          ErrorMsg*     GetNodeValue(string node_path, string *value);
          ErrorMsg*     GetControllerName(string nic, string *name);
          ErrorMsg*     GetFirmwareVersion(string nic, string *fw_version);
          ErrorMsg*     GetNode(string nic, string *node);
          ErrorMsg*     Program(string ethernet_name, string filePath);
          ErrorMsg*     ReadSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 Length, INT32 *data);
          ErrorMsg*     writeSFP(string WellKnownName, INT32 DeviceAddress, INT32 StartAddress, INT32 data);
          ErrorMsg*     GetPCIError(string nic, string *AER);
          ErrorMsg*     Clear(string nic);
          ErrorMsg*     GetPCIBDF(string nic, string *bdf);
          ErrorMsg*     GetPCISpeed(string nic, string *PCISpeed);
          ErrorMsg*     GetPCIWidth(string nic, string *PCIWidth);

    protected:

    private:
          EthernetDDAPI*    m_EthernetDriver;
          ErrorMsg*         m_pErrorMsg;

};

#endif
