#include "ExpanderDAPI.hpp"

ExpanderDAPI :: ExpanderDAPI(STRING className) : BaseClassDAPI(className)
{
    ErrorMsg    *pErrorMsg;

    m_expDriver = NULL;
    
    m_expDriver = new ExpanderDDAPI("ExpanderDDAPI");
    if ((pErrorMsg =  m_expDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        addErrorMsg(ERROR_EXPANDER_DAPI_CONSTRUCTOR, NULL, pErrorMsg);
    }
}

ExpanderDAPI :: ~ExpanderDAPI()
{
    if (m_expDriver != NULL)
    {
        delete m_expDriver;
        m_expDriver = NULL;
    }
}

void ExpanderDAPI :: listDevice(EXP_DEVICE devType)
{
    UINT32      index = 0;
    string      strDeviceName;

    while(1)
    {
        m_expDriver->getDeviceNameByID(devType, index, &strDeviceName);

        if (strDeviceName != "\0")
            cout << strDeviceName << endl;
        else if (index > 0)
        {
            break;
        }
        else
        {
        	cout << "NONE" << endl;
            break;
        }

        index++;
    }

}

ErrorMsg *ExpanderDAPI :: getLogicalIDByName(string wellKnownName, UINT32 *logicalID)
{
    ErrorMsg    *pErrorMsg;

    if ((pErrorMsg = m_expDriver->getLogicalIDByName(wellKnownName, logicalID)) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_EXPANDER_DAPI_GET_LOGICAL_ID_BY_NAME, NULL, pErrorMsg);
    }

    return NO_ERROR_MSG;
}

ErrorMsg *ExpanderDAPI :: showData(UINT32 logicalID, EXP_DATAELEMENT dataElement)
{
    ErrorMsg    *pErrorMsg;
    string      strReturnData;
    string      strPefix;
    
    for (INT32 elementIndex = 0; elementIndex <= (INT32)MAX_EXP_DATAELEMENT_NUM; elementIndex++)
    {
        if ((dataElement == (EXP_DATAELEMENT)elementIndex) || dataElement == EXP_DATAELEMENT_ALL)
        {
            switch((EXP_DATAELEMENT)elementIndex)
            {
                case EXP_DATAELEMENT_ATTACHED_TYPE:
                    strPefix.assign("ATTACHED TYPE     : ");
                    break;
                case EXP_DATAELEMENT_SPEED:
                    strPefix.assign("SPEED             : ");
                    break;
                case EXP_DATAELEMENT_ADDRESS:
                    strPefix.assign("SAS ADDRESS       : ");
                    break;
                case EXP_DATAELEMENT_LINK:
                    strPefix.assign("LINK STATUS       : ");
                    break;
                default:
                    break;
            }
            
            if ((pErrorMsg = m_expDriver->getDataByID(logicalID, (EXP_DATAELEMENT)elementIndex, &strReturnData)) == NO_ERROR_MSG)
            {
                cout << strPefix << strReturnData << endl;
            }
            else
                break;
        }
    }
    
    if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_EXPANDER_DAPI_SHOW_DATA, NULL, pErrorMsg);
    }
    
    return NO_ERROR_MSG;
}

ErrorMsg *ExpanderDAPI :: reset(UINT32 logicalID)
{
    ErrorMsg    *pErrorMsg;

    if ((pErrorMsg = m_expDriver->reset(logicalID)) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_EXPANDER_DAPI_RESET, NULL, pErrorMsg);
    }

    return NO_ERROR_MSG;
}

ErrorMsg *ExpanderDAPI :: showErrorData(UINT32 logicalID, EXP_ERRORELEMENT dataElement)
{
    ErrorMsg    *pErrorMsg;
    string      strReturnData;
    string      strPefix;
    
    for (INT32 elementIndex = 0; elementIndex <= (INT32)MAX_EXP_ERRORELEMENT_NUM; elementIndex++)
    {
        if ((dataElement == (EXP_ERRORELEMENT)elementIndex) || dataElement == EXP_ERRORELEMENT_ALL_ERROR)
        {
            switch((EXP_ERRORELEMENT)elementIndex)
            {
                case EXP_ERRORELEMENT_DISPARITY_ERR:
                    strPefix.assign("DISPARITY_ERR     : ");
                    break;
                case EXP_ERRORELEMENT_INVALID_DWORD_ERR:
                    strPefix.assign("INVALID_DWORD_ERR : ");
                    break;
                case EXP_ERRORELEMENT_LOSS_DWORD_SYNC:
                    strPefix.assign("LOSS_DWORD_SYNC   : ");
                    break;
                case EXP_ERRORELEMENT_PHY_RESET_PRBM:
                    strPefix.assign("PHY_RESET_PRBM    : ");
                    break;
                default:
                    break;
            }
            
            if ((pErrorMsg = m_expDriver->getErrorDataByID(logicalID, (EXP_ERRORELEMENT)elementIndex, &strReturnData)) == NO_ERROR_MSG)
            {
                cout << strPefix << strReturnData << endl;
            }
            else
                break;
        }
    }
    
    if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_EXPANDER_DAPI_SHOW_ERROR_DATA, NULL, pErrorMsg);
    }

    return NO_ERROR_MSG;
}

ErrorMsg *ExpanderDAPI :: clearPHYError()
{
    ErrorMsg    *pErrorMsg;

    if ((pErrorMsg = m_expDriver->clearPHYError()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_EXPANDER_DAPI_CLEAR_PHY_ERROR, NULL, pErrorMsg);
    }

    return NO_ERROR_MSG;
}
