/// @file       NVSRAMDAPI.hpp
/// @brief      NVSRAMDAPI class
/// @details    Provide diagnostic and utility APIs for NVSRAM module
/// @version    1.0.0
/// @date       2015.11.16
/// @author     JerryLee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#ifndef NVSRAM_DAPI_HPP
#define NVSRAM_DAPI_HPP

#include "BaseClassDAPI.hpp"
#include "MemoryDAPI.hpp"
#include "NVSRAMDDAPI.hpp"

/// @class      NVSRAMDAPI NVSRAMDAPI.hpp "NVSRAMDAPI.hpp"
/// @brief      Provide diagnostic and utility APIs for NVSRAM module
/// @param      m_pNVSRAMDDAPI is a pointer to NVSRAMDDAPI
class NVSRAMDAPI: public BaseClassDAPI
{
    public:
                NVSRAMDAPI(STRING className);
        virtual ~NVSRAMDAPI(void);

        ErrorMsg* getInformationByID(UINT8 logicalID, NvsramInfo* returnInfo);
        ErrorMsg* getInformationList(vector<NVSRAM_RESOURCE_INFO>* returnedList);
        ErrorMsg* nonVolatileTest(PHYSICAL_ADDRESS_T startAddress, PHYSICAL_ADDRESS_T endAddress, INT32 dataLineAmount);

    protected:

    private:
        MemoryDAPI* memDAPI;
        NVSRAMDDAPI* nvsramDDAPI;

        ErrorMsg* NVTestExecute(PHYSICAL_ADDRESS_T startAddress, PHYSICAL_ADDRESS_T endAddress, INT32 elementSize);
        ErrorMsg* NVTestExecute(PHYSICAL_ADDRESS_T startAddress, PHYSICAL_ADDRESS_T endAddress,
                                INT32 elementSize, UINT64 userTestPattern);
        ErrorMsg* NVTestCleanup(PHYSICAL_ADDRESS_T startAddress, PHYSICAL_ADDRESS_T endAddress, INT32 elementSize);
        ErrorMsg* WritePattern(INT32 elementSize, UINT64 userTestPattern);
        ErrorMsg* VerifyPattern(INT32 elementSize, UINT64 userTestPattern);

        vector<NVSRAM_RESOURCE_INFO> testNvsramList;
        vector<NVSRAM_RESOURCE_INFO>::iterator testIter;

        SYSTEM_ADDRESS_T bufferBackup;
        SYSTEM_ADDRESS_T nvsramBackup;
        PHYSICAL_ADDRESS_T testStartAddress;
        PHYSICAL_ADDRESS_T testEndAddress;
};

#endif

