#include "RTCDAPI.hpp"

RTCDAPI :: RTCDAPI(STRING className) : BaseClassDAPI(className)
{

}

RTCDAPI :: ~RTCDAPI()
{

}

ErrorMsg * RTCDAPI :: showDateTime(void)
{
    ErrorMsg *pErrorMsg;
    
    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SHOW_DATE_TIME, NULL, pErrorMsg);
    }

    if((pErrorMsg = m_rtcDriver->showDateTime()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SHOW_DATE_TIME, NULL, pErrorMsg);
    }

    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    return NO_ERROR_MSG;
}

ErrorMsg * RTCDAPI :: setDate(INT32 year, INT32 month, INT32 date, STRING day)
{
    ErrorMsg    *pErrorMsg;
    INT32       s32MaxDate;
    
    if ((strncmp(day, "MON", 3) != 0) && (strncmp(day, "TUE", 3) != 0) &&
        (strncmp(day, "WED", 3) != 0) && (strncmp(day, "THU", 3) != 0) &&
        (strncmp(day, "FRI", 3) != 0) && (strncmp(day, "SAT", 3) != 0) &&
        (strncmp(day, "SUN", 3) != 0))
    {
        sprintf(m_szTempString, "Invalid day \"%s\"\n", day);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_DATE | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }


    if ((year < 2000) || (year > 2099))
    {
        sprintf(m_szTempString, "Invalid year \"%d\"\n", year);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_DATE | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }

    switch (month)
    {
        case 4:
        case 6:
        case 9:
        case 11:
            s32MaxDate = 30;
            break;
        case 2:
            s32MaxDate = 28;
            if (year % 4 == 0)
            {
                if ((year % 100 == 0) && (year % 400 != 0))
                    s32MaxDate = 28;
                else
                    s32MaxDate = 29;
            }
            break;
        case 1:
        case 3:
        case 5:
        case 7:
        case 8:
        case 10:
        case 12:
            s32MaxDate = 31;
            break;
        default:
            sprintf(m_szTempString, "Invalid month \"%d\"\n", month);
            return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_DATE | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
            break;
    }

    // Date validation
    if ((date > s32MaxDate) || (date < 1))
    {       
        sprintf(m_szTempString, "Invalid date \"%d\"\n", date);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_DATE | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }
    
    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SET_DATE, "Construct RTC DDAPI failed", pErrorMsg);
    }

    if ((pErrorMsg = m_rtcDriver->setDate(year, month, date)) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SET_DATE, NULL, pErrorMsg);
    }

    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    return NO_ERROR_MSG;
}

ErrorMsg * RTCDAPI :: setTime(INT32 hour, INT32 minute, INT32 second)
{
    ErrorMsg    *pErrorMsg;

    if(hour < 0 || hour > 23)
    {
        sprintf(m_szTempString, "Invalid hour \"%d\"\n", hour);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_TIME | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }
        
    if(minute < 0 || minute > 59)
    {
        sprintf(m_szTempString, "Invalid minute \"%d\"\n", minute);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_TIME | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }
        
    if(second < 0 || second > 59)
    {
        sprintf(m_szTempString, "Invalid second \"%d\"\n", second);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_TIME | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }
    
    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SET_TIME, "Construct RTC DDAPI failed", pErrorMsg);
    }

    if((pErrorMsg = m_rtcDriver->setTime(hour, minute, second)) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SET_TIME, NULL, pErrorMsg);
    }

    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    return NO_ERROR_MSG;
}

ErrorMsg * RTCDAPI :: setDateTime(INT32 year, INT32 month, INT32 date, STRING day, INT32 hour, INT32 minute, INT32 second)
{
    ErrorMsg *pErrorMsg;
    
    if ((pErrorMsg = setDate(year, month, date, day)) == NO_ERROR_MSG)
    {
        pErrorMsg = setTime(hour, minute, second);
    }
    
    return pErrorMsg;
}

ErrorMsg * RTCDAPI :: rtcAccuracyTest(void)
{
    ErrorMsg    *pErrorMsg;     
    INT32       s32LowerBound;
    INT32       s32UpperBound;
    UINT32      u32Count;
    UINT32      u32CurrentCount;
    UINT32      u32TestTime;

    //Setup
    u32TestTime = TIEMER_ACCURACY_TEST_TIME;
    s32UpperBound = TIEMER_ACCURACY_UPPER_BOUND;
    s32LowerBound = TIEMER_ACCURACY_LOWER_BOUND;

    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_RTC_ACCURACY_TEST, "Construct RTC DDAPI failed", pErrorMsg);
    }

    m_rtcDriver->convertPITTimeToCount((FLOAT)u32TestTime, &u32Count);
    s32LowerBound += (INT32)u32Count;
    
    pErrorMsg = m_rtcDriver->setRTCRate((FLOAT)u32TestTime);
    
    if (pErrorMsg == NO_ERROR_MSG)
        pErrorMsg = m_rtcDriver->startRTC();                

    if (pErrorMsg == NO_ERROR_MSG)
        pErrorMsg = m_rtcDriver->waitRTCInterrupt();

    if (pErrorMsg == NO_ERROR_MSG)  
    {
        m_rtcDriver->startPIT(u32Count); 
        pErrorMsg = m_rtcDriver->waitRTCInterrupt();
        m_rtcDriver->readPIT(&u32CurrentCount);
    }
    
    if (pErrorMsg == NO_ERROR_MSG)
    {        
        if ((INT32)u32CurrentCount <= s32LowerBound && (INT32)u32CurrentCount >= s32UpperBound)
        {
            sprintf(m_szTempString, "Count %d should be less than %d or greater than %d.\n", 
                    (INT32)u32CurrentCount, s32UpperBound, s32LowerBound); 
            
            addErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_RTC_ACCURACY_TEST | ERROR_RTC_DAPI_RTC_UNEXPECTED_TEST_RESULT), m_szTempString, pErrorMsg);
        }
    }

    m_rtcDriver->stopRTC();
    m_rtcDriver->stopPIT();

    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    if (pErrorMsg != NO_ERROR_MSG)
    {        
        return returnErrorMsg(ERROR_RTC_DAPI_RTC_ACCURACY_TEST, NULL, pErrorMsg);
    }
    
    return getErrorMsg();
}

ErrorMsg * RTCDAPI :: rtcWDTTest(void)
{
    ErrorMsg *pErrorMsg;
    
    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) == NO_ERROR_MSG)
    {
        pErrorMsg = m_rtcDriver->rtcWDTTest();
    }
    
    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_RTC_DAPI_RTC_WDT_TEST, NULL, pErrorMsg);
    }
    
    return NO_ERROR_MSG;
}

