#ifndef StorageDAPI_HPP
#define StorageDAPI_HPP

#include <iostream>
#include <iomanip>
#include <sys/time.h>
#include <limits.h>

#include "BaseClassDAPI.hpp"
#include "StorageDDAPIOperation.hpp"
#include "NVMeDAPI.hpp"
#include "SCSIFactory.hpp"

#define HEXDUMP_COLUMS 32

/// @class      StorageDAPI StorageDAPI.hpp "StorageDAPI.hpp"
/// @brief      Provide diagnostic and utility APIs for Storage module
/// @param      m_pStorageDDAPI is a pointer to StorageDDAPI
class StorageDAPI : public BaseClassDAPI
{
public:
     string m_strOutputResultString;

     StorageDAPI(STRING in_szClassName);
     virtual ~StorageDAPI(void);

     ErrorMsg *IOCTL(UINT32 in_u32DefinedIndex, StorageIOCTLOperation in_eOperation);
     ErrorMsg *generateStorageList(void);
     ErrorMsg *getIndexByName(string in_strDefinedName, UINT32 *out_u32DefinedIndex);
     ErrorMsg *getElementDataByIndex(UINT32 in_u32DefinedIndex, StorageDataElement in_eStorageDataElement, string *out_strpReturnData);
     ErrorMsg *getElementDataByName(string in_strDefinedName, StorageDataElement in_eStorageDataElement, string *out_strpReturnData);
     ErrorMsg *getSMARTByIndex(UINT32 in_u32DefinedIndex);//, string *out_strpReturnData);

     
     ErrorMsg *readPerformanceTest(UINT32 in_u32DefinedIndex, UINT64 in_u64StartSector, UINT32 in_u32TestSizeMB, BOOL in_blsMultiModeOutputFormat = false);
     ErrorMsg *writePerformanceTest(UINT32 in_u32DefinedIndex, UINT64 in_u64StartSector, UINT32 in_u32TestSizeMB, BOOL in_blsMultiModeOutputFormat = false);
     ErrorMsg *readwriteTest(UINT32 in_u32DefinedIndex, UINT64 in_u64StartSector, UINT32 in_u32TestSizeMB, BOOL in_blsMultiModeOutputFormat = false);

     ErrorMsg *writeZeroByIndex(UINT32 in_u32DefinedIndex, UINT64 in_u64StartSector, BOOL in_blsMultiModeOutputFormat = false);
     ErrorMsg *getPGListByIndex(UINT32 in_u32DefinedIndex, UINT32 in_u32RequestPList, UINT32 in_u32RequestGList, UINT32 in_u32Format, BOOL in_blsMultiModeOutputFormat = false);

protected:

private:
     StorageDDAPIOperation *m_pStorageDDAPIOperation;
     SCSIFactory *m_pSCSIFactory;
     SCSISenseAnalyzer *m_pSCSISenseAnalyzer;
     NVMeDAPI *m_pNVMeDAPI;
     UINT32 m_u32StorageSectorCountPerRDWRAccess;

     ErrorMsg *setSectorCountPerRDWRAccess(UINT32 in_u32DefinedIndex);
     ERRORCODE translateHexToString(string *out_strpDataBuffer, UINT32 *in_u32pDataBuffer, UINT32 in_u32DataLengthInBytes, BOOL in_blsDumpAddressHead = false);
     long double int128_to_double(__u8 *data);
};

#endif
