#include "USBDAPI.hpp"

USBDAPI::USBDAPI(STRING className) : BaseClassDAPI(className)
{
    m_pDataBuffer = NULL;
    m_pFileHandle = -1;
    m_errorCode =  getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        addErrorMsg(ERROR_USB_DAPI_CONSTRUCTOR, NULL, m_errorCode);
    }

    if ((m_USBDiver = new USBDDAPI("USB_drv")) == NULL)
    {
        addErrorMsg( (ERRORCODE)(ERROR_USB_DAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR), "Create USB_DDAPI fail");
    }

    if((m_pMemoryDAPI = new MemoryDAPI("MemDAPI")) == NULL)
    {
        addErrorMsg( (ERRORCODE)(ERROR_USB_DAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_API_CONSTRUCTOR2_ERROR), "Create MEMORY_DDAPI fail");
    }

   
}



USBDAPI::~USBDAPI()
{
    if (m_USBDiver != NULL)
        delete m_USBDiver;
    if (m_pMemoryDAPI != NULL)
        delete m_pMemoryDAPI;
}


ErrorMsg * USBDAPI::displayUSBDevicesList(list<string> *pUSBList)
{
    m_errorCode =  m_USBDiver->getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_USB_DAPI_DISPLAY_USB_PORT_LIST, NULL, m_errorCode);
    }

    m_errorCode = m_USBDiver->getUSBDevicesList(pUSBList);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_USB_DAPI_DISPLAY_USB_PORT_LIST, NULL, m_errorCode);
    }

    return NO_ERROR_MSG;

}



ErrorMsg*   USBDAPI::getIndexByName(STRING WellKnownName, UINT32 *Index)
{
    if ((m_errorCode = m_USBDiver->getIndexByName(WellKnownName, Index)) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERRIR_USB_DAPI_GET_LOGICAL_ID_BY_NAME , NULL, m_errorCode);
    }

    return NO_ERROR_MSG;
}





ErrorMsg* USBDAPI::getElementByIndex(UINT32 Index,
                               USB_DATA_ELEMENT USBDataElement,
                               list<string> *returnDataInfo)
{
    USB_DATA_ELEMENT eCurrentType;
    ErrorMsg* pErrorMsg = NULL;

    switch(USBDataElement)
    {
        case USB_DATA_ALL_INFO:
            for(UINT32 index=1; index < MAX_DATA_ELEMENT_NUMBER; index++)
            {
                eCurrentType = USB_DATA_ELEMENT(index);
                pErrorMsg = m_USBDiver->getElementByIndex(Index, eCurrentType, returnDataInfo);
                if(pErrorMsg != NO_ERROR_MSG)
                {
                    return returnErrorMsg(ERRIR_USB_DAPI_GET_DATA_ID_BY_ID, NULL, pErrorMsg);
                }
            }
            returnDataInfo->reverse();
            break;
        case USB_DATA_NODE_INFO:
        case USB_DATA_SN_INFO:
        case USB_DATA_VENDOR_INFO:
        case USB_DATA_CAPACITY_INFO:
        case USB_DATA_LINK_INFO:
        case USB_DATA_SPEED_INFO:
        case USB_DATA_PHYSICAL_BLOCK_SIZE_INFO:
        case USB_DATA_LOGICAL_BLOCK_SIZE_INFO:
        case USB_DATA_SG_NODE_INFO:
            pErrorMsg = m_USBDiver->getElementByIndex(Index, USBDataElement, returnDataInfo);
            if(pErrorMsg != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERRIR_USB_DAPI_GET_DATA_ID_BY_ID, NULL, pErrorMsg);
            }
            break;
        default:
            return returnErrorMsg((ERRORCODE)(ERRIR_USB_DAPI_GET_DATA_ID_BY_ID | ERROR_USB_DAPI_DATA_ELEMENT_NOT_FOUND), NULL, pErrorMsg);
            break;
    }


    return NO_ERROR_MSG;
}




ErrorMsg*   USBDAPI::readNwriteTest(UINT32   Index,
                                   list<string> *returnDataInfo,
                                   UINT32   StartSector,
                                   UINT32   TestSize,
                                   UINT32   BlockSize)
{
    ErrorMsg* pErrorMsg = NULL;
    UINT64 intTestSize = TestSize * MEGA_BYTES_SIZE;

    pErrorMsg = readNwriteTestSetup(Index, StartSector, intTestSize, BlockSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST, NULL, pErrorMsg);
    }

    pErrorMsg = readNwriteTestExecute(Index, returnDataInfo, StartSector, intTestSize, BlockSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST, NULL, pErrorMsg);
    }

    pErrorMsg = readNwriteTestCleanup(Index);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST, NULL, pErrorMsg);
    }
    return NO_ERROR_MSG;

}





ErrorMsg*   USBDAPI::readNwriteTestSetup(UINT32   Index,
                                        UINT32   LogicalSector,
                                        UINT32   TestSize,
                                        UINT32   BlockSize)
{
    initProgress(TestSize);


    UINT64 u64DeviceCapacity = 0;

    ErrorMsg*       pErrorMsg = NULL;
    ErrorMsg*       tempErrorMsg = NULL;
    list<string>    strTempList;
    strTempList.clear();

    pErrorMsg = m_USBDiver->getElementByIndex(Index, USB_DATA_CAPACITY_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP , NULL, pErrorMsg);
    }
    u64DeviceCapacity = atoll(strTempList.front().c_str()) * MEGA_BYTES_SIZE;

    if(u64DeviceCapacity==0 || (u64DeviceCapacity - LogicalSector*USB_BLOCK_SIZE) < TestSize )
    {
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP|ERROR_OUT_OF_RANGE), NULL, NULL);
    }
    strTempList.clear();

    pErrorMsg = m_USBDiver->getElementByIndex(Index, USB_DATA_NODE_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP , NULL, pErrorMsg);
    }

    pErrorMsg = openDeviceFile(strTempList.front().c_str());
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, pErrorMsg);
    }
    strTempList.clear();

    pErrorMsg = m_pMemoryDAPI->getMemoryBuffer(BlockSize, (SYSTEM_ADDRESS_T*)&m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP | ERROR_USB_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL) , NULL, pErrorMsg);
    }
    memset(m_pDataBuffer, 0, BlockSize);

    return NO_ERROR_MSG;
}






ErrorMsg*   USBDAPI::readNwriteTestExecute(UINT32   Index,
                                          list<string> *returnDataInfo,
                                          UINT32   LogicalSector,
                                          UINT32   TestSize,
                                          UINT32   BlockSize)
{
    ErrorMsg*   pErrorMsg = NULL;
    ErrorMsg*   tempErrorMsg = NULL;

    timeval StartTime;
    timeval EndTime;
    timeval spendTime;
    double  Speed = 0;

    UINT32  intTempTestSize;
    intTempTestSize = TestSize;

    CHAR tempResultString[256];
    memset(tempResultString, 0, sizeof(tempResultString));

    UINT32  *pPatternBuffer = NULL;

    pErrorMsg = m_pMemoryDAPI->getMemoryBuffer(BlockSize, (SYSTEM_ADDRESS_T*)&pPatternBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
            if(pErrorMsg != NULL)
            {
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, pErrorMsg);
            }
            return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE, NULL, pErrorMsg);
        }

        pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
        if(pErrorMsg != NULL)
        {
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, pErrorMsg);
        }

        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_USB_DAPI_ALLOCATE_PATTERN_BUFFER_FAIL), NULL, pErrorMsg);
    }

    srand(time(NULL));

    gettimeofday(&StartTime, NULL);

    //generate random pattern
    while(intTempTestSize > 0)
    {

        for(UINT32 i = 0 ; i < BlockSize / sizeof(UINT32) ; i++)
        {
            *(pPatternBuffer + i) = rand() % UINT_MAX;

        }

        pErrorMsg = m_USBDiver->PWRITE(m_pFileHandle,
                                       pPatternBuffer,
                                       LogicalSector,
                                       BlockSize);
        if(pErrorMsg != NULL)
        {
            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                    if(tempErrorMsg != NULL)
                    {
                        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                    }
                    return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
                }

                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }

            tempErrorMsg = closeDeviceFile(m_pFileHandle);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
            }

            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
            if(tempErrorMsg != NULL)
            {
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_USB_DAPI_WRITE_DATA_TO_STORAGE_FAIL), NULL, pErrorMsg);
        }

        //read patter from disk
        pErrorMsg = m_USBDiver->PREAD(m_pFileHandle,
                                      m_pDataBuffer,
                                      LogicalSector,
                                      BlockSize);
        if(pErrorMsg != NULL)
        {
            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                    if(tempErrorMsg != NULL)
                    {
                        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                    }
                    return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
                }

                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }

            tempErrorMsg = closeDeviceFile(m_pFileHandle);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
            }

            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
            if(tempErrorMsg != NULL)
            {
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }
            //m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pCompareBuffer);
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_USB_DAPI_WRITE_DATA_TO_STORAGE_FAIL), NULL, pErrorMsg);
        }

        //compare pattern
        int compareResult=0;
        if(compareResult != 0)
        {
            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                    if(tempErrorMsg != NULL)
                    {
                        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                    }
                    return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
                }

                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }

            tempErrorMsg = closeDeviceFile(m_pFileHandle);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
            }

            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
            if(tempErrorMsg != NULL)
            {
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }
            return returnErrorMsg((ERRORCODE)(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_USB_DAPI_TEST_PATTERN_COMPARE_FAIL), NULL, NULL);
        }

        LogicalSector = LogicalSector + BlockSize/USB_BLOCK_SIZE;
        updateProgress(BlockSize);

    }
    gettimeofday(&EndTime, NULL);

    spendTime.tv_sec = EndTime.tv_sec - StartTime.tv_sec;
    spendTime.tv_usec = EndTime.tv_usec - StartTime.tv_usec;

    if(spendTime.tv_usec < 0)
    {
        --spendTime.tv_sec;
        spendTime.tv_usec += 1000000;
    }

    double dDataSize = (double)((double)TestSize/(MEGA_BYTES_SIZE));
    double dCostTime = (double)(spendTime.tv_sec)+ ((double)(spendTime.tv_usec)/1000000);

    if(dCostTime == 0)
    {
        m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_DIVIDED_ZERO_FAIL), NULL, NULL);
    }
    else
    {
        Speed = (double)dDataSize/dCostTime;
    }
    sprintf(tempResultString, "Spend time: %lf s\n", dCostTime);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);
    memset(tempResultString, 0, sizeof(tempResultString));
    sprintf(tempResultString,"Speed: %5.2f M/s\n", Speed);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);

    tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
    if(tempErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
    }
    return returnErrorMsg(NO_ERROR, NULL, NULL);

}






ErrorMsg*   USBDAPI::readNwriteTestCleanup(UINT32 Index)
{

    ErrorMsg*       tempErrorMsg = NULL;

    ErrorMsg* pErrorMsg = NULL;
    pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_CLEAN, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_CLEAN, NULL, pErrorMsg);
    }

    pErrorMsg = closeDeviceFile(m_pFileHandle);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST, NULL, pErrorMsg);
    }

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}





ErrorMsg*   USBDAPI::readPFMCTest(UINT32 Index,
                                         list<string> *returnDataInfo,
                                         UINT32 StartSector,
                                         UINT32 TestSize,
                                         UINT32 BlockSize)
{
    ErrorMsg* pErrorMsg = NULL;
    UINT64 intTestSize = TestSize * MEGA_BYTES_SIZE;

    pErrorMsg = readPFMCTestSetup(Index, StartSector, intTestSize, BlockSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST, NULL, pErrorMsg);
    }


    pErrorMsg = readPFMCTestExecute(Index, returnDataInfo, StartSector, intTestSize, BlockSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST, NULL, pErrorMsg);
    }
    pErrorMsg = readPFMCTestCleanup(Index);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST, NULL, pErrorMsg);
    }
    return NO_ERROR_MSG;
}






ErrorMsg*   USBDAPI::readPFMCTestSetup(UINT32   Index,
                                              UINT32   LogicalSector,
                                              UINT32   TestSize,
                                              UINT32   BlockSize)
{
    initProgress(TestSize);

    UINT64 u64DeviceCapacity = 0;

    ErrorMsg*       pErrorMsg = NULL;
    ErrorMsg*       tempErrorMsg = NULL;
    list<string>    strTempList;
    strTempList.clear();

    // Check Test Size valid
    pErrorMsg = m_USBDiver->getElementByIndex(Index, USB_DATA_CAPACITY_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP , NULL, pErrorMsg);
    }

    u64DeviceCapacity = atoll(strTempList.front().c_str()) * MEGA_BYTES_SIZE;
    if(u64DeviceCapacity==0 || (u64DeviceCapacity - LogicalSector*USB_BLOCK_SIZE) < TestSize )
    {
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP|ERROR_OUT_OF_RANGE), NULL, NULL);
    }
    strTempList.clear();

    // Get Device Node
    pErrorMsg = m_USBDiver->getElementByIndex(Index, USB_DATA_NODE_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP , NULL, pErrorMsg);
    }
    //Open Device node
    pErrorMsg = openDeviceFile(strTempList.front().c_str());
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP, NULL, pErrorMsg);
    }
    strTempList.clear();
    pErrorMsg = m_pMemoryDAPI->getMemoryBuffer(BlockSize, (SYSTEM_ADDRESS_T*)&m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP | ERROR_USB_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL) , NULL, pErrorMsg);
    }
    memset(m_pDataBuffer, 0, BlockSize);

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}





ErrorMsg*   USBDAPI::readPFMCTestExecute(UINT32   Index,
                                                list<string> *returnDataInfo,
                                                UINT32   LogicalSector,
                                                UINT32   TestSize,
                                                UINT32   BlockSize)
{
    ErrorMsg* pErrorMsg = NULL;
    ErrorMsg* tempErrorMsg = NULL;
    timeval StartTime;
    timeval EndTime;
    timeval spendTime;
    double  Speed = 0;
    UINT32  intTempTestSize;
    intTempTestSize = TestSize;

    CHAR tempResultString[256];
    memset(tempResultString, 0, sizeof(gettimeofday(&StartTime, NULL)));

    while(intTempTestSize > 0)
    {
        pErrorMsg = m_USBDiver->PREAD(m_pFileHandle,
                                      m_pDataBuffer,
                                      LogicalSector,
                                      BlockSize);


        if(pErrorMsg != NULL)
        {
            m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
            if(pErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE, NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, pErrorMsg);
            }
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_READ_DATA_FROM_STORAGE_FAIL),
                                  NULL,
                                  pErrorMsg);
        }

        LogicalSector = LogicalSector + BlockSize/USB_BLOCK_SIZE;
        intTempTestSize = intTempTestSize - BlockSize;

        updateProgress(BlockSize);
    }

    gettimeofday(&EndTime, NULL);

    spendTime.tv_sec = EndTime.tv_sec - StartTime.tv_sec;
    spendTime.tv_usec = EndTime.tv_usec - StartTime.tv_usec;

    if(spendTime.tv_usec < 0)
    {
        --spendTime.tv_sec;
        spendTime.tv_usec += 1000000;
    }

    double dDataSize = (double)((double)TestSize/(MEGA_BYTES_SIZE));
    double dCostTime = (double)(spendTime.tv_sec)+ ((double)(spendTime.tv_usec)/1000000);

    if(dCostTime == 0)
    {
        m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_DIVIDED_ZERO_FAIL), NULL, NULL);
    }
    else
    {
        Speed = (double)dDataSize/dCostTime;
    }
    sprintf(tempResultString, "Spend time: %lf s\n", dCostTime);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);
    memset(tempResultString, 0, sizeof(tempResultString));
    sprintf(tempResultString,"Speed: %5.2f M/s\n", Speed);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);


    return NO_ERROR_MSG;
}




ErrorMsg*   USBDAPI::readPFMCTestCleanup(UINT32 Index)
{
    ErrorMsg*       tempErrorMsg = NULL;
    ErrorMsg* pErrorMsg = NULL;
    pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_CLEAN, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_CLEAN, NULL, pErrorMsg);
    }

    pErrorMsg = closeDeviceFile(m_pFileHandle);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_CLEAN, NULL, pErrorMsg);
    }

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}





ErrorMsg*   USBDAPI::writePFMCTest(UINT32   Index,
                                          list<string> *returnDataInfo,
                                          UINT32 StartSector,
                                          UINT32 TestSize,
                                          UINT32   BlockSize)

{
    ErrorMsg* pErrorMsg = NULL;
    UINT64 intTestSize = TestSize * MEGA_BYTES_SIZE;



    pErrorMsg = writePFMCTestSetup(Index, StartSector, intTestSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST, NULL, pErrorMsg);
    }
    pErrorMsg = writePFMCTestExecute(Index, returnDataInfo, StartSector, intTestSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST, NULL, pErrorMsg);
    }
    pErrorMsg = writePFMCTestCleanup(Index);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST, NULL, pErrorMsg);
    }

    return NO_ERROR_MSG;
}





ErrorMsg*   USBDAPI::writePFMCTestSetup(UINT32   Index,
                                               UINT32   LogicalSector,
                                               UINT32   TestSize,
                                               UINT32   BlockSize)
{

    UINT64 u64DeviceCapacity = 0;

    ErrorMsg*       pErrorMsg = NULL;
    ErrorMsg*       tempErrorMsg = NULL;
    list<string>    strTempList;
    strTempList.clear();

    initProgress(TestSize);

    // Check Test Size valid
    pErrorMsg = m_USBDiver->getElementByIndex(Index, USB_DATA_CAPACITY_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP , NULL, pErrorMsg);
    }
    u64DeviceCapacity = atoll(strTempList.front().c_str()) * MEGA_BYTES_SIZE;

    if(u64DeviceCapacity==0 || (u64DeviceCapacity - LogicalSector*USB_BLOCK_SIZE) < TestSize )
    {
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP|ERROR_OUT_OF_RANGE), NULL, NULL);
    }
    strTempList.clear();

    // Get Device Node
    pErrorMsg = m_USBDiver->getElementByIndex(Index, USB_DATA_NODE_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP , NULL, pErrorMsg);
    }
    //Open Device node
    pErrorMsg = openDeviceFile(strTempList.front().c_str());
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP, NULL, pErrorMsg);
    }
    strTempList.clear();

    pErrorMsg = m_pMemoryDAPI->getMemoryBuffer(BlockSize, (SYSTEM_ADDRESS_T*)&m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP | ERROR_USB_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL) , NULL, pErrorMsg);
    }
    memset(m_pDataBuffer, 0, BlockSize);

    return NO_ERROR_MSG;
}





ErrorMsg*   USBDAPI::writePFMCTestExecute(UINT32   Index,
                                                 list<string> *returnDataInfo,
                                                 UINT32   LogicalSector,
                                                 UINT32   TestSize,
                                                 UINT32   BlockSize)
{
    ErrorMsg* pErrorMsg = NULL;
    ErrorMsg* tempErrorMsg = NULL;
    timeval StartTime;
    timeval EndTime;
    timeval spendTime;
    double  Speed = 0;
    UINT32  intTempTestSize;
    intTempTestSize = TestSize;

    CHAR tempResultString[256];
    memset(tempResultString, 0, sizeof(tempResultString));

    gettimeofday(&StartTime, NULL);

    while(intTempTestSize > 0)
    {

        pErrorMsg = m_USBDiver->PWRITE(m_pFileHandle,
                                       m_pDataBuffer,
                                       LogicalSector,
                                       BlockSize);


        if(pErrorMsg != NULL)
        {
            if(pErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE, NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, pErrorMsg);
            }
            m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_READ_DATA_FROM_STORAGE_FAIL),
                                  NULL,
                                  pErrorMsg);
        }

        LogicalSector = LogicalSector + BlockSize/USB_BLOCK_SIZE;
        intTempTestSize = intTempTestSize - BlockSize;

        updateProgress(BlockSize);

    }

    gettimeofday(&EndTime, NULL);

    spendTime.tv_sec = EndTime.tv_sec - StartTime.tv_sec;
    spendTime.tv_usec = EndTime.tv_usec - StartTime.tv_usec;

    if(spendTime.tv_usec < 0)
    {
        --spendTime.tv_sec;
        spendTime.tv_usec += 1000000;
    }

    double dDataSize = (double)((double)TestSize/(MEGA_BYTES_SIZE));
    double dCostTime = (double)(spendTime.tv_sec)+ ((double)(spendTime.tv_usec)/1000000);

    if(dCostTime == 0)
    {
        m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_DIVIDED_ZERO_FAIL), NULL, NULL);
    }
    else
    {
        Speed = (double)dDataSize/dCostTime;
    }
    sprintf(tempResultString, "Spend time: %lf s\n", dCostTime);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);
    memset(tempResultString, 0, sizeof(tempResultString));
    sprintf(tempResultString,"Speed:     %5.2f M/s\n", Speed);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);


    return NO_ERROR_MSG;
}





ErrorMsg*   USBDAPI::writePFMCTestCleanup(UINT32   Index)
{
    ErrorMsg*       tempErrorMsg = NULL;
    ErrorMsg* pErrorMsg = NULL;
    pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_CLEAN, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_CLEAN, NULL, pErrorMsg);
    }

    pErrorMsg = closeDeviceFile(m_pFileHandle);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_CLEAN, NULL, pErrorMsg);
    }

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}




ErrorMsg*
USBDAPI::openDeviceFile(STRING strDeviceNode)
{
    if((m_pFileHandle = open(strDeviceNode, O_RDWR | O_SYNC)) == -1)
    {
        //ADD Error Message
        return returnErrorMsg((ERRORCODE)(ERROR_OPEN_FILE|ERROR_USB_DAPI_OPEN_DEVICE),NULL,NULL);
    }
    return returnErrorMsg(NO_ERROR, NULL, NULL);
}




ErrorMsg*
USBDAPI::closeDeviceFile(UINT32 intFileNode)
{
    UINT32 error = 0;
    error = close(intFileNode);
    if(error != 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_CLOSE_FILE|ERROR_USB_DAPI_CLOSE_DEVICE), NULL, NULL);
    }
    return returnErrorMsg(NO_ERROR, NULL, NULL);
}
