#include "BroadwellDE.hpp"


BroadwellDE::BroadwellDE(STRING in_szClassName) : I2CTemperature(in_szClassName)
{
    m_errorCode = NO_ERROR_MSG;
}



BroadwellDE::~BroadwellDE()
{

}



ErrorMsg*
BroadwellDE::GetCurrntTemperature(string wellknow_name, string *result)
{
    char                     TemperatureChar[10];
    list<unsigned long long> TCC_Act, MSR_value;
    ProcessorDDAPI           CPU_Driver("CPU driver");

    //Read CPU TCC_ACTIVATION_TEMP
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_TEMPERATURE_TARGET, 1, &TCC_Act);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }

    //Read CPU IA32_PACKAGE_THERM_STATUS(TEMPERATURE)
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_THERMAL_STATUS, 1, &MSR_value);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }
    
    //printf("TCC_Act: %lld\n", TCC_Act.front());
    //printf("MSR_value: %lld\n", MSR_value.front());

    sprintf(TemperatureChar, "%lld", ((TCC_Act.front() >> 16) & 0xFF) - ((MSR_value.front() >> 16) & 0x7F ));
    *result = TemperatureChar;

    return NO_ERROR_MSG;
}



ErrorMsg*
BroadwellDE::GetThresholdHigh(string wellknow_name, string *result)
{
    char                     TemperatureChar[10];
    list<unsigned long long> TCC_Act, MSR_value;
    ProcessorDDAPI           CPU_Driver("CPU driver");

    //Read CPU TCC_ACTIVATION_TEMP
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_TEMPERATURE_TARGET, 1, &TCC_Act);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }

    //Read CPU THRESHOLD_1_REL_TEMP
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_THERMAL_INTERRUPT, 1, &MSR_value);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }
    
    //printf("TCC_Act: %lld\n", TCC_Act.front());
    //printf("MSR_value: %lld\n", MSR_value.front());

    sprintf(TemperatureChar, "%lld", ((TCC_Act.front() >> 16) & 0xFF) - ((MSR_value.front() >> 8) & 0x7F ));
    *result = TemperatureChar;

    return NO_ERROR_MSG;
}



ErrorMsg*
BroadwellDE::GetThresholdLow(string wellknow_name, string *result)
{
    char                     TemperatureChar[10];
    list<unsigned long long> TCC_Act, MSR_value;
    ProcessorDDAPI           CPU_Driver("CPU driver");

    //Read CPU TCC_ACTIVATION_TEMP
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_TEMPERATURE_TARGET, 1, &TCC_Act);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }

    //Read CPU THRESHOLD_2_REL_TEMP
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_THERMAL_INTERRUPT, 1, &MSR_value);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }

    //printf("TCC_Act: %lld\n", TCC_Act.front());
    //printf("MSR_value: %lld\n", MSR_value.front());

    sprintf(TemperatureChar, "%lld", ((TCC_Act.front() >> 16) & 0xFF) - ((MSR_value.front() >> 16) & 0x7F ));
    *result = TemperatureChar;

    return NO_ERROR_MSG;
}



ErrorMsg*
BroadwellDE::SetThresholdHigh(string wellknow_name, string data)
{
    list<unsigned long long> TCC_Act, MSR_value;
    unsigned long long       write_data;
    ProcessorDDAPI           CPU_Driver("CPU driver");
    int                      int_data;

    sscanf(data.c_str(), "%d", &int_data);
    //Read CPU TCC_ACTIVATION_TEMP
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_TEMPERATURE_TARGET, 1, &TCC_Act);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }

    //Read CPU THRESHOLD_1_REL_TEMP
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_THERMAL_INTERRUPT, 1, &MSR_value);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }

    int_data = ((TCC_Act.front() >> 16) & 0xFF) - int_data;

    write_data = (MSR_value.front() & 0xFFFF80FF) | (int_data << 8);

    //Write THRESHOLD_1_REL_TEMP to CPU
    m_errorCode = CPU_Driver.WriteCPURegister(wellknow_name, MSR_THERMAL_INTERRUPT, write_data);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }
    return NO_ERROR_MSG;
}



ErrorMsg*
BroadwellDE::SetThresholdLow(string wellknow_name, string data)
{
    list<unsigned long long> TCC_Act, MSR_value;
    unsigned long long       write_data;
    ProcessorDDAPI           CPU_Driver("CPU driver");
    int                      int_data;

    sscanf(data.c_str(), "%d", &int_data);
    //Read CPU TCC_ACTIVATION_TEMP
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_TEMPERATURE_TARGET, 1, &TCC_Act);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }

    //Read CPU THRESHOLD_2_REL_TEMP
    m_errorCode = CPU_Driver.ReadCPURegister(wellknow_name, MSR_THERMAL_INTERRUPT, 1, &MSR_value);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }

    int_data = ((TCC_Act.front() >> 16) & 0xFF) - int_data;

    write_data = (MSR_value.front() & 0xFF80FFFF) | (int_data << 16);

    //Write THRESHOLD_2_REL_TEMP to CPU
    m_errorCode = CPU_Driver.WriteCPURegister(wellknow_name, MSR_THERMAL_INTERRUPT, write_data);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return m_errorCode;
    }

    return NO_ERROR_MSG;
    return NO_ERROR_MSG;
}


