#include "INTEL_X552.hpp"



INTEL_X552::INTEL_X552(STRING nic_name):EthernetBaseClass(nic_name)
{
   Append_Node("/device/resource0");
}



INTEL_X552::~INTEL_X552()
{

}



ErrorMsg* INTEL_X552::Set_external_loopback()
{
   unsigned int *addr;
   int fd;

   fd = open(Get_Node().c_str(), O_RDWR | O_SYNC);
   if (fd < 0)
   {
       //printf("%s file open error!\n",Get_Node().c_str());
       return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), NULL, NULL);
   }

   addr = (unsigned int *)mmap(NULL, 0x1000, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0x00);
   MDIO_point = addr + (MDIO_CONTROL_REGISTER / 4);

   //printf("0x%04x\n",read_PHY_reg(0, 0));
   //1G setup
   write_PHY_reg(18, 6, 0x0008);
   write_PHY_reg(0, 0, 0x9340);

   //100/10M setup
   //write_PHY_reg(18, 6, 0x0000);
   //write_PHY_reg(0, 0, 0x1140);

   munmap(addr, 0x1000);
   close(fd);

   return NO_ERROR_MSG;
}



ErrorMsg* INTEL_X552::Remove_external_loopback()
{
   unsigned int *addr;
   int fd;

   fd = open(Get_Node().c_str(), O_RDWR | O_SYNC);
   if (fd < 0)
   {
       //printf("%s file open error!\n",Get_Node().c_str());
       return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), NULL, NULL);
   }

   addr = (unsigned int *)mmap(NULL, 0x1000, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0x00);
   MDIO_point = addr + (MDIO_CONTROL_REGISTER / 4);

   //printf("0x%04x\n",read_PHY_reg(0, 0));

   //100/10M setup
   write_PHY_reg(18, 6, 0x0000);
   write_PHY_reg(0, 0, 0x1140);

   munmap(addr, 0x1000);
   close(fd);

   return NO_ERROR_MSG;
}



int INTEL_X552::read_PHY_reg(int reg, int page)
{
   //write page(Register 22)
   if (reg != 22)
   {
      *MDIO_point =  MDIO_WRITE | ((0x16 & PHY_REG_NUM) << 16) | (page & PHY_DATA_LEN);
      usleep(1);
   }

   *MDIO_point =  MDIO_READ | ((reg & PHY_REG_NUM) << 16);
   usleep(1);
   return *MDIO_point & 0x0FFFF;
}



int INTEL_X552::write_PHY_reg(int reg, int page , int data)
{
   //write page(Register 22)
   if (reg != 22)
   {
      *MDIO_point =  MDIO_WRITE | ((0x16 & PHY_REG_NUM) << 16) | (page & PHY_DATA_LEN);
      usleep(1);
   }

   *MDIO_point =  MDIO_WRITE | ((reg & PHY_REG_NUM) << 16) | (data & PHY_DATA_LEN);
   usleep(1);

   return 0;
}



ErrorMsg* INTEL_X552::Get_self_test_result(struct ethtool_test *test, TestType index, INT32 *test_result)
{
    //printf("test->data[0]:%d  test->data[1]:%d  test->data[2]:%d  test->data[3]:%d  test->data[4]:%d\n"
    //       ,test->data[0], test->data[1], test->data[2], test->data[3], test->data[4]);

    switch(index)
    {
        case Register:
            *test_result = test->data[0];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_REGISTER_TEST_FAIL), NULL, NULL);
            break;
        case EepromValidation:
            *test_result = test->data[1];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_EEPROM_VALIDATION_TEST_FAIL), NULL, NULL);
            break;
        case Interrupt:
            *test_result = test->data[2];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_INTERRUPT_TEST_FAIL), NULL, NULL);
            break;
        case InternalLoopback:
            *test_result = test->data[3];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_INTERNAL_LOOPBACK_TEST_FAIL), NULL, NULL);
            break;
        case LinkStatus:
            *test_result = test->data[4];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_MDI_REGISTER_TEST_FAIL), NULL, NULL);
            break;
        default:
            return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_SENDIOCTL_UNDEFINED), NULL, NULL);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_X552::SetupSpeed(string nic, INT32 speed)
{
    return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED | ERROR_ETHERNET_DAPI_SETUP_SPEED_TIMEOUT), "Not support", NULL);
}



ErrorMsg* INTEL_X552::GetMAC(string ethernet_name, string *mac_address)
{
    INT32  err, fd, i, PCI_function_id;
    struct ethtool_eeprom *eeprom;
    struct ifreq ifr;
    char   string_temp[3];

    errno = 0;

    if (ethernet_name.length() == 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC), NULL, NULL);
    }

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL), NULL, NULL);
    }

    strncpy(ifr.ifr_name, ethernet_name.c_str(), sizeof(ifr.ifr_name));

    eeprom = (ethtool_eeprom *)calloc(1, sizeof(*eeprom) + 6);
    if (!eeprom)
    {
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL), NULL, NULL);
    }

    GetPCIFunction(ethernet_name, &PCI_function_id);

    //get configuration of LAN port 0(0x9)/1(0xa) pointer
    //get the MAC in EEPROM offset
    eeprom->cmd = ETHTOOL_GEEPROM;
    eeprom->len = 2;
    if (PCI_function_id == 0)
        eeprom->offset = 0x09 * 2; //LAN Core 0 Module Pointer
    else if (PCI_function_id == 1)
        eeprom->offset = 0x0a * 2; //LAN Core 1 Module Pointer

    ifr.ifr_data = (__caddr_t)eeprom;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        //perror("Cannot do get EEPROM");
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    //get the MAC in EEPROM
    eeprom->cmd = ETHTOOL_GEEPROM;
    eeprom->len = 6;
    eeprom->offset = (((eeprom->data[1] << 8) | eeprom->data[0]) + 1 ) * 2; //LAN Core 0 Module Pointer

    ifr.ifr_data = (__caddr_t)eeprom;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        //perror("Cannot do get EEPROM");
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    for (i = 0; i < eeprom->len; i++)
    {
        sprintf(string_temp, "%02X",eeprom->data[i]);

        *mac_address += string_temp;
        if ( i != 5 )
        {
            *mac_address += ":";
        }
    }

    free(eeprom);
    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_X552::SetMAC(string ethernet_name, string *mac_address)
{
    INT32  fd, PCI_function_id;
    struct ethtool_eeprom *eeprom;
    struct ifreq ifr;
    INT32  MACnumber;

    errno = 0;

    if (ethernet_name.length() == 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC), NULL, NULL);
    }

    strncpy(ifr.ifr_name, ethernet_name.c_str(), sizeof(ifr.ifr_name));

    // Open control socket.
    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL), NULL, NULL);
    }

    eeprom = (ethtool_eeprom *)calloc(1, sizeof(*eeprom) + 6);
    if (!eeprom)
    {
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL), NULL, NULL);
    }

    GetPCIFunction(ethernet_name, &PCI_function_id);

    //get configuration of LAN port 0(0x9)/1(0xa)
    //get the MAC in EEPROM offset
    eeprom->cmd = ETHTOOL_GEEPROM;
    eeprom->len = 2;
    if (PCI_function_id == 0)
        eeprom->offset = (0x09) * 2 ; //LAN Core 0 Module Pointer
    else if (PCI_function_id == 1)
        eeprom->offset = (0x0A) * 2 ; //LAN Core 1 Module Pointer

    eeprom->magic = 0;

    ifr.ifr_data = (caddr_t)eeprom;
    if (ioctl(fd, SIOCETHTOOL, &ifr) < 0)
    {
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    //Ready to write MAC
    eeprom->cmd = ETHTOOL_SEEPROM;
    eeprom->len = 6;
    eeprom->offset = (((eeprom->data[1] << 8) | eeprom->data[0]) + 1) * 2;;
    eeprom->magic = 0x15AB8086;

    sscanf((mac_address[0].c_str()), "%x", &MACnumber);
    eeprom->data[0] = MACnumber;
    sscanf((mac_address[1].c_str()), "%x", &MACnumber);
    eeprom->data[1] = MACnumber;
    sscanf((mac_address[2].c_str()), "%x", &MACnumber);
    eeprom->data[2] = MACnumber;
    sscanf((mac_address[3].c_str()), "%x", &MACnumber);
    eeprom->data[3] = MACnumber;
    sscanf((mac_address[4].c_str()), "%x", &MACnumber);
    eeprom->data[4] = MACnumber;
    sscanf((mac_address[5].c_str()), "%x", &MACnumber);
    eeprom->data[5] = MACnumber;

    ifr.ifr_data = (__caddr_t)eeprom;

    if (ioctl(fd, SIOCETHTOOL, &ifr) < 0)
    {
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_X552::GetEEPROM(string ethernet_name, INT32 StartAddress, INT32 Length, INT32 *data)
{
    struct ethtool_eeprom *eeprom;
    struct ifreq          ifr;
           INT32          err, fd, i;

    errno = 0;

    if (ethernet_name.length() == 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC), NULL, NULL);
    }

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL), NULL, NULL);
    }

    strncpy(ifr.ifr_name, ethernet_name.c_str(), sizeof(ifr.ifr_name));

    eeprom = (ethtool_eeprom *)calloc(1, sizeof(*eeprom) + Length);
    if (!eeprom)
    {
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL), NULL, NULL);
    }

    eeprom->cmd    = ETHTOOL_GEEPROM;
    eeprom->len    = Length;
    eeprom->offset = StartAddress;

    ifr.ifr_data = (__caddr_t)eeprom;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        //perror("Cannot do get EEPROM");
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    for (i = 0; i < eeprom->len; i++)
    {
        data[i] = eeprom->data[i];
    }

    free(eeprom);
    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_X552::SetEEPROM(string ethernet_name, INT32 StartAddress, INT32 *data, INT32 Length)
{
    struct ethtool_eeprom *eeprom;
    struct ifreq          ifr;
           INT32          err, fd, i;

    errno = 0;

    if (ethernet_name.length() == 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC), NULL, NULL);
    }

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL), NULL, NULL);
    }

    strncpy(ifr.ifr_name, ethernet_name.c_str(), sizeof(ifr.ifr_name));

    eeprom = (ethtool_eeprom *)calloc(1, sizeof(*eeprom) + Length);
    if (!eeprom)
    {
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL), NULL, NULL);
    }

    eeprom->cmd     = ETHTOOL_SEEPROM;
    eeprom->len     = Length;
    eeprom->offset  = StartAddress;
    eeprom->magic   = 0x15AB8086; //Intel device_id:vend_id
    for (i = 0; i < Length ; i++)
    {
        eeprom->data[i] = data[i];
    }

    ifr.ifr_data = (__caddr_t)eeprom;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        //perror("Cannot do get EEPROM");
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    free(eeprom);
    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_X552::ReadSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 Length, INT32 *data)
{
    return returnErrorMsg(ERROR_ETHERNET_DDAPI_SENDIOCTL, "Intel X552(KR) not support", NULL);
}


