#include "QLogic_EP8324.hpp"



QLOGIC_EP8324::QLOGIC_EP8324(STRING nic_name):EthernetBaseClass(nic_name)
{
   Append_Node("/device/resource0");
   m_errorCode = NO_ERROR_MSG;
}



QLOGIC_EP8324::~QLOGIC_EP8324()
{

}



ErrorMsg* QLOGIC_EP8324::Get_self_test_result(struct ethtool_test *test, TestType index, INT32 *test_result)
{
//    printf("test->data[0]:%d  test->data[1]:%d  test->data[2]:%d  test->data[3]:%d  test->data[5]:%d\n"
//           ,test->data[0], test->data[1], test->data[2], test->data[3], test->data[5]);

    switch(index)
    {
        case Register:
            *test_result = test->data[0];
            break;
        case EepromValidation:
            *test_result = test->data[5];
            break;
        case Interrupt:
            *test_result = test->data[2];
            break;
        case InternalLoopback:
            *test_result = test->data[3];
            break;
        /*case InternalLoopback:
            *test_result = test->data[4];
            break;*/
        case LinkStatus:
            *test_result = test->data[1];
            break;
        default:
            return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_SENDIOCTL_UNDEFINED), NULL, NULL);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* QLOGIC_EP8324::SetupSpeed(string nic, INT32 speed)
{
    string link_up;
    int    sleep_count = 0;

    GetLinkStatus(nic, &link_up);

    while (link_up.compare("Down") == 0)
    {
       //printf("Wait link UP!!!\n");
       sleep(1);
       sleep_count++;
       if (sleep_count > 6)
       {
          return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED | ERROR_ETHERNET_DAPI_SETUP_SPEED_TIMEOUT), nic.c_str(), NULL);
       }
       GetLinkStatus(nic, &link_up);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* QLOGIC_EP8324::GetMAC(string ethernet_name, string *mac_address)
{
    int         domain, bus, device, func;
    UINT32      u32Bar;
    UINT32      data;
    PCIDDAPI    *pciDriverAPI = NULL;
    MemoryDDAPI *m_memoryDriver = NULL;
    char        string_temp[3];

    pciDriverAPI = new PCIDDAPI("PCIDriver");

    //Get EP8324 PCI BDF
    m_errorCode = GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "EP8324 get MAC fail", m_errorCode);
    }

    //Get EP8324 PCI base address
    pciDriverAPI->getBarByBDF(domain, bus, device, 0, 0, &u32Bar);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "EP8324 get MAC fail", m_errorCode);
    }

    m_memoryDriver = new MemoryDDAPI("MemDriver");

    //Write the address where the MAC in EEPROM
    if (func == 2)
    {
        m_memoryDriver->write(u32Bar, (unsigned int)(SPI_FLASH_START_ADDRESS | (FUNCTION_2_MAC >> 2)) );
    }
    else if (func == 3)
    {
        m_memoryDriver->write(u32Bar, (unsigned int)(SPI_FLASH_START_ADDRESS | (FUNCTION_3_MAC >> 2)) );
    }

    //Wait for SPI to get EEPROM data
    while ( (m_memoryDriver->read64(u32Bar) & FLASH_ACCESS_DATA_FLAG) == 0 ) { }
    //Ready to get data
    data = m_memoryDriver->read64(u32Bar + 4);


    sprintf(string_temp, "%02X", ((unsigned int)data >> 24) & 0x0FF);
    mac_address->append(string_temp);
    mac_address->append(":");
    sprintf(string_temp, "%02X", ((unsigned int)data >> 16) & 0x0FF);
    mac_address->append(string_temp);
    mac_address->append(":");
    sprintf(string_temp, "%02X", ((unsigned int)data >>  8) & 0x0FF);
    mac_address->append(string_temp);
    mac_address->append(":");
    sprintf(string_temp, "%02X", ((unsigned int)data >>  0) & 0x0FF);
    mac_address->append(string_temp);


    //Write the address where the MAC in EEPROM
    if (func == 2)
    {
        m_memoryDriver->write(u32Bar, (unsigned int)(SPI_FLASH_START_ADDRESS | ((FUNCTION_2_MAC + 4) >> 2)) );
    }
    else if (func == 3)
    {
        m_memoryDriver->write(u32Bar, (unsigned int)(SPI_FLASH_START_ADDRESS | ((FUNCTION_3_MAC + 4) >> 2)) );
    }
    //Wait for SPI to get EEPROM data
    while ( (m_memoryDriver->read64(u32Bar) & FLASH_ACCESS_DATA_FLAG) == 0 ) { }
    //Ready to get data
    data = m_memoryDriver->read64(u32Bar + 4);


    mac_address->insert(0, ":");
    sprintf(string_temp, "%02X", ((unsigned int)data >>  0) & 0x0FF);
    mac_address->insert(0, string_temp);
    mac_address->insert(0, ":");
    sprintf(string_temp, "%02X", ((unsigned int)data >>  8) & 0x0FF);
    mac_address->insert(0, string_temp);

    delete m_memoryDriver;

    return NO_ERROR_MSG;
}



ErrorMsg* QLOGIC_EP8324::SetMAC(string ethernet_name, string *mac_address)
{
    UINT32  data32[2], string_to_int;
    UINT32  offset, i;
    UINT32  cnaBoardConfig[CNA_FLASH_SECTOR_SIZE];
    int     domain, bus, device, func;

    //Get PCI BDF data
    GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);

    //Transform MAC string to int for writing
    sscanf(mac_address[5].c_str(), "%x", &string_to_int);
    data32[0] = string_to_int;
    sscanf(mac_address[4].c_str(), "%x", &string_to_int);
    data32[0] |= string_to_int << 8;
    sscanf(mac_address[3].c_str(), "%x", &string_to_int);
    data32[0] |= string_to_int << 16;
    sscanf(mac_address[2].c_str(), "%x", &string_to_int);
    data32[0] |= string_to_int << 24;

    sscanf(mac_address[1].c_str(), "%x", &string_to_int);
    data32[1] = string_to_int;
    sscanf(mac_address[0].c_str(), "%x", &string_to_int);
    data32[1] |= string_to_int << 8;


    // store entire sector
    for (offset = FLASH_CNA_BOARD_CONFIG_START, i = 0;
         offset < FLASH_CNA_BOARD_CONFIG_START + CNA_FLASH_SECTOR_SIZE;
         offset += 4, i++)
    {
        m_errorCode = SPIRead(ethernet_name, offset, &cnaBoardConfig[i]);

        if (m_errorCode != NO_ERROR_MSG)
        {
            return returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, NULL, m_errorCode);
        }
    }


    // write what we want
    for (offset = FLASH_CNA_BOARD_CONFIG_START, i = 0;
         offset < FLASH_CNA_BOARD_CONFIG_START + CNA_FLASH_SECTOR_SIZE;
         offset += 4, i++)
    {
        if (offset == FLASH_CNA_BOARD_CONFIG_CHECKSUM)
            cnaBoardConfig[i] = 0xffffffff; // leave blank

        //=========== port 1 MAC =============
        if (offset == FUNCTION_2_MAC && func == 2)
            cnaBoardConfig[i] = data32[0];

        if (offset == (FUNCTION_2_MAC + 4) && func == 2)
            cnaBoardConfig[i] = data32[1];

        //=========== port 2 MAC =============
        if (offset == FUNCTION_3_MAC && func == 3)
            cnaBoardConfig[i] = data32[0];

        if (offset == (FUNCTION_3_MAC + 4) && func == 3)
            cnaBoardConfig[i] = data32[1];
    }


    m_errorCode = SPIBufferWrite(ethernet_name,
                                 FLASH_CNA_BOARD_CONFIG_START,
                                 cnaBoardConfig,
                                 CNA_FLASH_SECTOR_SIZE/4,
                                 0);


    if (m_errorCode == NO_ERROR_MSG)
    {
        // update checksum
        m_errorCode = UpdateFlashChecksum(ethernet_name,
                                          FLASH_CNA_BOARD_CONFIG_START,
                                          FLASH_CNA_BOARD_CONFIG_START + FLASH_CNA_BOARD_CONFIG_SIZE,
                                          FLASH_CNA_BOARD_CONFIG_CHECKSUM);
    }


    return NO_ERROR_MSG;
}



ErrorMsg* QLOGIC_EP8324::SPIRead(string ethernet_name, UINT32 spiAddress, UINT32 *returnedDataValue)
{
    int    ret;
    struct ep8324_ioctl_cb cb;
    int    domain, bus, device, func;


    /*if (spiAddress & 0x3)
    {
        m_errorCode = ERROR_CNA_SPI_READ;
        return m_errorCode;
    }*/

    memset(&cb, 0, sizeof(struct ep8324_ioctl_cb));

    if ((m_errorCode = OpenDevice()) != NO_ERROR_MSG)
        return m_errorCode;

    //A combination of the well known name by PCI BDF
    GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    //printf("Name:%s  %x  %x  %x  %x\n", ethernet_name.c_str(), domain, bus, device, func);
    sprintf(cb.well_known_name, "CNA-%02x:%02x.%02x", bus, device, 0);
    //printf("%s\n",cb.well_known_name);


    cb.spicb.faddr = spiAddress >> 2; // byte address to flash dword address
    cb.spicb.dwords = 1;


    ret = ioctl(deviceHandle, EP8324_IOCTL_SPI_READ, &cb);
    *returnedDataValue = cb.spicb.dword_buf[0];

    if (!ret)
        m_errorCode = NO_ERROR_MSG;
    else
        m_errorCode = returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, "SPI read fail", NULL);

    close(deviceHandle);

    return m_errorCode;
}



ErrorMsg* QLOGIC_EP8324::SPIWrite(string ethernet_name, UINT32 spiAddress, UINT32 dataToWrite)
{
    int    ret;
    struct ep8324_ioctl_cb cb;
    UINT32 data = dataToWrite;
    int    domain, bus, device, func;

    /*if (spiAddress & 0x3)
    {
        m_errorCode = ERROR_CNA_SPI_WRITE;
        return m_errorCode;
    }*/

    memset(&cb, 0, sizeof(struct ep8324_ioctl_cb));

    if ((m_errorCode = OpenDevice()) != NO_ERROR_MSG)
        return m_errorCode;

    //A combination of the well known name by PCI BDF
    GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    //printf("Name:%s  %x  %x  %x  %x\n", ethernet_name.c_str(), domain, bus, device, func);
    sprintf(cb.well_known_name, "CNA-%02x:%02x.%02x", bus, device, 0);
    //printf("%s\n",cb.well_known_name);

    cb.spicb.faddr = spiAddress >> 2; // byte address to flash dword address
    cb.spicb.dwords = 1;
    cb.spicb.dword_buf[0] = data;

    ret = ioctl(deviceHandle, EP8324_IOCTL_SPI_WRITE, &cb);
    if (!ret)
        m_errorCode = NO_ERROR_MSG;
    else
        m_errorCode = returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, "SPI write fail", NULL);

    close(deviceHandle);

    return m_errorCode;
}



ErrorMsg* QLOGIC_EP8324::SPIBufferWrite(string ethernet_name, UINT32 spiAddress, UINT32 *dataBuffer, UINT32 dwordsToWrite, UINT32 showProgress)
{
    int    ret;
    struct ep8324_ioctl_cb cb;
    UINT32 dwords;
    UINT32 totals = 0;
    UINT32 total_dwords = dwordsToWrite;
    UINT32 percentage = 0;
    int    domain, bus, device, func;
    //char   m_tempString[201];

    /*if (spiAddress & 0x3)
    {
        m_errorCode = ERROR_CNA_SPI_WRITE;
        return m_errorCode;
    }*/

    memset(&cb, 0, sizeof(struct ep8324_ioctl_cb));

    if ((m_errorCode = OpenDevice()) != NO_ERROR_MSG)
        return m_errorCode;

    //A combination of the well known name by PCI BDF
    GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    //printf("Name:%s  %x  %x  %x  %x\n", ethernet_name.c_str(), domain, bus, device, func);
    sprintf(cb.well_known_name, "CNA-%02x:%02x.%02x", bus, device, 0);
    //printf("%s\n",cb.well_known_name);


    while (dwordsToWrite)
    {
        if (dwordsToWrite > 64)
            dwords = 64;
        else
            dwords = dwordsToWrite;

        cb.spicb.faddr = spiAddress >> 2; // byte address to flash dword address
        cb.spicb.dwords = dwords;

        for (unsigned int i = 0; i < dwords; i++)
            cb.spicb.dword_buf[i] = dataBuffer[totals + i];

        ret = ioctl(deviceHandle, EP8324_IOCTL_SPI_WRITE, &cb);
        if (ret)
        {
            m_errorCode = returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, "SPI write fail", NULL);
            break;
        }

        totals += dwords;
        spiAddress += (dwords * 4);
        dwordsToWrite -= dwords;

        // display progress
        if (percentage < (totals*100/total_dwords) && showProgress == 1)
        {
            percentage = totals*100/total_dwords;
            //sprintf(m_tempString, "%3d%%\r", (INT32) percentage);
            if (percentage != 100)
                printf("%3d%%\r", percentage);
            else
                printf("%3d%%\n", percentage);
            fflush(stdout);
        }
    }

    close(deviceHandle);

    return m_errorCode;
}



ErrorMsg* QLOGIC_EP8324::UpdateFlashChecksum(string ethernet_name, UINT32 startAddress, UINT32 endAddress, UINT32 checksumAddress)
{
    UINT32 offset;
    UINT32 dataValue;
    UINT32 sum = 0;

    if (startAddress >= endAddress)
    {
        m_errorCode = returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, "Start address bigger than end", NULL);
        return m_errorCode;
    }

    offset = startAddress;

    while (offset < endAddress)
    {
        m_errorCode = SPIRead(ethernet_name, offset, &dataValue);

        if (m_errorCode != NO_ERROR_MSG)
            return m_errorCode;

        if (offset != checksumAddress)
            sum += dataValue;

        offset += 4;
    }

    // update new checksum value
    m_errorCode = SPIWrite(ethernet_name, checksumAddress, ~sum + 1);

    return m_errorCode;
}



ErrorMsg* QLOGIC_EP8324::OpenDevice(void)
{
    m_errorCode = NO_ERROR_MSG;

    deviceHandle = open("/dev/ep8324", O_RDWR);
    if (deviceHandle < 0)
        m_errorCode = returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, "Open /dev/ep8324 fail", NULL);

    return m_errorCode;
}



ErrorMsg* QLOGIC_EP8324::GetEEPROM(string ethernet_name, INT32 StartAddress, INT32 Length, INT32 *data)
{
    UINT32  *CNAData = NULL;
    UINT32  *TempData = NULL;
    INT32   AlignmentStartAddress, AlignmentEndAddress, ByteOffset, offset, i;


    CNAData = new UINT32[EP8324_FLASH_SIZE];
    TempData = new UINT32[EP8324_FLASH_SIZE];

    AlignmentStartAddress = (StartAddress / 4) * 4;
    AlignmentEndAddress = ((StartAddress + Length) / 4) * 4;
    ByteOffset = StartAddress - AlignmentStartAddress;


    for (offset = AlignmentStartAddress, i = 0;
         offset <= AlignmentEndAddress;
         offset += 4, i++)
    {
        m_errorCode = SPIRead(ethernet_name, offset, &CNAData[i]);
        if (m_errorCode != NO_ERROR_MSG)
        {
            if (CNAData != NULL)
            {
                delete CNAData;
                delete TempData;
            }
            return m_errorCode;
        }
    }


    for (i = 0; i <= (Length / 4)+ByteOffset ; i++)
    {
        TempData[i*4+0] = (CNAData[i] >> 0)  & 0xFF;
        TempData[i*4+1] = (CNAData[i] >> 8)  & 0xFF;
        TempData[i*4+2] = (CNAData[i] >> 16) & 0xFF;
        TempData[i*4+3] = (CNAData[i] >> 24) & 0xFF;
    }

    for (i = 0; i < Length ; i++)
    {
        data[i] =  TempData[i + ByteOffset];
    }

    if (CNAData != NULL)
    {
        delete CNAData;
        delete TempData;
    }

    return m_errorCode;
}



ErrorMsg* QLOGIC_EP8324::SetEEPROM(string ethernet_name, INT32 StartAddress, INT32 *data, INT32 Length)
{
    INT32   i, offset;
    INT32   AlignmentStartAddress, AlignmentEndAddress, ByteOffset;
    UINT32  *CNAData = new UINT32[EP8324_FLASH_SIZE / 4];
    UINT32  *TempData = new UINT32[EP8324_FLASH_SIZE];

    m_errorCode = NO_ERROR_MSG;

    AlignmentStartAddress = (StartAddress / 4) * 4;
    AlignmentEndAddress = ((StartAddress + Length - 1) / 4) * 4;
    ByteOffset = StartAddress - AlignmentStartAddress;

    //Read out the alignment data first
    for (offset = AlignmentStartAddress, i = 0;
         offset <= AlignmentEndAddress;
         offset += 4, i++)
    {
        m_errorCode = SPIRead(ethernet_name, offset, &CNAData[i]);
        if (m_errorCode != NO_ERROR_MSG)
        {
            if (CNAData != NULL)
            {
                delete CNAData;
                delete TempData;
            }
            return m_errorCode;
        }

    }
    //Transform CNA data to one byte data
    for (i = 0; i <= (AlignmentEndAddress - AlignmentStartAddress)/4 ; i++)
    {
        TempData[i*4+0] = (CNAData[i] >> 0)   & 0xFF;
        TempData[i*4+1] = (CNAData[i] >> 8)   & 0xFF;
        TempData[i*4+2] = (CNAData[i] >> 16)  & 0xFF;
        TempData[i*4+3] = (CNAData[i] >> 24)  & 0xFF;

    }
    //Modify what we want to write byte
    for (i = 0; i < Length; i++)
    {
        TempData[i + ByteOffset] = data[i];
    }
    //Assemble four byte to one dword for writing
    for (i = 0; i <= (AlignmentEndAddress - AlignmentStartAddress)/4 ; i++)
    {
        CNAData[i] = (TempData[i*4+3] << 24) | (TempData[i*4+2] << 16) | (TempData[i*4+1] << 8) | (TempData[i*4+0] << 0);
    }
    //Ready to write
    for (offset = AlignmentStartAddress, i = 0;
         offset <= AlignmentEndAddress;
         offset += 4, i++)
    {

        m_errorCode = SPIWrite(ethernet_name, offset, CNAData[i]);
        if (m_errorCode != NO_ERROR_MSG)
        {
            if (CNAData != NULL)
            {
                delete CNAData;
                delete TempData;
            }
            return m_errorCode;
        }
    }


    delete CNAData;
    delete TempData;

    return m_errorCode;
}



ErrorMsg* QLOGIC_EP8324::Program(string ethernet_name, string filePath)
{
    UINT32  *WriteData = NULL;
    UINT32  *FlashBinary = NULL;
    int     fileCount = 0, i = 0 , binaryData, offset;
    string  errorMessage;
    FILE    *fileHandle;

    m_errorCode = NO_ERROR_MSG;

    //printf("name:%s   file:%s\n", ethernet_name.c_str(), filePath.c_str());

    fileHandle = fopen(filePath.c_str(), "rb");
    if (fileHandle < 0)
    {
        errorMessage = "Open " + filePath + " fail";
        return returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, errorMessage.c_str(), NULL);
    }

    WriteData = new UINT32[EP8324_FLASH_SIZE / 4];
    FlashBinary = new UINT32[EP8324_FLASH_SIZE];

    //Read out the binary file
    while ((binaryData = fgetc(fileHandle)) != EOF)
    {
        FlashBinary[fileCount] = binaryData;
        fileCount++;
    }

    //Combine four byte to one byte for writing to flash
    for (i = 0, offset = 0;
         i < (EP8324_FLASH_SIZE / 4);
         i++, offset += 4)
    {
      WriteData[i] = FlashBinary[offset + 3] << 24 |
                     FlashBinary[offset + 2] << 16 |
                     FlashBinary[offset + 1] <<  8 |
                     FlashBinary[offset + 0] <<  0;
    }

    /*for (i = 0; i < (EP8324_FLASH_SIZE / 4); i++)
    {
        printf("%08x ", WriteData[i]);
        if (i == 10)
        {
           printf("\n");
           break;
        }
    }*/

    //Start to write binary to flash
    m_errorCode = SPIBufferWrite(ethernet_name,
                                 0,
                                 WriteData,
                                 EP8324_FLASH_SIZE/4,
                                 1);

    delete WriteData;
    delete FlashBinary;
    fclose(fileHandle);

    return m_errorCode;
}



ErrorMsg* QLOGIC_EP8324::ReadSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 Length, INT32 *data)
{
    int    ret, i;
    struct ep8324_ioctl_cb cb;
    int    domain, bus, device, func;


    /*if (spiAddress & 0x3)
    {
        m_errorCode = ERROR_CNA_SPI_READ;
        return m_errorCode;
    }*/
    m_errorCode = NO_ERROR_MSG;

    memset(&cb, 0, sizeof(struct ep8324_ioctl_cb));

    if ((m_errorCode = OpenDevice()) != NO_ERROR_MSG)
        return m_errorCode;

    //A combination of the well known name by PCI BDF
    GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    //printf("Name:%s  %x  %x  %x  %x\n", ethernet_name.c_str(), domain, bus, device, func);
    sprintf(cb.well_known_name, "CNA-%02x:%02x.%02x", bus, device, func - 2);
    //printf("%s\n",cb.well_known_name);


    for (i = 0; i < Length; i++)
    {
        cb.sfpcb.device_address = DeviceAddress;
        cb.sfpcb.size = 1;
        cb.sfpcb.offset = StartAddress + i;

        ret = ioctl(deviceHandle, EP8324_IOCTL_SFP_READ, &cb);

        if (!ret)
        {
            //memcpy(data, &cb.sfpcb.data[0], Length);
            data[i] = cb.sfpcb.data[0];
        }
        else
        {
            close(deviceHandle);
            return returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, "SPI read fail", NULL);
        }
    }

    close(deviceHandle);

    return m_errorCode;
}



ErrorMsg* QLOGIC_EP8324::WriteSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 data)
{
    int    ret;
    struct ep8324_ioctl_cb cb;
    int    domain, bus, device, func;


    /*if (spiAddress & 0x3)
    {
        m_errorCode = ERROR_CNA_SPI_READ;
        return m_errorCode;
    }*/
    m_errorCode = NO_ERROR_MSG;

    memset(&cb, 0, sizeof(struct ep8324_ioctl_cb));

    if ((m_errorCode = OpenDevice()) != NO_ERROR_MSG)
        return m_errorCode;

    //A combination of the well known name by PCI BDF
    GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    //printf("Name:%s  %x  %x  %x  %x\n", ethernet_name.c_str(), domain, bus, device, func);
    sprintf(cb.well_known_name, "CNA-%02x:%02x.%02x", bus, device, func - 2);
    //printf("%s\n",cb.well_known_name);


    cb.sfpcb.device_address = DeviceAddress;
    cb.sfpcb.size = 1;
    cb.sfpcb.offset = StartAddress;
    memcpy(&cb.sfpcb.data[0], (UINT8 *)&data, 1);

    ret = ioctl(deviceHandle, EP8324_IOCTL_SFP_WRITE, &cb);

    close(deviceHandle);
    
    if (!ret)
    {
        if (cb.sfpcb.mb0 != 0x4000)
        {
            return returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, "SPI write mailbox fail", NULL);
        }
    }
    else
    {
        return returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, "SPI write IOCTL fail", NULL);
    }

    return m_errorCode;
}


