/*
 * GPIOGeneric.hpp
 *
 *  Created on: Dec 2, 2015
 *      Author: ianfang
 */

#ifndef GPIO_PCHDRIVER_HPP
#define GPIO_PCHDRIVER_HPP
#include "GPIOGeneric.hpp"
#include "PCIDDAPI.hpp"
#include "MemoryDDAPI.hpp"


#define LOW_IO_PRIVILEGE    1   /* ioport <= 0x3ff */
#define HIGH_IO_PRIVILEGE   2   /* ioport >0x3ff && ioport < 0xffff */
#define IO_DISABLE          0
#define IO_ENABLE           1
#define HIGH_IO_ENABLE      3   /* io port > 0x3ff */

#define RW_DELAY            1000 /*delay n microseconds for reading and writing I/O ports. */
#define BLOCK_SIZE          1024
#define BASE_PORT           0x3FF
#define PCI_ADDR_PORT       0xCF8
#define PCI_DATA_PORT       0xCFC

#define MAX_PCH_GPIO_CONFIG_ENTRIES    95
#define PCH_NOT_FOUND                  10000



enum {
//    READ = 0,
//    WRITE = -1,

    ReadWrite_BYTE = 1,
	ReadWrite_WORD = 2,
	ReadWrite_DWORD = 4,
};



class GPIO_PCHDriver : public GPIOGeneric
{
    public:
            GPIO_PCHDriver(STRING className);
            ~GPIO_PCHDriver();


        ErrorMsg*   AddEntry(STRING dataElementName,
                             PCH_DRIVER_DATA_TYPE dataElementType,
                             GPIO_RW_MODE gpioRWMode,
                             UINT32 offset,
                             BOOL bitState,
                             BOOL disableWriteProtect = TRUE);
         ErrorMsg*  readINIFile(CHAR *Line);

         ErrorMsg*  getGPIOList(list<string> *pGPIOList);
         ErrorMsg*  readBit(string gpioName, BOOL*  readValue);
         ErrorMsg*  isMyGPIO(string    gpioName);
         ErrorMsg*  writeBit(string  gpioName, BOOL Status);
//        virtual ERRORCODE AddEntry(STRING gpioName,
//                                   UINT32 gpioIndex,
//                                   UINT32 bWritable
//                                   );
//        virtual ERRORCODE GetWellKnownNameByIndex(INT32 index, CHAR* gpioName, INT32 stringLength);
//        virtual ERRORCODE IsMyGPIO(STRING gpioName);
//        virtual ERRORCODE ReadBit(STRING gpioName, BOOL* readValue);
//        virtual ERRORCODE ResetBit(STRING gpioName);
//        virtual ERRORCODE SetBit(STRING gpioName);

    protected:

    private:
         struct PCHGPIOConfiguration{
                            CHAR                    dataElementName[(PCH_DATA_ELEMENT_NAME_LENGTH + 10)];
                            PCH_DRIVER_DATA_TYPE   dataElementType;
                            GPIO_RW_MODE            gpioRWMode;
                            UINT32                  offset;
                            BOOL                    bitState;
                            BOOL                    disableWriteProtect;
                    };
                    typedef struct PCHGPIOConfiguration PCH_GPIO_CONFIGURATION;

                    UINT32                  m_BaseAddressRegister;
                    ErrorMsg                *m_errorCode;
                    INT32                   m_ConfigurationTableIndex;
                    PCIDDAPI                *m_PCIDriver;
                    MemoryDDAPI             *m_MemoryDriver;
                    PCH_GPIO_CONFIGURATION m_gpioConfiguration[MAX_PCH_GPIO_CONFIG_ENTRIES];

                    UINT32 GetGPIOIndexByName(string gpioName);

                    INT16   readIOPort(UINT32 Port, UINT32 *ReturnData, UINT32 ByteSize);
                    INT16   writeIOPort(UINT32 Port, UINT32 DataInfo, UINT32 ByteSize);

                    void 	delay(int us);
};
#endif /* GPIO_PCHDRIVER_HPP */
