#include "I2CDDAPI.hpp"
#include "MemoryDDAPI.hpp"

I2CDDAPI :: I2CDDAPI(STRING className) : BaseClassDDAPI(className)
{
    FILE  *pFile;
    int   index, I2C_device_count;
    char  line[500], teg[20];
    int   teg_number, string_length;


    //Initial device/bus table
    for (index=0; index<DEVICE_NUMBER; index++)
        m_I2C_device_table[index] = NULL;

    for (index=0; index<MASTER_NUMBER; index++)
        m_I2C_bus_table[index] = NULL;

    I2C_device_count = 0;
    m_mux_counter = 0;
    m_errorCode = NO_ERROR_MSG;

    //Open the I2C_profile.ini for parser
    #ifdef DEBUG
        pFile = fopen ("I2C_profile.ini","r");
    #else
        pFile = fopen ("/etc/lemur/I2C_profile.ini","r");
    #endif
    if (pFile == NULL)
    {
        //printf("Error opening file\n");
        addErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_CONSTRUCTOR | ERROR_I2C_DDAPI_OPEN_PROFILE_FAIL), NULL);
        return;
    }

    //Read each line of I2C_profile.ini
    while (fgets(line, 500, pFile) != NULL)
    {
        if (line[0] == '#')
            continue;

        teg_number = sscanf(line, " [%[^]]", teg); //paser the teg:BUS, Topology, Protocol
        if ( (strlen(line) == 1 && line[0] == '\n') || (teg_number != 0))
        {
            continue;
        }
        //printf("Teg(%d):%s line:%s",teg_number, teg, line);

        //Remove the '\n' at the end of line
        string_length = strlen(line);
        if (line[string_length-1] == '\n')
        {
            line[string_length-1] = '\0';
        }


        //Start parser by teg
        if ( strcmp(teg,"BUS") == 0 )
        {
            Initial_I2C_BUS(line);
        }

        if ( strcmp(teg,"Topology") == 0 )
        {
            Initial_I2C_device(line, I2C_device_count);
            I2C_device_count++;
        }

        if ( strcmp(teg,"Protocol") == 0 )
        {
            m_errorCode = Initial_I2C_Protocol(line);
            if (m_errorCode != NO_ERROR_MSG)
            {
                addErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_CONSTRUCTOR | ERROR_I2C_DDAPI_OPEN_PROFILE_FAIL), "Initial I2C Protocol fail", m_errorCode);
                return;
            }
        }
    }
    fclose(pFile);

    //Prepare to setup the each I2C device master bus 
    for (index=0; m_I2C_device_table[index] ; index++)
    {
        int Master_bus;

        Master_bus = Find_physical_master(m_I2C_device_table[index]->get_Bus());
        //printf("Find Master:%d\n", Master_bus);

        if (m_I2C_bus_table[Master_bus] == NULL)
        {
            //printf("Master BUS %d is NULL(logical:0x%x)!!!\n",Master_bus, m_I2C_device_table[index]->get_Bus());
            addErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_CONSTRUCTOR | ERROR_I2C_DDAPI_BUS_NULL), NULL);
            return;
        }
        else if (Master_bus == -1)
        {
            //printf("Can't find the master bus %d\n", m_I2C_device_table[index]->get_Bus());
            addErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_CONSTRUCTOR | ERROR_I2C_DDAPI_BUS_NOT_FOUND), NULL);
            return;
        }
        else
        {
            m_I2C_device_table[index]->set_Master_bus(m_I2C_bus_table[Master_bus]);
        }

    }

    //For debug. Show the I2C device protocol 
    /*for (index=0; m_I2C_device_table[index] ; index++)
    {
        printf("%s\n",m_I2C_device_table[index]->get_Name());
        for(int j=0; j<REGISTER_NUMBER; j++)
        {
            printf("%3d:%d ",j, m_I2C_device_table[index]->get_Register_protocol(j));
            if ((j+1) % 25 == 0)
                printf("\n");
        }
        printf("\n");
    }*/

    //For debug. Show BUS information
    /*for (index=0; m_I2C_bus_table[index]; index++)
        if (m_I2C_bus_table[index] != NULL)
            printf("BUS name:%s ID:%d  frequency:%dKHz\n",m_I2C_bus_table[index]->get_Bus_name(), m_I2C_bus_table[index]->get_Bus_ID(), m_I2C_bus_table[index]\
            ->get_frequency());*/

    //For debug. Show I2C device information 
    /*for (int i=0; m_I2C_device_table[i] ; i++)
    {
        I2CBus  *p_I2CBus;

        printf("Name:%-7s  bus:0x%x  address:0x%x  ",m_I2C_device_table[i]->get_Name(), m_I2C_device_table[i]->get_Bus(), m_I2C_device_table[i]->get_Address());
        p_I2CBus = m_I2C_device_table[i]->get_Master_bus();
        printf("PHY. Bus name:%s ID:%d ",p_I2CBus->get_Bus_name(),p_I2CBus->get_Bus_ID());

        if (m_I2C_device_table[i]->IsSwitch())
        {
            I2CSwitch *p_switch;

            p_switch = (I2CSwitch *)m_I2C_device_table[i];
            //p_switch->set_input_port(2);
            printf(" output:");
            for(int j=0; j<p_switch->get_output_port_counter(); j++)
                printf("%d ",p_switch->get_output_port(j));
        }
        printf("\n");
    }*/

    //For debug get device by wellknown name/bus address
    /*I2CDevice *p_i2c_device;
    I2CBus    *p_i2c_bus;

    //p_i2c_device = get_device_by_well_known_name("VPD01");
    p_i2c_device = get_device_by_bus_address(0x07, 0x23);
    if (p_i2c_device == NULL)
    {
        m_errorCode = NO_ERROR;
        printf("There is no this I2C device.\n");
    }
    else
    {
        p_i2c_bus = p_i2c_device->get_Master_bus();
        printf("Name:%-7sBUS:0x%x  address:0x%x  Mastet bus:%d\n",p_i2c_device->get_Name(),p_i2c_device->get_Bus(),p_i2c_device->get_Address(), p_i2c_bus->get_Bus_ID());
    }*/

}


I2CDDAPI :: ~I2CDDAPI()
{
    int index;

    for (index=0; index<DEVICE_NUMBER; index++)
        if (m_I2C_device_table[index] != NULL)
            delete m_I2C_device_table[index];

    for (index=0; index<MASTER_NUMBER; index++)
        if (m_I2C_bus_table[index] != NULL)
            delete m_I2C_bus_table[index];

//    if (m_errorCode != NULL)
//        delete m_errorCode;

}



int I2CDDAPI :: Is_physical_bus(int bus_check)
{
    int index;

    for (index=0; m_I2C_bus_table[index]; index++)
    {
        if ( bus_check == m_I2C_bus_table[index]->get_Bus_ID() )
        {
            return 1;
        }
    }
    return -1;
}



int I2CDDAPI :: Find_physical_master(int logical_bus)
{
    int       index, index2, current_bus, master_count;
    I2CSwitch *p_switch;

    current_bus = logical_bus;
    master_count = 0;

    while( (Is_physical_bus(current_bus) != 1) && (master_count <= m_mux_counter))
    {
        for (index=0; m_I2C_device_table[index]; index++)
        {
            if (m_I2C_device_table[index]->IsSwitch())
            {
                p_switch = (I2CSwitch *)m_I2C_device_table[index];
                for (index2=0; index2<p_switch->get_output_port_counter(); index2++)
                {
                    if ( current_bus == p_switch->get_output_port(index2) )
                    {
                        current_bus = p_switch->get_Bus();
                        break;
                    }
                }
            }
        }
        if ( Is_physical_bus(current_bus) == 1 )
            break;
        master_count++;
    }

    if (master_count <= m_mux_counter)
        return current_bus;
    else
        return -1;
}



I2CDevice * I2CDDAPI :: get_device_by_well_known_name(string well_know_name)
{
    int index;

    for (index=0; m_I2C_device_table[index] ; index++)
    {
        if (well_know_name.compare(m_I2C_device_table[index]->get_Name()) == 0)
        {
            return m_I2C_device_table[index];
        }
    }
    return NULL;
}



I2CDevice * I2CDDAPI :: get_device_by_bus_address(int bus, int address)
{
    int index;

    for (index=0; m_I2C_device_table[index] ; index++)
    {
        if ( (m_I2C_device_table[index]->get_Bus() == bus) && (m_I2C_device_table[index]->get_Address() == address) )
        {
            return m_I2C_device_table[index];
        }
    }
    return NULL;
}



ErrorMsg * I2CDDAPI :: read(int bus, int address, int StartRegister, int EndRegister, int *data, int *count)
{
    I2CDevice *p_i2c_device;
    I2CBus    *p_i2c_bus;


    p_i2c_device = get_device_by_bus_address(bus, address);
    if (p_i2c_device == NULL)
    {
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        //printf("Read:There is no this I2C device.\n");
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_READ | ERROR_I2C_DDAPI_NO_DEVICE), "by bus_address", NULL);
    }
    else
    {
        p_i2c_bus = p_i2c_device->get_Master_bus();
        /*printf("Name:%-7sBUS:0x%x  Address:0x%x protocol:%d  Mastet bus:%d\n",p_i2c_device->get_Name(),p_i2c_device->get_Bus(), \
               p_i2c_device->get_Address(), p_i2c_device->get_Register_protocol(i2c_register), p_i2c_bus->get_Bus_ID());*/
        Route_path(p_i2c_device->get_Bus());

        m_errorCode = p_i2c_bus->read(p_i2c_device->get_Address(), StartRegister, EndRegister, p_i2c_device->get_Register_protocol(StartRegister), data, count, p_i2c_device->get_type());
        /*if(m_errorCode != (ErrorMsg *)NO_ERROR)
        {
            return m_errorCode;
        }*/
    }

    return m_errorCode;
}



ErrorMsg * I2CDDAPI :: read(string well_know_name, int StartRegister, int EndRegister, int *data, int *count)
{
    I2CDevice *p_i2c_device;

    p_i2c_device = get_device_by_well_known_name(well_know_name);
    if (p_i2c_device == NULL)
    {
        //printf("Read: Can't find the I2C device by Wellknown name.\n");
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_READ | ERROR_I2C_DDAPI_NO_DEVICE), "by well_know_name", NULL);
    }

    m_errorCode = read(p_i2c_device->get_Bus(), p_i2c_device->get_Address(), StartRegister, EndRegister, data, count);

    return m_errorCode;
}



ErrorMsg * I2CDDAPI :: write(int bus, int address, int StartRegister, int *data, int count)
{
    I2CDevice *p_i2c_device;
    I2CBus    *p_i2c_bus;

    p_i2c_device = get_device_by_bus_address(bus, address);
    if (p_i2c_device == NULL)
    {
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        //printf("Write:There is no this I2C device.\n");
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_WRITE | ERROR_I2C_DDAPI_NO_DEVICE), "by bus_address", NULL);
    }
    else
    {
        p_i2c_bus = p_i2c_device->get_Master_bus();
        /*printf("Name:%-7sBUS:0x%x  Address:0x%x protocol:%d  Mastet bus:%d\n",p_i2c_device->get_Name(),p_i2c_device->get_Bus(), \
               p_i2c_device->get_Address(), p_i2c_device->get_Register_protocol(i2c_register), p_i2c_bus->get_Bus_ID());*/
        Route_path(p_i2c_device->get_Bus());
        m_errorCode = p_i2c_bus->write(p_i2c_device->get_Address(), StartRegister, p_i2c_device->get_Register_protocol(StartRegister), data, count, p_i2c_device->get_type());
    }

    return m_errorCode;
}



ErrorMsg  * I2CDDAPI :: write(int bus, int address, int StartRegister, int *data, int count, int *numberRealWrite)
{
    I2CDevice *p_i2c_device;
    I2CBus    *p_i2c_bus;

    p_i2c_device = get_device_by_bus_address(bus, address);
    if (p_i2c_device == NULL)
    {
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        //printf("Write:There is no this I2C device.\n");
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_WRITE | ERROR_I2C_DDAPI_NO_DEVICE), "by bus_address", NULL);
    }
    else
    {
        p_i2c_bus = p_i2c_device->get_Master_bus();
        /*printf("Name:%-7sBUS:0x%x  Address:0x%x protocol:%d  Mastet bus:%d\n",p_i2c_device->get_Name(),p_i2c_device->get_Bus(), \
               p_i2c_device->get_Address(), p_i2c_device->get_Register_protocol(i2c_register), p_i2c_bus->get_Bus_ID());*/
        Route_path(p_i2c_device->get_Bus());
        m_errorCode = p_i2c_bus->write(p_i2c_device->get_Address(), StartRegister, p_i2c_device->get_Register_protocol(StartRegister), data, count, numberRealWrite, p_i2c_device->get_type());
    }

    return m_errorCode;
}



ErrorMsg * I2CDDAPI :: write(string well_know_name, int StartRegister, int *data, int count)
{
    I2CDevice *p_i2c_device;

    p_i2c_device = get_device_by_well_known_name(well_know_name);
    if (p_i2c_device == NULL)
    {
        //printf("Write: Can't find the I2C device by Wellknown name.\n");
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_WRITE | ERROR_I2C_DDAPI_NO_DEVICE), "by well_know_name", NULL);
    }

    m_errorCode = write(p_i2c_device->get_Bus(), p_i2c_device->get_Address(), StartRegister, data, count);

    return m_errorCode;
}



ErrorMsg  * I2CDDAPI :: write(string well_know_name, int StartRegister, int *data, int count, int *numberRealWrite)
{
    I2CDevice *p_i2c_device;

    p_i2c_device = get_device_by_well_known_name(well_know_name);
    if (p_i2c_device == NULL)
    {
        //printf("Write: Can't find the I2C device by Wellknown name.\n");
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_WRITE | ERROR_I2C_DDAPI_NO_DEVICE), "by well_know_name", NULL);
    }

    m_errorCode = write(p_i2c_device->get_Bus(), p_i2c_device->get_Address(), StartRegister, data, count, numberRealWrite);

    return m_errorCode;
}



int I2CDDAPI :: Route_path(int logical_bus)
{
    int       index, index2, current_bus, master_count;
    I2CSwitch *p_switch;
    I2CBus    *p_i2c_bus;

    current_bus = logical_bus;
    master_count = 0;

    while( (Is_physical_bus(current_bus) != 1) && (master_count <= m_mux_counter))
    {
        for (index=0; m_I2C_device_table[index]; index++)
        {
            if (m_I2C_device_table[index]->IsSwitch())
            {
                p_switch = (I2CSwitch *)m_I2C_device_table[index];
                for (index2=0; index2<p_switch->get_output_port_counter(); index2++)
                {
                    if ( current_bus == p_switch->get_output_port(index2) )
                    {
                        m_route_table[master_count].switchLogicalID = p_switch->get_Address();
                        m_route_table[master_count].logicalOutputBusNumber = 1 << index2;
                        p_i2c_bus = m_I2C_device_table[index]->get_Master_bus();
                        current_bus = p_switch->get_Bus();
                        break;
                    }
                }
            }
        }
        master_count++;
    }

    for (index=master_count-1; index>=0 ;index--)
    {
        //printf("Switch Address:0x%x  Port:0x%x\n",m_route_table[index].switchLogicalID, m_route_table[index].logicalOutputBusNumber);
        p_i2c_bus->write(m_route_table[index].switchLogicalID, 0x00 , 0 ,(int *)&m_route_table[index].logicalOutputBusNumber, 1, MUX);
    }

    return 1;
}



ErrorMsg * I2CDDAPI :: Initial_I2C_device(char *line, int device_count)
{
    int    counter, switch_output_counter, output_port;
    int    bus, address, switch_output[32];
    char   *pch;
    char   *p;
    string name;
    I2C_DEVICE_TYPE type = GENERAL;
    string vender, model;

    pch = strtok(line, " ");
    counter = 0;               //counter for calculate the item number of line
    switch_output_counter = 0; //calculate the switch output number

    //Start parse the line
    while (pch != NULL)
    {
        counter++;
        if (counter == 1)
        {
            name = pch;
            //string to lower case
            transform(name.begin(), name.end(), name.begin(), ::tolower);
        }
        if (counter == 2)
        {
            bus = strtoul( pch, &p, 16 );
            if ( *p != 0 )
            {  
                //printf("bus:not a number\n");
                //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_DEVICE | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "bus is not a number", NULL);
            }
        }
        if (counter == 3)
        {
            address = strtoul( pch, &p, 16 );
            if ( *p != 0 )
            {
                //printf("address:not a number\n");
                //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_DEVICE | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "address is not number", NULL);
            }
        }
        if (counter == 4)
        {
            if (strcmp(pch, "temperature") == 0)
               type = TEMPERATURE;
            else if (strcmp(pch, "mux") == 0)
               type = MUX;
            else if (strcmp(pch, "battery") == 0)
               type = BATTERY;
            else if (strcmp(pch, "spd_ddr1") == 0)
               type = SPD_DDR1;
            else if (strcmp(pch, "spd_ddr2") == 0)
               type = SPD_DDR2;
            else if (strcmp(pch, "spd_ddr3") == 0)
               type = SPD_DDR3;
            else if (strcmp(pch, "spd_ddr4") == 0)
               type = SPD_DDR4;
            else if (strcmp(pch, "vpd") == 0)
               type = VPD;
            else
               type = GENERAL;
        }
        if (counter == 5 && type != MUX)
        {
            vender = pch;
        }
        if (counter == 6 && type != MUX)
        {
            model = pch;
        }
        //MUX setup
        if (counter > 4 && type == MUX)
        {
            output_port = strtoul( pch, &p, 16 );
            if ( *p != 0 )
            {  
                //printf("switch:not a number\n");
                //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_DEVICE | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "switch is not number", NULL);
            }
            switch_output[switch_output_counter] = output_port;
            switch_output_counter++;
        }

        pch = strtok (NULL, " ");
    }


    //Ready for add I2C to table
    //if (counter == 3)  //This is normal I2C device
    if (type != MUX)
    {
        m_I2C_device_table[device_count] = new I2CDevice(name, bus, address, type, vender, model);
    }
    else
    {                  //This is I2C switch(MUX)
        m_I2C_device_table[device_count] = new I2CSwitch(name, bus, address, switch_output,  switch_output_counter, type, vender, model);
        m_mux_counter++;
    }
    m_I2C_device_table[device_count]->set_Master_bus(m_I2C_bus_table[0]); //This is temporarily setting, we will set real after the for loop.

    return m_errorCode;
}



ErrorMsg * I2CDDAPI :: Initial_I2C_BUS(char *line)
{
    char   *pch;
    char   *p;
    int    counter, bus, frequency;
    string name, master;

    pch = strtok(line, " ");
    counter = 0;            //counter for calculate the number of item in the line.

    //Start parse the line
    while (pch != NULL)
    {
        counter++;
        if (counter == 1)
        {
            name = pch;
        }
        if (counter == 2)
        {
            bus = strtoul( pch, &p, 16 );
            if ( *p != 0 )
            {  
                //printf("bus:not a number\n");
                //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_BUS | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "bus is not number", NULL);
            }
        }
        if (counter == 3)
        {
            frequency = strtoul( pch, &p, 10 );
            if ( *p != 0 )
            {  
                //printf("frequency:not a number\n");
                //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_BUS | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "frequency is not number", NULL);
            }
        }
        if (counter == 4)
        {
            master = pch;
        }

        pch = strtok (NULL, " ");
    }

    if (counter == 4)
    {
        m_I2C_bus_table[bus] = new I2CBus("I2C_BUS",name, bus, frequency, master);
    }

    return m_errorCode;
}



ErrorMsg * I2CDDAPI :: Initial_I2C_Protocol(char *line)
{
    I2CDevice *p_i2c_device;
    string    name;
    int       register_address, counter, index;
    char      *pch, *p, register_address_string[10], protocol_string[100];
    FILE      *pFile_external = NULL;
    char      line_external[2000];
    string    file_path = "/etc/lemur/";

    p_i2c_device = NULL;
    counter = 0;
    pch = strtok(line, " ");

    //Start parse the line
    while (pch != NULL)
    {
        counter++;
        if (counter == 1)
        {
            name = pch;
            //string to lower case
            transform(name.begin(), name.end(), name.begin(), ::tolower);
        }
        if (counter > 1)
        {
            sscanf(pch, "%[^=]=%s", register_address_string, protocol_string);
            //printf("reg:%s  protocol:%s\n",register_address_string, protocol_string);

            p_i2c_device = get_device_by_well_known_name(name);
            if (p_i2c_device == NULL)
            {
                //printf("Initial_I2C_Protocol: Can't find the I2C device by Wellknown name(%s).\n",name.c_str());
                //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_PROTOCOL | ERROR_I2C_DDAPI_NO_DEVICE), "Can't find device by Wellknown name", NULL);
            }

            if (strcmp(register_address_string,"all") == 0)
            {
                for (index=0; index<REGISTER_NUMBER; index++)
                {
                    if (strcmp(protocol_string, "byte") == 0)
                    {
                        p_i2c_device->set_Register_protocol(index, I2C_DEVICE_PROTOCOL_SMBUS_BYTE);
                    }
                    if (strcmp(protocol_string, "byte_data") == 0)
                    {
                        p_i2c_device->set_Register_protocol(index, I2C_DEVICE_PROTOCOL_SMBUS_BYTE_DATA);
                    }
                    if (strcmp(protocol_string, "word") == 0)
                    {
                        p_i2c_device->set_Register_protocol(index, I2C_DEVICE_PROTOCOL_SMBUS_WORD_DATA);
                    }
                    if (strcmp(protocol_string, "block") == 0)
                    {
                        p_i2c_device->set_Register_protocol(index, I2C_DEVICE_PROTOCOL_SMBUS_BLOCK);
                    }
                    if (strcmp(protocol_string, "i2c_block") == 0)
                    {
                        p_i2c_device->set_Register_protocol(index, I2C_DEVICE_PROTOCOL_I2C_BLOCK);
                    }
                    if (strcmp(protocol_string, "vpd_block_data") == 0)
                    {
                        p_i2c_device->set_Register_protocol(index, I2C_DEVICE_PROTOCOL_VPD_BLOCK);
                    }
                    if (strcmp(protocol_string, "pmbus_block") == 0)
                    {
                        p_i2c_device->set_Register_protocol(index, I2C_DEVICE_PROTOCOL_PMBUS_BLOCK);
                    }
                }
            }
            else if (strcmp(register_address_string,"file") == 0)
            {
                //Open the external .ini for parser
                #ifdef DEBUG
                    pFile_external = fopen (protocol_string, "r");
                #else
                    file_path = file_path + protocol_string;
                    pFile_external = fopen (file_path.c_str(), "r");
                    strncpy(protocol_string, file_path.c_str(), file_path.length());
                #endif
                if (pFile_external == NULL)
                {
                    //printf("Error opening file\n");
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_PROTOCOL | ERROR_I2C_DDAPI_OPEN_PROFILE_FAIL), protocol_string, NULL);
                }
                while (fgets(line_external, 2000, pFile_external) != NULL)
                {
                    if (line_external[0] == '#')
                    {
                       continue;
                    }
                    else
                    {
                       pch = strtok(line_external, " ");
                       break;
                    }
                }
                fclose(pFile_external);
            }
            else
            {
                register_address = strtoul( register_address_string, &p, 16 );
                if ( *p != 0 )
                {
                    //printf("Protocol:\"%s\" is not a number.\n",register_address_string);
                    //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_PROTOCOL | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "Protocol is not a number", NULL);
                }

                if (strcmp(protocol_string, "byte") == 0)
                {
                    p_i2c_device->set_Register_protocol(register_address, I2C_DEVICE_PROTOCOL_SMBUS_BYTE);
                }
                if (strcmp(protocol_string, "byte_data") == 0)
                {
                    p_i2c_device->set_Register_protocol(register_address, I2C_DEVICE_PROTOCOL_SMBUS_BYTE_DATA);
                }
                if (strcmp(protocol_string, "word") == 0)
                {
                    p_i2c_device->set_Register_protocol(register_address, I2C_DEVICE_PROTOCOL_SMBUS_WORD_DATA);
                }
                if (strcmp(protocol_string, "block") == 0)
                {
                    p_i2c_device->set_Register_protocol(register_address, I2C_DEVICE_PROTOCOL_SMBUS_BLOCK);
                }
                if (strcmp(protocol_string, "i2c_block") == 0)
                {
                    p_i2c_device->set_Register_protocol(register_address, I2C_DEVICE_PROTOCOL_I2C_BLOCK);
                }
                if (strcmp(protocol_string, "pmbus_block") == 0)
                {
                    p_i2c_device->set_Register_protocol(register_address, I2C_DEVICE_PROTOCOL_PMBUS_BLOCK);
                }
            }
        }
        if (strcmp(register_address_string,"file") != 0)
            pch = strtok (NULL, " ");
    }
    
    return m_errorCode;
}



ErrorMsg * I2CDDAPI :: get_i2c_devices_list(list<string> *list)
{
    int       index1,index2;
    char      tmp_string[10];
    I2CSwitch *p_switch;
    string    device_result;


    list->clear();

    for (index1=0; m_I2C_device_table[index1] ; index1++)
    {
        /*list[index1].append(m_I2C_device_table[index1]->get_Name());
        list[index1].append(" ");
        sprintf(tmp_string,"%x",m_I2C_device_table[index1]->get_Bus());
        list[index1].append(tmp_string);
        list[index1].append(" ");
        sprintf(tmp_string,"0x%0x",m_I2C_device_table[index1]->get_Address());
        list[index1].append(tmp_string);
        list[index1].append(" ");
        if (m_I2C_device_table[index1]->IsSwitch() == 1)
        {
            p_switch = (I2CSwitch *)m_I2C_device_table[index1];
            for (index2=0; index2<p_switch->get_output_port_counter(); index2++)
            {
                sprintf(tmp_string,"0x%0x",p_switch->get_output_port(index2));
                list[index1].append(tmp_string);
                list[index1].append(" ");
            }
        }*/
        device_result.clear();
        device_result.append(m_I2C_device_table[index1]->get_Name());
        device_result.append(" ");

        sprintf(tmp_string,"%x",m_I2C_device_table[index1]->get_Bus());
        device_result.append(tmp_string);
        device_result.append(" ");
 
        sprintf(tmp_string,"0x%0x",m_I2C_device_table[index1]->get_Address());
        device_result.append(tmp_string);
        device_result.append(" ");

        if (m_I2C_device_table[index1]->IsSwitch() == 1)
        {
            p_switch = (I2CSwitch *)m_I2C_device_table[index1];
            for (index2=0; index2<p_switch->get_output_port_counter(); index2++)
            {
                sprintf(tmp_string,"0x%0x",p_switch->get_output_port(index2));
                device_result.append(tmp_string);
                device_result.append(" ");
            }
        }
        list->push_back(device_result);
    }

    return m_errorCode;
}



ErrorMsg * I2CDDAPI ::  get_i2c_devices_list(I2CDevice *list[])
{
    int             i, j;
    int             bus, address;
    string          name, vender, model;
    I2C_DEVICE_TYPE type;

    int       switch_output[32], switch_output_counter;
    I2CSwitch *p_switch;


    for (i=0; m_I2C_device_table[i] ; i++)
    {
        name = m_I2C_device_table[i]->get_Name();
        bus = m_I2C_device_table[i]->get_Bus();
        address = m_I2C_device_table[i]->get_Address();
        type = m_I2C_device_table[i]->get_type();
        vender = m_I2C_device_table[i]->get_vender();
        model = m_I2C_device_table[i]->get_model();
        //printf("name:%s bus:%x add.:%x type:%d vender:%s model:%s\n",name.c_str(), bus, address, type, vender.c_str(), model.c_str());
        if (type != MUX)
        {
           list[i] = new I2CDevice(name, bus, address, type, vender, model);
        }
        else
        {
           p_switch = (I2CSwitch *)m_I2C_device_table[i];
           switch_output_counter = p_switch->get_output_port_counter();
           for (j=0; j<switch_output_counter; j++)
              switch_output[j] = p_switch->get_output_port(j);
           list[i] = new I2CSwitch(name, bus, address, switch_output,  switch_output_counter, type, vender, model);
        }
    }

    return m_errorCode;
}



ErrorMsg * I2CDDAPI :: mux_test(string well_know_name)
{
    I2CDevice *p_i2c_device;
    int       data_write = 0x02, data_read = 0x00, count = 1;


    p_i2c_device = get_device_by_well_known_name(well_know_name);
    if (p_i2c_device == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_WRITE | ERROR_I2C_DDAPI_NO_DEVICE), "by well_know_name", NULL);
    }

    if (p_i2c_device->get_type() != MUX)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_MUX_TEST | ERROR_I2C_DDAPI_TYPE_UN_MATCH), "Type is not MUX", NULL);
    }

    count = 1;
    m_errorCode = write(p_i2c_device->get_Bus(), p_i2c_device->get_Address(), 0x00, &data_write, count);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DDAPI_MUX_TEST, NULL, m_errorCode);
    }

    m_errorCode = read(p_i2c_device->get_Bus(), p_i2c_device->get_Address(), 0x00, 0x00, &data_read, &count);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DDAPI_MUX_TEST, NULL, m_errorCode);
    }

    if (data_write != data_read)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_MUX_TEST | ERROR_I2C_DDAPI_MUX_TEST_FAIL), NULL, NULL);
    }


    return NO_ERROR_MSG;
}



ErrorMsg  * I2CDDAPI :: get_i2c_device_path(string well_know_name, string *route_path)
{
    int          index, index2, current_bus, master_count;
    I2CSwitch    *p_switch;
    string       s;
    stringstream ss(s);
    I2CBus       *p_i2c_bus = NULL;
    I2CDevice    *p_i2c_device = NULL;

    //Initial path value
    *route_path = well_know_name;

    p_i2c_device = get_device_by_well_known_name(well_know_name);
    if (p_i2c_device == NULL)
    {
        //printf("Read: Can't find the I2C device by Wellknown name.\n");
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_READ | ERROR_I2C_DDAPI_NO_DEVICE), "by well_know_name", NULL);
    }

    current_bus = p_i2c_device->get_Bus();
    master_count = 0;
    p_i2c_bus = p_i2c_device->get_Master_bus();

    //printf("%s\n",(p_i2c_bus->get_master()).c_str());

    //Save the bus information
    ss << std::hex << current_bus;
    route_path->insert(0, "Bus" + ss.str() + "->");
    ss.str("");

    while( (Is_physical_bus(current_bus) != 1) && (master_count <= m_mux_counter))
    {
        for (index=0; m_I2C_device_table[index]; index++)
        {
            if (m_I2C_device_table[index]->IsSwitch())
            {
                p_switch = (I2CSwitch *)m_I2C_device_table[index];
                for (index2=0; index2<p_switch->get_output_port_counter(); index2++)
                {
                    if ( current_bus == p_switch->get_output_port(index2) )
                    {
                        //Save the device information
                        route_path->insert(0, "->");
                        route_path->insert(0, p_switch->get_Name());

                        m_route_table[master_count].switchLogicalID = p_switch->get_Address();
                        m_route_table[master_count].logicalOutputBusNumber = 1 << index2;
                        current_bus = p_switch->get_Bus();

                        //Save the bus information
                        ss << std::hex << current_bus;
                        route_path->insert(0, "Bus" + ss.str() + "->");
                        ss.str("");
                        break;
                    }
                }
            }
        }
        master_count++;
    }

    route_path->insert(0, p_i2c_bus->get_master() + "->");
    ss.str("");

    p_i2c_bus = NULL;

    return NO_ERROR_MSG;
}



ErrorMsg  * I2CDDAPI :: stuckPrevention(int busID)
{
    UINT32        u32Bar;
    UINT8         ret;
    PCIDDAPI      *m_pciDriverAPI;
    MemoryDDAPI   *m_memoryDriver;

    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_MUX_TEST), "Not Support", NULL);

    if (busID < 1 || busID > 3)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_MUX_TEST | ERROR_I2C_DDAPI_MUX_TEST_FAIL), NULL, NULL);
    }

    //New a PCI driver
    m_pciDriverAPI = new PCIDDAPI("I2C_FPGA_PCI");

    m_errorCode = m_pciDriverAPI->getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        delete m_pciDriverAPI;
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "MUX test", m_errorCode);
    }

    //New a memory driver
    m_memoryDriver = new MemoryDDAPI("I2C_MEM");

    m_errorCode = m_memoryDriver->getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        delete m_memoryDriver;
        delete m_pciDriverAPI;
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "MUX test", m_errorCode);
    }

    //Get the FPGA bar address
    m_pciDriverAPI->getBarByBDF(0, 0x72, 0x00, 0x00, 0, &u32Bar);
    //printf("Bar[0] -- 0x%x\n", u32Bar);

    //Get I2C Master SDA SCL status first
    ret = m_memoryDriver->read8(u32Bar + 0x54);
    //printf("0x%02x\n", ret);

    ret = ret & 0xFC;

    //Set I2C Master SDA SCL to Low
    m_errorCode = m_memoryDriver->write(u32Bar, ret);
    if (m_errorCode != NO_ERROR_MSG)
    {
        delete m_memoryDriver;
        delete m_pciDriverAPI;
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "MUX test", m_errorCode);
    }

    //Get I2C Master SDA SCL status again
    ret = m_memoryDriver->read8(u32Bar + 0x54);
    //printf("0x%02x\n", ret);


    //Free unused point
    delete m_pciDriverAPI;
    delete m_memoryDriver;

    return NO_ERROR_MSG;
}



ErrorMsg *
I2CDDAPI::getTemperatureDataByID(string                   well_know_name,
                                 TEMPERATURE_OPERATION_ID ID,
                                 string                   *result)
{
    I2CTemperature *i2c_temperature = NULL;
    I2CDevice      *device = NULL;
    string         model;

    m_errorCode = NO_ERROR_MSG;

    //Get I2C device object by name
    device = get_device_by_well_known_name(well_know_name);
    if (device == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_READ | ERROR_I2C_DDAPI_NO_DEVICE), "by well_know_name", NULL);
    }

    //Get the temperature sensor model name
    model = device->get_model();

    //Base on different model to create the essential object
    if (model == "MAX6657")
    {
        i2c_temperature = new MAX6657("MAX6657 i2c driver");
    }
    else if (model == "SPDTMP")
    {
        i2c_temperature = new DIMMTemperature("DIMM Temperature i2c driver");
    }
    else
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Not support temperature sensor", m_errorCode);
    }


    switch (ID)
    {
        case CURRENT_TEMPERATURE:  //Get sensor currrnt temperature
            m_errorCode = i2c_temperature->GetCurrntTemperature(well_know_name, result);
            break;
        case THRESHOLD_HIGH:  //Get temperature threshold high
            m_errorCode = i2c_temperature->GetThresholdHigh(well_know_name, result);
            break;
        case THRESHOLD_LOW:  //Get temperature threshold low
            m_errorCode = i2c_temperature->GetThresholdLow(well_know_name, result);
            break;
        default:
            break;
    }

    delete i2c_temperature;

    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, NULL, m_errorCode);
    }

    return NO_ERROR_MSG;
}



ErrorMsg *
I2CDDAPI::setTemperatureDataByID(string                   well_know_name,
                                 TEMPERATURE_OPERATION_ID ID,
                                 string                   data)
{
    I2CTemperature *i2c_temperature = NULL;
    I2CDevice      *device = NULL;
    string         model;

    m_errorCode = NO_ERROR_MSG;

    //Get I2C device object by name
    device = get_device_by_well_known_name(well_know_name);
    if (device == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_READ | ERROR_I2C_DDAPI_NO_DEVICE), "by well_know_name", NULL);
    }

    //Get the temperature sensor model name
    model = device->get_model();

    //Base on different model to create the essential object
    if (model == "MAX6657")
    {
        i2c_temperature = new MAX6657("MAX6657 i2c driver");
    }
    else if (model == "SPDTMP")
    {
        i2c_temperature = new DIMMTemperature("DIMM Temperature i2c driver");
    }
    else
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Not support temperature sensor", m_errorCode);
    }


    switch (ID)
    {
        case THRESHOLD_HIGH:  //Set temperature threshold high
            m_errorCode = i2c_temperature->SetThresholdHigh(well_know_name, data);
            break;
        case THRESHOLD_LOW:  //Set temperature threshold low
            m_errorCode = i2c_temperature->SetThresholdLow(well_know_name, data);
            break;
        default:
            break;
    }

    delete i2c_temperature;

    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, NULL, m_errorCode);
    }

    return NO_ERROR_MSG;
}



ErrorMsg *
I2CDDAPI::DisplayI2cDevicesListByType(I2CDevice        *i2c_list[],
                                      I2C_DEVICE_TYPE  ptype)
{
    int             i, j, count;
    int             bus, address;
    string          name, vender, model;
    I2C_DEVICE_TYPE type;
    I2CDevice       *my_i2c_list[DEVICE_NUMBER];

    int       switch_output[32], switch_output_counter;
    I2CSwitch *p_switch;

    //Initial device/bus table
    for (i=0; i<DEVICE_NUMBER; i++)
    {
        my_i2c_list[i] = NULL;
        i2c_list[i] = NULL;
    }

    m_errorCode = get_i2c_devices_list(my_i2c_list);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_DISPLAY_I2C_DEVICES_LIST_BY_TYPE, NULL, m_errorCode);
    }

    count = 0;

    for (i=0; my_i2c_list[i]; i++)
    {
        if (my_i2c_list[i]->get_type() == ptype)
        {
           //printf("%s\n",i2c_list[i]->get_Name());
           name = my_i2c_list[i]->get_Name();
           bus = my_i2c_list[i]->get_Bus();
           address = my_i2c_list[i]->get_Address();
           type = my_i2c_list[i]->get_type();
           vender = my_i2c_list[i]->get_vender();
           model = my_i2c_list[i]->get_model();
           //printf("name:%s bus:%x add.:%x type:%d vender:%s model:%s\n",name.c_str(), bus, address, type, vender.c_str(), model.c_str());
           if (type != MUX)
           {
              i2c_list[count] = new I2CDevice(name, bus, address, type, vender, model);
           }
           else
           {
              p_switch = (I2CSwitch *)my_i2c_list[i];
              switch_output_counter = p_switch->get_output_port_counter();
              for (j=0; j<switch_output_counter; j++)
                 switch_output[j] = p_switch->get_output_port(j);
              i2c_list[count] = new I2CSwitch(name, bus, address, switch_output,  switch_output_counter, type, vender, model);
           }
           count++;
        }
    }

    return NO_ERROR_MSG;
}
