#ifndef NTBDDAPI_H
#define NTBDDAPI_H

#include <string>
#include <vector>
#include <iostream>


#include "BaseClassDDAPI.hpp"
#include "PCIDDAPI.hpp"
using namespace std;

#define SIZE_1G         (1<<30)
#define NTB_MAX_LINE    1024

#define NTB_INFO_FILE   "/tmp/ntbInfo"
//#define NTB_INFO_FILE   "/usr/local/share/diag_env"

#define NTB_DRIVER_NAME "/dev/plx/Plx8000_NT-0"

typedef struct _NTB_DEVICE_TABLE
{
    UINT16  u16VendorID;
    UINT16  u16DeviceID;
}NTB_DEVICE_TABLE;

#if 0
const NTB_DEVICE_TABLE  ntbDeviceTable[] = 
{
    {0x8086, 0x6F0D}, //Intel Broadwell
    {0x10B5, 0x87B0}, //PLX PCIe switch
    {0xFFFF, 0xFFFF},
};
#endif 


typedef struct _NTB_INFO
{
    UINT16              u16LogicalID;
    string              ntbName;
    PCIInformantion*    pciInfo;
}NTB_INFO;

typedef struct _NTB_WINDOW_INFO
{
    UINT32  u32Size;
    UINT32  u32SizeDW;
    UINT64* u64pUserAddrRead;
    UINT64* u64pUserAddrWrite;
}NTB_WINDOW_INFO;


class NTBDDAPI : public BaseClassDDAPI
{
    public:
        NTBDDAPI(STRING);
        ~NTBDDAPI();

        ErrorMsg*   searchDevice(vector<NTB_INFO>*  out_pNtbList);

        ErrorMsg*   server(string   in_name,
                           UINT64   in_u64PatternSend,
                           UINT64   in_u64PatternRev,
                           UINT32   in_u32Loop);

        ErrorMsg*   client(string   in_name,
                           UINT64   in_u64PatternRev,
                           UINT64   in_u64PatternSend,
                           UINT32   in_u32Loop);
        
        ErrorMsg*   initialWindow();

        ErrorMsg*   stress(UINT64   in_u64PatternSend,
                           UINT64   in_u64PatternRev,
                           UINT64   in_u64Loop,
                           bool     in_bCheckData);

        ErrorMsg*   testPerformance(UINT64 in_u64PatternSend,
                                    UINT64 in_u64BlockSize,
                                    UINT64 in_u64Loop);

        ErrorMsg*   receiver(UINT64   in_u64PatternRev,
                             UINT64   in_u64Loop);
        
    private:
        PCIDDAPI*           m_pPCIDriverAPI;
        ErrorMsg*           m_pErrorMsg;
        ERRORCODE           m_eErrorCode;
        NTB_WINDOW_INFO     m_windowInfo;
        UINT64*             m_u64pBufferSend;
        UINT64*             m_u64pBufferInit;
        UINT64*             m_u64pBufferRev;
        int                 m_fd;

        //bool    isNTB(UINT16  in_u16VendorID,
        //              UINT16  in_u16DeviceID);

        static void signalHandler(int signum);

        ErrorMsg*   getWindowInfo(UINT32  in_u32Size);

        ErrorMsg*   createDataBlocks(UINT64    in_u64PatternSend,
                                     UINT64    in_u64PatternRev);

        ErrorMsg*   receiveData(UINT64*   in_u64pBuffer,
                                UINT64*   in_u64pReadAddr,
                                UINT32    in_u32SizeDW);

        void        releaseWindowResource();
        ErrorMsg*   releaseDataBlocks();

        ErrorMsg*   setForIoTransfer(UINT64     in_u64PatternSend,
                                     UINT64     in_u64PatternRev);
};

#endif
