/// @file       PCIDev.hpp
/// @brief      PCIDev class
/// @details    Provide basic information and operation for a PCI device
/// @author     KenHu
/// @copyright  Copyright 2017 Hon Hai Technology Group. All Rights Reserved.

#ifndef PCIDEV_HPP
#define PCIDEV_HPP

//#include <list>
#include "common.hpp"
#include "PCIBus.hpp"
#include "PCICommonFunction.hpp"

class PCIBus;

#define UNKNOWN_PCI_VALUE    0xFFFFFFFF

#define INTEL_NTB_LNK_STA_REG   0x1A2
#define INTEL_NTB_LNK_CTL_REG   0x19C

#define LINK_STATUS_REG_OFFSET          0x12
#define PCIE_LNKCTL_REG_OFFSET          0x10
#define LINK_CAPABILITY_REG_OFFSET      0xC
#define PCIE_CAPABILITIES_REG_OFFSET    0x2

#define PCIE_CAPABILITY_ID              0x10
#define PCIE_AER_CAPABILITY_ID          0x0001

#define PCI_BAR_START_OFFSET    0x10

//define port type
#define PORT_TYPE_PCIE_ENDPOINT                     0x0
#define PORT_TYPE_LEGACY_PCIE_ENDPOINT              0x1
#define PORT_TYPE_PCIE_ROOT_COMPLEX_ROOT_PORT       0x4
#define PORT_TYPE_PCIE_SWITCH_UPSTREAM_PORT         0x5
#define PORT_TYPE_PCIE_SWITCH_DOWNSTREAM_PORT       0x6
#define PORT_TYPE_PCIE_TO_PCI_PCIX_BRIDGE           0x7
#define PORT_TYPE_PCI_PCIX_TO_PCIE_BRIDGE           0x8
#define PORT_TYPE_ROOT_COMPLEX_INTEGRATED_ENDPOINT  0x9
#define PORT_TYPE_ROOT_COMPLEX_EVENT_COLLECTOR      0xa

/// @class      PCIBus PCIBus.hpp "PCIBus.hpp"
/// @brief      Used to store information of a PCI bus
/// @param      m_u16Domain is UINT16 value for domain
/// @param      m_u8Bus is UINT8 value for bus
/// @param      m_u8Dev is UINT8 value for device
/// @param      m_u8Func is UINT8 value for function
/// @param      m_u16VendorID is UINT16 for vendor ID
/// @param      m_u16DeviceID is UINT16 for device ID
/// @param      m_u32Class is UINT32 for class
/// @param      m_pLowerBus is pointer to lower layer bus connected to this device
class PCIDev : public PCICommonFunction
{
    public:
        PCIDev();
        PCIDev(UINT16 in_u16Domain, UINT8 in_u8Bus, UINT8 in_u8Dev, UINT8 in_u8Func);
        ~PCIDev();

        list<UINT32>    getBARList();
        UINT64          getBAR64(INT32 in_s32BarID);
        UINT32          getCapabilityOffset(UINT32  in_u32CapID);
        UINT16          getDeviceID();
        UINT16          getVendorID();
        UINT8           getPortTypeRawData();

        void            initializeCapList();
        void            initializeExtendedCapList();
        void            initializeInfo();

        UINT16          m_u16Domain;
        UINT8           m_u8Bus;
        UINT8           m_u8Dev;
        UINT8           m_u8Func;
        UINT16          m_u16VendorID;
        UINT16          m_u16DeviceID;
        UINT32          m_u32Class;
        PCIBus          *m_pLowerBus;
    private:
        list<UINT32>    m_barList;
        list<CapInfo>   m_capList;
};

#endif
