#ifndef PCIDevice_H
#define PCIDevice_H

#include "BaseClassDDAPI.hpp"

//#include "ErrorMsg.hpp"

#include <linux/ioctl.h>
#include <sys/ioctl.h>

//#include "PCILibrary.hpp"

#include "PCICommonFunction.hpp"

#include <list>
#include <vector>
#include <stdio.h>
#include <string>
using namespace std;

#define TRUE    1
#define FALSE   0

#define INTEL_NTB_LNK_STA_REG   0x1A2
#define INTEL_NTB_LNK_CTL_REG   0x19C

#define LINK_STATUS_REG_OFFSET          0x12
#define PCIE_LNKCTL_REG_OFFSET          0x10 
#define LINK_CAPABILITY_REG_OFFSET      0xC
#define PCIE_CAPABILITIES_REG_OFFSET    0x2

#define PCIE_CAPABILITY_ID              0x10
#define PCIE_AER_CAPABILITY_ID          0x0001

#define NODE_NAME              "/dev/pciDrv"

#define PCI_MODULE_IOC_MAGIC                'P'
#define READ_PCI_CONFIG_BYTE       _IO(PCI_MODULE_IOC_MAGIC,   1)

#define PCI_BAR_START_OFFSET    0x10

//define port type 
#define PORT_TYPE_PCIE_ENDPOINT                     0x0
#define PORT_TYPE_LEGACY_PCIE_ENDPOINT              0x1
#define PORT_TYPE_PCIE_ROOT_COMPLEX_ROOT_PORT       0x4
#define PORT_TYPE_PCIE_SWITCH_UPSTREAM_PORT         0x5
#define PORT_TYPE_PCIE_SWITCH_DOWNSTREAM_PORT       0x6
#define PORT_TYPE_PCIE_TO_PCI_PCIX_BRIDGE           0x7
#define PORT_TYPE_PCI_PCIX_TO_PCIE_BRIDGE           0x8
#define PORT_TYPE_ROOT_COMPLEX_INTEGRATED_ENDPOINT  0x9
#define PORT_TYPE_ROOT_COMPLEX_EVENT_COLLECTOR      0xa




/*
class CapInfo
{
    public:
        CapInfo(unsigned int in_u32CapID, unsigned int in_u32CapOffset, unsigned int in_u32NextCapOffset):
        CapID(in_u32CapID), CapOffset(in_u32CapOffset), NextCapOffset(in_u32NextCapOffset)
        {}

        unsigned int GetCapID()
        {
            return CapID;
        };

        unsigned int GetNextCapOffset()
        {
            return NextCapOffset;
        };

        unsigned int GetCapOffset()
        {
            return CapOffset;
        };

    private:
        unsigned int CapID;
        unsigned int CapOffset;
        unsigned int NextCapOffset;
};
*/

/*
class PCIDeviceInformation 
{
    public:
        PCIDeviceInformation();
        ~PCIDeviceInformation();
       
        string         GetDeviceName();
        unsigned short GetDomain();
        unsigned short GetBus();
        unsigned short GetDevice();
        unsigned char  GetFunction();

        unsigned short GetVendorID();
        unsigned short GetDeviceID();
        unsigned char  GetRevisionID();
        unsigned int   GetClassCode();
        unsigned char  GetHeaderType();
        unsigned char  GetCapPtrValue();

    private:
        string          szDeviceName;
        unsigned short  u16Domain;
        unsigned short  u16Bus;
        unsigned short  u16Device;
        unsigned char   u8Function;
        
        //common configuration space header
        unsigned short  u16VendorID;
        unsigned short  u16DeviceID;
        unsigned char   u8RevisionID;
        unsigned int    u32ClassCode;
        unsigned char   u8HeaderType;
        unsigned char   u8CapPtrValue;
};
*/

/*
typedef struct _config_pci_dev
{
    //unsigned int  domain;
    unsigned int  bus;
    unsigned int  dev;
    unsigned int  func;
    unsigned int  offset;
    unsigned int  device_id;
    unsigned int  vendor_id;
    unsigned int  link_speed;
    unsigned int  link_width;
    unsigned int  location;
    unsigned int  reserved;
}__attribute__((packed)) config_pci_dev;
*/


typedef enum _PCIE_TYPE
{
    PCIE_ENDPOINT = 0x0,
    LEGACY_PCIE_ENDPOINT,
    ROOT_COMPLEX_ROOT_PORT = 0x4, //only valid for Type01 config 
    PCIE_SWITCH_UPSTREAM_PORT, //only valid for Type01 config 
    PCIE_SWITCH_DOWNSTREAM_PORT, //only valid for Type01 config 
    PCIE_TO_PCI_PCIX_BRIDGE, //only valid for Type01 config 
    PCI_PCIX_TO_PCIE_BRIDGE, //only valid for Type01 config 
    ROOT_COMPLEX_INTEGRATED_ENDPOINT,
    ROOT_COMPLEX_EVENT_COLLECTOR,
}PCIE_TYPE;


typedef enum _DATA_TYPE
{
    PCI_DATA_TYPE_DOMAIN = 0,
    PCI_DATA_TYPE_BUS,
    PCI_DATA_TYPE_DEVICE,
    PCI_DATA_TYPE_FUNCTION,
    PCI_DATA_TYPE_VENDOR_ID,
    PCI_DATA_TYPE_DEVICE_ID,
    PCI_DATA_TYPE_CLASS_CODE,
    PCI_DATA_TYPE_CLASS_TYPE,
    PCI_DATA_TYPE_REVISION_ID,
    PCI_DATA_TYPE_HEADER_TYPE,
    PCI_DATA_TYPE_CURRENT_WIDTH,
    PCI_DATA_TYPE_MAX_WIDTH,
    PCI_DATA_TYPE_CURRENT_SPEED,
    PCI_DATA_TYPE_MAX_SPEED,
    PCI_DATA_TYPE_PORT_TYPE,
    PCI_DATA_TYPE_PORT_NUMBER,
    PCI_DATA_TYPE_LINK_STATUS,
}DATA_TYPE;




typedef struct _PCIInformantion
{
    string          szDeviceName;
    unsigned short  u16Domain;
    unsigned short  u16Bus;
    unsigned short  u16Device;
    unsigned char   u8Function;
    unsigned short  u16VendorID;
    unsigned short  u16DeviceID;
    unsigned int    u32ClassCode;
    unsigned char   u8RevisionID;
    unsigned char   u8HeaderType;
    unsigned char   u8CapPtrValue;
    UINT8           u8PortType;
    UINT8           u8PortNumber;
}PCIInformantion;


class PCIDevice 
{
    public:
        PCIDevice(PCIInformantion in_pciInfo);
        ~PCIDevice();

        string          GetDeviceName();
        unsigned short  GetDomain();
        unsigned short  GetBus();
        unsigned short  GetDevice();
        unsigned char   GetFunction();

        unsigned short  GetVendorID();
        unsigned short  GetDeviceID();
        unsigned char   GetRevisionID();
        unsigned int    GetClassCode();
        unsigned char   GetHeaderType();
        unsigned char   GetCapPtrValue();

        float           GetCurrentLinkSpeed();
        unsigned short  GetCurrentLinkWidth();
        float           GetMaxLinkSpeed();
        unsigned short  GetMaxLinkWidth();

        UINT8  getPortTypeRawData();
        string  getPortTypeString();

        ERRORCODE getErrorCode();

        unsigned char   ConfigReadByte(unsigned int in_u32Offset);
        unsigned short  ConfigReadWord(unsigned int in_u32Offset);
        unsigned int    ConfigReadDword(unsigned int in_u32Offset);
        
        int ConfigWriteByte(unsigned int     in_u32Offset,
                            unsigned char    in_u8WriteData);
        int ConfigWriteWord(unsigned int     in_u32Offset,
                            unsigned short   in_u16WriteData);
        int ConfigWriteDword(unsigned int    in_u32Offset,
                             unsigned int    in_u32WriteData);

        void            GetBARList(vector<unsigned int> *pBarList);
        
        //unsigned long   GetMMIO();
        //void            GetMMIO(list<unsigned long> *pMMIOList);

        bool            IsPCIe();
        bool            LinkStatus();
        PCIE_TYPE       GetPCIeType();

        int             GetCapabilityOffset(unsigned int   in_u32CapID,
                                            unsigned int   *CapOffset);

        int             getExtendedCapabilityOffset(unsigned int   in_u32CapID,
                                                    unsigned int   *CapOffset);

        void            GetCapList();
        //void            GetCapList(list<CapInfo> *pCapInfo);

        /*
        ERRORCODE   checkPresenceOfDevice(UINT16 in_u16Domain,
                                          UINT16 in_u16Bus,
                                          UINT16 in_u16Device,
                                          UINT8  in_u8Function);
        */
        
        float   GetLinkSpeed(unsigned int in_u32LinkStatusRegAddr);

        bool    checkRetrainCap();

        unsigned char getPortNumber();
    
    private:
        ERRORCODE       m_eErrorCode;

        struct pci_dev  *m_dev;

        //PCILibrary   *m_pPciLibrary;
        PCICommonFunction   *m_pPciLibrary;
        //PCILibrary   m_pciLibrary;
        PCICommonFunction   m_pciLibrary;

        string          m_szDeviceName;
        unsigned short  m_u16Domain;
        unsigned short  m_u16Bus;
        unsigned short  m_u16Device;
        unsigned char   m_u8Function;
        
        //common configuration space header
        unsigned short  m_u16VendorID;
        unsigned short  m_u16DeviceID;
        unsigned int    m_u32ClassCode;
        unsigned char   m_u8RevisionID;
        unsigned char   m_u8HeaderType;
        unsigned char   m_u8CapPtrValue;

        //for PCIE
        //bool            m_bIsPCIe;
        //bool            m_bLinkStatus;
        unsigned short  m_u32CurrentWidth;
        float           m_fCurrentSpeed;
        unsigned short  m_u32WidthCap;
        float           m_fSpeedCap;

        list<unsigned int>  m_listBAR;
        list<CapInfo>       m_listCap;

        void    GetLinkInfo();
        void    getExtendedCapList();

};

#endif
