/// @file       BaseClassPSUDDAPI.cpp
/// @brief      BaseClassPSUDDAPI class
/// @details    Provide low layer diagnostic and utility for PSU device
/// @author     KenHu
/// @copyright  Copyright 2015-2016 Hon Hai Technology Group. All Rights Reserved.

#include "BaseClassPSUDDAPI.hpp"

/// @fn         BaseClassPSUDDAPI::BaseClassPSUDDAPI(void) :  BaseClassDDAPI("PSUDDAPI")
/// @brief      BaseClassPSUDDAPI constructor
/// @exception  Nothing
/// @return     Nothing
BaseClassPSUDDAPI::BaseClassPSUDDAPI(void) :  BaseClassDDAPI("PSUDDAPI")
{                                

}

/// @fn         BaseClassPSUDDAPI::~BaseClassPSUDDAPI(void)
/// @brief      BaseClassPSUDDAPI destructor
/// @exception  Nothing
/// @return     Nothing                                
BaseClassPSUDDAPI::~BaseClassPSUDDAPI(void)
{
}

/// @fn         BaseClassPSUDDAPI::getName(void)
/// @brief      Get PSU device name
/// @exception  Nothing
/// @return     A string for device name  
string
BaseClassPSUDDAPI::getName(void)
{
    return m_strDeviceName;
}

/// @fn         BaseClassPSUDDAPI::setName(string in_strName)
/// @brief      Set PSU device name
/// @param      in_strName is a string for device name
/// @exception  Nothing
/// @return     Nothing
void
BaseClassPSUDDAPI::setName(string in_strName)
{
    m_strDeviceName = in_strName;
}

/// @fn         BaseClassPSUDDAPI::getFanList(list<string> *out_pList)
///
/// @brief      Get list of FAN names
/// @param      out_pList is a pointer to a list of strings for FAN names
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getFanList(list<string> *out_pList)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_FAN_LIST | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getFanSpeed(string  in_strDeviceName,
///                                            INT32   *out_s32pValue)
///
/// @brief      Get current speed for the given FAN
/// @param      in_strDeviceName is a string for FAN name
/// @param      out_s32pValue is a pointer to a INT32 for the current speed
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getFanSpeed(string  in_strDeviceName,
                               INT32   *out_s32pValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_FAN_SPEED | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getFWVersion(string *out_strpValue)
///
/// @brief      Get FW version
/// @param      out_strpValue is a pointer to a string for the FW version
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getFWVersion(string *out_strpValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_FW_VERSION | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getLEDList(list<string> *out_pList)
///
/// @brief      Get list of LED names
/// @param      out_pList is a pointer to a list of strings for LED names
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getLEDList(list<string> *out_pList)
{
    return NO_ERROR_MSG;
}

/// @fn         BaseClassPSUDDAPI::getLEDStatus(string in_strDeviceName,
///                                    string *out_strpStatus)
///
/// @brief      Get status of the given LED
/// @param      in_strDeviceName is a string for LED
/// @param      out_strpStatus is a pointer to a string for LED status
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getLEDStatus(string in_strDeviceName,
                                string *out_strpStatus)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_LED_STATUS | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getMaxPower(string *out_strpValue)
///
/// @brief      Get maximum power
/// @param      out_strpValue is a pointer to a string for the maximum power
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getMaxPower(string *out_strpValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_MAX_POWER | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getPartNumber(string *out_strpValue)
///
/// @brief      Get part number
/// @param      out_strpValue is a pointer to a string for the part number
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getPartNumber(string *out_strpValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_PART_NUMBER | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getSerialNumber(string *out_strpValue)
///
/// @brief      Get serial number
/// @param      out_strpValue is a pointer to a string for the serial number
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getSerialNumber(string *out_strpValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_SERIAL_NUMBER | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getSupportFanSpeed(string  in_strDeviceName,
///                                                   list<string>   *out_pList)
///
/// @brief      Get supported speeds for the given FAN
/// @param      in_strDeviceName is a string for FAN name
/// @param      out_pList is a pointer to a list of strings for supported speeds
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getSupportFanSpeed(string         in_strDeviceName,
                                      list<string>   *out_pList)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_SUPPORT_FAN_SPEED | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getTemperature(string   in_strDeviceName,
///                                               FLOAT    *out_fpValue)
///
/// @brief      Get current temperature
/// @param      in_strDeviceName is a string for temperature sensor name
/// @param      out_fpValue is a pointer to a FLOAT for the current temperature
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getTemperature(string   in_strDeviceName,
                                  FLOAT    *out_fpValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getTemperatureList(list<string> *out_pList)
///
/// @brief      Get the list of temperature sensor names
/// @param      out_pList is a pointer to a list of strings for temperature sensor names
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getTemperatureList(list<string> *out_pList)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE_LIST | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getTemperatureThreshold(string   in_strDeviceName,
///                                                        FLOAT    *out_fpValue)
///
/// @brief      Get temperature threshold
/// @param      in_strDeviceName is a string for temperature threshold
/// @param      out_fpValue is a pointer to a FLOAT for the threshold
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getTemperatureThreshold(string   in_strDeviceName,
                                           FLOAT    *out_fpValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         PSUDDAPI::getTemperatureThresholdList(list<string> *out_pList)
///
/// @brief      Get the list of temperature sensor thresholds
/// @param      out_pList is a pointer to a list of strings for temperature sensor names
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getTemperatureThresholdList(list<string> *out_pList)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD_LIST | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getType(string *out_strpValue)
///
/// @brief      Get protocol type
/// @param      out_strpValue is a pointer to a string for the protocol type
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getType(string *out_strpValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_TYPE | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::getVendor(string *out_strpValue)
///
/// @brief      Get vendor name
/// @param      out_strpValue is a pointer to a string for the vendor name
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::getVendor(string *out_strpValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_GET_VENDOR_NAME | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::isPresent(BOOL *out_bpPresent)
///
/// @brief      Check if PSU is present
/// @param      out_bpPresent is a pointer to a BOOL for the status
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::isPresent(BOOL *out_bpPresent)
{
    return NO_ERROR_MSG;
}

/// @fn         BaseClassPSUDDAPI::program(UINT8   in_u8Sector,
///                                        CHAR*   in_chpFwBuffer,
///                                        INT32   in_s32FwSize)
/// @brief      Program new FW to the given sector
/// @param      in_u8Sector is a UINT8 for the sector
/// @param      in_chpFwBuffer is a pointer to CHAR for the memory buffer stores new FW
/// @param      in_s32FwSize is a INT32 for the size of FW
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::program(UINT8   in_u8Sector,
                           CHAR*   in_chpFwBuffer,
                           INT32   in_s32FwSize)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_PROGRAM | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::setFanSpeed(string   in_strDeviceName,
///                                            INT32    in_s32Value)
/// @brief      Set FAN speed to the given FAN
/// @param      in_strDeviceName is a string for the FAN
/// @param      in_s32Value is a INT32 for the FAN speed
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::setFanSpeed(string   in_strDeviceName,
                               INT32    in_s32Value)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_SET_FAN_SPEED | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::setLEDStatus(string   in_strDeviceName,
///                                    string   in_strStatus)
///
/// @brief      Set LED to be the given status
/// @param      in_strDeviceName is a string for LED
/// @param      in_strStatus is a string for LED status
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::setLEDStatus(string   in_strDeviceName,
                                string   in_strStatus)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_SET_LED_STATUS | ERROR_FUNCTION_NOT_IMPLEMENT));
}

/// @fn         BaseClassPSUDDAPI::setTemperatureThreshold(string   in_strDeviceName,
///                                                        FLOAT    in_fValue)
/// @brief      Set temperature threshold for temperature sensors
/// @param      in_strDeviceName is a string for the temperature threshold name
/// @param      in_s32Value is a FLOAT for the threshold value
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
BaseClassPSUDDAPI::setTemperatureThreshold(string   in_strDeviceName,
                                           FLOAT    in_fValue)
{
    return returnErrorMsg((ERRORCODE)(ERROR_PSU_DDAPI_SET_TEMPERATURE_THRESHOLD | ERROR_FUNCTION_NOT_IMPLEMENT));
}
