/// @file       BaseClassPSUDDAPI.hpp
/// @brief      BaseClassPSUDDAPI class
/// @details    Provide low layer diagnostic and utility for PSU device
/// @author     KenHu
/// @copyright  Copyright 2015-2016 Hon Hai Technology Group. All Rights Reserved.

#ifndef BASE_CLASS_PSU_DDAPI_HPP
#define BASE_CLASS_PSU_DDAPI_HPP

#include <string>
#include <list>

#include "BaseClassDDAPI.hpp"
#include "PSUConfig.hpp"

using::std::string;
using::std::list;

/// @class      BaseClassPSUDDAPI BaseClassPSUDDAPI.hpp "BaseClassPSUDDAPI.hpp"
/// @brief      Provide low layer diagnostic and utility for PSU device
/// @param      m_strDeviceName is a string for device name
class BaseClassPSUDDAPI : public BaseClassDDAPI
{
    public:
                                BaseClassPSUDDAPI(void);
        virtual                 ~BaseClassPSUDDAPI(void);

        string                  getName(void);
        void                    setName(string in_strName);

        virtual ErrorMsg*       getFanList(list<string> *out_pList);
        virtual ErrorMsg*       getFanSpeed(string  in_strDeviceName,
                                            INT32   *out_s32pValue);
        virtual ErrorMsg*       getFWVersion(string *out_strpValue);
        virtual ErrorMsg*       getLEDList(list<string> *out_pList);
        virtual ErrorMsg*       getLEDStatus(string in_strDeviceName,
                                             string *out_strpStatus);
        virtual ErrorMsg*       getMaxPower(string *out_strpValue);
        virtual ErrorMsg*       getPartNumber(string *out_strpValue);
        virtual ErrorMsg*       getSerialNumber(string *out_strpValue);
        virtual ErrorMsg*       getSupportFanSpeed(string  in_strDeviceName,
                                                   list<string> *out_pList);
        virtual ErrorMsg*       getTemperature(string   in_strDeviceName,
                                               FLOAT    *out_fpValue);
        virtual ErrorMsg*       getTemperatureList(list<string> *out_pList);
        virtual ErrorMsg*       getTemperatureThreshold(string   in_strDeviceName,
                                                        FLOAT    *out_fpValue);
        virtual ErrorMsg*       getTemperatureThresholdList(list<string> *out_pList);
        virtual ErrorMsg*       getType(string *out_strpValue);
        virtual ErrorMsg*       getVendor(string *out_strpValue);

        virtual ErrorMsg*       isPresent(BOOL *out_bpPresent);

        virtual ErrorMsg*       program(UINT8   in_u8Sector,
                                        CHAR*   in_chpFwBuffer,
                                        INT32   in_s32FwSize);

        virtual ErrorMsg*       setFanSpeed(string  in_strDeviceName,
                                            INT32   in_s32Value);
        virtual ErrorMsg*       setLEDStatus(string   in_strDeviceName,
                                             string   in_strStatus);
        virtual ErrorMsg*       setTemperatureThreshold(string  in_strDeviceName,
                                                        FLOAT   in_fValue);
    protected:
        
    private:
        string                  m_strDeviceName;
};
#endif

