/// @file       PSUDDAPI.hpp
/// @brief      PSUDDAPI class
/// @details    Provide low layer diagnostic and utility APIs for PSU module
/// @author     KenHu
/// @copyright  Copyright 2015-2016 Hon Hai Technology Group. All Rights Reserved.

#ifndef PSU_DDAPI_HPP
#define PSU_DDAPI_HPP

#include "BaseClassDDAPI.hpp"
#include "BaseClassPSUDDAPI.hpp"

#define TEMP_FILE_PSU_INFO  "psu/psuinfo"

/// @class      PSUDDAPI PSUDDAPI.hpp "PSUDDAPI.hpp"
/// @brief      Provide low layer diagnostic and utility APIs for PSU module
/// @param
class PSUDDAPI : public BaseClassDDAPI
{
    public: 
                            PSUDDAPI();
        virtual             ~PSUDDAPI(void);

        ErrorMsg*           getDeviceList(list<string> *out_pList);
        ErrorMsg*           getFanList(list<string> *out_pList);
        ErrorMsg*           getFanSpeed(string  in_strDeviceName,
                                        INT32   *out_s32pValue);
        ErrorMsg*           getFWVersion(string in_strDevice,
                                         string *out_strpValue);
        ErrorMsg*           getLEDList(list<string> *out_pList);
        ErrorMsg*           getLEDStatus(string in_strDeviceName,
                                         string *out_strpStatus);
        ErrorMsg*           getPartNumber(string in_strDevice,
                                          string *out_strpValue);
        ErrorMsg*           getMaxPower(string  in_strDevice,
                                        string  *out_strpValue);
        ErrorMsg*           getSerialNumber(string in_strDevice,
                                            string *out_strpValue);
        ErrorMsg*           getSupportFanSpeed(string  in_strDeviceName,
                                               list<string> *out_pList);
        ErrorMsg*           getTemperature(string   in_strDeviceName,
                                           string   *out_strpValue);
        ErrorMsg*           getTemperatureList(list<string> *out_pList);
        ErrorMsg*           getTemperatureThreshold(string   in_strDeviceName,
                                                    string   *out_strpValue);
        ErrorMsg*           getTemperatureThresholdList(list<string> *out_pList);
        ErrorMsg*           getType(string in_strDevice,
                                    string *out_strpValue);
        ErrorMsg*           getVendor(string in_strDevice,
                                      string *out_strpValue);

        ErrorMsg*           program(string    in_strDevice,
                                    UINT8     in_u8Region,
                                    CHAR      *in_chpFwBuffer,
                                    INT32     in_s32FwSize);

        ErrorMsg*           setFanSpeed(string  in_strDeviceName,
                                        INT32   in_s32Value);
        ErrorMsg*           setLEDStatus(string in_strDeviceName,
                                         string in_strStatus);
        ErrorMsg*           setTemperatureThreshold(string  in_strDeviceName,
                                                    string  in_strValue);
    protected:

    private:
        string              getOutPSUName(string *inout_strpString);
        ErrorMsg*           initializePSUInfoFile(void);

        list<BaseClassPSUDDAPI *>   m_psuList;
};

#endif

