#ifndef SCSI_SENSE_ANALYZER_HPP
#define SCSI_SENSE_ANALYZER_HPP

#include <iostream>
#include <sstream>
#include <string>
#include <list>

#include "StorageInfo.hpp"

/* defines for useful SCSI Status codes */
#define SCSI_STATUS_CHECK_CONDITION     0x2

/* defines for useful Sense Key codes */
#define SCSI_SK_NO_SENSE                0x0
#define SCSI_SK_RECOVERED_ERR           0x1
#define SCSI_SK_NOT_READY               0x2
#define SCSI_SK_MEDIUM_ERROR            0x3
#define SCSI_SK_HARDWARE_ERROR          0x4
#define SCSI_SK_ILLEGAL_REQUEST         0x5
#define SCSI_SK_UNIT_ATTENTION          0x6
#define SCSI_SK_ABORTED_COMMAND         0xb

/* defines for useful Additional Sense Codes (ASCs) */
#define SCSI_ASC_NOT_READY              0x4     /* more info in ASCQ code */
#define SCSI_ASC_NO_MEDIUM              0x3a    /* more info in ASCQ code */
#define SCSI_ASC_UNKNOWN_OPCODE         0x20
#define SCSI_ASC_INVALID_FIELD          0x24
#define SCSI_ASC_UNKNOWN_PARAM          0x26
#define SCSI_ASC_WARNING                0xb
#define SCSI_ASC_IMPENDING_FAILURE      0x5d

#define MAX_SENSE_BUFFER_LENGTH         512

typedef struct SCSISenseData
{
     string strCategory;
     string strErrorCondition;
     
     UINT32 u32SenseKey;
     UINT32 u32AdditionalSenseCode;
     UINT32 u32AdditionalSenseCodeQualifier;
     UINT32 u32ErrorCount;
} SCSISenseDataStruct;

typedef struct SCSISenseDrive
{
     UINT32 u32ControllerType; //0:SAS 1:SATA 2:USB
     UINT32 u32DefinedIndex;
     string strDriveDefinedName;
     list <SCSISenseDataStruct> SCSISenseDataList;
} SCSISenseDriveStruct;

using namespace std;

/// @class      SCSISenseAnalyzer SCSISenseAnalyzer.hpp "SCSISenseAnalyzer.hpp""
/// @brief      Analyze SCSI sense buffer
class SCSISenseAnalyzer : public BaseClassDDAPI
{
public:
     SCSISenseAnalyzer(STRING className);
     ~SCSISenseAnalyzer(void);

     string m_strOutputResultString;

     ErrorMsg *setSenseDataByIndex(UINT32 in_u32DefinedIndex, UCHAR* in_uchaSenseBuffer);
     ErrorMsg *listSenseDataByIndex(UINT32 in_u32DefinedIndex, BOOL in_blsMultiModeOutputFormat = false);
     ErrorMsg *clearSenseDataByIndex(UINT32 in_u32DefinedIndex);

     ErrorMsg *setSenseDataByName(string in_strDefinedName, UCHAR in_uchaSenseBuffer);
     ErrorMsg *listSenseDataByName(string in_strDefinedName);
     ErrorMsg *clearSenseDataByName(string in_strDefinedName);
     ERRORCODE translateSenseDataToErrorCode(UCHAR *in_uchpSenseBuffer, UINT32 in_u32BufferSize);
protected:

private:
     ERRORCODE parseSenseData(UCHAR* in_uchaSenseBuffer, UINT32* out_u32pSenseKey, UINT32* out_u32pAdditionalSenseCode, UINT32* out_u32pAdditionalSenseCodeQualifier);
     void initSenseKeyTable(void);

     SCSISenseDataStruct m_SCSISenseKeyTable[15][255][255];//[KEY:0~0xf][ASC:0~0xff][ASCQ:0~0xff]
     bool m_blsSenseKeyTableInit;
     list <SCSISenseDriveStruct> m_DriveList;
};
#endif
