/// @file       error_code.hpp
/// @brief      All error code and message definition
/// @details    Provide all modules error code with message
/// @version    1.1.0-bata
/// @date       2014.01.13
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#ifndef ERROR_CODE_H
#define ERROR_CODE_H

#include "common.hpp"

#define MEF_HEADER              0xC0000000

#define SHIFT_BIT_MODULE        24
#define SHIFT_BIT_TYPE          18
#define SHIFT_BIT_API           10
#define SHIFT_BIT_COMMON_ERROR  9

/// @def    ERRORCODE_MASK_MODULE   ((0x3F) << SHIFT_BIT_MODULE)
///         your Macro description...
#define ERRORCODE_MASK_MODULE   ((0x3F) << SHIFT_BIT_MODULE)
#define ERRORCODE_MASK_TYPE     ((0x3) << SHIFT_BIT_TYPE)
#define ERRORCODE_MASK_API      ((0xFF) << SHIFT_BIT_API)
#define ERRORCODE_MASK_ERROR    0x1FF

#define ERROR_CODE_MODULE   0
#define ERROR_CODE_COMMON   1

/// @emun   MEF_INDEX
///         your emun description
typedef enum {
    MEF_UNKNOWN = 0,
    MEF_PROCESSOR,
    MEF_MEMORY,
    MEF_ETHERNET,
    MEF_STORAGE,
    MEF_USB,
    MEF_GPIO,
    MEF_SAS,
    MEF_PCI,
    MEF_RTC,
    MEF_SCSI,
    MEF_UART,
    MEF_PSU,
    MEF_LED,
    MEF_I2C,
    MEF_TEMPERATURE,
    MEF_SPD,
    MEF_NVSRAM,
    MEF_TPM,
    MEF_NTB,
    MEF_EXPANDER,
    MEF_FPGA,
    MEF_NVME,
    MEF_CONTROLLER,
} MEF_INDEX;

/// @emun   TEF_INDEX
///         your emun description
typedef enum {
    TEF_UNKNOWN = 0,
    TEF_DAPI,
    TEF_DDAPI,
    TEF_DRIVER,
} TEF_INDEX;

// module, type, api, common
#define G_ME_HLD(module,type,api,common) (MEF_HEADER | ((module) << SHIFT_BIT_MODULE)  | ((type) << SHIFT_BIT_TYPE) | ((api) << SHIFT_BIT_API) | ((common) << SHIFT_BIT_COMMON_ERROR))

typedef enum  {
    NO_ERROR = 0,
    // Common error code definition
    ERROR_COMMON_ERROR_CODE                             = G_ME_HLD(MEF_UNKNOWN,TEF_UNKNOWN,0,ERROR_CODE_COMMON),
    ERROR_ILLEGAL_REQUEST,
    ERROR_TOO_FEW_PARAMETERS,
    ERROR_TOO_MANY_PARAMETERS,
    ERROR_BAD_PARAMETER,
    ERROR_BAD_PARAMETER_ONE,
    ERROR_BAD_PARAMETER_TWO,
    ERROR_BAD_PARAMETER_THREE,
    ERROR_BAD_PARAMETER_FOUR,
    ERROR_BAD_PARAMETER_FIVE,
    ERROR_BAD_PARAMETER_SIX,
    ERROR_BAD_PARAMETER_SEVEN,
    ERROR_BAD_PARAMETER_EIGHT,
    ERROR_BAD_PARAMETER_NINE,
    ERROR_FUNCTION_NOT_FINISHED,
    ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR,
    ERROR_DIAGNOSTIC_API_CONSTRUCTOR2_ERROR,
    ERROR_DIAGNOSTIC_DRIVER_API_CONSTRUCTOR1_ERROR,
    ERROR_ILLEGAL_ELEMENT_SIZE,
    ERROR_ILLEGAL_BUFFER_SIZE,
    ERROR_ALLOCATE_BUFFER_FAIL,
    ERROR_OPEN_FILE,
    ERROR_CLOSE_FILE,
    ERROR_READ_FILE_FAIL,
    ERROR_OPEN_DIRECTORY,
    ERROR_OUT_OF_RANGE,
    ERROR_IOCTL,
    ERROR_IOCTL_UNSUPPORT_CLASS,
    ERROR_IOCTL_GET_FILE_HANDLE_FAIL,
    ERROR_IOCTL_CLOSE_FILE_HANDLE_FAIL,
    ERROR_IOCTL_SET_CONTROLLER_ID_FAIL,
    ERROR_IOCTL_UNSUPPORT_ALLOCATEIOCTLARGUMENT,
    ERROR_IOCTL_UNSUPPORT_ARGUMENT,
    ERROR_IOCTL_UNSUPPORT_SENTIOCTL,
    ERROR_IOCTL_INVALID_MAGIC_NUMBER,
    ERROR_IOCTL_INVALID_COMMAND_CODE,
    ERROR_IOCTL_COPY_FROM_USER,
    ERROR_IOCTL_COPY_TO_USER,
    ERROR_IOCTL_SAS_INVALID_IOC_NUMBER,
    ERROR_IOCTL_SAS_INVALID_PHY_NUMBER,
    ERROR_SCSI_UPSUPPORT_OPCODE,
    ERROR_SCSI_GET_FILE_HANDLE_FAIL,
    ERROR_SCSI_IOCTL_ACCESS_FAIL,
    ERROR_SCSI_BUFFER_TOO_SMALL,
    ERROR_SCSI_BAD_DIRECTION,
    ERROR_SAS_IOCTL_ACCESS_FAIL,
    ERROR_SAS_IOCTL_MEM_ALLOC_FAILED,
    ERROR_NO_DEVICE_FOUND,
    ERROR_XML_COMMAND_PROCESSING_FAIL,
    ERROR_UNEXPECTED_VALUE,
    ERROR_UNEXPECTED_DATA_AMOUNT,
    ERROR_PRIVILEGE,
    ERROR_IO_ACCESS,
    ERROR_FUNCTION_NOT_IMPLEMENT,
    ERROR_FUNCTION_NOT_SUPPORTED,
    ERROR_CONFIGURATION_INCORRECT_FORMAT,
    ERROR_CONFIGURATION_NOT_FOUND,
    ERROR_TIME_OUT,
    ERROR_UNKNOWN_ARCH,
    ERROR_CREATE_OBJECT_FAIL,
    ERROR_REGISTER_IS_READ_ONLY,
    ERROR_REGISTER_IS_WRITE_ONLY,

    //--------------------- Memory ---------------------// 
    // Memory Diagnostic API error code definition
    ERROR_MEMORY_DAPI_ERRORCODE                         = G_ME_HLD(MEF_MEMORY,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_NO_SPD_INFO,
    ERROR_MEMORY_DAPI_NO_DIMM_INFO,
    ERROR_MEMORY_DAPI_NO_USABLE_MEMORY,
    ERROR_MEMORY_DAPI_NO_TESTABLE_MEMORY,
    ERROR_MEMORY_DAPI_TEST_PATTERN_COMPARE_FAIL,
    ERROR_MEMORY_DAPI_NO_DIMM_MATCH_TO_ECC_ERROR,
    ERROR_MEMORY_DAPI_REMAP_MEMORY_RANGE_UNMATCH,
    ERROR_MEMORY_DAPI_CONSTRUCTOR                       = G_ME_HLD(MEF_MEMORY,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_GET_INFORMATION                   = G_ME_HLD(MEF_MEMORY,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_READ_MEMORY                       = G_ME_HLD(MEF_MEMORY,TEF_DAPI,3,ERROR_CODE_MODULE),                    
    ERROR_MEMORY_DAPI_WRITE_MEMORY                      = G_ME_HLD(MEF_MEMORY,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_GET_MEMORY_BUFFER                 = G_ME_HLD(MEF_MEMORY,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER             = G_ME_HLD(MEF_MEMORY,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_DATA_LINE_TEST                    = G_ME_HLD(MEF_MEMORY,TEF_DAPI,7,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_ADDRESS_LINE_TEST                 = G_ME_HLD(MEF_MEMORY,TEF_DAPI,8,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_MARCHC_TEST                       = G_ME_HLD(MEF_MEMORY,TEF_DAPI,9,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_CHECKSUM_TEST                     = G_ME_HLD(MEF_MEMORY,TEF_DAPI,10,ERROR_CODE_MODULE),
    ERROR_MEMROY_DAPI_GET_TESTABLE_MEMORY               = G_ME_HLD(MEF_MEMORY,TEF_DAPI,11,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_GET_ECC_REPORT                    = G_ME_HLD(MEF_MEMORY,TEF_DAPI,12,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_CLEAR_ECC_REPORT                  = G_ME_HLD(MEF_MEMORY,TEF_DAPI,13,ERROR_CODE_MODULE),        
    ERROR_MEMORY_DAPI_ECC_ERROR_DETECTION_TEST          = G_ME_HLD(MEF_MEMORY,TEF_DAPI,14,ERROR_CODE_MODULE), 
    ERROR_MEMORY_DAPI_PATTERN_TEST                      = G_ME_HLD(MEF_MEMORY,TEF_DAPI,15,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_VERIFY_MEMORY_BLOCK               = G_ME_HLD(MEF_MEMORY,TEF_DAPI,16,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_EXECUTE_MARCHC_TEST               = G_ME_HLD(MEF_MEMORY,TEF_DAPI,17,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_VERIFY_MEMORY_BLOCK_BY_MARCHC     = G_ME_HLD(MEF_MEMORY,TEF_DAPI,18,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_RANDOM_ADDRESS_TEST               = G_ME_HLD(MEF_MEMORY,TEF_DAPI,19,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_GET_DIMM_ADDRESS_INFORMATION      = G_ME_HLD(MEF_MEMORY,TEF_DAPI,20,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_GET_DEVICE_LIST                   = G_ME_HLD(MEF_MEMORY,TEF_DAPI,21,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_GET_RANK_RANGE                    = G_ME_HLD(MEF_MEMORY,TEF_DAPI,22,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_INITIALIZE_MEMORY_LAYOUT_FILE     = G_ME_HLD(MEF_MEMORY,TEF_DAPI,23,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_INITIALIZE_TESTABLE_MEMORY_FILE   = G_ME_HLD(MEF_MEMORY,TEF_DAPI,24,ERROR_CODE_MODULE),
    ERROR_MEMORY_DAPI_CHECK_DEVICE_BY_SMBIOS            = G_ME_HLD(MEF_MEMORY,TEF_DAPI,25,ERROR_CODE_MODULE),

    // Memory Diagnostic Driver API error code definition
    ERROR_MEMORY_DDAPI_ERRORCODE                        = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_DECODE_DIMM_SPD_FAIL,
    ERROR_MEMORY_DDAPI_DMIDECODE_MEMORY_DEVICE_FAIL,
    ERROR_MEMORY_DDAPI_DMIDECODE_MEMORY_DEVICE_NOT_FOUND,
    ERROR_MEMORY_DDAPI_OPEN_MEM_DEVICE_FAIL,
    ERROR_MEMORY_DDAPI_MMAP_FAIL,
    ERROR_MEMORY_DDAPI_EDAC_DEVICE_NOT_FOUND,
    ERROR_MEMORY_DDAPI_ADDRESS_TRANSLATION_FAIL,
    ERROR_MEMORY_DDAPI_INCORRECT_LAYOUT_INFO,
    ERROR_MEMORY_DDAPI_GET_DIMM_INFORMATION_BY_SPD      = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_GET_DIMM_INFORMATION_BY_SMBIOS   = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_IOREMAP                          = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_READ_MEMORY                      = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_WRITE_MEMORY                     = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_GET_MEMORY_BUFFER                = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_RELEASE_MEMORY_BUFFER            = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_GET_ECC_REPORT                   = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_CLEAR_ECC_REPORT                 = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_INJECT_ECC                       = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,10,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_GET_HIGH_RAM_RANGE               = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,11,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_GET_CONTROLLER_CHANNEL_COUNT     = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,12,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_GET_CONTROLLER_SLOT_COUNT        = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,13,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_INITIALIZE_MEMORY_LAYOUT_FILE    = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,14,ERROR_CODE_MODULE), 
    ERROR_MEMORY_DDAPI_GET_PHYSICAL_ADDRESS             = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,15,ERROR_CODE_MODULE),
    ERROR_MEMORY_DDAPI_GET_RANK_OFFSET                  = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,16,ERROR_CODE_MODULE),

    // Memory Driver error code definition
    ERROR_MEMORY_DRIVER_ERRORCODE                       = G_ME_HLD(MEF_MEMORY,TEF_DRIVER,0,ERROR_CODE_MODULE),
    ERROR_MEMORY_DRIVER_ADDR_TRANSLATION_FAIL,
    ERROR_MEMORY_DRIVER_IOREMAP_FAIL,
    ERROR_MEMORY_DRIVER_ECC_INJECT_FAIL,
    ERROR_MEMORY_DRIVER_COMPARE_DATA_FAIL,
    ERROR_MEMORY_DRIVER_ENABLE_ECC_INJECT_FAIL,
    ERROR_MEMORY_DRIVER_INCORRECT_ECC_COUNT,
    ERROR_MEMORY_DRIVER_NO_ECC_COUNT,
    ERROR_MEMORY_DRIVER_ECC_INJECT_LOCK,

    //--------------------- Processor ---------------------//
    //Processor Diagnostic API error code definition
    ERROR_PROCESSOR_DAPI_ERRORCODE                      = G_ME_HLD(MEF_PROCESSOR,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DAPI_OPEN_NODE_FAIL,
    ERROR_PROCESSOR_DAPI_OPEN_MCE_NODE_FAIL,
    ERROR_PROCESSOR_DAPI_MCE_GET_RECORD_LEN,
    ERROR_PROCESSOR_DAPI_MCE_GET_LOG_LEN,
    ERROR_PROCESSOR_DAPI_GET_MCE_PROCESS_FAIL,
    ERROR_PROCESSOR_DAPI_CONSTRUCTOR                    = G_ME_HLD(MEF_PROCESSOR,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DAPI_GET_INFORMATIOM                = G_ME_HLD(MEF_PROCESSOR,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DAPI_STABILITY_TEST                 = G_ME_HLD(MEF_PROCESSOR,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DAPI_GET_ERROR_REGISTER             = G_ME_HLD(MEF_PROCESSOR,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DAPI_GET_MCE                        = G_ME_HLD(MEF_PROCESSOR,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DAPI_GET_MCE_EXECUTE                = G_ME_HLD(MEF_PROCESSOR,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DAPI_GET_MCE_PROCESS                = G_ME_HLD(MEF_PROCESSOR,TEF_DAPI,7,ERROR_CODE_MODULE),

    //Processor Diagnostic Driver API error code definition
    ERROR_PROCESSOR_DDAPI_ERRORCODE                     = G_ME_HLD(MEF_PROCESSOR,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DDAPI_OPEN_CPU_INFO_FAIL,
    ERROR_PROCESSOR_DDAPI_REGREAD_FAIL,
    ERROR_PROCESSOR_DDAPI_GET_CPU_INFORMATION           = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DDAPI_GET_CPU_NUMBER                = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DDAPI_GET_ERROR_REGISTERS_EXECUTE   = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DDAPI_REG_READ                      = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DDAPI_READ_CPU_REGISTER             = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DDAPI_WRITE_CPU_REGISTER            = G_ME_HLD(MEF_MEMORY,TEF_DDAPI,6,ERROR_CODE_MODULE),

    //Processor Driver error code definition
    ERROR_PROCESSOR_DRIVER_ERRORCODE                    = G_ME_HLD(MEF_PROCESSOR,TEF_DRIVER,0,ERROR_CODE_MODULE),
    ERROR_PROCESSOR_DRIVER_OPEN_NODE_FAIL,


    //--------------------- SAS ---------------------//
    //SAS
    ERROR_SAS_DAPI_ERRORCODE                            = G_ME_HLD(MEF_SAS,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_TEST_PATTERN_COMPARE_FAIL,
    ERROR_SAS_DAPI_DISPLAY_DATA,
    ERROR_SAS_DAPI_GET_MEMORY_BUFFER,
    ERROR_SAS_DAPI_RELEASE_MEMORY_BUFFER,
    ERROR_SAS_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL,
    ERROR_SAS_DAPI_ALLOCATE_PATTERN_BUFFER_FAIL,
    ERROR_SAS_DAPI_ALLOCATE_PCOMPARE_BUFFER_FAIL,
    ERROR_SAS_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_SAS,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_DATA_LINE_TEST                       = G_ME_HLD(MEF_SAS,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_ADDRESS_LINE_TEST                    = G_ME_HLD(MEF_SAS,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_CLEAR_PHY_ERROR_LOG                  = G_ME_HLD(MEF_SAS,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_INFORMATION                      = G_ME_HLD(MEF_SAS,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_TEMPERATURE                      = G_ME_HLD(MEF_SAS,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_LOGICALID_BY_NAME                = G_ME_HLD(MEF_SAS,TEF_DAPI,7,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_READWRITE_TEST                       = G_ME_HLD(MEF_SAS,TEF_DAPI,8,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_READWRITE_TEST_SETUP                 = G_ME_HLD(MEF_SAS,TEF_DAPI,9,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_READWRITE_TEST_EXECUTE               = G_ME_HLD(MEF_SAS,TEF_DAPI,10,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_READWRITE_TEST_CLEAN                 = G_ME_HLD(MEF_SAS,TEF_DAPI,11,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_READ_PERFORMANCE_TEST                = G_ME_HLD(MEF_SAS,TEF_DAPI,12,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_READ_PERFORMANCE_TEST_SETUP          = G_ME_HLD(MEF_SAS,TEF_DAPI,13,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_READ_PERFORMANCE_TEST_EXECUTE        = G_ME_HLD(MEF_SAS,TEF_DAPI,14,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_READ_PERFORMANCE_TEST_CLEAN          = G_ME_HLD(MEF_SAS,TEF_DAPI,15,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST               = G_ME_HLD(MEF_SAS,TEF_DAPI,16,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_SETUP         = G_ME_HLD(MEF_SAS,TEF_DAPI,17,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE       = G_ME_HLD(MEF_SAS,TEF_DAPI,18,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_CLEAN         = G_ME_HLD(MEF_SAS,TEF_DAPI,19,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_LOOPBACK_TEST                        = G_ME_HLD(MEF_SAS,TEF_DAPI,20,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_RESETIOC                             = G_ME_HLD(MEF_SAS,TEF_DAPI,21,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_PROGRAM_ADDRESS                      = G_ME_HLD(MEF_SAS,TEF_DAPI,22,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_DO_IOCTL			        = G_ME_HLD(MEF_SAS,TEF_DAPI,23,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_ATTACHED_DEVICE                  = G_ME_HLD(MEF_SAS,TEF_DAPI,24,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_ERROR_COUNT                      = G_ME_HLD(MEF_SAS,TEF_DAPI,25,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_LINK_SPEED                       = G_ME_HLD(MEF_SAS,TEF_DAPI,26,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_LINK_STATUS                      = G_ME_HLD(MEF_SAS,TEF_DAPI,27,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_DEVICE_NODE_LIST                 = G_ME_HLD(MEF_SAS,TEF_DAPI,28,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_DRIVE_INFORMATION                = G_ME_HLD(MEF_SAS,TEF_DAPI,29,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_GET_HOST_SAS_ADDRESS                 = G_ME_HLD(MEF_SAS,TEF_DAPI,30,ERROR_CODE_MODULE),
    ERROR_SAS_DAPI_PROGRAM_FIRMWARE                     = G_ME_HLD(MEF_SAS,TEF_DAPI,31,ERROR_CODE_MODULE),


    ERROR_SAS_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_SAS,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_SAS_DDAPI_READ_MEMORY,
    ERROR_SAS_DDAPI_WRITE_MEMORY,
    ERROR_SAS_DDAPI_OPEN_SAS_DEVICE_FAIL,
    ERROR_SAS_DDAPI_MMAP_FAIL,
    ERROR_SAS_DDAPI_FILE_SEEK,
    ERROR_SAS_DDAPI_NOT_SUPPORT_SPECIFIC_PHY,
    ERROR_SAS_DDAPI_READ_LINK_ERROR,
    ERROR_SAS_DDAPI_NO_SAS_CONTROLLER,
    ERROR_SAS_DDAPI_GET_DRIVE_SIZE,
	ERROR_SAS_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_SAS,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_SAS_DDAPI_EXECUTE_IOCTL                       = G_ME_HLD(MEF_SAS,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_SAS_DDAPI_SEEK_READ                           = G_ME_HLD(MEF_SAS,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_SAS_DDAPI_SEEK_WRITE                          = G_ME_HLD(MEF_SAS,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_SAS_DDAPI_PROGRAMIMAGE                        = G_ME_HLD(MEF_SAS,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_SAS_DDAPI_SETFILEHANDLE                       = G_ME_HLD(MEF_SAS,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_SAS_DDAPI_CLOSEFILEHANDLE                     = G_ME_HLD(MEF_SAS,TEF_DDAPI,7,ERROR_CODE_MODULE),
	ERROR_SAS_DDAPI_GET_SAS_LIST						= G_ME_HLD(MEF_SAS,TEF_DDAPI,8,ERROR_CODE_MODULE),
	ERROR_SAS_DDAPI_TEMPERATURE_NO_EXIST				= G_ME_HLD(MEF_SAS,TEF_DDAPI,9,ERROR_CODE_MODULE),
	ERROR_SAS_DDAPI_GET_DEVICE_NODE_LIST                = G_ME_HLD(MEF_SAS,TEF_DDAPI,10,ERROR_CODE_MODULE),
	ERROR_SAS_DDAPI_GET_DRIVE_INFO                      = G_ME_HLD(MEF_SAS,TEF_DDAPI,11,ERROR_CODE_MODULE),
	ERROR_SAS_DDAPI_GET_DRIVE_PHY_INFO                  = G_ME_HLD(MEF_SAS,TEF_DDAPI,12,ERROR_CODE_MODULE),
	ERROR_SAS_DDAPI_GET_SAS_INFO                        = G_ME_HLD(MEF_SAS,TEF_DDAPI,13,ERROR_CODE_MODULE),

    ERROR_SAS_DRIVER_ERRORCODE                          = G_ME_HLD(MEF_SAS,TEF_DRIVER,0,ERROR_CODE_MODULE),
    ERROR_SAS_DRIVER_IOCTL_FAIL,

    ERROR_SCSI_FACTORY_ERRORCODE                        = G_ME_HLD(MEF_SCSI,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_ERR_NOT_READY,
    ERROR_SCSI_FACTORY_ERR_BAD_OPCODE,
    ERROR_SCSI_FACTORY_ERR_BAD_FIELD,
    ERROR_SCSI_FACTORY_ERR_BAD_PARAM,
    ERROR_SCSI_FACTORY_ERR_BAD_RESP,
    ERROR_SCSI_FACTORY_ERR_NO_MEDIUM,
    ERROR_SCSI_FACTORY_ERR_BECOMING_READY,
    ERROR_SCSI_FACTORY_ERR_TRY_AGAIN,
    ERROR_SCSI_FACTORY_ERR_MEDIUM_HARDWARE,
    ERROR_SCSI_FACTORY_ERR_UNKNOWN,
    ERROR_SCSI_FACTORY_ERR_ABORTED_COMMAND,
    ERROR_SCSI_FACTORY_CONSTRUCTOR                      = G_ME_HLD(MEF_SCSI,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_READ_SCSI                        = G_ME_HLD(MEF_SCSI,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_WRITE_SCSI                       = G_ME_HLD(MEF_SCSI,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_TEST_UNIT_READY                  = G_ME_HLD(MEF_SCSI,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_STD_INQUIRY                      = G_ME_HLD(MEF_SCSI,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_INQUIRY_VPD                      = G_ME_HLD(MEF_SCSI,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_LOG_SENSE                        = G_ME_HLD(MEF_SCSI,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_LOG_SELECT                       = G_ME_HLD(MEF_SCSI,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_MODE_SENSE                       = G_ME_HLD(MEF_SCSI,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_MODE_SELECT                      = G_ME_HLD(MEF_SCSI,TEF_DDAPI,10,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_MODE_SENSE10                     = G_ME_HLD(MEF_SCSI,TEF_DDAPI,11,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_MODE_SELECT10                    = G_ME_HLD(MEF_SCSI,TEF_DDAPI,12,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_MODE_PAGE_OFFSET                 = G_ME_HLD(MEF_SCSI,TEF_DDAPI,13,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_REQUEST_SENSE                    = G_ME_HLD(MEF_SCSI,TEF_DDAPI,14,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_SEND_DIAGNOSTIC                  = G_ME_HLD(MEF_SCSI,TEF_DDAPI,15,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_READ_CAPACITY10                  = G_ME_HLD(MEF_SCSI,TEF_DDAPI,16,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_READ_CAPACITY16                  = G_ME_HLD(MEF_SCSI,TEF_DDAPI,17,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_GET_RPM                          = G_ME_HLD(MEF_SCSI,TEF_DDAPI,18,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_GET_TRANSPORT_PROTOCAL           = G_ME_HLD(MEF_SCSI,TEF_DDAPI,19,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_GET_DRIVE_TEMPERATURE            = G_ME_HLD(MEF_SCSI,TEF_DDAPI,20,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_GET_ERROR_LOG                    = G_ME_HLD(MEF_SCSI,TEF_DDAPI,21,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_GET_SMART_HEALTH_STATUS          = G_ME_HLD(MEF_SCSI,TEF_DDAPI,22,ERROR_CODE_MODULE),
    ERROR_SCSI_FACTORY_READ_DEFECT12                    = G_ME_HLD(MEF_SCSI,TEF_DDAPI,23,ERROR_CODE_MODULE),

    //--------------------- USB ---------------------//
    // USB Diagnostic API error code definition
    ERROR_USB_DAPI_ERRORCODE                            = G_ME_HLD(MEF_USB,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_DATA_ELEMENT_NOT_FOUND,
    ERROR_USB_DAPI_TEST_PATTERN_COMPARE_FAIL,
    ERROR_USB_DAPI_DIVIDED_ZERO_FAIL,
    ERROR_USB_DAPI_BACKUP_DATA_FAIL,
    ERROR_USB_DAPI_WRITE_DATA_TO_STORAGE_FAIL,
    ERROR_USB_DAPI_READ_DATA_FROM_STORAGE_FAIL,
    ERROR_USB_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL,
    ERROR_USB_DAPI_ALLOCATE_PATTERN_BUFFER_FAIL,
    ERROR_USB_DAPI_ALLOCATE_PCOMPARE_BUFFER_FAIL,
    ERROR_USB_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_USB,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_DISPLAY_USB_PORT_LIST                = G_ME_HLD(MEF_USB,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_OPEN_DEVICE                          = G_ME_HLD(MEF_USB,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_CLOSE_DEVICE                         = G_ME_HLD(MEF_USB,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_READ_WRITE_TEST                      = G_ME_HLD(MEF_USB,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_READ_WRITE_TEST_SETUP                = G_ME_HLD(MEF_USB,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE              = G_ME_HLD(MEF_USB,TEF_DAPI,7,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_READ_WRITE_TEST_CLEAN                = G_ME_HLD(MEF_USB,TEF_DAPI,8,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_READ_PERFORMANCE_TEST                = G_ME_HLD(MEF_USB,TEF_DAPI,9,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP          = G_ME_HLD(MEF_USB,TEF_DAPI,10,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE        = G_ME_HLD(MEF_USB,TEF_DAPI,11,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_READ_PERFORMANCE_TEST_CLEAN          = G_ME_HLD(MEF_USB,TEF_DAPI,12,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST               = G_ME_HLD(MEF_USB,TEF_DAPI,13,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP         = G_ME_HLD(MEF_USB,TEF_DAPI,14,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE       = G_ME_HLD(MEF_USB,TEF_DAPI,15,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_CLEAN         = G_ME_HLD(MEF_USB,TEF_DAPI,16,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_PREAD                                = G_ME_HLD(MEF_USB,TEF_DAPI,17,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_PWRITE                               = G_ME_HLD(MEF_USB,TEF_DAPI,18,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_READ_FROM_DEVICE                     = G_ME_HLD(MEF_USB,TEF_DAPI,19,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_WRITE_TO_DEVICE                      = G_ME_HLD(MEF_USB,TEF_DAPI,20,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_LIST_DEVICE_NODE                     = G_ME_HLD(MEF_USB,TEF_DAPI,21,ERROR_CODE_MODULE),
    ERROR_USB_DAPI_IS_DEVICE_NODE                       = G_ME_HLD(MEF_USB,TEF_DAPI,22,ERROR_CODE_MODULE),
    ERRIR_USB_DAPI_GET_LOGICAL_ID_BY_NAME               = G_ME_HLD(MEF_USB,TEF_DAPI,23,ERROR_CODE_MODULE),
    ERRIR_USB_DAPI_GET_DATA_ID_BY_ID                    = G_ME_HLD(MEF_USB,TEF_DAPI,24,ERROR_CODE_MODULE),

    // USB Diagnostic Driver API error code definition
    ERROR_USB_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_USB,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_DEVICE_NAME_NOT_FOUND,
    ERROR_USB_DDAPI_DATA_ELEMENT_NOT_FOUND,
    ERROR_USB_DDAPI_LIBUSB_INIT,
    ERROR_USB_DDAPI_LINK_SYMBOLIC_FAIL,
    ERROR_USB_DDAPI_FILE_SEEK,
    ERROR_USB_DDAPI_READ_DATA_FAIL,
    ERROR_USB_DDAPI_WRITE_DATA_FAIL,
	ERROR_USB_DDAPI_OPEN_DIR_FAIL,
    ERROR_USB_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_USB,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_INITIALIZE_USB_NAME                 = G_ME_HLD(MEF_USB,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_INITIALIZE_USB_INFO                 = G_ME_HLD(MEF_USB,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_DISPLAY_USB_PORT_LIST               = G_ME_HLD(MEF_USB,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_SCAN_USB_DEVICE                     = G_ME_HLD(MEF_USB,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_ADD_USB_BUS                         = G_ME_HLD(MEF_USB,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_INSPECT_BUS_ENTRY                   = G_ME_HLD(MEF_USB,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_ADD_USB_INTERFACE                   = G_ME_HLD(MEF_USB,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_ADD_USB_DEVICE                      = G_ME_HLD(MEF_USB,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_WALK_USB_DEVICES                    = G_ME_HLD(MEF_USB,TEF_DDAPI,10,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_GET_ROOT_HUB_DRIVER                 = G_ME_HLD(MEF_USB,TEF_DDAPI,11,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_CLASS                           = G_ME_HLD(MEF_USB,TEF_DDAPI,12,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_PARSE_USB_VENDOR_PRODUCT_NAME       = G_ME_HLD(MEF_USB,TEF_DDAPI,13,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_USBInfo_CONSTRUCTOR                 = G_ME_HLD(MEF_USB,TEF_DDAPI,14,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_GENERIC_STRING_TABLE            = G_ME_HLD(MEF_USB,TEF_DDAPI,15,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_PHY_DESCRIPTION                 = G_ME_HLD(MEF_USB,TEF_DDAPI,16,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_BIAS                            = G_ME_HLD(MEF_USB,TEF_DDAPI,17,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_HUTUS                           = G_ME_HLD(MEF_USB,TEF_DDAPI,18,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_LANG_ID                         = G_ME_HLD(MEF_USB,TEF_DDAPI,19,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_COUNTRY_CODE                    = G_ME_HLD(MEF_USB,TEF_DDAPI,20,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_HID                             = G_ME_HLD(MEF_USB,TEF_DDAPI,21,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_HUTS                            = G_ME_HLD(MEF_USB,TEF_DDAPI,22,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_REPORT_TAG                      = G_ME_HLD(MEF_USB,TEF_DDAPI,23,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_AUDIO_TERMINAL                  = G_ME_HLD(MEF_USB,TEF_DDAPI,24,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_VIDEO_TERMINAL                  = G_ME_HLD(MEF_USB,TEF_DDAPI,25,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_VENDOR                          = G_ME_HLD(MEF_USB,TEF_DDAPI,26,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_PRODUCT                         = G_ME_HLD(MEF_USB,TEF_DDAPI,27,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_SUB_CLASS                       = G_ME_HLD(MEF_USB,TEF_DDAPI,28,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_NEW_PROTOCOL                        = G_ME_HLD(MEF_USB,TEF_DDAPI,29,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_SEEK_READ                           = G_ME_HLD(MEF_USB,TEF_DDAPI,30,ERROR_CODE_MODULE),
    ERROR_USB_DDAPI_SEEK_WRITE                          = G_ME_HLD(MEF_USB,TEF_DDAPI,31,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_GET_DEVICE_CAPACITY                 = G_ME_HLD(MEF_USB,TEF_DDAPI,32,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_GET_LOGICAL_ID_BY_NAME              = G_ME_HLD(MEF_USB,TEF_DDAPI,33,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_GET_DATA_ID_BY_ID                   = G_ME_HLD(MEF_USB,TEF_DDAPI,34,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_GET_USB_DEVICE_NODE                 = G_ME_HLD(MEF_USB,TEF_DDAPI,35,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_GET_USB_INFO_FROM_SYS_BUS           = G_ME_HLD(MEF_USB,TEF_DDAPI,36,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_GET_USB_INFO_FROM_SYS_BLOCK         = G_ME_HLD(MEF_USB,TEF_DDAPI,37,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_PREAD                               = G_ME_HLD(MEF_USB,TEF_DDAPI,38,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_PWRITE                              = G_ME_HLD(MEF_USB,TEF_DDAPI,39,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_SCSI_READ                           = G_ME_HLD(MEF_USB,TEF_DDAPI,40,ERROR_CODE_MODULE),
    ERRIR_USB_DDAPI_SCSI_WRITE                          = G_ME_HLD(MEF_USB,TEF_DDAPI,41,ERROR_CODE_MODULE),
	ERRIR_USB_DDAPI_GET_USB_SG_DEVICE_NODE				= G_ME_HLD(MEF_USB,TEF_DDAPI,42,ERROR_CODE_MODULE),

    // USB Driver error code definition
    ERROR_USB_DRIVER_ERRORCODE                          = G_ME_HLD(MEF_USB,TEF_DRIVER,0,ERROR_CODE_MODULE),
    ERROR_USB_DRIVER_IOCTL_FAIL,

    //--------------------- GPIO ---------------------//
    //GPIO Diagnostic API error code definition
    ERROR_GPIO_DAPI_ERRORCODE                           = G_ME_HLD(MEF_GPIO,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_GPIO_DAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_GPIO,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_GPIO_DAPI_DISPLAY_GPIO_LIST                   = G_ME_HLD(MEF_GPIO,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_GPIO_DAPI_GPIO_GET_STATUS                     = G_ME_HLD(MEF_GPIO,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_GPIO_DAPI_GPIO_SET_STATUS                     = G_ME_HLD(MEF_GPIO,TEF_DAPI,4,ERROR_CODE_MODULE),




    //GPIO Diagnostic Driver API error code definition
    ERROR_GPIO_DDAPI_ERRORCODE                          = G_ME_HLD(MEF_GPIO,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_GPIO_DDAPI_DEVICE_NAME_NOT_FOUND,
    ERROR_GPIO_DDAPI_OPEN_PROFILE_FAIL,
    ERROR_GPIO_DDAPI_GET_GPIO_INDEX_FAIL,
    ERROR_GPIO_DDAPI_GPIO_READ_ONLY,
    ERROR_GPIO_DDAPI_GPIO_NOT_SUPPORT,
    ERROR_GPIO_DDAPI_GPIO_UNSUPPORT_TYPE,
	ERROR_GPIO_DDAPI_GPIO_GET_FAIL,
	ERROR_GPIO_DDAPI_GPIO_SET_FAIL,
    ERROR_GPIO_DDAPI_CONSTRUCTOR                        = G_ME_HLD(MEF_GPIO,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_GPIO_DDAPI_GET_GPIO_LIST                      = G_ME_HLD(MEF_GPIO,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_GPIO_DDAPI_GPIO_FPGA_DRIVER_CONSTRUCTOR       = G_ME_HLD(MEF_GPIO,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_GPIO_DDAPI_GPIO_FPGA_READ_BIT                 = G_ME_HLD(MEF_GPIO,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_GPIO_DDAPI_GPIO_FPGA_IS_MY_GPIO               = G_ME_HLD(MEF_GPIO,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_GPIO_DDAPI_GPIO_READ_BIT                      = G_ME_HLD(MEF_GPIO,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_GPIO_DDAPI_GPIO_SET_BIT                      	= G_ME_HLD(MEF_GPIO,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_GPIO_DDAPI_GPIO_FPGA_SET_BIT                 	= G_ME_HLD(MEF_GPIO,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_GPIO_PCH_DDAPI_DRIVER_CONSTRUCTOR       		= G_ME_HLD(MEF_GPIO,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_GPIO_PCH_DDAPI_READ_BIT                 		= G_ME_HLD(MEF_GPIO,TEF_DDAPI,10,ERROR_CODE_MODULE),
    ERROR_GPIO_PCH_DDAPI_IS_MY_GPIO               		= G_ME_HLD(MEF_GPIO,TEF_DDAPI,11,ERROR_CODE_MODULE),
	ERROR_GPIO_PCH_DDAPI_SET_BIT                  		= G_ME_HLD(MEF_GPIO,TEF_DDAPI,12,ERROR_CODE_MODULE),

	ERROR_GPIO_PCH_DDAPI_GET_GPIO_LIST					= G_ME_HLD(MEF_GPIO,TEF_DDAPI,12,ERROR_CODE_MODULE),


    //GPIO Driver error code definition
    ERROR_GPIO_DRIVER_ERRORCODE                         = G_ME_HLD(MEF_GPIO,TEF_DRIVER,0,ERROR_CODE_MODULE),
    ERROR_GPIO_DRIVER_IOCTL_FAIL,




    //--------------------- Ethernet ---------------------//
    //Ethernet Diagnostic API error code definition
    ERROR_ETHERNET_DAPI_ERRORCODE                       = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_NO_SPECIFY_NIC,
    ERROR_ETHERNET_DAPI_GET_ETHERNET_DRIVER_SET_FAIL,
    ERROR_ETHERNET_DAPI_SET_ETHERNET_DRIVER_SET_FAIL,
    ERROR_ETHERNET_DAPI_IOCTL_FAIL,
    ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL,
    ERROR_ETHERNET_DAPI_NODE_OPEN_FAIL,
    ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK_ABNORMAL,
    ERROR_ETHERNET_DAPI_SETUP_SPEED_TIMEOUT,
    ERROR_ETHERNET_DAPI_CONSTRUCTOR                     = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_GET_DEVICE_NAMES                = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_READ_MAC                        = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_READ_IP                         = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_READ_MASK                       = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_READ_MTU                        = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_READ_EEPROM                     = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,7,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_READ_EEPROM_SIZE                = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,8,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_EEPROM_VALIDATION_TEST          = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,9,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_MDI_REGISTER_TEST               = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,10,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_INTERRUPT_TEST                  = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,11,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_INTERNAL_LOOPBACK_TEST          = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,12,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_WRITE_EEPROM                    = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,13,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_WRITE_MAC                       = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,14,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK               = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,15,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_SETUP_SPEED                     = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,16,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_GET_SUPPORT_LINK_MODE           = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,17,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_GET_LINK_STATUS                 = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,18,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_GET_STATISTICS                  = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,19,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_GET_NODE_VALUE                  = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,20,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_GET_CONTROLLER_NAME             = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,21,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_GET_FIRMWARE_VERSION            = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,22,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DAPI_CLEAR                           = G_ME_HLD(MEF_ETHERNET,TEF_DAPI,23,ERROR_CODE_MODULE),

    //Ethernet Diagnostic Driver API error code definition
    ERROR_ETHERNET_DDAPI_ERRORCODE                      = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL,
    ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL,
    ERROR_ETHERNET_DDAPI_IOCTL_NOT_SUPPORT,
    ERROR_ETHERNET_DDAPI_IOCTL_FAIL,
    ERROR_ETHERNET_DDAPI_IOCTL_ETHTOOL_FAIL,
    ERROR_ETHERNET_DDAPI_SENDIOCTL_UNDEFINED,
    ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC,
    ERROR_ETHERNET_DDAPI_NIC_NAME_TOO_LONG,
    ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL,
    ERROR_ETHERNET_DDAPI_SET_MAC_ONE,
    ERROR_ETHERNET_DDAPI_SET_MAC_TWO,
    ERROR_ETHERNET_DDAPI_SET_MAC_THREE,
    ERROR_ETHERNET_DDAPI_SET_MAC_FOUR,
    ERROR_ETHERNET_DDAPI_SET_MAC_FIVE,
    ERROR_ETHERNET_DDAPI_SET_MAC_SIX,
    ERROR_ETHERNET_DDAPI_REGISTER_TEST_FAIL,
    ERROR_ETHERNET_DDAPI_EEPROM_VALIDATION_TEST_FAIL,
    ERROR_ETHERNET_DDAPI_MDI_REGISTER_TEST_FAIL,
    ERROR_ETHERNET_DDAPI_INTERRUPT_TEST_FAIL,
    ERROR_ETHERNET_DDAPI_INTERNAL_LOOPBACK_TEST_FAIL,
    ERROR_ETHERNET_DDAPI_SEND_SOCKET_OPEN_FAIL,
    ERROR_ETHERNET_DDAPI_RECEIVE_SOCKET_OPEN_FAIL,
    ERROR_ETHERNET_DDAPI_IOCTL_SIOCGIFINDEX,
    ERROR_ETHERNET_DDAPI_IOCTL_SIOCGIFHWADDR,
    ERROR_ETHERNET_DDAPI_IOCTL_SIOCGIFMTU,
    ERROR_ETHERNET_DDAPI_SOCKET_BIND,
    ERROR_ETHERNET_DDAPI_SENDTO,
    ERROR_ETHERNET_DDAPI_RECV_TIMEOUT,
    ERROR_ETHERNET_DDAPI_RECVFROM,
    ERROR_ETHERNET_DDAPI_READ_LINK_FAIL,
    ERROR_ETHERNET_DDAPI_SENDIOCTL                      = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_MAC                        = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_NAME_LIST                  = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_IP                         = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_MASK                       = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_MTU                        = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_EEPROM                     = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_EEPROM_SIZE                = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_SET_EEPROM                     = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_SET_MAC                        = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,10,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_MAIC                       = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,11,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK          = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,12,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_EXECUTE_EXTERNAL_LOOPBACK      = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,13,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_DEVICE_ID                  = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,14,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_ENABLE_EXTERNAL_LOOPBACK       = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,15,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_DISABLE_EXTERNAL_LOOPBACK      = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,16,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_DEVICE_BUS_TYPE            = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,17,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_GET_STATISTICS                 = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,18,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_CLEAR_AER                      = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,19,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DDAPI_CLEAR_STATISTICS               = G_ME_HLD(MEF_ETHERNET,TEF_DDAPI,20,ERROR_CODE_MODULE),

    //Ethernet Driver error code definition
    ERROR_ETHERNET_DRIVER_ERRORCODE                     = G_ME_HLD(MEF_ETHERNET,TEF_DRIVER,0,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DRIVER_OPEN_NODE_FAIL,
    ERROR_ETHERNET_DRIVER_XL710_SET_MAC                 = G_ME_HLD(MEF_ETHERNET,TEF_DRIVER,1,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DRIVER_XL710_GET_MAC                 = G_ME_HLD(MEF_ETHERNET,TEF_DRIVER,2,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DRIVER_XL710_SET_EEPROM              = G_ME_HLD(MEF_ETHERNET,TEF_DRIVER,3,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DRIVER_XL710_GET_EEPROM              = G_ME_HLD(MEF_ETHERNET,TEF_DRIVER,4,ERROR_CODE_MODULE),
    ERROR_ETHERNET_DRIVER_SET_MAC_FAIL                  = G_ME_HLD(MEF_ETHERNET,TEF_DRIVER,5,ERROR_CODE_MODULE),


    //--------------------- PCI ---------------------//
    //PCI_DAPI
    ERROR_PCI_DAPI_ERRORCODE                            = G_ME_HLD(MEF_PCI,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_READ_CONFIG_DATA_BYTE,
    ERROR_PCI_DAPI_READ_CONFIG_DATA_WORD,
    ERROR_PCI_DAPI_READ_CONFIG_DATA_DWORD,
    ERROR_PCI_DAPI_LINK_SPEED_NOT_MATCH,
    ERROR_PCI_DAPI_LINK_WIDTH_NOT_MATCH,
    //ERROR_PCI_DAPI_NO_DEVICE_SUPPORT_RETRAIN,
    ERROR_PCI_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_PCI,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_READ_CONFIG_DATA                     = G_ME_HLD(MEF_PCI,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_DEVICE_LIST                          = G_ME_HLD(MEF_PCI,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RETRAIN                              = G_ME_HLD(MEF_PCI,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RETRAIN_ONE_DEVICE                   = G_ME_HLD(MEF_PCI,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RETRAIN_CAP_CHECK                    = G_ME_HLD(MEF_PCI,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RETRAIN_LOOP                         = G_ME_HLD(MEF_PCI,TEF_DAPI,7,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_ERROR_REPORT                         = G_ME_HLD(MEF_PCI,TEF_DAPI,8,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_AER_REPORT                           = G_ME_HLD(MEF_PCI,TEF_DAPI,9,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RESET_STATISTICS                     = G_ME_HLD(MEF_PCI,TEF_DAPI,10,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RESET_STATISTICS_RUN                 = G_ME_HLD(MEF_PCI,TEF_DAPI,11,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RESET_STATISTICS_BRI                 = G_ME_HLD(MEF_PCI,TEF_DAPI,12,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RESET_STATISTICS_PCIE_CAP            = G_ME_HLD(MEF_PCI,TEF_DAPI,13,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RESET_STATISTICS_AER                 = G_ME_HLD(MEF_PCI,TEF_DAPI,14,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RESET_BRIDGE_ERR                     = G_ME_HLD(MEF_PCI,TEF_DAPI,15,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RESET_COM_ERR                        = G_ME_HLD(MEF_PCI,TEF_DAPI,16,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RESET_PCIE_CAP_ERR                   = G_ME_HLD(MEF_PCI,TEF_DAPI,17,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_RESET_AER                            = G_ME_HLD(MEF_PCI,TEF_DAPI,18,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_GET_DEVICE_INFO_BY_CLASS             = G_ME_HLD(MEF_PCI,TEF_DAPI,19,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_DEV_SUPPORT_RETRAIN                  = G_ME_HLD(MEF_PCI,TEF_DAPI,20,ERROR_CODE_MODULE),
//PCIE switch eeprom
    ERROR_PCI_DAPI_READ_EEPROM                          = G_ME_HLD(MEF_PCI,TEF_DAPI,21,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_GET_EEPROM_BY_ID                     = G_ME_HLD(MEF_PCI,TEF_DAPI,22,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_GET_EEPROM_LIST                      = G_ME_HLD(MEF_PCI,TEF_DAPI,23,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_GET_EEPROM_SIZE                      = G_ME_HLD(MEF_PCI,TEF_DAPI,24,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_WRITE_EEPROM                         = G_ME_HLD(MEF_PCI,TEF_DAPI,25,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_EEPROM_TEST                          = G_ME_HLD(MEF_PCI,TEF_DAPI,26,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_EEPROM_DUMP                          = G_ME_HLD(MEF_PCI,TEF_DAPI,27,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_SAVE_EEPROM                          = G_ME_HLD(MEF_PCI,TEF_DAPI,28,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_PROGRAM_EEPROM                       = G_ME_HLD(MEF_PCI,TEF_DAPI,29,ERROR_CODE_MODULE),

    ERROR_PCI_DAPI_TEST_SPEED                           = G_ME_HLD(MEF_PCI,TEF_DAPI,30,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_TEST_WIDTH                           = G_ME_HLD(MEF_PCI,TEF_DAPI,31,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_GET_CLASS                            = G_ME_HLD(MEF_PCI,TEF_DAPI,32,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_SCAN_DEVICE                          = G_ME_HLD(MEF_PCI,TEF_DAPI,33,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_SHOW_ERROR                           = G_ME_HLD(MEF_PCI,TEF_DAPI,34,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_SHOW_EEPROM_FW_VER                   = G_ME_HLD(MEF_PCI,TEF_DAPI,35,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_LIST_SWITCH                          = G_ME_HLD(MEF_PCI,TEF_DAPI,36,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_GET_SWITCH_LIST                      = G_ME_HLD(MEF_PCI,TEF_DAPI,37,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_SHOW_SW_ERROR                        = G_ME_HLD(MEF_PCI,TEF_DAPI,38,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_GET_SW_PORT_INFO                     = G_ME_HLD(MEF_PCI,TEF_DAPI,39,ERROR_CODE_MODULE),
    ERROR_PCI_DAPI_DISPLAY_HBAS                         = G_ME_HLD(MEF_PCI,TEF_DAPI,40,ERROR_CODE_MODULE),
    
    //PCI_DDAPI
    ERROR_PCI_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_PCI,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_INVALID_CLASS_CODE,
    ERROR_PCI_DDAPI_WRITE_CONFIG,
    ERROR_PCI_DDAPI_CAPABILITY_UNSUPPORT,
    ERROR_PCI_DDAPI_ERROR_TYPE_NOT_FOUND,
    ERROR_PCI_DDAPI_NO_DEVICE_FOUND,
    ERROR_PCI_DDAPI_COMPARE_DATA,
    ERROR_PCI_DDAPI_RETRAIN_UNSUPPORT,
    ERROR_PCI_DDAPI_UNEXPECTED_DEVICE_COUNT,
    ERROR_PCI_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_PCI,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_RETRAIN                             = G_ME_HLD(MEF_PCI,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_RETRAIN_EXECUTE                     = G_ME_HLD(MEF_PCI,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_RETRAIN_RESULT_CHECK                = G_ME_HLD(MEF_PCI,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_AER_REPORT                          = G_ME_HLD(MEF_PCI,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_PCIE_CAP_ERR                        = G_ME_HLD(MEF_PCI,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_RESET_COM_ERR                       = G_ME_HLD(MEF_PCI,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_RESET_BRI_ERR                       = G_ME_HLD(MEF_PCI,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_RESET_PCIE_CAP_ERR                  = G_ME_HLD(MEF_PCI,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_RESET_AER_ERR                       = G_ME_HLD(MEF_PCI,TEF_DDAPI,10,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_SEND_COMMAND                 = G_ME_HLD(MEF_PCI,TEF_DDAPI,11,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_GET_EEPROM_DATA_SIZE                = G_ME_HLD(MEF_PCI,TEF_DDAPI,12,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_WAIT_EEPROM_IDLE                    = G_ME_HLD(MEF_PCI,TEF_DDAPI,13,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_GET_EEPROM_DATA_SIZE_CMD            = G_ME_HLD(MEF_PCI,TEF_DDAPI,14,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_READ_EEPROM                         = G_ME_HLD(MEF_PCI,TEF_DDAPI,15,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPRON_NOT_PRESENT                  = G_ME_HLD(MEF_PCI,TEF_DDAPI,16,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_PROGRAM                      = G_ME_HLD(MEF_PCI,TEF_DDAPI,17,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_SAVE                         = G_ME_HLD(MEF_PCI,TEF_DDAPI,18,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_READ                         = G_ME_HLD(MEF_PCI,TEF_DDAPI,19,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_READ_BLOCK                   = G_ME_HLD(MEF_PCI,TEF_DDAPI,20,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_TEST                         = G_ME_HLD(MEF_PCI,TEF_DDAPI,21,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_WAIT_IDLE                    = G_ME_HLD(MEF_PCI,TEF_DDAPI,22,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_GET_DEVICE                   = G_ME_HLD(MEF_PCI,TEF_DDAPI,23,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_GET_SIZE                     = G_ME_HLD(MEF_PCI,TEF_DDAPI,24,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_WRITE                        = G_ME_HLD(MEF_PCI,TEF_DDAPI,25,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_WRITE_BLOCK                  = G_ME_HLD(MEF_PCI,TEF_DDAPI,26,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_EEPROM_RESTORE                      = G_ME_HLD(MEF_PCI,TEF_DDAPI,27,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_TEST_SPEED                          = G_ME_HLD(MEF_PCI,TEF_DDAPI,28,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_TEST_WIDTH                          = G_ME_HLD(MEF_PCI,TEF_DDAPI,29,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_GET_BAR                             = G_ME_HLD(MEF_PCI,TEF_DDAPI,30,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_GET_PCI_INFO                        = G_ME_HLD(MEF_PCI,TEF_DDAPI,31,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_SCAN_DEVICE                         = G_ME_HLD(MEF_PCI,TEF_DDAPI,32,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_SHOW_EEPROM_FW_VER                  = G_ME_HLD(MEF_PCI,TEF_DDAPI,33,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_GET_NTB                             = G_ME_HLD(MEF_PCI,TEF_DDAPI,34,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_GET_SWITCH_LIST                     = G_ME_HLD(MEF_PCI,TEF_DDAPI,35,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_GET_NTB_ERROR                       = G_ME_HLD(MEF_PCI,TEF_DDAPI,36,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_GET_PORT_ERROR_LIST                 = G_ME_HLD(MEF_PCI,TEF_DDAPI,37,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_SCAN_SLOT                           = G_ME_HLD(MEF_PCI,TEF_DDAPI,38,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_BUILD_TREE                          = G_ME_HLD(MEF_PCI,TEF_DDAPI,39,ERROR_CODE_MODULE),
    ERROR_PCI_DDAPI_GET_EEPROM_LIST                     = G_ME_HLD(MEF_PCI,TEF_DDAPI,40,ERROR_CODE_MODULE),


    //--------------------- UART ---------------------//
    //UART DAPI
    ERROR_UART_DAPI_ERRORCODE                           = G_ME_HLD(MEF_UART, TEF_DAPI, 0, ERROR_CODE_MODULE),
    ERROR_UART_DAPI_DEVICE_TYPE_UNKNOWN,
    ERROR_UART_DAPI_DEVICE_LIST                         = G_ME_HLD(MEF_UART, TEF_DAPI, 1, ERROR_CODE_MODULE),
    ERROR_UART_DAPI_SCRATCHPAD_TEST                     = G_ME_HLD(MEF_UART, TEF_DAPI, 2, ERROR_CODE_MODULE),
    ERROR_UART_DAPI_INTERNAL_LOOPBACK_TEST              = G_ME_HLD(MEF_UART, TEF_DAPI, 3, ERROR_CODE_MODULE),
    ERROR_UART_DAPI_GET_INFO                            = G_ME_HLD(MEF_UART, TEF_DAPI, 4, ERROR_CODE_MODULE),
    ERROR_UART_DAPI_GET_LIST                            = G_ME_HLD(MEF_UART, TEF_DAPI, 5, ERROR_CODE_MODULE),

        

    //UART DDAPI
    ERROR_UART_DDAPI_ERRORCODE                          = G_ME_HLD(MEF_UART, TEF_DDAPI, 0, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_FUNCTION_UNSUPPORTED,
    ERROR_UART_DDAPI_TIMEOUT,
    ERROR_UART_DDAPI_DATA_NOT_READY,
    ERROR_UART_DDAPI_IOCTL_GET_SERIAL_INFO,
    ERROR_UART_DDAPI_DATA_NOT_MATCH,
    ERROR_UART_DDAPI_SCRATCHPAD_TEST                    = G_ME_HLD(MEF_UART, TEF_DDAPI, 1, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_SCRATCHPAD_DEFAULT_RUN             = G_ME_HLD(MEF_UART, TEF_DDAPI, 2, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_LOOPBACK                           = G_ME_HLD(MEF_UART, TEF_DDAPI, 3, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_INTERNAL_LOOPBACK_RUN              = G_ME_HLD(MEF_UART, TEF_DDAPI, 4, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_INTERNAL_LOOPBACK_DEFAULT          = G_ME_HLD(MEF_UART, TEF_DDAPI, 5, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_INTERNAL_LOOPBACK_DEFAULT_POLLING  = G_ME_HLD(MEF_UART, TEF_DDAPI, 6, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_SCAN_UARTS                         = G_ME_HLD(MEF_UART, TEF_DDAPI, 7, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_GET_DEVICE                         = G_ME_HLD(MEF_UART, TEF_DDAPI, 8, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_READ_REGISTER                      = G_ME_HLD(MEF_UART, TEF_DDAPI, 9, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_WRITE_REGISTER                     = G_ME_HLD(MEF_UART, TEF_DDAPI, 10, ERROR_CODE_MODULE),
    ERROR_UART_DDAPI_SEARCH_DEVICE                      = G_ME_HLD(MEF_UART, TEF_DDAPI, 11, ERROR_CODE_MODULE),



    //--------------------- Storage ---------------------//
    //StorageDAPI
/*    ERROR_STORAGE_DAPI_ERRORCODE                        = G_ME_HLD(MEF_STORAGE,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_ILLEGAL_START_LOCATION,
    ERROR_STORAGE_DAPI_ILLEGAL_TEST_SIZE,
    ERROR_STORAGE_DAPI_DATA_COMPARE_MISMATCH,
    ERROR_STORAGE_DAPI_BUFFER_TOO_SMALL,
    ERROR_STORAGE_DAPI_UNDEFINED_BLOCK_SIZE,
    ERROR_STORAGE_DAPI_LOOPBACK_PATH_ERROR,
    ERROR_STORAGE_DAPI_LINK_SPEED_ERROR,
    ERROR_STORAGE_DAPI_UNDEFINED_CONTROLLER_TYPE,
    ERROR_STORAGE_DAPI_CONSTRUCTOR                      = G_ME_HLD(MEF_STORAGE,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_READ_WRITE_TEST                  = G_ME_HLD(MEF_STORAGE,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_READ_PERFORMANCE_TEST            = G_ME_HLD(MEF_STORAGE,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_WRITE_PERFORMANCE_TEST           = G_ME_HLD(MEF_STORAGE,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_DISPLAY_STORAGE_DEVICE_ON_HOST   = G_ME_HLD(MEF_STORAGE,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_READ_FROM_DEVICE                 = G_ME_HLD(MEF_STORAGE,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_WRITE_TO_DEVICE                  = G_ME_HLD(MEF_STORAGE,TEF_DAPI,7,ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_DISPLAY_BY_CONTROLER_TYPE        = G_ME_HLD(MEF_STORAGE,TEF_DAPI,8,ERROR_CODE_MODULE),

    ERROR_STORAGE_DDAPI_ERRORCODE                       = G_ME_HLD(MEF_STORAGE,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_ILLEGAL_ID_REQUEST,
    ERROR_STORAGE_DDAPI_INVALID_OP_CODE,
    ERROR_STORAGE_DDAPI_FILE_OPEN_FAILED,
    ERROR_STORAGE_DDAPI_UNSUPPORTED_IO_CONTROLLER,
    ERROR_STORAGE_DDAPI_READSCSI                        = G_ME_HLD(MEF_STORAGE,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_WRITESCSI                       = G_ME_HLD(MEF_STORAGE,TEF_DDAPI,2,ERROR_CODE_MODULE),

    ERROR_STORAGE_DRIVER_ERRORCODE                      = G_ME_HLD(MEF_STORAGE,TEF_DRIVER,0,ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_IOCTL_FAILED,
*/

    //--------------------- STORAGE ---------------------//
    // STORAGE DAPI error code definition
    ERROR_STORAGE_DAPI_ERRORCODE                           = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 0, ERROR_CODE_MODULE),

    ERROR_STORAGE_DAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 1, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_IOCTL                               = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 2, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_GENERATE_STORAGE_LIST               = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 3, ERROR_CODE_MODULE),

    ERROR_STORAGE_DAPI_READ_BY_SCSI                        = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 4, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_WRITE_BY_SCSI                       = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 5, ERROR_CODE_MODULE),

    ERROR_STORAGE_DAPI_GET_DEFINED_INDEX_BY_NAME           = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 6, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_GET_ELEMENT_DATA_BY_INDEX           = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 7, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_GET_SMART_BY_INDEX                  = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 8, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_GET_ERROR_REGISTER_BY_INDEX         = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 9, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_GET_ELEMENT_DATA_BY_NAME            = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 10, ERROR_CODE_MODULE),


    ERROR_STORAGE_DAPI_IDENTIFY_COMMAND_TEST               = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 11, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_READ_PERFORMANCE_TEST               = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 12, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_WRITE_PERFORMANCE_TEST              = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 13, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_READ_WRITE_TEST                     = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 14, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_SET_SECTOR_COUNT_PER_RDWR_ACCESS    = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 15, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_WRITE_ZERO_BY_INDEX                 = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 16, ERROR_CODE_MODULE),
    ERROR_STORAGE_DAPI_GET_PGLIST_BY_INDEX                 = G_ME_HLD(MEF_STORAGE, TEF_DAPI, 17, ERROR_CODE_MODULE),

    ERROR_STORAGE_DAPI_NO_SUCH_IOCTL_OPERATION,
    ERROR_STORAGE_DAPI_COMPARE_DATA_ERROR,
    ERROR_STORAGE_DAPI_OPEN_DEVICE_FAIL,
    ERROR_STORAGE_DAPI_CLOSE_DEVICE_FAIL,
    ERROR_STORAGE_DAPI_UNKNOWN_CONTROLLER_TYPE,
    ERROR_STORAGE_DAPI_DUMP_HEX_TYPE_WRONG,
    ERROR_STORAGE_DAPI_DEVICE_NOT_SUPPORT_SMART,

    // STORAGE DDAPI error code definition
    ERROR_STORAGE_DDAPI_ERRORCODE                          = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 0, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_WRONG_DEVICE_NAME_FORMAT,
    ERROR_STORAGE_DDAPI_NO_SUCH_IOCTL_OPERATION,
    ERROR_STORAGE_DDAPI_OPEN_MMAP_FAIL,
    ERROR_STORAGE_DDAPI_MMAP_FAIL,
    ERROR_STORAGE_DDAPI_MUNMAP_FAIL,

    ERROR_STORAGE_DDAPI_OPEN_DEVICE_FAIL,
    ERROR_STORAGE_DDAPI_CLOSE_DEVICE_FAIL,

    ERROR_STORAGE_DDAPI_OPEN_PIPE_FAIL,
    ERROR_STORAGE_DDAPI_OPEN_DIR_FAIL,
    ERROR_STORAGE_DDAPI_READ_LINK_ERROR,
    ERROR_STORAGE_DDAPI_DUMP_SMART_INFO_ERROR,
    ERROR_STORAGE_DDAPI_NO_ANY_STORAGE_CONTROLLER,

    ERROR_STORAGE_DDAPI_DEFINED_INDEX_NOT_FOUND,
    ERROR_STORAGE_DDAPI_NO_SUCH_ERROR_ELEMENT,
    ERROR_STORAGE_DDAPI_NO_SUCH_DATA_ELEMENT,
    ERROR_STORAGE_DDAPI_WRONG_DEFINED_INDEX,
    ERROR_STORAGE_DDAPI_BLOCKSIZE_IS_NOT_MULTIPLE_OF_512,
    ERROR_STORAGE_DDAPI_SCSICDBSEND_IOCTL_FAIL,
    ERROR_STORAGE_DDAPI_SCSICDBSEND_SENSE_ERROR,
    ERROR_STORAGE_DDAPI_LINK_STATUS_UNKNOWN,
    ERROR_STORAGE_DDAPI_LINK_SPEED_UNKNOWN,
    ERROR_STORAGE_DDAPI_NO_SUCH_DUMP_HEX_FORMAT,
    ERROR_STORAGE_DDAPI_NO_SUCH_CONTROLLER_TYPE,


    ERROR_STORAGE_DDAPI_CONSTRUCTOR                        = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 1, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_IOCTL                              = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 2, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_GENERATE_STORAGE_LIST              = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 3, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_INIT_CONTROLLER_INFO               = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 4, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_INIT_PORT_INFO                     = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 5, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_LIST_DRIVE                         = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 6, ERROR_CODE_MODULE),

    ERROR_STORAGE_DDAPI_FLUSH_BUFFER_CACHE                 = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 7, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_IDENTIFY_COMMAND_TEST              = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 8, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_GET_DEFINED_INDEX_BY_NAME          = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 9, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_INDEX          = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 10, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_GET_ERROR_REGISTER_BY_INDEX        = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 11, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_GET_SMART_BY_INDEX                 = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 12, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_NAME           = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 13, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_READ_CAPACITY_16                   = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 14, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_SAT_IDENTIFY_16                    = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 15, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_PREAD                              = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 16, ERROR_CODE_MODULE),
    ERROR_STORAGE_DDAPI_PWRITE                             = G_ME_HLD(MEF_STORAGE, TEF_DDAPI, 17, ERROR_CODE_MODULE),

    // STORAGE Driver error code definition
    ERROR_STORAGE_DRIVER_ERRORCODE                         = G_ME_HLD(MEF_STORAGE, TEF_DRIVER, 0, ERROR_CODE_MODULE),


    //--------------------- RTC ---------------------//
    // RTC Diagnostic API error code definition
    ERROR_RTC_DAPI_ERRORCODE                            = G_ME_HLD(MEF_RTC,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_RTC_DAPI_RTC_UNEXPECTED_TEST_RESULT,
    ERROR_RTC_DAPI_DATA_OUT_OF_RANGE,
    ERROR_RTC_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_RTC,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_RTC_DAPI_SHOW_DATE_TIME                       = G_ME_HLD(MEF_RTC,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_RTC_DAPI_SET_DATE                             = G_ME_HLD(MEF_RTC,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_RTC_DAPI_SET_TIME                             = G_ME_HLD(MEF_RTC,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_RTC_DAPI_RTC_ACCURACY_TEST                    = G_ME_HLD(MEF_RTC,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_RTC_DAPI_RTC_WDT_TEST                         = G_ME_HLD(MEF_RTC,TEF_DAPI,6,ERROR_CODE_MODULE),
    
    // RTC Diagnostic Driver API error code definition
    ERROR_RTC_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_RTC,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_RTC_DDAPI_RTC_DEVICE_OPEN_FAIL,
    ERROR_RTC_DDAPI_ALTACC_DEVICE_OPEN_FAIL,
    ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL,
    ERROR_RTC_DDAPI_ALTACC_IOCTL_ACCESS_FAIL,
    ERROR_RTC_DDAPI_PORT_PERMISSION,
    ERROR_RTC_DDAPI_BATTERY_CHARGED_FAIL,
    ERROR_RTC_DDAPI_UNKNOWN_DATA_ELEMENT,
    ERROR_RTC_DDAPI_RTC_DEVICE_READ_FAIL,
    ERROR_RTC_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_RTC,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_RTC_DDAPI_SHOW_DATE_TIME                      = G_ME_HLD(MEF_RTC,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_RTC_DDAPI_SET_RTC_TIMER_RATE                  = G_ME_HLD(MEF_RTC,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_RTC_DDAPI_START_RTC_TIMER                     = G_ME_HLD(MEF_RTC,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_RTC_DDAPI_READ_RTC_TIMER_STATUS               = G_ME_HLD(MEF_RTC,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_RTC_DDAPI_SET_DATE                            = G_ME_HLD(MEF_RTC,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_RTC_DDAPI_SET_TIME                            = G_ME_HLD(MEF_RTC,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_RTC_DDAPI_RTC_WDT_TEST                        = G_ME_HLD(MEF_RTC,TEF_DDAPI,8,ERROR_CODE_MODULE),
    //--------------------- PSU ---------------------//
    // PSU Diagnostic API error code definition
    ERROR_PSU_DAPI_ERRORCODE                            = G_ME_HLD(MEF_PSU,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_PSU_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_PSU,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_PSU_DAPI_GET_DEVICE_LIST                      = G_ME_HLD(MEF_PSU,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_PSU_DAPI_GET_DATA_BY_ELEMENT                  = G_ME_HLD(MEF_PSU,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_PSU_DAPI_GET_SUPPORT_VALUE_BY_ELEMENT         = G_ME_HLD(MEF_PSU,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_PSU_DAPI_SET_DATA_BY_ELEMENT                  = G_ME_HLD(MEF_PSU,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_PSU_DAPI_UPDATE_FW_BY_FILE                    = G_ME_HLD(MEF_PSU,TEF_DAPI,6,ERROR_CODE_MODULE),

    // PSU Diagnostic Driver API error code definition
    ERROR_PSU_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_PSU,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_UNSUPPORTED_FAN_SPEED,
    ERROR_PSU_DDAPI_UNKNOWN_LED_STATUS,
    ERROR_PSU_DDAPI_PROGRAM_DATA_FAIL,
    ERROR_PSU_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_PSU,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_INITIALIZEPSU_INFO_FILE             = G_ME_HLD(MEF_PSU,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_IS_PRESENT                          = G_ME_HLD(MEF_PSU,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_DEVICE_LIST                     = G_ME_HLD(MEF_PSU,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_VENDOR_NAME                     = G_ME_HLD(MEF_PSU,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_FW_VERSION                      = G_ME_HLD(MEF_PSU,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_PART_NUMBER                     = G_ME_HLD(MEF_PSU,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_SERIAL_NUMBER                   = G_ME_HLD(MEF_PSU,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_MAX_POWER                       = G_ME_HLD(MEF_PSU,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_FAN_LIST                        = G_ME_HLD(MEF_PSU,TEF_DDAPI,10,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_FAN_SPEED                       = G_ME_HLD(MEF_PSU,TEF_DDAPI,11,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_SUPPORT_FAN_SPEED               = G_ME_HLD(MEF_PSU,TEF_DDAPI,12,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_SET_FAN_SPEED                       = G_ME_HLD(MEF_PSU,TEF_DDAPI,13,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_TEMPERATURE_LIST                = G_ME_HLD(MEF_PSU,TEF_DDAPI,14,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_TEMPERATURE                     = G_ME_HLD(MEF_PSU,TEF_DDAPI,15,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD_LIST      = G_ME_HLD(MEF_PSU,TEF_DDAPI,16,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD           = G_ME_HLD(MEF_PSU,TEF_DDAPI,17,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_SET_TEMPERATURE_THRESHOLD           = G_ME_HLD(MEF_PSU,TEF_DDAPI,18,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_LED_LIST                        = G_ME_HLD(MEF_PSU,TEF_DDAPI,19,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_LED_STATUS                      = G_ME_HLD(MEF_PSU,TEF_DDAPI,20,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_SET_LED_STATUS                      = G_ME_HLD(MEF_PSU,TEF_DDAPI,21,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_GET_TYPE                            = G_ME_HLD(MEF_PSU,TEF_DDAPI,22,ERROR_CODE_MODULE),
    ERROR_PSU_DDAPI_PROGRAM                             = G_ME_HLD(MEF_PSU,TEF_DDAPI,23,ERROR_CODE_MODULE),

    //--------------------- LED ---------------------//
    // LED Diagnostic API error code definition
    ERROR_LED_DAPI_ERRORCODE                            = G_ME_HLD(MEF_LED,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_LED_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_LED,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_LED_DAPI_GET_DEVICE_LIST                      = G_ME_HLD(MEF_LED,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_LED_DAPI_GET_LED_STATUS                       = G_ME_HLD(MEF_LED,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_LED_DAPI_SET_LED_STATUS                       = G_ME_HLD(MEF_LED,TEF_DAPI,4,ERROR_CODE_MODULE),

    // LED Diagnostic Driver API error code definition
    ERROR_LED_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_LED,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_LED_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_LED,TEF_DDAPI,1,ERROR_CODE_MODULE),

    // LED Driver error code definition
    ERROR_LED_DRIVER_ERRORCODE                          = G_ME_HLD(MEF_LED, TEF_DRIVER, 0, ERROR_CODE_MODULE),

    //--------------------- TEMPERATURE ---------------------//
    // TEMPERATURE Diagnostic API error code definition
    ERROR_TEMPERATURE_DAPI_ERRORCODE                            = G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 0, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DAPI_OPEN_PROFILE_FAIL,
    ERROR_TEMPERATURE_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 1, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE              = G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 2, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DAPI_GET_THRESHOLD_HIGHT                  = G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 3, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DAPI_GET_THRESHOLD_LOW                    = G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 4, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DAPI_SET_THRESHOLD_HIGHT                  = G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 5, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DAPI_SET_THRESHOLD_LOW                    = G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 6, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_DAPI_GET_DATA_BY_NAME                     = G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 7, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_DAPI_SET_DATA_BY_NAME						= G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 8, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_DAPI_DISPLAY_TEMPERATURE_LIST				= G_ME_HLD(MEF_TEMPERATURE, TEF_DAPI, 9, ERROR_CODE_MODULE),

    // TEMPERATURE Diagnostic Driver API error code definition
    ERROR_TEMPERATURE_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 0, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 1, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_GET_NAME                            = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 2, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_GET_CURRENT_TEMPERATURE             = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 3, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_GET_THRESHOLD_HIGHT                 = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 4, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_GET_THRESHOLD_LOW                   = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 5, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_GET_TYPE                            = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 5, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_SET_NAME                            = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 6, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_SET_THRESHOLD_HIGHT                 = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 7, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_SET_THRESHOLD_LOW                   = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 8, ERROR_CODE_MODULE),
    ERROR_TEMPERATURE_DDAPI_SET_TYPE                            = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 9, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_DDAPI_GET_DATA_BY_NAME                    = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 10, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_DDAPI_GET_TEMPERATURE_NAME_LIST			= G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 11, ERROR_CODE_MODULE),

	ERROR_TEMPERATURE_I2C_DDAPI_GET_TEMPERATURE_NAME_LIST		= G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 12, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_GET_CURRENT_TEMPERATURE			= G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 13, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_GET_DATA_BY_NAME                = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 14, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_SET_DATA_BY_NAME                = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 15, ERROR_CODE_MODULE),

	ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_HIGH0_LIST		= G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 16, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_LOW0_LIST		    = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 17, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_HIGH1_LIST		= G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 18, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_LOW1_LIST		    = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 19, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_HIGH0_LIST		= G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 20, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_LOW0_LIST		    = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 21, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_HIGH1_LIST		= G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 22, ERROR_CODE_MODULE),
	ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_LOW1_LIST		    = G_ME_HLD(MEF_TEMPERATURE, TEF_DDAPI, 23, ERROR_CODE_MODULE),



    // TEMPERATURE Driver error code definition
    ERROR_TEMPERATURE_DRIVER_ERRORCODE                          = G_ME_HLD(MEF_TEMPERATURE, TEF_DRIVER, 0, ERROR_CODE_MODULE),

    //--------------------- I2C ---------------------//
    // I2C Diagnostic API error code definition
    ERROR_I2C_DAPI_ERRORCODE                            = G_ME_HLD(MEF_I2C,TEF_DAPI,0,ERROR_CODE_MODULE),

    ERROR_I2C_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_I2C,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_I2C_DAPI_DISPLAY_I2C_DEVICES_LIST             = G_ME_HLD(MEF_I2C,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_I2C_DAPI_READ                                 = G_ME_HLD(MEF_I2C,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_I2C_DAPI_WRITE                                = G_ME_HLD(MEF_I2C,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_I2C_DAPI_DISPLAY_I2C_DEVICES_LIST_BY_TYPE     = G_ME_HLD(MEF_I2C,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_I2C_DAPI_MUX_TEST                             = G_ME_HLD(MEF_I2C,TEF_DAPI,6,ERROR_CODE_MODULE),

    // I2C Diagnostic Driver API error code definition
    ERROR_I2C_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_I2C,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_I2C_DDAPI_OPEN_NODE_FAIL,
    ERROR_I2C_DDAPI_OPEN_PROFILE_FAIL,
    ERROR_I2C_DDAPI_NO_DEVICE,
    ERROR_I2C_DDAPI_SET_SLAVE_ADDRESS_FAIL,
    ERROR_I2C_DDAPI_READ_BYTE_FAIL,
    ERROR_I2C_DDAPI_READ_BYTE_DATA_FAIL,
    ERROR_I2C_DDAPI_READ_WORD_DATA_FAIL,
    ERROR_I2C_DDAPI_READ_BLOCK_DATA_FAIL,
    ERROR_I2C_DDAPI_READ_I2C_BLOCK_DATA_FAIL,
    ERROR_I2C_DDAPI_WRITE_BYTE_FAIL,
    ERROR_I2C_DDAPI_WRITE_BYTE_DATA_FAIL,
    ERROR_I2C_DDAPI_WRITE_WORD_DATA_FAIL,
    ERROR_I2C_DDAPI_WRITE_BLOCK_DATA_FAIL,
    ERROR_I2C_DDAPI_WRITE_I2C_BLOCK_DATA_FAIL,
    ERROR_I2C_DDAPI_BUS_NULL,
    ERROR_I2C_DDAPI_BUS_NOT_FOUND,
    ERROR_I2C_DDAPI_ADDRESS_NOT_FOUND,
    ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER,
    ERROR_I2C_DDAPI_TYPE_UN_MATCH,
    ERROR_I2C_DDAPI_MUX_TEST_FAIL,
    ERROR_I2C_DDAPI_I2CBUS_READ                         = G_ME_HLD(MEF_I2C,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_I2C_DDAPI_I2CBUS_WRITE                        = G_ME_HLD(MEF_I2C,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_I2C_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_I2C,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_I2C_DDAPI_READ                                = G_ME_HLD(MEF_I2C,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_I2C_DDAPI_WRITE                               = G_ME_HLD(MEF_I2C,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_I2C_DDAPI_INITIAL_I2C_DEVICE                  = G_ME_HLD(MEF_I2C,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_I2C_DDAPI_INITIAL_I2C_BUS                     = G_ME_HLD(MEF_I2C,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_I2C_DDAPI_INITIAL_I2C_PROTOCOL                = G_ME_HLD(MEF_I2C,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_I2C_DDAPI_MUX_TEST                            = G_ME_HLD(MEF_I2C,TEF_DDAPI,9,ERROR_CODE_MODULE),

    // LED Driver error code definition
    ERROR_I2C_DRIVER_ERRORCODE                          = G_ME_HLD(MEF_I2C, TEF_DRIVER, 0, ERROR_CODE_MODULE),

    // SPD Diagnostic Driver API error code definition
    ERROR_SPD_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_SPD,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_NO_SPD_DEVICE,
    ERROR_SPD_DDAPI_UNKNOWN_VENDOR,
    ERROR_SPD_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_SPD,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_DEVICE_LIST                     = G_ME_HLD(MEF_SPD,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_DEVICE_TYPE                     = G_ME_HLD(MEF_SPD,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_MODULE_TYPE                     = G_ME_HLD(MEF_SPD,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_SIZE                            = G_ME_HLD(MEF_SPD,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_SPEED                           = G_ME_HLD(MEF_SPD,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_RANK_COUNT                      = G_ME_HLD(MEF_SPD,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_PART_NUMBER                     = G_ME_HLD(MEF_SPD,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_SERIAL_NUMBER                   = G_ME_HLD(MEF_SPD,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_DRAM_VENDOR                     = G_ME_HLD(MEF_SPD,TEF_DDAPI,10,ERROR_CODE_MODULE),
    ERROR_SPD_DDAPI_GET_MODULE_VENDOR                   = G_ME_HLD(MEF_SPD,TEF_DDAPI,11,ERROR_CODE_MODULE),

    //--------------------- NVSRAM ---------------------//
    // NVSRAM Diagnostic API error code definition
    ERROR_NVSRAM_DAPI_ERRORCODE                         = G_ME_HLD(MEF_NVSRAM, TEF_DAPI, 0, ERROR_CODE_MODULE),
    ERROR_NVSRAM_DAPI_NO_NVSRAM_DEVICE,
    ERROR_NVSRAM_DAPI_CONSTRUCTOR                       = G_ME_HLD(MEF_NVSRAM, TEF_DAPI, 1, ERROR_CODE_MODULE),
    ERROR_NVSRAM_DAPI_GET_INFORMATION                   = G_ME_HLD(MEF_NVSRAM, TEF_DAPI, 2, ERROR_CODE_MODULE),
    ERROR_NVSRAM_DAPI_DATA_LINE_TEST                    = G_ME_HLD(MEF_NVSRAM, TEF_DAPI, 3, ERROR_CODE_MODULE),
    ERROR_NVSRAM_DAPI_ADDRESS_LINE_TEST                 = G_ME_HLD(MEF_NVSRAM, TEF_DAPI, 4, ERROR_CODE_MODULE),
    ERROR_NVSRAM_DAPI_NVRAM_TEST                        = G_ME_HLD(MEF_NVSRAM, TEF_DAPI, 5, ERROR_CODE_MODULE),

    // NVSRAM Diagnostic Driver API error code definition
    ERROR_NVSRAM_DDAPI_ERRORCODE                        = G_ME_HLD(MEF_NVSRAM,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_NVSRAM_DDAPI_OPEN_MEM_DEVICE_FAIL,
    ERROR_NVSRAM_DDAPI_MMAP_FAIL,
	ERROR_NVSRAM_DDAPI_CONSTRUCTOR                      = G_ME_HLD(MEF_NVSRAM,TEF_DDAPI,1,ERROR_CODE_MODULE),
	ERROR_NVSRAM_DDAPI_GET_BASE_ADDRESS					= G_ME_HLD(MEF_NVSRAM,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_NVSRAM_DDAPI_GET_RESOURE_SIZE                 = G_ME_HLD(MEF_NVSRAM,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_NVSRAM_DDAPI_GET_ELEMENT_SIZE                 = G_ME_HLD(MEF_NVSRAM,TEF_DDAPI,4,ERROR_CODE_MODULE),
	ERROR_NVSRAM_DDAPI_GET_NVSRAM_LIST					= G_ME_HLD(MEF_NVSRAM,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_NVSRAM_DDAPI_RECALL_NVSRAM                    = G_ME_HLD(MEF_NVSRAM,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_NVSRAM_DDAPI_STORE_NVSRAM                     = G_ME_HLD(MEF_NVSRAM,TEF_DDAPI,7,ERROR_CODE_MODULE),

    // NVSRAM Driver error code definition
    ERROR_NVSRAM_DRIVER_ERRORCODE                       = G_ME_HLD(MEF_NVSRAM,TEF_DRIVER,0,ERROR_CODE_MODULE),

    //--------------------- TPM ---------------------//
    //DAPI
    ERROR_TPM_DAPI_ERRORCODE                            = G_ME_HLD(MEF_TPM,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_TPM_DAPI_ACCESS_CMD                           = G_ME_HLD(MEF_TPM,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_TPM_DAPI_SEND_CMD_MANUALLY                    = G_ME_HLD(MEF_TPM,TEF_DAPI,2,ERROR_CODE_MODULE),


    //DDAPI 
    ERROR_TPM_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_TPM,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_TPM_DDAPI_ACCESS_CMD                          = G_ME_HLD(MEF_TPM,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_TPM_DDAPI_SEND_CMD_MANUALLY                   = G_ME_HLD(MEF_TPM,TEF_DDAPI,2,ERROR_CODE_MODULE),

    //--------------------- NTB ---------------------//
    //DAPI 
    ERROR_NTB_DAPI_ERRORCODE                            = G_ME_HLD(MEF_NTB,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_NTB_DAPI_SEARCH_DEVICE                        = G_ME_HLD(MEF_NTB,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_NTB_DAPI_SEND                                 = G_ME_HLD(MEF_NTB,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_NTB_DAPI_RECEIVER                             = G_ME_HLD(MEF_NTB,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_NTB_DAPI_INIT                                 = G_ME_HLD(MEF_NTB,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_NTB_DAPI_SERVER                               = G_ME_HLD(MEF_NTB,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_NTB_DAPI_CLIENT                               = G_ME_HLD(MEF_NTB,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_NTB_DAPI_PERFORMANCE                          = G_ME_HLD(MEF_NTB,TEF_DAPI,7,ERROR_CODE_MODULE),
    
    //DDAPI 
    ERROR_NTB_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_NTB,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_TIMEOUT,
    ERROR_NTB_DDAPI_SEARCH_DEVICE                       = G_ME_HLD(MEF_NTB,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_SEND                                = G_ME_HLD(MEF_NTB,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_RECEIVER                            = G_ME_HLD(MEF_NTB,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_INIT                                = G_ME_HLD(MEF_NTB,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_GET_INFO                            = G_ME_HLD(MEF_NTB,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_GET_WINDOW_INFO                     = G_ME_HLD(MEF_NTB,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_SERVER                              = G_ME_HLD(MEF_NTB,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_CLIENT                              = G_ME_HLD(MEF_NTB,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_IO_SETTING                          = G_ME_HLD(MEF_NTB,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_RLS_BLOCK                           = G_ME_HLD(MEF_NTB,TEF_DDAPI,10,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_CREATE_BLOCK                        = G_ME_HLD(MEF_NTB,TEF_DDAPI,11,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_PERFORMANCE                         = G_ME_HLD(MEF_NTB,TEF_DDAPI,12,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_STRESS                              = G_ME_HLD(MEF_NTB,TEF_DDAPI,13,ERROR_CODE_MODULE),
    ERROR_NTB_DDAPI_RECEIVE_DATA                        = G_ME_HLD(MEF_NTB,TEF_DDAPI,14,ERROR_CODE_MODULE),



    
    //--------------------- Expander ---------------------//
    // Expander Diagnostic API error code definition
    ERROR_EXPANDER_DAPI_ERRORCODE                            = G_ME_HLD(MEF_EXPANDER,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_EXPANDER,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DAPI_LIST_DEVICE                          = G_ME_HLD(MEF_EXPANDER,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DAPI_GET_LOGICAL_ID_BY_NAME               = G_ME_HLD(MEF_EXPANDER,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DAPI_SHOW_DATA                            = G_ME_HLD(MEF_EXPANDER,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DAPI_RESET                                = G_ME_HLD(MEF_EXPANDER,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DAPI_SHOW_ERROR_DATA                      = G_ME_HLD(MEF_EXPANDER,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DAPI_CLEAR_PHY_ERROR                      = G_ME_HLD(MEF_EXPANDER,TEF_DAPI,7,ERROR_CODE_MODULE),
    
    // Expander Diagnostic Driver API error code definition
    ERROR_EXPANDER_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_EXPANDER,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DDAPI_POPEN_FAIL,
    ERROR_EXPANDER_DDAPI_UNKNOWN_NAME,
    ERROR_EXPANDER_DDAPI_UNKNOWN_ID,
    ERROR_EXPANDER_DDAPI_DIFF_PHY_SPEED,
    ERROR_EXPANDER_DDAPI_UNSUPPORT_DEV_TYPE,
    ERROR_EXPANDER_DDAPI_SYS_CMD_FAIL,
    ERROR_EXPANDER_DDAPI_SES_CMD_FAIL,
    ERROR_EXPANDER_DDAPI_SMP_CMD_FAIL,
    ERROR_EXPANDER_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_EXPANDER,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DDAPI_GET_ID_BY_NAME                      = G_ME_HLD(MEF_EXPANDER,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DDAPI_GET_DATA_BY_ID                      = G_ME_HLD(MEF_EXPANDER,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DDAPI_GET_ERROR_DATA_BY_ID                = G_ME_HLD(MEF_EXPANDER,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DDAPI_CLEAR_PHY_ERROR                     = G_ME_HLD(MEF_EXPANDER,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_EXPANDER_DDAPI_RESET      	                     = G_ME_HLD(MEF_EXPANDER,TEF_DDAPI,6,ERROR_CODE_MODULE),

    //--------------------- FPGA ---------------------//
    //FPGA Diagnostic API error code definition
    ERROR_FPGA_DAPI_ERRORCODE                           = G_ME_HLD(MEF_FPGA,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_FPGA_DAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_FPGA,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_FPGA_DAPI_GET_DEVICE_LIST                     = G_ME_HLD(MEF_FPGA,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_FPGA_DAPI_READ_REG                            = G_ME_HLD(MEF_FPGA,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_FPGA_DAPI_WRITE_REG                           = G_ME_HLD(MEF_FPGA,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_FPGA_DAPI_GET_SPAD_INFO                       = G_ME_HLD(MEF_FPGA,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_FPGA_DAPI_READ_SPAD                           = G_ME_HLD(MEF_FPGA,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_FPGA_DAPI_WRITE_SPAD                          = G_ME_HLD(MEF_FPGA,TEF_DAPI,7,ERROR_CODE_MODULE),

    //FPGA Diagnostic Driver API error code definition
    ERROR_FPGA_DDAPI_ERRORCODE                          = G_ME_HLD(MEF_FPGA,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_FPGA_DDAPI_UNKNOWN_REGISTER,
    ERROR_FPGA_DDAPI_UNKNOWN_BASE_ADDRESS,
    ERROR_FPGA_DDAPI_CONSTRUCTOR                        = G_ME_HLD(MEF_FPGA,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_FPGA_DDAPI_GET_DEVICE_LIST                    = G_ME_HLD(MEF_FPGA,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_FPGA_DDAPI_GET_REG_INFO                       = G_ME_HLD(MEF_FPGA,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_FPGA_DDAPI_READ                               = G_ME_HLD(MEF_FPGA,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_FPGA_DDAPI_WRITE                              = G_ME_HLD(MEF_FPGA,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_FPGA_DDAPI_GET_SPAD_INFO                      = G_ME_HLD(MEF_FPGA,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_FPGA_DDAPI_READ_SPAD                          = G_ME_HLD(MEF_FPGA,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_FPGA_DDAPI_WRITE_SPAD                         = G_ME_HLD(MEF_FPGA,TEF_DDAPI,8,ERROR_CODE_MODULE),

    //--------------------- NVME ---------------------//
    //NVME
    ERROR_NVME_DAPI_ERRORCODE                            = G_ME_HLD(MEF_NVME,TEF_DAPI,0,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_TEST_PATTERN_COMPARE_FAIL,
    ERROR_NVME_DAPI_DISPLAY_DATA,
    ERROR_NVME_DAPI_GET_MEMORY_BUFFER,
    ERROR_NVME_DAPI_RELEASE_MEMORY_BUFFER,
    ERROR_NVME_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL,
    ERROR_NVME_DAPI_ALLOCATE_PATTERN_BUFFER_FAIL,
    ERROR_NVME_DAPI_ALLOCATE_PCOMPARE_BUFFER_FAIL,
    ERROR_NVME_DAPI_CONSTRUCTOR                          = G_ME_HLD(MEF_NVME,TEF_DAPI,1,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_FW_LOG                           = G_ME_HLD(MEF_NVME,TEF_DAPI,2,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_ADDRESS_LINE_TEST                    = G_ME_HLD(MEF_NVME,TEF_DAPI,3,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_INFORMATION                      = G_ME_HLD(MEF_NVME,TEF_DAPI,4,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_SMART_LOG                        = G_ME_HLD(MEF_NVME,TEF_DAPI,5,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_LOGICALID_BY_NAME                = G_ME_HLD(MEF_NVME,TEF_DAPI,6,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_READWRITE_TEST                       = G_ME_HLD(MEF_NVME,TEF_DAPI,7,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_READWRITE_TEST_SETUP                 = G_ME_HLD(MEF_NVME,TEF_DAPI,8,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_READWRITE_TEST_EXECUTE               = G_ME_HLD(MEF_NVME,TEF_DAPI,9,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_READWRITE_TEST_CLEAN                 = G_ME_HLD(MEF_NVME,TEF_DAPI,10,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_READ_PERFORMANCE_TEST                = G_ME_HLD(MEF_NVME,TEF_DAPI,11,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_READ_PERFORMANCE_TEST_SETUP          = G_ME_HLD(MEF_NVME,TEF_DAPI,12,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_READ_PERFORMANCE_TEST_EXECUTE        = G_ME_HLD(MEF_NVME,TEF_DAPI,13,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_READ_PERFORMANCE_TEST_CLEAN          = G_ME_HLD(MEF_NVME,TEF_DAPI,14,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_WRITE_PERFORMANCE_TEST               = G_ME_HLD(MEF_NVME,TEF_DAPI,15,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_WRITE_PERFORMANCE_TEST_SETUP         = G_ME_HLD(MEF_NVME,TEF_DAPI,16,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE       = G_ME_HLD(MEF_NVME,TEF_DAPI,17,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_WRITE_PERFORMANCE_TEST_CLEAN         = G_ME_HLD(MEF_NVME,TEF_DAPI,18,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_RESET_CTRL                           = G_ME_HLD(MEF_NVME,TEF_DAPI,19,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_FW_DOWNLOAD                          = G_ME_HLD(MEF_NVME,TEF_DAPI,20,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_DO_IOCTL                             = G_ME_HLD(MEF_NVME,TEF_DAPI,21,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_ERROR_COUNT                      = G_ME_HLD(MEF_NVME,TEF_DAPI,22,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_DEVICE_NODE_LIST                 = G_ME_HLD(MEF_NVME,TEF_DAPI,23,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_BAR_RESOURCE                     = G_ME_HLD(MEF_NVME,TEF_DAPI,24,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_ERROR_LOG                        = G_ME_HLD(MEF_NVME,TEF_DAPI,25,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_NAMESPACE_ID                     = G_ME_HLD(MEF_NVME,TEF_DAPI,26,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_IDENTITY_CONTROLLER                  = G_ME_HLD(MEF_NVME,TEF_DAPI,27,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_IDENTITY_NAMESPACE                   = G_ME_HLD(MEF_NVME,TEF_DAPI,28,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_FW_ACTIVATE                          = G_ME_HLD(MEF_NVME,TEF_DAPI,29,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_WRITE_ZERO                           = G_ME_HLD(MEF_NVME,TEF_DAPI,30,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_GET_FEATURES                         = G_ME_HLD(MEF_NVME,TEF_DAPI,31,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_SET_FEATURES                         = G_ME_HLD(MEF_NVME,TEF_DAPI,32,ERROR_CODE_MODULE),
    ERROR_NVME_DAPI_RESET_SUBSYS                         = G_ME_HLD(MEF_NVME,TEF_DAPI,33,ERROR_CODE_MODULE),

    ERROR_NVME_DDAPI_ERRORCODE                           = G_ME_HLD(MEF_NVME,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_READ_MEMORY,
    ERROR_NVME_DDAPI_WRITE_MEMORY,
    ERROR_NVME_DDAPI_OPEN_NVME_DEVICE_FAIL,
    ERROR_NVME_DDAPI_MMAP_FAIL,
    ERROR_NVME_DDAPI_FILE_SEEK,
    ERROR_NVME_DDAPI_READ_LINK_ERROR,
    ERROR_NVME_DDAPI_NO_NVME_CONTROLLER,
    ERROR_NVME_IOCTL_INVALID_OPCODE,
    ERROR_NVME_IOCTL_INVALID_FIELD,
    ERROR_NVME_IOCTL_CMDID_CONFLICT,
    ERROR_NVME_IOCTL_DATA_XFER_ERROR,
    ERROR_NVME_IOCTL_POWER_LOSS,
    ERROR_NVME_IOCTL_INTERNAL,
    ERROR_NVME_IOCTL_ABORT_REQ,
    ERROR_NVME_IOCTL_ABORT_QUEUE,
    ERROR_NVME_IOCTL_FUSED_FAIL,
    ERROR_NVME_IOCTL_FUSED_MISSING,
    ERROR_NVME_IOCTL_INVALID_NS,
    ERROR_NVME_IOCTL_CMD_SEQ_ERROR,
    ERROR_NVME_IOCTL_LBA_RANGE,
    ERROR_NVME_IOCTL_CAP_EXCEEDED,
    ERROR_NVME_IOCTL_NS_NOT_READY,
    ERROR_NVME_IOCTL_CQ_INVALID,
    ERROR_NVME_IOCTL_QID_INVALID,
    ERROR_NVME_IOCTL_QUEUE_SIZE,
    ERROR_NVME_IOCTL_ABORT_LIMIT,
    ERROR_NVME_IOCTL_ABORT_MISSING,
    ERROR_NVME_IOCTL_ASYNC_LIMIT,
    ERROR_NVME_IOCTL_FW_SLOT,
    ERROR_NVME_IOCTL_FW_IMAGE,
    ERROR_NVME_IOCTL_FW_NEEDS_MAX_TIME_VIOLATION,
    ERROR_NVME_IOCTL_FW_PROHIBITED,
    ERROR_NVME_IOCTL_FW_IMAGE_OVERLAP,
    ERROR_NVME_IOCTL_INVALID_VECTOR,
    ERROR_NVME_IOCTL_INVALID_LOG_PAGE,
    ERROR_NVME_IOCTL_INVALID_FORMAT,
    ERROR_NVME_IOCTL_BAD_ATTRIBUTES,
    ERROR_NVME_IOCTL_WRITE_FAULT,
    ERROR_NVME_IOCTL_READ_ERROR,
    ERROR_NVME_IOCTL_GUARD_CHECK,
    ERROR_NVME_IOCTL_APPTAG_CHECK,
    ERROR_NVME_IOCTL_REFTAG_CHECK,
    ERROR_NVME_IOCTL_COMPARE_FAILED,
    ERROR_NVME_IOCTL_ACCESS_DENIED,
    ERROR_NVME_INVALID_FILE_MODE,
    ERROR_NVME_DRIVER_IOCTL_FAIL,
    ERROR_NVME_DDAPI_CONSTRUCTOR                         = G_ME_HLD(MEF_NVME,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_EXECUTE_IOCTL                       = G_ME_HLD(MEF_NVME,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_DOWNLOAD_FW                         = G_ME_HLD(MEF_NVME,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_SETFILEHANDLE                       = G_ME_HLD(MEF_NVME,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_CLOSEFILEHANDLE                     = G_ME_HLD(MEF_NVME,TEF_DDAPI,5,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_GET_NVME_LIST                       = G_ME_HLD(MEF_NVME,TEF_DDAPI,6,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_TEMPERATURE_NO_EXIST                = G_ME_HLD(MEF_NVME,TEF_DDAPI,7,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST                = G_ME_HLD(MEF_NVME,TEF_DDAPI,8,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_RESET_CTRL                          = G_ME_HLD(MEF_NVME,TEF_DDAPI,9,ERROR_CODE_MODULE),
    ERROR_NVME_DDAPI_RESET_SUBSYS                        = G_ME_HLD(MEF_NVME,TEF_DDAPI,10,ERROR_CODE_MODULE),



    //----------------- CONTROLLER -----------------//
    //CONTROLLER Diagnostic Driver API error code definition
    ERROR_CONTROLLER_DDAPI_ERRORCODE                     = G_ME_HLD(MEF_CONTROLLER,TEF_DDAPI,0,ERROR_CODE_MODULE),
    ERROR_CONTROLLER_DDAPI_CONSTRUCTOR                   = G_ME_HLD(MEF_CONTROLLER,TEF_DDAPI,1,ERROR_CODE_MODULE),
    ERROR_CONTROLLER_DDAPI_GET_LED_LIST                  = G_ME_HLD(MEF_CONTROLLER,TEF_DDAPI,2,ERROR_CODE_MODULE),
    ERROR_CONTROLLER_DDAPI_GET_LED_STATUS                = G_ME_HLD(MEF_CONTROLLER,TEF_DDAPI,3,ERROR_CODE_MODULE),
    ERROR_CONTROLLER_DDAPI_SET_LED_STATUS                = G_ME_HLD(MEF_CONTROLLER,TEF_DDAPI,4,ERROR_CODE_MODULE),
    ERROR_CONTROLLER_DDAPI_GET_SLOT_ID                   = G_ME_HLD(MEF_CONTROLLER,TEF_DDAPI,5,ERROR_CODE_MODULE),
} ERRORCODE;


#endif

#ifdef INCLUDE_ERRORTEXT

typedef struct {
    UINT32      u32ErrorCode;
    CHAR        strErrorText[65]; // Max length of text is 64
} ErrorCodeText;

typedef struct {
    UINT32              u32Index;
    ErrorCodeText       *errorCodeTextTable;
    INT32               s32TableSize;
} ErrorCodeTextTable;

// Module error code text definition
/// @struct ErrorCodeText error_code.hpp "error_code.hpp"
///         Module error code text definition
ErrorCodeText ModuleErrorCodeText[] = {
    { (MEF_PROCESSOR    << SHIFT_BIT_MODULE),   "Processor"},
    { (MEF_MEMORY       << SHIFT_BIT_MODULE),   "Memory"},
    { (MEF_ETHERNET     << SHIFT_BIT_MODULE),   "Ethernet"},
    { (MEF_STORAGE      << SHIFT_BIT_MODULE),   "Storage"},
    { (MEF_USB          << SHIFT_BIT_MODULE),   "USB"},
    { (MEF_PCI          << SHIFT_BIT_MODULE),   "PCI"},
    { (MEF_RTC          << SHIFT_BIT_MODULE),   "RTC"},
    { (MEF_SAS          << SHIFT_BIT_MODULE),   "SAS"},
    { (MEF_SCSI         << SHIFT_BIT_MODULE),   "SCSI"},
    { (MEF_UART         << SHIFT_BIT_MODULE),   "UART"},  
    { (MEF_PSU          << SHIFT_BIT_MODULE),   "PSU"},
    { (MEF_LED          << SHIFT_BIT_MODULE),   "LED"},
    { (MEF_I2C          << SHIFT_BIT_MODULE),   "I2C"},
    { (MEF_TEMPERATURE  << SHIFT_BIT_MODULE),   "TEMPERATURE"},
    { (MEF_SPD          << SHIFT_BIT_MODULE),   "SPD"},
    { (MEF_NVSRAM       << SHIFT_BIT_MODULE),   "NVSRAM"},
    { (MEF_TPM          << SHIFT_BIT_MODULE),   "TPM"},
    { (MEF_NTB          << SHIFT_BIT_MODULE),   "NTB"},
    { (MEF_EXPANDER     << SHIFT_BIT_MODULE),   "Expander"},
    { (MEF_FPGA         << SHIFT_BIT_MODULE),   "FPGA"},
    { (MEF_NVME         << SHIFT_BIT_MODULE),   "NVMe"},
    { (MEF_GPIO     	<< SHIFT_BIT_MODULE),   "GPIO"},
    { (MEF_CONTROLLER   << SHIFT_BIT_MODULE),   "CONTROLLER"},
};

// Type error code text definition
ErrorCodeText TypeErrorCodeText[] = {
    { (TEF_DAPI         << SHIFT_BIT_TYPE),     "DAPI"},
    { (TEF_DDAPI        << SHIFT_BIT_TYPE),     "DDAPI"},
    { (TEF_DRIVER       << SHIFT_BIT_TYPE),     "Driver"},
};

// Common error code text definition
ErrorCodeText CommonErrorCodeText[] = {
    { ERROR_ILLEGAL_REQUEST,                            "Illegal request"},
    { ERROR_TOO_FEW_PARAMETERS,                         "Too few parameters"},
    { ERROR_TOO_MANY_PARAMETERS,                        "Too many parameters"},
    { ERROR_BAD_PARAMETER,                              "Bad parameter"},
    { ERROR_BAD_PARAMETER_ONE,                          "Bad parameter 1"},
    { ERROR_BAD_PARAMETER_TWO,                          "Bad parameter 2"},
    { ERROR_BAD_PARAMETER_THREE,                        "Bad parameter 3"},
    { ERROR_BAD_PARAMETER_FOUR,                         "Bad parameter 4"},
    { ERROR_BAD_PARAMETER_FIVE,                         "Bad parameter 5"},
    { ERROR_BAD_PARAMETER_SIX,                          "Bad parameter 6"},
    { ERROR_BAD_PARAMETER_SEVEN,                        "Bad parameter 7"},
    { ERROR_BAD_PARAMETER_EIGHT,                        "Bad parameter 8"},
    { ERROR_BAD_PARAMETER_NINE,                         "Bad parameter 9"},
    { ERROR_FUNCTION_NOT_FINISHED,                      "Function not finished"},
    { ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR,          "Constructor 1 error"},
    { ERROR_DIAGNOSTIC_API_CONSTRUCTOR2_ERROR,          "Constructor 2 error"},
    { ERROR_ILLEGAL_ELEMENT_SIZE,                       "Illegal element size"},
    { ERROR_ILLEGAL_BUFFER_SIZE,                        "Illegal buffer size"},
    { ERROR_ALLOCATE_BUFFER_FAIL,                       "Allocate buffer fail"},
    { ERROR_OPEN_FILE,                                  "Fail to open file node"},
    { ERROR_CLOSE_FILE,                                 "Fail to close file node"},
    { ERROR_READ_FILE_FAIL,                             "fail to read file"},
    { ERROR_OPEN_DIRECTORY,                             "Fail to open directory node"},
    { ERROR_OUT_OF_RANGE,                               "out of range"},
    { ERROR_IOCTL,                                      "Ioctl fail"},
    { ERROR_IOCTL_INVALID_MAGIC_NUMBER,                 "invalid IOCTL magic number"},
    { ERROR_IOCTL_INVALID_COMMAND_CODE,                 "invalid IOCTL command code"},
    { ERROR_IOCTL_COPY_FROM_USER,                       "fail to copy IOCTL argument from user"},
    { ERROR_IOCTL_COPY_TO_USER,                         "fail to copy IOCTL argument to user"},
    { ERROR_IOCTL_SAS_INVALID_IOC_NUMBER,               "Invalid SAS IOC number"},
    { ERROR_IOCTL_SAS_INVALID_PHY_NUMBER,               "Invalid SAS PHY number"},
    { ERROR_SCSI_UPSUPPORT_OPCODE,                      "Invalid SCSI IOCTL command code"},
    { ERROR_SCSI_GET_FILE_HANDLE_FAIL,                  "Fail to get SCSI file handle"},
    { ERROR_SCSI_IOCTL_ACCESS_FAIL,                     "Fail to access SCSI ioctl"},
    { ERROR_SCSI_BUFFER_TOO_SMALL,                      "SCSI allocate buffer too small"},
    { ERROR_SCSI_BAD_DIRECTION,                         "BAD SG IO dxfer_dir"},
    { ERROR_SAS_IOCTL_ACCESS_FAIL,                      "Fail to access SAS ioctl"},
    { ERROR_SAS_IOCTL_MEM_ALLOC_FAILED,                 "SAS IOTCL allocate memory fail"},
    { ERROR_NO_DEVICE_FOUND,                            "No device is found"},
    { ERROR_UNEXPECTED_VALUE,                           "unexpected value"},
    { ERROR_UNEXPECTED_DATA_AMOUNT,                     "unexpected data amount"},        
    { ERROR_PRIVILEGE,                                  "privilege error"},
    { ERROR_IO_ACCESS,                                  "I/O access fail"},
    { ERROR_FUNCTION_NOT_IMPLEMENT,                     "function is not implement"},
    { ERROR_FUNCTION_NOT_SUPPORTED,                     "function is not supported"},
    { ERROR_CONFIGURATION_INCORRECT_FORMAT,             "incorrect format in the configuration file"},
    { ERROR_CONFIGURATION_NOT_FOUND,                    "configuration is not found"},
    { ERROR_TIME_OUT,                                   "Time out"},
    { ERROR_UNKNOWN_ARCH,                               "unknown arch"},
    { ERROR_CREATE_OBJECT_FAIL,                         "create object fail"},
    { ERROR_REGISTER_IS_READ_ONLY,                      "register is read-only"},
    { ERROR_REGISTER_IS_WRITE_ONLY,                     "register is write-only"},
};

// Memory DAPI error code text definition
ErrorCodeText MemoryDAPIErrorCodeText[] = {
    { ERROR_MEMORY_DAPI_NO_SPD_INFO,                    "no SPD information of DIMMs"},
    { ERROR_MEMORY_DAPI_NO_DIMM_INFO,                   "no DIMM information"},        
    { ERROR_MEMORY_DAPI_TEST_PATTERN_COMPARE_FAIL,      "test pattern compare fail"},
    { ERROR_MEMORY_DAPI_NO_USABLE_MEMORY,               "no usable memory information"},
    { ERROR_MEMORY_DAPI_NO_TESTABLE_MEMORY,             "no testable memory information"},
    { ERROR_MEMORY_DAPI_NO_DIMM_MATCH_TO_ECC_ERROR,     "no DIMM match to ECC error"},
    { ERROR_MEMORY_DAPI_REMAP_MEMORY_RANGE_UNMATCH,     "remap memory range is unmatchable"},        
    { ERROR_MEMORY_DAPI_CONSTRUCTOR,                    "Constructor"},
    { ERROR_MEMORY_DAPI_GET_INFORMATION,                "getInformation"},
    { ERROR_MEMORY_DAPI_READ_MEMORY,                    "readMemory"},                 
    { ERROR_MEMORY_DAPI_WRITE_MEMORY,                   "writeMemory"},
    { ERROR_MEMORY_DAPI_GET_MEMORY_BUFFER,              "getMemoryBuffer"},
    { ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER,          "releaseMemoryBuffer"},
    { ERROR_MEMORY_DAPI_DATA_LINE_TEST,                 "dataLineTest"},
    { ERROR_MEMORY_DAPI_ADDRESS_LINE_TEST,              "addressLineTest"},
    { ERROR_MEMORY_DAPI_MARCHC_TEST,                    "MarchCTest"},
    { ERROR_MEMORY_DAPI_CHECKSUM_TEST,                  "ChecksumTest"},
    { ERROR_MEMROY_DAPI_GET_TESTABLE_MEMORY,            "getTestableMemory"},
    { ERROR_MEMORY_DAPI_GET_ECC_REPORT,                 "getECCReport"}, 
    { ERROR_MEMORY_DAPI_CLEAR_ECC_REPORT,               "clearECCReport"},
    { ERROR_MEMORY_DAPI_ECC_ERROR_DETECTION_TEST,       "ECCErrorDetectionTest"},
    { ERROR_MEMORY_DAPI_PATTERN_TEST,                   "patternTest"},
    { ERROR_MEMORY_DAPI_VERIFY_MEMORY_BLOCK,            "verifyMemoryBlock"},
    { ERROR_MEMORY_DAPI_EXECUTE_MARCHC_TEST,            "executeMarchCTest"},
    { ERROR_MEMORY_DAPI_VERIFY_MEMORY_BLOCK_BY_MARCHC,  "verifyMemoryBlockByMarchC"},
    { ERROR_MEMORY_DAPI_RANDOM_ADDRESS_TEST,            "randomAddressTest"},
    { ERROR_MEMORY_DAPI_GET_DIMM_ADDRESS_INFORMATION,   "getDIMMAddressInformation"},
    { ERROR_MEMORY_DAPI_GET_DEVICE_LIST,                "getDeviceList"},
    { ERROR_MEMORY_DAPI_GET_RANK_RANGE,                 "getRankRange"},
    { ERROR_MEMORY_DAPI_INITIALIZE_MEMORY_LAYOUT_FILE,  "initializeMemoryLayoutFile" },
    { ERROR_MEMORY_DAPI_INITIALIZE_TESTABLE_MEMORY_FILE,"initializeTestableMemoryFile" },
    { ERROR_MEMORY_DAPI_CHECK_DEVICE_BY_SMBIOS,         "checkDeviceBySMBIOS"},
};

// Memory DDAPI error code text definition
ErrorCodeText MemoryDDAPIErrorCodeText[] = {
    { ERROR_MEMORY_DDAPI_DECODE_DIMM_SPD_FAIL,              "decode DIMM SPD fail"},
    { ERROR_MEMORY_DDAPI_DMIDECODE_MEMORY_DEVICE_FAIL,      "dmidecode memory device fail"},
    { ERROR_MEMORY_DDAPI_DMIDECODE_MEMORY_DEVICE_NOT_FOUND, "memory device can not be found in dmidecode"},
    { ERROR_MEMORY_DDAPI_OPEN_MEM_DEVICE_FAIL,              "open /dev/mem fail"},
    { ERROR_MEMORY_DDAPI_MMAP_FAIL,                         "mmap fail"},
    { ERROR_MEMORY_DDAPI_EDAC_DEVICE_NOT_FOUND,             "edac device can not be found"},
    { ERROR_MEMORY_DDAPI_ADDRESS_TRANSLATION_FAIL,          "address translation fail"},
    { ERROR_MEMORY_DDAPI_INCORRECT_LAYOUT_INFO,             "incorrect layout information"},
    { ERROR_MEMORY_DDAPI_GET_DIMM_INFORMATION_BY_SPD,       "getDIMMInformationBySPD"},
    { ERROR_MEMORY_DDAPI_GET_DIMM_INFORMATION_BY_SMBIOS,    "getDIMMInformationBySMBIOS"},
    { ERROR_MEMORY_DDAPI_IOREMAP,                           "IORemap"},
    { ERROR_MEMORY_DDAPI_READ_MEMORY,                       "readMemory"},
    { ERROR_MEMORY_DDAPI_WRITE_MEMORY,                      "writeMemory"},
    { ERROR_MEMORY_DDAPI_GET_MEMORY_BUFFER,                 "getMemroryBuffer"},
    { ERROR_MEMORY_DDAPI_RELEASE_MEMORY_BUFFER,             "releaseMemoryBuffer"},
    { ERROR_MEMORY_DDAPI_GET_ECC_REPORT,                    "getECCReport"},
    { ERROR_MEMORY_DDAPI_CLEAR_ECC_REPORT,                  "clearECCReport"},
    { ERROR_MEMORY_DDAPI_INJECT_ECC,                        "injectECC"},
    { ERROR_MEMORY_DDAPI_GET_HIGH_RAM_RANGE,                "getHighRamRange" },
    { ERROR_MEMORY_DDAPI_GET_CONTROLLER_CHANNEL_COUNT,      "getControllerChannelCount"},
    { ERROR_MEMORY_DDAPI_GET_CONTROLLER_SLOT_COUNT,         "getControllerSlotCount"},
    { ERROR_MEMORY_DDAPI_INITIALIZE_MEMORY_LAYOUT_FILE,     "initializeMemoryLayoutFile"},
    { ERROR_MEMORY_DDAPI_GET_PHYSICAL_ADDRESS,              "getPhysicalAddress"},
    { ERROR_MEMORY_DDAPI_GET_RANK_OFFSET,                   "getRankOffset"},
};

// Memory Driver error code text definition
ErrorCodeText MemoryDriverErrorCodeText[] = {
    { ERROR_MEMORY_DRIVER_ADDR_TRANSLATION_FAIL,            "fail to translate address"},
    { ERROR_MEMORY_DRIVER_IOREMAP_FAIL,                     "fail to ioremap"},
    { ERROR_MEMORY_DRIVER_ECC_INJECT_FAIL,                  "fail to inject ecc"},
    { ERROR_MEMORY_DRIVER_COMPARE_DATA_FAIL,                "data is inconsistent"},
    { ERROR_MEMORY_DRIVER_ENABLE_ECC_INJECT_FAIL,           "fail to enable injection"},
    { ERROR_MEMORY_DRIVER_INCORRECT_ECC_COUNT,              "incorrect ecc counts"},
    { ERROR_MEMORY_DRIVER_NO_ECC_COUNT,                     "no ecc count" },
    { ERROR_MEMORY_DRIVER_ECC_INJECT_LOCK,                  "ecc injection is locked" },
};



// SAS DAPI error code text definition
ErrorCodeText SASDAPIErrorCodeText[] =
{
    { ERROR_SAS_DAPI_TEST_PATTERN_COMPARE_FAIL,     "testPatternCompareFail"},
    { ERROR_SAS_DAPI_GET_INFORMATION,               "getInformation"},
    { ERROR_SAS_DAPI_DISPLAY_DATA,                  "displayData"},
    { ERROR_SAS_DAPI_GET_MEMORY_BUFFER,             "getMemoryBuffer"},
    { ERROR_SAS_DAPI_GET_TEMPERATURE,               "getTemperature"},
    { ERROR_SAS_DAPI_GET_LOGICALID_BY_NAME,         "getLogicalIDByName"},
    { ERROR_SAS_DAPI_RELEASE_MEMORY_BUFFER,         "releaseMemoryBuffer"},
    { ERROR_SAS_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL,   "allocate backup buffer fail"},
    { ERROR_SAS_DAPI_ALLOCATE_PATTERN_BUFFER_FAIL,  "allocate patter Buffer fail"},
    { ERROR_SAS_DAPI_ALLOCATE_PCOMPARE_BUFFER_FAIL, "allocate compare buffer fail"},
    { ERROR_SAS_DAPI_CONSTRUCTOR,                   "constructor"},
    { ERROR_SAS_DAPI_DATA_LINE_TEST,                "dataLineTest"},
    { ERROR_SAS_DAPI_ADDRESS_LINE_TEST,             "addressLineTest"},
    { ERROR_SAS_DAPI_CLEAR_PHY_ERROR_LOG,           "clearPHYErrorLog"},
    { ERROR_SAS_DAPI_READWRITE_TEST,                "readWriteTest"},
    { ERROR_SAS_DAPI_READWRITE_TEST_SETUP,          "readWriteTestSetup"},
    { ERROR_SAS_DAPI_READWRITE_TEST_EXECUTE,        "readWriteTestExecute"},
    { ERROR_SAS_DAPI_READWRITE_TEST_CLEAN,          "readWriteTestClean"},
    { ERROR_SAS_DAPI_READ_PERFORMANCE_TEST,         "readPerformanceTest"},
    { ERROR_SAS_DAPI_READ_PERFORMANCE_TEST_SETUP,   "readPerformanceTestSetup"},
    { ERROR_SAS_DAPI_READ_PERFORMANCE_TEST_EXECUTE, "readPerformanceTestExecute"},
    { ERROR_SAS_DAPI_READ_PERFORMANCE_TEST_CLEAN,   "readPerformanceTestClean"},
    { ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST,        "writePerformanceTest"},
    { ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_SETUP,  "writePerformanceTestSetup"},
    { ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE,"writePerformanceTestExecute"},
    { ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_CLEAN,  "writePerformanceTestClean"},
    { ERROR_SAS_DAPI_LOOPBACK_TEST,                 "loopbackTest"},
    { ERROR_SAS_DAPI_RESETIOC,                      "resetIOC"},
    { ERROR_SAS_DAPI_PROGRAM_ADDRESS,               "program SAS address"},
    { ERROR_SAS_DAPI_PROGRAM_FIRMWARE,              "program SAS new firmware"},
    { ERROR_SAS_DAPI_DO_IOCTL,                      "ioctl command fail"},
    { ERROR_SAS_DAPI_GET_ATTACHED_DEVICE,           "getAttachDevice"},
    { ERROR_SAS_DAPI_GET_ERROR_COUNT,               "getErrorCount"},
    { ERROR_SAS_DAPI_GET_LINK_STATUS,               "getLinkStatu"},
    { ERROR_SAS_DAPI_GET_LINK_SPEED,                "getLinkSpeed"},
    { ERROR_SAS_DAPI_GET_DEVICE_NODE_LIST,          "getSASDeviceNodeList"},
    { ERROR_SAS_DAPI_GET_DRIVE_INFORMATION,         "getSASDriveInformation"},
    { ERROR_SAS_DAPI_GET_HOST_SAS_ADDRESS,          "getIOCSASAddress"}
};

// SAS DDAPI error code text definition
ErrorCodeText SASDDAPIErrorCodeText[] = {
    { ERROR_SAS_DDAPI_READ_MEMORY,                      "Read memory fail"                },
    { ERROR_SAS_DDAPI_WRITE_MEMORY,                     "Write memory fail"               },
    { ERROR_SAS_DDAPI_OPEN_SAS_DEVICE_FAIL,             "Can not open SAS device"         },
    { ERROR_SAS_DDAPI_MMAP_FAIL,                        "SAS mmap fail"                   },
    { ERROR_SAS_DDAPI_FILE_SEEK,                        "File seek position fail"         },
    { ERROR_SAS_DDAPI_NOT_SUPPORT_SPECIFIC_PHY,         "Can not specific PHY number"     },
    { ERROR_SAS_DDAPI_TEMPERATURE_NO_EXIST,             "SAS temperature sensor not exist"},
    { ERROR_SAS_DDAPI_EXECUTE_IOCTL,                    "SAS execute IOCTL fail"          },
    { ERROR_SAS_DDAPI_SEEK_READ,                        "seekRead"                        },
    { ERROR_SAS_DDAPI_SEEK_WRITE,                       "seekWrite"                       },
    { ERROR_SAS_DDAPI_PROGRAMIMAGE,                     "programImage"                    },
    { ERROR_SAS_DDAPI_SETFILEHANDLE,                    "setFileHandle"                   },
    { ERROR_SAS_DDAPI_CLOSEFILEHANDLE,                  "closeFileHandle"                 },
    { ERROR_SAS_DDAPI_GET_SAS_LIST,                     "Cannot get SAS list"             },
    { ERROR_SAS_DDAPI_GET_DEVICE_NODE_LIST,             "Cannot get SAS drive device node"},
    { ERROR_SAS_DDAPI_GET_DRIVE_INFO,                   "Get SAS drive information fail"  },
    { ERROR_SAS_DDAPI_GET_DRIVE_PHY_INFO,               "Get SAS drive PHY info fail"     },
    { ERROR_SAS_DDAPI_GET_SAS_INFO,                     "Get SAS IOC information fail"    },
    { ERROR_SAS_DDAPI_READ_LINK_ERROR,                  "Read symbolic link fail"         },
    { ERROR_SAS_DDAPI_NO_SAS_CONTROLLER,                "Host has no SAS controller"      },
    { ERROR_SAS_DDAPI_GET_DRIVE_SIZE,                   "Get SAS drive size fail"         },

};

// SAS Driver error code text definition
ErrorCodeText SASDriverErrorCodeText[] = {
    { ERROR_SAS_DRIVER_IOCTL_FAIL,                      "SAS ioctl fail"                 },
};

// SCSI factory error code text definition
ErrorCodeText SCSIFactoryErrorCodeText[] = {
    { ERROR_SCSI_FACTORY_ERR_NOT_READY,                 "device not ready"               },
    { ERROR_SCSI_FACTORY_ERR_BAD_OPCODE,                "unsupported scsi opcode"        },
    { ERROR_SCSI_FACTORY_ERR_BAD_FIELD,                 "unsupported field in command"   },
    { ERROR_SCSI_FACTORY_ERR_BAD_PARAM,                 "badly formed scsi parameters"   },
    { ERROR_SCSI_FACTORY_ERR_BAD_RESP,                  "scsi response fails sanity test"},
    { ERROR_SCSI_FACTORY_ERR_NO_MEDIUM,                 "no medium present"              },
    { ERROR_SCSI_FACTORY_ERR_BECOMING_READY,            "device will be ready soon"      },
    { ERROR_SCSI_FACTORY_ERR_TRY_AGAIN,                 "attention reported, try again"  },
    { ERROR_SCSI_FACTORY_ERR_MEDIUM_HARDWARE,           "medium or hardware error"       },
    { ERROR_SCSI_FACTORY_ERR_UNKNOWN,                   "unexpected sense key"           },
    { ERROR_SCSI_FACTORY_ERR_ABORTED_COMMAND,           "aborted command"                },
    { ERROR_SCSI_FACTORY_CONSTRUCTOR,                   "SCSI Factory construct fail"    },
    { ERROR_SCSI_FACTORY_READ_SCSI,                     "SCSI read function fail"        },
    { ERROR_SCSI_FACTORY_WRITE_SCSI,                    "SCSI write function fail"       },
    { ERROR_SCSI_FACTORY_TEST_UNIT_READY,               "SCSI test unit ready fail"      },
    { ERROR_SCSI_FACTORY_STD_INQUIRY,                   "SCSI standard inquiry fail"     },
    { ERROR_SCSI_FACTORY_INQUIRY_VPD,                   "SCSI inquiry VPD command fail"  },
    { ERROR_SCSI_FACTORY_LOG_SENSE,                     "SCSI log sense command fail"    },
    { ERROR_SCSI_FACTORY_LOG_SELECT,                    "SCSI log select command fail"   },
    { ERROR_SCSI_FACTORY_MODE_SENSE,                    "SCSI mode sense command fail"   },
    { ERROR_SCSI_FACTORY_MODE_SELECT,                   "SCSI mode select command fail"  },
    { ERROR_SCSI_FACTORY_MODE_SENSE10,                  "SCSI mode sense10 command fail" },
    { ERROR_SCSI_FACTORY_MODE_SELECT10,                 "SCSI mode select10 command fail"},
    { ERROR_SCSI_FACTORY_MODE_PAGE_OFFSET,              "SCSI get mode page offset fail" },
    { ERROR_SCSI_FACTORY_REQUEST_SENSE,                 "SCSI request sense command fail"},
    { ERROR_SCSI_FACTORY_SEND_DIAGNOSTIC,               "SCSI send DIAG command fail"    },
    { ERROR_SCSI_FACTORY_READ_CAPACITY10,               "SCSI read capacity10 fail"      },
    { ERROR_SCSI_FACTORY_READ_CAPACITY16,               "SCSI read capacity16 fail"      },
    { ERROR_SCSI_FACTORY_GET_RPM,                       "SCSI get drive RPM fail"        },
    { ERROR_SCSI_FACTORY_GET_TRANSPORT_PROTOCAL,        "SCSI get protocol fail"         },
    { ERROR_SCSI_FACTORY_GET_DRIVE_TEMPERATURE,         "SCSI get drive temperature"     },
    { ERROR_SCSI_FACTORY_GET_ERROR_LOG,                 "SCSI get smart error log"       },
    { ERROR_SCSI_FACTORY_GET_SMART_HEALTH_STATUS,       "SCSI get smart health status"   },
    { ERROR_SCSI_FACTORY_READ_DEFECT12,                 "SCSI read defect12"             },
};

// USB DAPI error code text definition
ErrorCodeText USBDAPIErrorCodeText[] = {
    { ERROR_USB_DAPI_DATA_ELEMENT_NOT_FOUND,            "USB Data Element Not Found"},
    { ERROR_USB_DAPI_TEST_PATTERN_COMPARE_FAIL,         "Test pattern compare fail"},
    { ERROR_USB_DAPI_DIVIDED_ZERO_FAIL,                 "divided zero fail"},
    { ERROR_USB_DAPI_BACKUP_DATA_FAIL,                  "USB storage backup data fail"},
    { ERROR_USB_DAPI_WRITE_DATA_TO_STORAGE_FAIL,        "Write data to USB storage fail"},
    { ERROR_USB_DAPI_READ_DATA_FROM_STORAGE_FAIL,       "Read data from USB storage fail"},
    { ERROR_USB_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL,       "allocate backup buffer fail"},
    { ERROR_USB_DAPI_ALLOCATE_PATTERN_BUFFER_FAIL,      "allocate patter Buffer fail"},
    { ERROR_USB_DAPI_ALLOCATE_PCOMPARE_BUFFER_FAIL,     "allocate compare buffer fail"},
    { ERROR_USB_DAPI_CONSTRUCTOR,                       "Constructor"},
    { ERROR_USB_DAPI_DISPLAY_USB_PORT_LIST,             "displayUSBPortList"},
    { ERROR_USB_DAPI_OPEN_DEVICE,                       "openDeviceFile"},
    { ERROR_USB_DAPI_CLOSE_DEVICE,                      "closeDeviceFile"},
    { ERROR_USB_DAPI_READ_WRITE_TEST,                   "readwriteTest"},
    { ERROR_USB_DAPI_READ_WRITE_TEST_SETUP,             "readwriteTestSetup"},
    { ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE,           "readwriteTestExecute"},
    { ERROR_USB_DAPI_READ_WRITE_TEST_CLEAN,             "readwriteTestClean"},
    { ERROR_USB_DAPI_READ_PERFORMANCE_TEST,             "readPerformanceTest"},
    { ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP,       "readPerformanceTestSetup"},
    { ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE,     "readPerformanceTestExecute"},
    { ERROR_USB_DAPI_READ_PERFORMANCE_TEST_CLEAN,       "readPerformanceTestClean"},
    { ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST,            "writePerformanceTest"},
    { ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP,      "writePerformanceTestSetup"},
    { ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE,    "writePerformanceTestExecute"},
    { ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_CLEAN,      "writePerformanceTestClean"},
    { ERROR_USB_DAPI_PREAD,                             "pread"},
    { ERROR_USB_DAPI_PWRITE,                            "pwrite"},
    { ERROR_USB_DAPI_READ_FROM_DEVICE,                  "readFromDevice"},
    { ERROR_USB_DAPI_WRITE_TO_DEVICE,                   "writeToDevice"},
    { ERROR_USB_DAPI_LIST_DEVICE_NODE,                  "listDeviceNode"},
    { ERROR_USB_DAPI_IS_DEVICE_NODE,                    "isDeviceNode"},
    { ERRIR_USB_DAPI_GET_LOGICAL_ID_BY_NAME,            "getLogicalIdByName"},
    { ERRIR_USB_DAPI_GET_DATA_ID_BY_ID,                 "getDataByID"},



};

// USB DDAPI error code text definition
ErrorCodeText USBDDAPIErrorCodeText[] = {
    { ERROR_USB_DDAPI_LIBUSB_INIT,                      "libusb initialize fail"},
    { ERROR_USB_DDAPI_DEVICE_NAME_NOT_FOUND,            "USB Device Name Not Found"},
    { ERROR_USB_DDAPI_DATA_ELEMENT_NOT_FOUND,           "USB Data Element Not Found"},
    { ERROR_USB_DDAPI_LINK_SYMBOLIC_FAIL,               "link symbolic fail"},
    { ERROR_USB_DDAPI_FILE_SEEK,                        "File seek position fail"},
    { ERROR_USB_DDAPI_READ_DATA_FAIL,                   "Read device node fail"},
    { ERROR_USB_DDAPI_WRITE_DATA_FAIL,                  "Write device node fail"},
    { ERROR_USB_DDAPI_OPEN_DIR_FAIL,                   "open directory fail"},
    { ERROR_USB_DDAPI_CONSTRUCTOR,                      "Constructor"},
    { ERROR_USB_DDAPI_INITIALIZE_USB_NAME,              "initializeUSBNames"},
    { ERROR_USB_DDAPI_INITIALIZE_USB_INFO,              "InitializeUSBInfo"},
    { ERROR_USB_DDAPI_DISPLAY_USB_PORT_LIST,            "displayUSBPortList"},
    { ERROR_USB_DDAPI_SCAN_USB_DEVICE,                  "scanUSBDevice"},
    { ERROR_USB_DDAPI_ADD_USB_BUS,                      "addUSBBus"},
    { ERROR_USB_DDAPI_INSPECT_BUS_ENTRY,                "inspectBusEntry"},
    { ERROR_USB_DDAPI_ADD_USB_INTERFACE,                "addUSBInterface"},
    { ERROR_USB_DDAPI_ADD_USB_DEVICE,                   "addUSBDevice"},
    { ERROR_USB_DDAPI_WALK_USB_DEVICES,                 "walkUSBDevices"},
    { ERROR_USB_DDAPI_GET_ROOT_HUB_DRIVER,              "getRootHubDriver"},
    { ERROR_USB_DDAPI_NEW_CLASS,                        "newClass"},
    { ERROR_USB_DDAPI_PARSE_USB_VENDOR_PRODUCT_NAME,    "parseUSBVendorProductName"},
    { ERROR_USB_DDAPI_USBInfo_CONSTRUCTOR,              "Constructor"},
    { ERROR_USB_DDAPI_NEW_GENERIC_STRING_TABLE,         "newGenericStrtable"},
    { ERROR_USB_DDAPI_NEW_PHY_DESCRIPTION,              "newPhysDescription"},
    { ERROR_USB_DDAPI_NEW_BIAS,                         "newBias"},
    { ERROR_USB_DDAPI_NEW_HUTUS,                        "newHutus"},
    { ERROR_USB_DDAPI_NEW_LANG_ID,                      "newLangID"},
    { ERROR_USB_DDAPI_NEW_COUNTRY_CODE,                 "newCountryCode"},
    { ERROR_USB_DDAPI_NEW_HID,                          "newHid"},
    { ERROR_USB_DDAPI_NEW_HUTS,                         "newHuts"},
    { ERROR_USB_DDAPI_NEW_REPORT_TAG,                   "newReportTag"},
    { ERROR_USB_DDAPI_NEW_AUDIO_TERMINAL,               "newAudioTerminal"},
    { ERROR_USB_DDAPI_NEW_VIDEO_TERMINAL,               "newVideoTerminal"},
    { ERROR_USB_DDAPI_NEW_VENDOR,                       "newVendor"},
    { ERROR_USB_DDAPI_NEW_PRODUCT,                      "newProduct"},
    { ERROR_USB_DDAPI_NEW_SUB_CLASS,                    "newSubclass"},
    { ERROR_USB_DDAPI_NEW_PROTOCOL,                     "newProtocol"},
    { ERROR_USB_DDAPI_SEEK_READ,                        "seekRead"},
    { ERROR_USB_DDAPI_SEEK_WRITE,                       "seekWrite"},
    { ERRIR_USB_DDAPI_GET_DEVICE_CAPACITY,              "getDeviceCapacity"},
    { ERRIR_USB_DDAPI_GET_LOGICAL_ID_BY_NAME,           "getLogicalIdByName"},
    { ERRIR_USB_DDAPI_GET_DATA_ID_BY_ID,                "getDataByID"},
    { ERRIR_USB_DDAPI_GET_USB_DEVICE_NODE,              "getUSBDeviceNode"},
    { ERRIR_USB_DDAPI_GET_USB_INFO_FROM_SYS_BUS,        "getUSBInfoFromSysBus"},
    { ERRIR_USB_DDAPI_GET_USB_INFO_FROM_SYS_BLOCK,      "getUSBInfoFromSysBlock"},
    { ERRIR_USB_DDAPI_PREAD,                            "PREAD"},
    { ERRIR_USB_DDAPI_PWRITE,                           "PWRITE"},
    { ERRIR_USB_DDAPI_SCSI_READ,                        "SCSIRead"},
    { ERRIR_USB_DDAPI_SCSI_WRITE,                       "SCSIWrite"},
	{ ERRIR_USB_DDAPI_GET_USB_SG_DEVICE_NODE,			"getUSBSGDeviceNode"}


};


// USB Driver error code text definition
ErrorCodeText USBDriverErrorCodeText[] = {
    { ERROR_USB_DRIVER_IOCTL_FAIL,                   "ioctl fail"},
};



//GPIO DAPI error code text definition
ErrorCodeText GPIODAPIErrorCodeText[] = {
    { ERROR_GPIO_DAPI_CONSTRUCTOR,                  "Constructor"},
    { ERROR_GPIO_DAPI_DISPLAY_GPIO_LIST,            "displayGPIOList"},
    { ERROR_GPIO_DAPI_GPIO_GET_STATUS,              "getStatus"},
    { ERROR_GPIO_DAPI_GPIO_SET_STATUS,              "setStatus"},


};


//GPIO DDAPI error code text definition
ErrorCodeText GPIODDAPIErrorCodeText[] = {
    { ERROR_GPIO_DDAPI_CONSTRUCTOR,                     "Constructor"},
    { ERROR_GPIO_DDAPI_DEVICE_NAME_NOT_FOUND,           "GPIO Name Not Found"},
    { ERROR_GPIO_DDAPI_OPEN_PROFILE_FAIL,               "Open GPIO_profile.ini fail"},
    { ERROR_GPIO_DDAPI_GET_GPIO_INDEX_FAIL,             "Get GPIO Index Fail"},
    { ERROR_GPIO_DDAPI_GPIO_READ_ONLY,                  "GPIO Read Only"},
    { ERROR_GPIO_DDAPI_GPIO_NOT_SUPPORT,                "GPIO Not Support"},
    { ERROR_GPIO_DDAPI_GPIO_UNSUPPORT_TYPE,             "GPIO Unsupport Type"},
	{ ERROR_GPIO_DDAPI_GPIO_GET_FAIL,					"GPIO Get Fail"},
	{ ERROR_GPIO_DDAPI_GPIO_SET_FAIL,					"GPIO Set Fail"},
    { ERROR_GPIO_DDAPI_GET_GPIO_LIST,                   "getGPIOList"},
    { ERROR_GPIO_DDAPI_GPIO_FPGA_DRIVER_CONSTRUCTOR,    "GPIOFPGADriver Constructor"},
    { ERROR_GPIO_DDAPI_GPIO_FPGA_READ_BIT,              "GPIOFPGADriver readBit"},
    { ERROR_GPIO_DDAPI_GPIO_FPGA_IS_MY_GPIO,            "GPIOFPGADriver isMyGPIO"},
    { ERROR_GPIO_DDAPI_GPIO_READ_BIT,                   "GPIO readBit"},
    { ERROR_GPIO_DDAPI_GPIO_SET_BIT,                    "GPIO setBit"},
    { ERROR_GPIO_DDAPI_GPIO_FPGA_SET_BIT,               "GPIOFPGADriver setBit"},

    { ERROR_GPIO_PCH_DDAPI_DRIVER_CONSTRUCTOR,    		"GPIO_PCHDriver Constructor"},
    { ERROR_GPIO_PCH_DDAPI_READ_BIT,              		"GPIO_PCHDriver readBit"},
    { ERROR_GPIO_PCH_DDAPI_IS_MY_GPIO,            		"GPIO_PCHDriver isMyGPIO"},
    { ERROR_GPIO_PCH_DDAPI_SET_BIT,               		"GPIO_PCHDriver setBit"},
    { ERROR_GPIO_PCH_DDAPI_GET_GPIO_LIST,               "GPIO_PCHDriver getGPIOList"},








};

//Processor Driver error code text definition
ErrorCodeText GPIODriverErrorCodeText[] = {
    { ERROR_GPIO_DRIVER_IOCTL_FAIL,                   "ioctl fail"},

};



// Storage DAPI error code text definition
ErrorCodeText StorageDAPIErrorCodeText[] = {
/*    { ERROR_STORAGE_DAPI_LINK_SPEED_ERROR,              "link speed error"           },
    { ERROR_STORAGE_DAPI_LOOPBACK_PATH_ERROR,           "loopback path error"        },
    { ERROR_STORAGE_DAPI_ILLEGAL_START_LOCATION,        "illegal start location"     },
    { ERROR_STORAGE_DAPI_ILLEGAL_TEST_SIZE,             "illegal test size"          },
    { ERROR_STORAGE_DAPI_DATA_COMPARE_MISMATCH,         "data compare mismatch"      },
    { ERROR_STORAGE_DAPI_UNDEFINED_BLOCK_SIZE,          "block size error"           },
    { ERROR_STORAGE_DAPI_BUFFER_TOO_SMALL,              "call buffer too small"      },
    { ERROR_STORAGE_DAPI_UNDEFINED_CONTROLLER_TYPE,     "undefined controller type"  },
    { ERROR_STORAGE_DAPI_CONSTRUCTOR,                   "constructor"                },
    { ERROR_STORAGE_DAPI_READ_WRITE_TEST,               "readWriteTest"              },
    { ERROR_STORAGE_DAPI_READ_PERFORMANCE_TEST,         "readPerformanceTest"        },
    { ERROR_STORAGE_DAPI_WRITE_PERFORMANCE_TEST,        "writePerformanceTest"       },
    { ERROR_STORAGE_DAPI_DISPLAY_STORAGE_DEVICE_ON_HOST,"displayStorageDeviceOnHost" },
    { ERROR_STORAGE_DAPI_READ_FROM_DEVICE,              "readFromDevice"             },
    { ERROR_STORAGE_DAPI_WRITE_TO_DEVICE,               "writeToDevice"              },
    { ERROR_STORAGE_DAPI_DISPLAY_BY_CONTROLER_TYPE,     "displayyByControllerType"   },*/
    { ERROR_STORAGE_DAPI_CONSTRUCTOR,                      "Constructor"},
    { ERROR_STORAGE_DAPI_IOCTL,                            "IOCTL"},
    { ERROR_STORAGE_DAPI_GENERATE_STORAGE_LIST,            "generateStorageList"},
    { ERROR_STORAGE_DAPI_READ_BY_SCSI,                     "readBySCSI"},
    { ERROR_STORAGE_DAPI_WRITE_BY_SCSI,                    "writeBySCSI"},
    { ERROR_STORAGE_DAPI_GET_DEFINED_INDEX_BY_NAME,        "getDefinedIndexByName"},
    { ERROR_STORAGE_DAPI_GET_ELEMENT_DATA_BY_INDEX,        "getElementDataByIndex"},
    { ERROR_STORAGE_DAPI_GET_SMART_BY_INDEX,               "getSMARTByIndex"},
    { ERROR_STORAGE_DAPI_GET_ERROR_REGISTER_BY_INDEX,      "getErrorRegisterByIndex"},
    { ERROR_STORAGE_DAPI_GET_ELEMENT_DATA_BY_NAME,         "getElementDataByName"},
    { ERROR_STORAGE_DAPI_IDENTIFY_COMMAND_TEST,            "identifyCommandTest"},
    { ERROR_STORAGE_DAPI_READ_PERFORMANCE_TEST,            "readPerformanceTest"},
    { ERROR_STORAGE_DAPI_WRITE_PERFORMANCE_TEST,           "writePerformanceTest"},
    { ERROR_STORAGE_DAPI_READ_WRITE_TEST,                  "readWriteTest"},
    { ERROR_STORAGE_DAPI_SET_SECTOR_COUNT_PER_RDWR_ACCESS, "setSectorCountPerRDWRAccess"},
    { ERROR_STORAGE_DAPI_WRITE_ZERO_BY_INDEX,              "writeZeroByIndex"},
    { ERROR_STORAGE_DAPI_GET_PGLIST_BY_INDEX,              "getPGListByIndex"},

    //other error msg
    { ERROR_STORAGE_DAPI_NO_SUCH_IOCTL_OPERATION,          "wrong device node name"},
    { ERROR_STORAGE_DAPI_COMPARE_DATA_ERROR,               "read write test compare data fail"},
    { ERROR_STORAGE_DAPI_OPEN_DEVICE_FAIL,                 "open device node fail"},
    { ERROR_STORAGE_DAPI_CLOSE_DEVICE_FAIL,                "close device node fail"},
    { ERROR_STORAGE_DAPI_UNKNOWN_CONTROLLER_TYPE,          "get unknown controller type"},
    { ERROR_STORAGE_DAPI_DEVICE_NOT_SUPPORT_SMART,         "device not support smart"},

};

// Storage DDAPI error code text definition
ErrorCodeText StorageDDAPIErrorCodeText[] = {
/*
    { ERROR_STORAGE_DDAPI_ILLEGAL_ID_REQUEST,           "illegal ID"                 },
    { ERROR_STORAGE_DDAPI_INVALID_OP_CODE,              "invalid OP code"            },
    { ERROR_STORAGE_DDAPI_FILE_OPEN_FAILED,             "file open failed"           },
    { ERROR_STORAGE_DDAPI_UNSUPPORTED_IO_CONTROLLER,    "unsupported IOC"            },
    { ERROR_STORAGE_DDAPI_IOCTL_FAILED,                 "ioctl failed"               },
    { ERROR_STORAGE_DDAPI_READSCSI,                     "readSCSI"                   },
    { ERROR_STORAGE_DDAPI_WRITESCSI,                    "writeSCSI"                  },
*/
    //API error msg
    { ERROR_STORAGE_DDAPI_CONSTRUCTOR,                 "Constructor"},
    { ERROR_STORAGE_DDAPI_IOCTL,                       "IOCTL"},
    { ERROR_STORAGE_DDAPI_GENERATE_STORAGE_LIST,       "generateStorageList"},
    { ERROR_STORAGE_DDAPI_INIT_CONTROLLER_INFO,        "initControllerInfo"},
    { ERROR_STORAGE_DDAPI_INIT_PORT_INFO,              "initPortInfo"},
    { ERROR_STORAGE_DDAPI_LIST_DRIVE,                  "listDrive"},
    { ERROR_STORAGE_DDAPI_FLUSH_BUFFER_CACHE,          "flushBufferCache"},
    { ERROR_STORAGE_DDAPI_IDENTIFY_COMMAND_TEST,       "identifyCommandTest"},
    { ERROR_STORAGE_DDAPI_GET_DEFINED_INDEX_BY_NAME,   "getDefinedIndexByName"},
    { ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_INDEX,   "getElementDataByIndex"},
    { ERROR_STORAGE_DDAPI_GET_ERROR_REGISTER_BY_INDEX, "getErrorRegisterByIndex"},
    { ERROR_STORAGE_DDAPI_GET_SMART_BY_INDEX,          "getSMARTByIndex"},
    { ERROR_STORAGE_DDAPI_GET_ELEMENT_DATA_BY_NAME,    "getElementDataByName"},
    { ERROR_STORAGE_DDAPI_READ_CAPACITY_16,            "readCapacity16"},
    { ERROR_STORAGE_DDAPI_SAT_IDENTIFY_16,             "SATIdentify16"},
    { ERROR_STORAGE_DDAPI_PREAD,                       "PREADE"},
    { ERROR_STORAGE_DDAPI_PWRITE,                      "PWRITE"},

    //other error msg
    { ERROR_STORAGE_DDAPI_WRONG_DEVICE_NAME_FORMAT,        "wrong device node name format"},
    { ERROR_STORAGE_DDAPI_NO_SUCH_IOCTL_OPERATION,         "no such operation"},

    { ERROR_STORAGE_DDAPI_OPEN_MMAP_FAIL,                  "open memory map fail"},
    { ERROR_STORAGE_DDAPI_MMAP_FAIL,                       "memory map fail"},
    { ERROR_STORAGE_DDAPI_MUNMAP_FAIL,                     "memory unmap fail"},

    { ERROR_STORAGE_DDAPI_OPEN_DEVICE_FAIL,                "open device node fail"},
    { ERROR_STORAGE_DDAPI_CLOSE_DEVICE_FAIL,               "close device node fail"},
    { ERROR_STORAGE_DDAPI_OPEN_PIPE_FAIL,                  "open pipe fail" },
    { ERROR_STORAGE_DDAPI_OPEN_DIR_FAIL,                   "open directory fail"},
    { ERROR_STORAGE_DDAPI_READ_LINK_ERROR,                 "read link error"},

    { ERROR_STORAGE_DDAPI_DUMP_SMART_INFO_ERROR,           "dump SMART info error"},
    { ERROR_STORAGE_DDAPI_NO_ANY_STORAGE_CONTROLLER,       "no any STORAGE controller"},
    { ERROR_STORAGE_DDAPI_DEFINED_INDEX_NOT_FOUND,         "defined index not found"},
    { ERROR_STORAGE_DDAPI_NO_SUCH_ERROR_ELEMENT,           "no such error element"},
    { ERROR_STORAGE_DDAPI_NO_SUCH_DATA_ELEMENT,            "no such data element"},
    { ERROR_STORAGE_DDAPI_WRONG_DEFINED_INDEX,             "wrong defined index"},
    { ERROR_STORAGE_DDAPI_BLOCKSIZE_IS_NOT_MULTIPLE_OF_512,"block isn't multiple of 512"},
    { ERROR_STORAGE_DDAPI_SCSICDBSEND_IOCTL_FAIL,          "SG_IO ioctl fail"},
    { ERROR_STORAGE_DDAPI_SCSICDBSEND_SENSE_ERROR,         "SG_IO ioctl sense error"},
    { ERROR_STORAGE_DDAPI_LINK_STATUS_UNKNOWN,             "link status unknown"},
    { ERROR_STORAGE_DDAPI_LINK_SPEED_UNKNOWN,              "link speed unknown"},
    { ERROR_STORAGE_DDAPI_NO_SUCH_CONTROLLER_TYPE,         "no such controller type"},

};

// Storage Driver error code text definition
ErrorCodeText StorageDriverErrorCodeText[] =
{

};

ErrorCodeText IOCTLErrorCodeText[] = {
    { ERROR_IOCTL_GET_FILE_HANDLE_FAIL,                 "IOCTL get file handler failed"   },
    { ERROR_IOCTL_CLOSE_FILE_HANDLE_FAIL,               "IOCTL close file handler failed" },
    { ERROR_IOCTL_SET_CONTROLLER_ID_FAIL,               "IOCTL set controller ID  failed" },
    { ERROR_IOCTL_UNSUPPORT_ALLOCATEIOCTLARGUMENT,      "UNSUPPORT_ALLOCATEIOCTLARGUMENT" },
    { ERROR_IOCTL_UNSUPPORT_ARGUMENT,                   "UNSUPPORT_SETIOCTLOPERATIONARGUMENT" },
    { ERROR_IOCTL_UNSUPPORT_SENTIOCTL,                  "UNSUPPORT_SENTIOCTL"             },
};

//Processor DDAPI error code text definition
ErrorCodeText ProcessorDDAPIErrorCodeText[] = {
    { ERROR_PROCESSOR_DDAPI_READ_CPU_REGISTER,              "ReadCPURegister"             },
    { ERROR_PROCESSOR_DDAPI_WRITE_CPU_REGISTER,             "WriteCPURegister"            },

    { ERROR_PROCESSOR_DDAPI_OPEN_CPU_INFO_FAIL,             "Open /proc/cpuinfo failed"   },
    { ERROR_PROCESSOR_DDAPI_REGREAD_FAIL,                   "Call RegRead fail"           },
};

//Processor DAPI error code text definition
ErrorCodeText ProcessorDAPIErrorCodeText[] = {
    { ERROR_PROCESSOR_DAPI_OPEN_NODE_FAIL,                 "Open /proc/cpuinfo failed"   },
    { ERROR_PROCESSOR_DAPI_CONSTRUCTOR,                    "Constructor"                 },
    { ERROR_PROCESSOR_DAPI_GET_INFORMATIOM,                "GetInformation"              },
    { ERROR_PROCESSOR_DAPI_STABILITY_TEST,                 "StabilityTest"               },
    { ERROR_PROCESSOR_DAPI_GET_ERROR_REGISTER,             "GetErrorRegisters"           },
    { ERROR_PROCESSOR_DAPI_GET_MCE,                        "GetMCE"                      },
    { ERROR_PROCESSOR_DAPI_GET_MCE_EXECUTE,                "Execute"                     },
    { ERROR_PROCESSOR_DAPI_GET_MCE_PROCESS,                "process"                     },
    { ERROR_PROCESSOR_DAPI_OPEN_MCE_NODE_FAIL,             "Open /dev/mcelog failed"     },
    { ERROR_PROCESSOR_DAPI_MCE_GET_RECORD_LEN,             "Can't get MCE record lenth"  },
    { ERROR_PROCESSOR_DAPI_MCE_GET_LOG_LEN,                "Can't get MCE log lenth"     },
    { ERROR_PROCESSOR_DAPI_MCE_GET_LOG_LEN,                "Can't get MCE log lenth"     },
    { ERROR_PROCESSOR_DAPI_GET_MCE_PROCESS_FAIL,           "Consider an update"          },
};

//Processor Driver error code text definition
ErrorCodeText   ProcessorDriverErrorCodeText[] = {
    { ERROR_PROCESSOR_DRIVER_OPEN_NODE_FAIL,                 "Open /proc/cpuinfo failed"   },
};


//Ethernet DDAPI error code text definition
ErrorCodeText EthernetDDAPIErrorCodeText[] = {
    { ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL,                 "Open ethernet node failed"             },
    { ERROR_ETHERNET_DDAPI_SENDIOCTL,                      "SendIoctl"                             },
    { ERROR_ETHERNET_DDAPI_GET_MAC,                        "GetEthernetMAC"                        },
    { ERROR_ETHERNET_DDAPI_GET_NAME_LIST,                  "GetEthernetNamesList"                  },
    { ERROR_ETHERNET_DDAPI_GET_IP,                         "GetEthernetIP"                         },
    { ERROR_ETHERNET_DDAPI_GET_MASK,                       "GetEthernetMask"                       },
    { ERROR_ETHERNET_DDAPI_GET_MTU,                        "GetEthernetMTU"                        },
    { ERROR_ETHERNET_DDAPI_GET_EEPROM,                     "GetEthernetEEPROM"                     },
    { ERROR_ETHERNET_DDAPI_GET_EEPROM_SIZE,                "GetEthernetEEPROMSize"                 },
    { ERROR_ETHERNET_DDAPI_SET_EEPROM,                     "SetEthernetEEPROM"                     },
    { ERROR_ETHERNET_DDAPI_SET_MAC,                        "SetEthernetMAC"                        },
    { ERROR_ETHERNET_DDAPI_GET_MAIC,                       "GetEthernetMagic"                      },
    { ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK,          "Set_external_loopback"                 },
    { ERROR_ETHERNET_DDAPI_EXECUTE_EXTERNAL_LOOPBACK,      "Execute_external_loopback"             },
    { ERROR_ETHERNET_DDAPI_ENABLE_EXTERNAL_LOOPBACK,       "Enable_external_loopback"              },
    { ERROR_ETHERNET_DDAPI_DISABLE_EXTERNAL_LOOPBACK,      "Disable_external_loopback"             },
    { ERROR_ETHERNET_DDAPI_GET_DEVICE_ID,                  "get_device_ID"                         },
    { ERROR_ETHERNET_DDAPI_GET_DEVICE_BUS_TYPE,            "get_device_bus_type"                   },
    { ERROR_ETHERNET_DDAPI_GET_STATISTICS,                 "getStatistics"                         },
    { ERROR_ETHERNET_DDAPI_CLEAR_AER,                      "clearAER"                              },
    { ERROR_ETHERNET_DDAPI_CLEAR_STATISTICS,               "clearStatistics"                       },
    { ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL,               "Open socket fail"                      },
    { ERROR_ETHERNET_DDAPI_IOCTL_NOT_SUPPORT,              "Operation not supported on IOCTL"      },
    { ERROR_ETHERNET_DDAPI_IOCTL_FAIL,                     "IOCTL operation execute fail"          },
    { ERROR_ETHERNET_DDAPI_IOCTL_ETHTOOL_FAIL,             "Execute IOCTL of ETHTOOL fail"         },
    { ERROR_ETHERNET_DDAPI_SENDIOCTL_UNDEFINED,            "Undefined test option"                 },
    { ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC,                 "No specify ethernet name"              },
    { ERROR_ETHERNET_DDAPI_NIC_NAME_TOO_LONG,              "Interface's name is too long"          },
    { ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL,            "Can't allocate memory for EEPROM data" },
    { ERROR_ETHERNET_DDAPI_SET_MAC_ONE,                    "Set MAC one fail"                      },
    { ERROR_ETHERNET_DDAPI_SET_MAC_TWO,                    "Set MAC two fail"                      },
    { ERROR_ETHERNET_DDAPI_SET_MAC_THREE,                  "Set MAC three fail"                    },
    { ERROR_ETHERNET_DDAPI_SET_MAC_FOUR,                   "Set MAC four fail"                     },
    { ERROR_ETHERNET_DDAPI_SET_MAC_FIVE,                   "Set MAC five fail"                     },
    { ERROR_ETHERNET_DDAPI_SET_MAC_SIX,                    "Set MAC six fail"                      },
    { ERROR_ETHERNET_DDAPI_REGISTER_TEST_FAIL,             "Register test fail"                    },
    { ERROR_ETHERNET_DDAPI_EEPROM_VALIDATION_TEST_FAIL,    "EEPROM validation test fail"           },
    { ERROR_ETHERNET_DDAPI_MDI_REGISTER_TEST_FAIL,         "Link status test fail"                 },
    { ERROR_ETHERNET_DDAPI_INTERRUPT_TEST_FAIL,            "Interrupt test fail"                   },
    { ERROR_ETHERNET_DDAPI_INTERNAL_LOOPBACK_TEST_FAIL,    "Internal loopback test fail"           },
    { ERROR_ETHERNET_DDAPI_SEND_SOCKET_OPEN_FAIL,          "Open send socket fail"                 },
    { ERROR_ETHERNET_DDAPI_RECEIVE_SOCKET_OPEN_FAIL,       "Open receive socket fail"              },
    { ERROR_ETHERNET_DDAPI_IOCTL_SIOCGIFINDEX,             "Get network interface index ioctl fail"},
    { ERROR_ETHERNET_DDAPI_IOCTL_SIOCGIFHWADDR,            "Get MAC address ioctl fail"            },
    { ERROR_ETHERNET_DDAPI_IOCTL_SIOCGIFMTU,               "Get MTU ioctl fail"                    },
    { ERROR_ETHERNET_DDAPI_SOCKET_BIND,                    "Bind socket to network fail"           },
    { ERROR_ETHERNET_DDAPI_SENDTO,                         "Sendto API fail"                       },
    { ERROR_ETHERNET_DDAPI_RECV_TIMEOUT,                   "Recvfrom timeout"                      },
    { ERROR_ETHERNET_DDAPI_RECVFROM,                       "Recvfrom API fail"                     },
    { ERROR_ETHERNET_DDAPI_READ_LINK_FAIL,                 "Read symbolic link fail"               },
};

//Ethernet DAPI error code text definition
ErrorCodeText EthernetDAPIErrorCodeText[] = {
    { ERROR_ETHERNET_DAPI_CONSTRUCTOR,                     "Constructor"              },
    { ERROR_ETHERNET_DAPI_GET_DEVICE_NAMES,                "GetDeviceNames"           },
    { ERROR_ETHERNET_DAPI_READ_MAC,                        "ReadMacAddress"           },
    { ERROR_ETHERNET_DAPI_READ_IP,                         "ReadIPAddress"            },
    { ERROR_ETHERNET_DAPI_READ_MASK,                       "ReadIPMask"               },
    { ERROR_ETHERNET_DAPI_READ_MTU,                        "ReadMTU"                  },
    { ERROR_ETHERNET_DAPI_READ_EEPROM,                     "ReadEEPROM"               },
    { ERROR_ETHERNET_DAPI_READ_EEPROM_SIZE,                "ReadEEPROMSize"           },
    { ERROR_ETHERNET_DAPI_EEPROM_VALIDATION_TEST,          "EepromValidationTest"     },
    { ERROR_ETHERNET_DAPI_MDI_REGISTER_TEST,               "MDIRegisterTest"          },
    { ERROR_ETHERNET_DAPI_INTERRUPT_TEST,                  "InterruptTest"            },
    { ERROR_ETHERNET_DAPI_INTERNAL_LOOPBACK_TEST,          "InternalLoopbackTest"     },
    { ERROR_ETHERNET_DAPI_WRITE_EEPROM,                    "WriteEEPROM"              },
    { ERROR_ETHERNET_DAPI_WRITE_MAC,                       "WriteMacAddress"          },
    { ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK,               "Extrenal_loopback"        },
    { ERROR_ETHERNET_DAPI_SETUP_SPEED,                     "Setup_speed"              },
    { ERROR_ETHERNET_DAPI_GET_SUPPORT_LINK_MODE,           "GetSupportedLinkModes"    },
    { ERROR_ETHERNET_DAPI_GET_LINK_STATUS,                 "GetLinkStatus"            },
    { ERROR_ETHERNET_DAPI_GET_STATISTICS,                  "GetStatistics"            },
    { ERROR_ETHERNET_DAPI_GET_NODE_VALUE,                  "GetNodeValue"             },
    { ERROR_ETHERNET_DAPI_GET_CONTROLLER_NAME,             "GetControllerName"        },
    { ERROR_ETHERNET_DAPI_GET_FIRMWARE_VERSION,            "GetFirmwareVersion"       },
    { ERROR_ETHERNET_DAPI_CLEAR,                           "Clear"                    },
    { ERROR_ETHERNET_DAPI_NO_SPECIFY_NIC,                  "No specify ethernet name" },
    { ERROR_ETHERNET_DAPI_GET_ETHERNET_DRIVER_SET_FAIL,    "get driver info. fail"    },
    { ERROR_ETHERNET_DAPI_SET_ETHERNET_DRIVER_SET_FAIL,    "set network speed fail"   },
    { ERROR_ETHERNET_DAPI_IOCTL_FAIL,                      "IOCTL execute fail"       },
    { ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL,                "Open socket fail"         },
    { ERROR_ETHERNET_DAPI_NODE_OPEN_FAIL,                  "Open node fail fail"      },
    { ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK_ABNORMAL,      "External loopback throughput is zero"},
    { ERROR_ETHERNET_DAPI_SETUP_SPEED_TIMEOUT,             "Wait for link up timeout" },

};

//Ethernet Driver error code text definition
ErrorCodeText EthernetDriverErrorCodeText[] = {
    { ERROR_ETHERNET_DRIVER_OPEN_NODE_FAIL,                "Open /proc/cpuinfo failed"   },
    { ERROR_ETHERNET_DRIVER_XL710_SET_MAC,                 "XL710 set mac"               },
    { ERROR_ETHERNET_DRIVER_XL710_GET_MAC,                 "XL710 get mac"               },
    { ERROR_ETHERNET_DRIVER_XL710_SET_EEPROM,              "XL710 set EEPROM"            },
    { ERROR_ETHERNET_DRIVER_XL710_GET_EEPROM,              "XL710 get EEPROM"            },
    { ERROR_ETHERNET_DRIVER_SET_MAC_FAIL,                  "Set MAC address failed"      },
};



ErrorCodeText RTCDAPIErrorCodeText[] = {
    { ERROR_RTC_DAPI_RTC_UNEXPECTED_TEST_RESULT,            "unexpected test result"},
    { ERROR_RTC_DAPI_DATA_OUT_OF_RANGE,                     "value out of range"},
    { ERROR_RTC_DAPI_CONSTRUCTOR,                           "Constructor"},
    { ERROR_RTC_DAPI_SHOW_DATE_TIME,                        "showDateTime"},
    { ERROR_RTC_DAPI_SET_DATE,                              "setDate"},
    { ERROR_RTC_DAPI_SET_TIME,                              "setTime"},
    { ERROR_RTC_DAPI_RTC_ACCURACY_TEST,                     "rtcAccuracyTest"},
    { ERROR_RTC_DAPI_RTC_WDT_TEST,                          "rtcWDTTest"},
    
};

// RTC DDAPI error code text definition
ErrorCodeText RTCDDAPIErrorCodeText[] = {
    { ERROR_RTC_DDAPI_RTC_DEVICE_OPEN_FAIL,                 "RTC device open fail"},
    { ERROR_RTC_DDAPI_ALTACC_DEVICE_OPEN_FAIL,              "ALTACC dev open fail"},
    { ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL,                "RTC IOCTL access fail"},
    { ERROR_RTC_DDAPI_ALTACC_IOCTL_ACCESS_FAIL,             "ALTACC IOCTL access fail"},
    { ERROR_RTC_DDAPI_PORT_PERMISSION,                      "port permission error"},
    { ERROR_RTC_DDAPI_BATTERY_CHARGED_FAIL,                 "battery charged fail"},
    { ERROR_RTC_DDAPI_UNKNOWN_DATA_ELEMENT,                 "unknown data element"},
    { ERROR_RTC_DDAPI_RTC_DEVICE_READ_FAIL,                 "read rtc dev fail"},
    { ERROR_RTC_DDAPI_CONSTRUCTOR,                          "Constructor"},
    { ERROR_RTC_DDAPI_SHOW_DATE_TIME,                       "showDateTime"},
    { ERROR_RTC_DDAPI_SET_RTC_TIMER_RATE,                   "setRTCTimerRate"},
    { ERROR_RTC_DDAPI_START_RTC_TIMER,                      "startRTCTimer"},
    { ERROR_RTC_DDAPI_READ_RTC_TIMER_STATUS,                "readRTCTimerStatus"},   
    { ERROR_RTC_DDAPI_SET_DATE,                             "setDate"}, 
    { ERROR_RTC_DDAPI_SET_TIME,                             "setTime"}, 
	{ ERROR_RTC_DDAPI_RTC_WDT_TEST,  						"rtcWDTTest"}, 

};


// PCI DAPI error code test definition
ErrorCodeText PCIDAPIErrorCodeText[] = {
    {ERROR_PCI_DAPI_CONSTRUCTOR,                "Constructor"},
    {ERROR_PCI_DAPI_READ_CONFIG_DATA,           "Fail to read config data"},
    {ERROR_PCI_DAPI_READ_CONFIG_DATA_BYTE,      "Fail to read config data by byte"},
    {ERROR_PCI_DAPI_READ_CONFIG_DATA_WORD,      "Fail to read config data by word"},
    {ERROR_PCI_DAPI_READ_CONFIG_DATA_DWORD,     "Fail to read config data by dword"},
    {ERROR_PCI_DAPI_LINK_SPEED_NOT_MATCH,       "Link speed is not match"},
    {ERROR_PCI_DAPI_LINK_WIDTH_NOT_MATCH,       "Link width is not match"},
    //{ERROR_PCI_DAPI_NO_DEVICE_SUPPORT_RETRAIN,  "no device supports retrain"},
    {ERROR_PCI_DAPI_DEVICE_LIST,                "DisplayDeviceList"},
    {ERROR_PCI_DAPI_RETRAIN,                    "testRetrain"},
    {ERROR_PCI_DAPI_RETRAIN_ONE_DEVICE,         "PCIERetrainTestOneDevice"},
    {ERROR_PCI_DAPI_RETRAIN_CAP_CHECK,          "RetrainDeviceCheck"},
    {ERROR_PCI_DAPI_RETRAIN_LOOP,               "PCIERetrainTestLoop"},
    {ERROR_PCI_DAPI_ERROR_REPORT,               "PCIErrorReport"},
    {ERROR_PCI_DAPI_AER_REPORT,                 "PCIAERReport"},
    {ERROR_PCI_DAPI_RESET_STATISTICS,           "PCIResetStatistics"},
    {ERROR_PCI_DAPI_RESET_STATISTICS_RUN,       "PCIResetStatisticsRun"},
    {ERROR_PCI_DAPI_RESET_STATISTICS_BRI,       "PCIResetStatisticsBri"},
    {ERROR_PCI_DAPI_RESET_STATISTICS_PCIE_CAP,  "PCIResetStatisticsPCIeCap"},
    {ERROR_PCI_DAPI_RESET_STATISTICS_AER,       "PCIResetStatisticsAER"},
    {ERROR_PCI_DAPI_RESET_BRIDGE_ERR,           "PCIResetBridgeErr"},
    {ERROR_PCI_DAPI_RESET_COM_ERR,              "PCIResetCompatibleErr"},
    {ERROR_PCI_DAPI_RESET_PCIE_CAP_ERR,         "PCIResetPCIeCapErr"},
    {ERROR_PCI_DAPI_RESET_AER,                  "PCIResetAER"},
    {ERROR_PCI_DAPI_GET_DEVICE_INFO_BY_CLASS,   "GetDeviceInfoByClass"},
    {ERROR_PCI_DAPI_DEV_SUPPORT_RETRAIN,        "GetRetrainSupportDevices"},

    //pcie switch eeprom
    {ERROR_PCI_DAPI_READ_EEPROM,                "Failt to read EEPROM"},
    {ERROR_PCI_DAPI_GET_EEPROM_BY_ID,           "GetEepInfoByLogicalID"},
    {ERROR_PCI_DAPI_GET_EEPROM_LIST,            "GetAttachedFlashList"},
    {ERROR_PCI_DAPI_GET_EEPROM_SIZE,            "getEepromSize"},
    {ERROR_PCI_DAPI_WRITE_EEPROM,               "Fil to write EEPROM"},
    {ERROR_PCI_DAPI_EEPROM_TEST,                "eeprom test fail"},
    {ERROR_PCI_DAPI_EEPROM_DUMP,                "dumpEeprom"},
    {ERROR_PCI_DAPI_SAVE_EEPROM,                "saveEeprom"},
    {ERROR_PCI_DAPI_PROGRAM_EEPROM,             "programEeprom"},
    {ERROR_PCI_DAPI_TEST_SPEED,                 "test speed fail"},
    {ERROR_PCI_DAPI_TEST_WIDTH,                 "test width fail"},
    {ERROR_PCI_DAPI_GET_CLASS,                  "convertStringToClassCode"},
    {ERROR_PCI_DAPI_SCAN_DEVICE,                "scanDevicesFromIO"},
    {ERROR_PCI_DAPI_SHOW_ERROR,                 "showSpecificErrorStatistics"},
    
    {ERROR_PCI_DAPI_SHOW_EEPROM_FW_VER,         "showEepromImageVersion"},
    {ERROR_PCI_DAPI_LIST_SWITCH,                "getPCIeSwitcheList"},
    {ERROR_PCI_DAPI_GET_SWITCH_LIST,            "getPCIeSwitchList"},
    {ERROR_PCI_DAPI_SHOW_SW_ERROR,              "showSwitchErrorStatistics"},
    {ERROR_PCI_DAPI_GET_SW_PORT_INFO,           "getSwitchPortsInformation"},
    {ERROR_PCI_DAPI_DISPLAY_HBAS,               "displayHBAs"},
};

// PCI DDAPI error code test definition
ErrorCodeText PCIDDAPIErrorCodeText[] = {
    {ERROR_PCI_DDAPI_INVALID_CLASS_CODE,        "invalid PCI class code"},
    {ERROR_PCI_DDAPI_WRITE_CONFIG,              "fail to access config space"},
    {ERROR_PCI_DDAPI_CAPABILITY_UNSUPPORT,      "capability unsupported"},
    {ERROR_PCI_DDAPI_ERROR_TYPE_NOT_FOUND,      "error type is not found"},
    {ERROR_PCI_DDAPI_NO_DEVICE_FOUND,           "no device is found"},
    {ERROR_PCI_DDAPI_COMPARE_DATA,              "compare data fail"},
    {ERROR_PCI_DDAPI_RETRAIN_UNSUPPORT,         "retrain is not supported"},
    {ERROR_PCI_DDAPI_UNEXPECTED_DEVICE_COUNT,   "device count is unexpected"},

    {ERROR_PCI_DDAPI_RETRAIN,                   "testRetrain"},
    {ERROR_PCI_DDAPI_RETRAIN_EXECUTE,           "RetrainExecute"},
    {ERROR_PCI_DDAPI_RETRAIN_RESULT_CHECK,      "RetrainResultCheck"},
    {ERROR_PCI_DDAPI_AER_REPORT,                "PCIEAERError"},
    {ERROR_PCI_DDAPI_PCIE_CAP_ERR,              "PCIECapError"}, 
    {ERROR_PCI_DDAPI_RESET_COM_ERR,             "PCICompatibleErrorReset"},
    {ERROR_PCI_DDAPI_RESET_BRI_ERR,             "PCIBridgeErrorReset"},
    {ERROR_PCI_DDAPI_RESET_PCIE_CAP_ERR,        "PCIECapErrorReset"},
    {ERROR_PCI_DDAPI_RESET_AER_ERR,             "PCIEAERErrorReset"},
    {ERROR_PCI_DDAPI_EEPROM_SEND_COMMAND,       "fail to send eeprom command"},
    {ERROR_PCI_DDAPI_GET_EEPROM_DATA_SIZE,      "GetEepDataSize"},
    {ERROR_PCI_DDAPI_WAIT_EEPROM_IDLE,          "EepWaitIdle"},
    {ERROR_PCI_DDAPI_GET_EEPROM_DATA_SIZE_CMD,  "EepSendCmdToGetEepSize"},
    {ERROR_PCI_DDAPI_READ_EEPROM,               "ReadAttachedFlash"},
    {ERROR_PCI_DDAPI_EEPRON_NOT_PRESENT,        "EepPresentCheck"},
    {ERROR_PCI_DDAPI_EEPROM_PROGRAM,            "EepProgrm"},
    {ERROR_PCI_DDAPI_EEPROM_SAVE,               "saveEeprom"},
    {ERROR_PCI_DDAPI_EEPROM_READ,               "readEepromDword"},
    {ERROR_PCI_DDAPI_EEPROM_READ_BLOCK,         "readEepromBlock"},
    {ERROR_PCI_DDAPI_EEPROM_TEST,               "testEeprom"},
    {ERROR_PCI_DDAPI_EEPROM_SEND_COMMAND,       "send eeprom command fail"},
    {ERROR_PCI_DDAPI_EEPROM_WAIT_IDLE,          "time out to wait eeprom idle"},
    {ERROR_PCI_DDAPI_EEPROM_GET_DEVICE,         "getEeprom"},
    {ERROR_PCI_DDAPI_EEPROM_GET_SIZE,           "getEepromSize"},
    {ERROR_PCI_DDAPI_EEPROM_WRITE,              "writeEeprom"},
    {ERROR_PCI_DDAPI_EEPROM_WRITE_BLOCK,        "writeEepromBlock"},
    {ERROR_PCI_DDAPI_EEPROM_RESTORE,            "restoreEeprom"},
    {ERROR_PCI_DDAPI_TEST_SPEED,                "test speed fail"},
    {ERROR_PCI_DDAPI_TEST_WIDTH,                "test width fail"},
    {ERROR_PCI_DDAPI_GET_BAR,                   "get Bar fail"},
    {ERROR_PCI_DDAPI_GET_PCI_INFO,              "get pci info fail"},
    {ERROR_PCI_DDAPI_SCAN_DEVICE,               "scanDevicesFromIO"},
    {ERROR_PCI_DDAPI_SHOW_EEPROM_FW_VER,        "showEepromImageVersion"},
    {ERROR_PCI_DDAPI_GET_NTB,                   "getNtbDevice"},
    {ERROR_PCI_DDAPI_GET_SWITCH_LIST,           "getPCIeSwitchList"},
    {ERROR_PCI_DDAPI_GET_NTB_ERROR,             "getNtbErrorStatistics"},
    {ERROR_PCI_DDAPI_GET_PORT_ERROR_LIST,       "getPortErrorList"},
    {ERROR_PCI_DDAPI_SCAN_SLOT,                 "scanSlot"},
    {ERROR_PCI_DDAPI_BUILD_TREE,                "buildTree"},
    {ERROR_PCI_DDAPI_GET_EEPROM_LIST,           "getEepromList"},
};


// UART DAPI error code test definition
ErrorCodeText UARTDAPIErrorCodeText[] = {
    {ERROR_UART_DAPI_ERRORCODE,                     "Constructor"},
    {ERROR_UART_DAPI_DEVICE_TYPE_UNKNOWN,           "UART device type unknown"},
    {ERROR_UART_DAPI_DEVICE_LIST,                   "DisplayDeviceList"},
    {ERROR_UART_DAPI_SCRATCHPAD_TEST,               "testScratchPad"},
    {ERROR_UART_DAPI_INTERNAL_LOOPBACK_TEST,        "doInternalLoopbackTest"},
    {ERROR_UART_DAPI_GET_INFO,                      "getUartInfoByName"},
    {ERROR_UART_DAPI_GET_LIST,                      "getUartList"},
};

// UART DDAPI error code test definition
ErrorCodeText UARTDDAPIErrorCodeText[] = {
    {ERROR_UART_DDAPI_ERRORCODE,                            "Constructor"},
    {ERROR_UART_DDAPI_FUNCTION_UNSUPPORTED,                 "function is not supported"},
    {ERROR_UART_DDAPI_TIMEOUT,                              "timeout for checking register data"},
    {ERROR_UART_DDAPI_DATA_NOT_READY,                       "data is not ready"},
    {ERROR_UART_DDAPI_IOCTL_GET_SERIAL_INFO,                "fail to get serial infomation through ioctl"},
    {ERROR_UART_DDAPI_DATA_NOT_MATCH,                       "data is not match"},
    {ERROR_UART_DDAPI_SCRATCHPAD_TEST,                      "testScratchPad"},
    {ERROR_UART_DDAPI_SCRATCHPAD_DEFAULT_RUN,               "ScratchpadDefaultRun"},
    {ERROR_UART_DDAPI_LOOPBACK,                             "doInternalLoopbackTest"},
    {ERROR_UART_DDAPI_INTERNAL_LOOPBACK_RUN,                "InternalLoopbackDefaultRun"},
    {ERROR_UART_DDAPI_INTERNAL_LOOPBACK_DEFAULT,            "InternalLoopbackTestDefault"},
    {ERROR_UART_DDAPI_INTERNAL_LOOPBACK_DEFAULT_POLLING,    "InternalLoopbackDefaultPolling"},
    {ERROR_UART_DDAPI_SCAN_UARTS,                           "scanUarts"},
    {ERROR_UART_DDAPI_GET_DEVICE,                           "getDevice"},
    {ERROR_UART_DDAPI_READ_REGISTER,                        "readRegister"},
    {ERROR_UART_DDAPI_WRITE_REGISTER,                       "writeRegister"},
    {ERROR_UART_DDAPI_SEARCH_DEVICE,                        "searchUART"},
};

// PSU DAPI error code text definition
ErrorCodeText PSUDAPIErrorCodeText[] = {
    {ERROR_PSU_DAPI_CONSTRUCTOR,                        "Constructor"},
    {ERROR_PSU_DAPI_GET_DEVICE_LIST,                    "getDeviceList"},
    {ERROR_PSU_DAPI_GET_DATA_BY_ELEMENT,                "getDataByElement"},
    {ERROR_PSU_DAPI_GET_SUPPORT_VALUE_BY_ELEMENT,       "getSupportValueByElement"},
    {ERROR_PSU_DAPI_SET_DATA_BY_ELEMENT,                "setDataByElement"},
    {ERROR_PSU_DAPI_UPDATE_FW_BY_FILE,                  "updateFwByFile"},
};

// PSU DDAPI error code test definition
ErrorCodeText PSUDDAPIErrorCodeText[] = {
    {ERROR_PSU_DDAPI_UNSUPPORTED_FAN_SPEED,         "unsupported FAN speed"},
    {ERROR_PSU_DDAPI_UNKNOWN_LED_STATUS,            "unknown LED status"},
    {ERROR_PSU_DDAPI_PROGRAM_DATA_FAIL,             "fail to program data"},
    {ERROR_PSU_DDAPI_CONSTRUCTOR,                   "Constructor"},
    {ERROR_PSU_DDAPI_INITIALIZEPSU_INFO_FILE,       "initializePSUInfoFile"},
    {ERROR_PSU_DDAPI_IS_PRESENT,                    "isPresent"},
    {ERROR_PSU_DDAPI_GET_DEVICE_LIST,               "getDeviceList"},
    {ERROR_PSU_DDAPI_GET_VENDOR_NAME,               "getVendorName"},
    {ERROR_PSU_DDAPI_GET_FW_VERSION,                "getFWVersion"},
    {ERROR_PSU_DDAPI_GET_PART_NUMBER,               "getPartNumber"},
    {ERROR_PSU_DDAPI_GET_SERIAL_NUMBER,             "getSerialNumber"},
    {ERROR_PSU_DDAPI_GET_MAX_POWER,                 "getMaxPower"},
    {ERROR_PSU_DDAPI_GET_FAN_LIST,                  "getFanList"},
    {ERROR_PSU_DDAPI_GET_FAN_SPEED,                 "getFanSpeed"},
    {ERROR_PSU_DDAPI_GET_SUPPORT_FAN_SPEED,         "getSupportFanSpeed"},
    {ERROR_PSU_DDAPI_SET_FAN_SPEED,                 "setFanSpeed"},
    {ERROR_PSU_DDAPI_GET_TEMPERATURE_LIST,          "getTemperatureList"},
    {ERROR_PSU_DDAPI_GET_TEMPERATURE,               "getTemperature"},
    {ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD_LIST,"getTemperatureThresholdList"},
    {ERROR_PSU_DDAPI_GET_TEMPERATURE_THRESHOLD,     "getTemperatureThreshold"},
    {ERROR_PSU_DDAPI_SET_TEMPERATURE_THRESHOLD,     "setTemperatureThreshold"},
    {ERROR_PSU_DDAPI_GET_LED_LIST,                  "getLEDList"},
    {ERROR_PSU_DDAPI_GET_LED_STATUS,                "getLEDStatus"},
    {ERROR_PSU_DDAPI_SET_LED_STATUS,                "setLEDStatus"},
    {ERROR_PSU_DDAPI_GET_TYPE,                      "getType"},
    {ERROR_PSU_DDAPI_PROGRAM,                       "program"},
};

// LED DAPI error code text definition
ErrorCodeText LEDDAPIErrorCodeText[] = {
    {ERROR_LED_DAPI_CONSTRUCTOR,                      "CONSTRUCTOR"},
    {ERROR_LED_DAPI_GET_DEVICE_LIST,                  "getDeviceList"},
    {ERROR_LED_DAPI_GET_LED_STATUS,                   "getLEDStatus"},
    {ERROR_LED_DAPI_SET_LED_STATUS,                   "setLEDStatus"},
};

// LED DDAPI error code test definition
ErrorCodeText LEDDDAPIErrorCodeText[] = {
    {ERROR_LED_DDAPI_CONSTRUCTOR,                     "CONSTRUCTOR"                  },
};

// LED Driver error code test definition
ErrorCodeText LEDDriverErrorCodeText[] = {

};

// TEMPERATURE DAPI error code text definition
ErrorCodeText TEMPERATUREDAPIErrorCodeText[] = {
    {ERROR_TEMPERATURE_DAPI_CONSTRUCTOR,                      "CONSTRUCTOR"                     },
    {ERROR_TEMPERATURE_DAPI_OPEN_PROFILE_FAIL,                "Open profile fail"               },
    {ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE,          "Get_Current_Temperature"         },
    {ERROR_TEMPERATURE_DAPI_GET_THRESHOLD_HIGHT,              "Get_Threshold_Hight_Temperature" },
    {ERROR_TEMPERATURE_DAPI_GET_THRESHOLD_LOW,                "Get_Threshold_Low_Temperature"   },
    {ERROR_TEMPERATURE_DAPI_SET_THRESHOLD_HIGHT,              "Set_Threshold_Hight"             },
    {ERROR_TEMPERATURE_DAPI_SET_THRESHOLD_LOW,                "Set_Threshold_Low"               },
	{ERROR_TEMPERATURE_DAPI_GET_DATA_BY_NAME,                 "Get_Data_By_Name"                },
	{ERROR_TEMPERATURE_DAPI_SET_DATA_BY_NAME,                 "Set_Data_By_Name"                },
	{ERROR_TEMPERATURE_DAPI_DISPLAY_TEMPERATURE_LIST,         "Display_Temperature_Name_List"                },


};
// TEMPERATURE DDAPI error code test definition
ErrorCodeText TEMPERATUREDDAPIErrorCodeText[] = {
    {ERROR_TEMPERATURE_DDAPI_CONSTRUCTOR,                     "CONSTRUCTOR"                  },
    {ERROR_TEMPERATURE_DDAPI_GET_NAME,                        "Get_name"                     },
    {ERROR_TEMPERATURE_DDAPI_GET_CURRENT_TEMPERATURE,         "Get_current_temperature"      },
    {ERROR_TEMPERATURE_DDAPI_GET_THRESHOLD_HIGHT,             "Get_threshold_hight"          },
    {ERROR_TEMPERATURE_DDAPI_GET_THRESHOLD_LOW,               "Get_threshold_low"            },
    {ERROR_TEMPERATURE_DDAPI_GET_TYPE,                        "Get_type"                     },
    {ERROR_TEMPERATURE_DDAPI_SET_NAME,                        "Set_name"                     },
    {ERROR_TEMPERATURE_DDAPI_SET_THRESHOLD_HIGHT,             "Set_threshold_hight"          },
    {ERROR_TEMPERATURE_DDAPI_SET_THRESHOLD_LOW,               "Set_threshold_low"            },
    {ERROR_TEMPERATURE_DDAPI_SET_TYPE,                        "Set_type"                     },
	{ERROR_TEMPERATURE_DDAPI_GET_DATA_BY_NAME,                "Get_Data_By_Name"             },
	{ERROR_TEMPERATURE_DDAPI_GET_TEMPERATURE_NAME_LIST,       "Get_Temperature_Name_List"    },

	{ERROR_TEMPERATURE_I2C_DDAPI_GET_TEMPERATURE_NAME_LIST,   "Get_i2c_Temperature_Name_List"},
	{ERROR_TEMPERATURE_I2C_DDAPI_GET_CURRENT_TEMPERATURE,     "Get_current_i2c_temperature"  },
	{ERROR_TEMPERATURE_I2C_DDAPI_GET_DATA_BY_NAME,     		  "Get_i2c_Data_By_Name"  		 },
	{ERROR_TEMPERATURE_I2C_DDAPI_SET_DATA_BY_NAME,     		  "Set_i2c_Data_By_Name"  		 },

	{ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_HIGH0_LIST,    "Get_i2c_Get_Threshold_High0_List"},
	{ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_LOW0_LIST,     "Get_i2c_Get_Threshold_Low0_List"},
	{ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_HIGH1_LIST,    "Get_i2c_Get_Threshold_High1_List"},
	{ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_LOW1_LIST,     "Get_i2c_Get_Threshold_Low1_List"},
	{ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_HIGH0_LIST,    "Get_i2c_Set_Threshold_High0_List"},
	{ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_LOW0_LIST,     "Get_i2c_Set_Threshold_Low0_List"},
	{ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_HIGH1_LIST,    "Get_i2c_Set_Threshold_High1_List"},
	{ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_LOW1_LIST,     "Get_i2c_Set_Threshold_Low1_List"},



};

// TEMPERATURE Driver error code test definition
ErrorCodeText TEMPERATUREDriverErrorCodeText[] = {

};

// I2C DAPI error code text definition
ErrorCodeText I2CDAPIErrorCodeText[] = {
    {ERROR_I2C_DAPI_CONSTRUCTOR,                      "Constructor"                    },
    {ERROR_I2C_DAPI_DISPLAY_I2C_DEVICES_LIST,         "DisplayI2cDevicesList"          },
    {ERROR_I2C_DAPI_READ,                             "Read"                           },
    {ERROR_I2C_DAPI_WRITE,                            "Write"                          },
    {ERROR_I2C_DAPI_DISPLAY_I2C_DEVICES_LIST_BY_TYPE, "DisplayI2cDevicesListByType"    },
    {ERROR_I2C_DAPI_MUX_TEST,                         "MUX_Test"                       },
};

// I2C DDAPI error code test definition
ErrorCodeText I2CDDAPIErrorCodeText[] = {
    {ERROR_I2C_DDAPI_OPEN_NODE_FAIL,                  "I2C node open fail"            },
    {ERROR_I2C_DDAPI_OPEN_PROFILE_FAIL,               "Open I2C_profile.ini fail"     },
    {ERROR_I2C_DDAPI_NO_DEVICE,                       "No I2C device"                 },
    {ERROR_I2C_DDAPI_SET_SLAVE_ADDRESS_FAIL,          "I2C set slave address fail"    },
    {ERROR_I2C_DDAPI_READ_BYTE_FAIL,                  "I2C read byte fail"            },
    {ERROR_I2C_DDAPI_READ_BYTE_DATA_FAIL,             "I2C read byte data fail"       },
    {ERROR_I2C_DDAPI_READ_WORD_DATA_FAIL,             "I2C read word data fail"       },
    {ERROR_I2C_DDAPI_READ_BLOCK_DATA_FAIL,            "I2C read block data fail"      },
    {ERROR_I2C_DDAPI_READ_I2C_BLOCK_DATA_FAIL,        "I2C read i2c block data fail"  },
    {ERROR_I2C_DDAPI_WRITE_BYTE_FAIL,                 "I2C write byte fail"           },
    {ERROR_I2C_DDAPI_WRITE_BYTE_DATA_FAIL,            "I2C write byte data fail"      },
    {ERROR_I2C_DDAPI_WRITE_WORD_DATA_FAIL,            "I2C write word data fail"      },
    {ERROR_I2C_DDAPI_WRITE_BLOCK_DATA_FAIL,           "I2C write block data fail"     },
    {ERROR_I2C_DDAPI_WRITE_I2C_BLOCK_DATA_FAIL,       "I2C write i2c block data fail" },
    {ERROR_I2C_DDAPI_BUS_NULL,                        "I2C bus is null"               },
    {ERROR_I2C_DDAPI_BUS_NOT_FOUND,                   "I2C bus is not found"          },
    {ERROR_I2C_DDAPI_ADDRESS_NOT_FOUND,               "I2C address not found"         },
    {ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER,            "Parameter is not number"       },
    {ERROR_I2C_DDAPI_TYPE_UN_MATCH,                   "Type is not match"             },
    {ERROR_I2C_DDAPI_MUX_TEST_FAIL,                   "MUX test fail"                 },
    {ERROR_I2C_DDAPI_I2CBUS_READ,                     "read"                          },
    {ERROR_I2C_DDAPI_I2CBUS_WRITE,                    "write"                         },
    {ERROR_I2C_DDAPI_CONSTRUCTOR,                     "Constructor"                   },
    {ERROR_I2C_DDAPI_READ,                            "read"                          },
    {ERROR_I2C_DDAPI_WRITE,                           "write"                         },
    {ERROR_I2C_DDAPI_INITIAL_I2C_DEVICE,              "Initial_I2C_device"            },
    {ERROR_I2C_DDAPI_INITIAL_I2C_BUS,                 "Initial_I2C_BUS"               },
    {ERROR_I2C_DDAPI_INITIAL_I2C_PROTOCOL,            "Initial_I2C_Protocol"          },
    {ERROR_I2C_DDAPI_MUX_TEST,                        "mux_test"                      },
};

// I2C Driver error code test definition
ErrorCodeText I2CDriverErrorCodeText[] = {

};

// SPD DDAPI error code test definition
ErrorCodeText SPDDDAPIErrorCodeText[] = {
    {ERROR_SPD_DDAPI_NO_SPD_DEVICE,                   "no SPD device"},
    {ERROR_SPD_DDAPI_UNKNOWN_VENDOR,                  "unknown vendor"},
    {ERROR_SPD_DDAPI_CONSTRUCTOR,                     "Constructor"},
    {ERROR_SPD_DDAPI_GET_DEVICE_LIST,                 "getDeviceList"},
    {ERROR_SPD_DDAPI_GET_DEVICE_TYPE,                 "getDeviceType"},
    {ERROR_SPD_DDAPI_GET_MODULE_TYPE,                 "getModuleType"},
    {ERROR_SPD_DDAPI_GET_SIZE,                        "getSize"},
    {ERROR_SPD_DDAPI_GET_SPEED,                       "getSpeed"},
    {ERROR_SPD_DDAPI_GET_RANK_COUNT,                  "getRankCount"},
    {ERROR_SPD_DDAPI_GET_PART_NUMBER,                 "getPartNumber"},
    {ERROR_SPD_DDAPI_GET_SERIAL_NUMBER,               "getSerialNumber"},
    {ERROR_SPD_DDAPI_GET_DRAM_VENDOR,                 "getDRAMVendor"},
    {ERROR_SPD_DDAPI_GET_MODULE_VENDOR,               "getModuleVendor"},
};

//--------------------- NVSRAM ---------------------//
// NVSRAM Diagnostic API error code definition
ErrorCodeText NVSRAMDAPIErrorCodeText[] = {
    {ERROR_NVSRAM_DAPI_NO_NVSRAM_DEVICE,            "no NVSRAM device"},
    {ERROR_NVSRAM_DAPI_CONSTRUCTOR,                 "Constructor"},
    {ERROR_NVSRAM_DAPI_GET_INFORMATION,             "getInformation"},
    {ERROR_NVSRAM_DAPI_DATA_LINE_TEST,              "dataLineTest"},
    {ERROR_NVSRAM_DAPI_ADDRESS_LINE_TEST,           "addressLineTest"},
    {ERROR_NVSRAM_DAPI_NVRAM_TEST,                  "NVRAMTest"},
};

// NVSRAM Diagnostic Driver API error code definition
ErrorCodeText NVSRAMDDAPIErrorCodeText[] = {
    {ERROR_NVSRAM_DDAPI_OPEN_MEM_DEVICE_FAIL,       "openMemoryDeviceFail"},
    {ERROR_NVSRAM_DDAPI_MMAP_FAIL,                  "mmapFail"},
    {ERROR_NVSRAM_DDAPI_CONSTRUCTOR,                "Constructor"},
    {ERROR_NVSRAM_DDAPI_GET_BASE_ADDRESS,           "getBaseAddress"},
    {ERROR_NVSRAM_DDAPI_GET_RESOURE_SIZE,           "getResourceSize"},
    {ERROR_NVSRAM_DDAPI_GET_ELEMENT_SIZE,           "getElementSize"},
    {ERROR_NVSRAM_DDAPI_GET_NVSRAM_LIST,            "getNVSRAMList"},
    {ERROR_NVSRAM_DDAPI_RECALL_NVSRAM,              "recallNVSRAM"},
    {ERROR_NVSRAM_DDAPI_STORE_NVSRAM,               "restoreNVSRAM"},
};

// NVSRAM Driver error code definition
ErrorCodeText NVSRAMDriverErrorCodeText[] = {

};

//--------------------- TPM ---------------------//
//TPM DAPI error code definition 
ErrorCodeText TPMDAPIErrorCodeText[] = {
    {ERROR_TPM_DAPI_ACCESS_CMD,         "commandInterface"},
    {ERROR_TPM_DAPI_SEND_CMD_MANUALLY,  "sendCommandManually"},
};
//TPM DDAPI error code test definition
ErrorCodeText TPMDDAPIErrorCodeText[] = {
    {ERROR_TPM_DDAPI_ACCESS_CMD,        "commandInterface"},
    {ERROR_TPM_DDAPI_SEND_CMD_MANUALLY, "sendCommandManually"},
};


//--------------------- NTB ---------------------//
//NTB DAPI error code definition
ErrorCodeText NTBDAPIErrorCodeText[] = {
    {ERROR_NTB_DAPI_ERRORCODE,      "NTB DAPI"},
    {ERROR_NTB_DAPI_SEARCH_DEVICE,  "searchDevice"},
    {ERROR_NTB_DAPI_SEND,           "send"},
    {ERROR_NTB_DAPI_RECEIVER,       "receiver"},
    {ERROR_NTB_DAPI_INIT,           "initialWindow"},
    {ERROR_NTB_DAPI_SERVER,         "server"},
    {ERROR_NTB_DAPI_CLIENT,         "client"},
    {ERROR_NTB_DAPI_PERFORMANCE,    "testPerformance"},
};
//NTB DDAPI error code test definition
ErrorCodeText NTBDDAPIErrorCodeText[] = {
    {ERROR_NTB_DDAPI_ERRORCODE,         "NTB DDAPI"},
    //{ERROR_NTB_DDAPI_TIMEOUT,           "receive data timeout"},
    {ERROR_NTB_DDAPI_TIMEOUT,           "timeout"},
    {ERROR_NTB_DDAPI_SEARCH_DEVICE,     "searchDevice"},
    {ERROR_NTB_DDAPI_SEND,              "send"},
    {ERROR_NTB_DDAPI_RECEIVER,          "receiver"},
    {ERROR_NTB_DDAPI_INIT,              "initialWindow"},
    {ERROR_NTB_DDAPI_GET_WINDOW_INFO,   "getWindowInfo"},
    {ERROR_NTB_DDAPI_SERVER,            "server"},
    {ERROR_NTB_DDAPI_CLIENT,            "client"},
    {ERROR_NTB_DDAPI_IO_SETTING,        "setForIoTransfer"},
    {ERROR_NTB_DDAPI_RLS_BLOCK,         "releaseDataBlocks"},
    {ERROR_NTB_DDAPI_CREATE_BLOCK,      "createDataBlocks"},
    {ERROR_NTB_DDAPI_PERFORMANCE,       "testPerformance"},
    {ERROR_NTB_DDAPI_STRESS,            "stress"},
    {ERROR_NTB_DDAPI_RECEIVE_DATA,      "receiveData"},
};

//--------------------- Expander ---------------------//
// Expander Diagnostic API error code definition
ErrorCodeText EXPANDERDAPIErrorCodeText[] = {
    {ERROR_EXPANDER_DAPI_CONSTRUCTOR,                    "Constructor"},
    {ERROR_EXPANDER_DAPI_LIST_DEVICE,                    "listDevice"},
    {ERROR_EXPANDER_DAPI_GET_LOGICAL_ID_BY_NAME,         "getLogicalIDByName"},
    {ERROR_EXPANDER_DAPI_SHOW_DATA,                      "showData"},
    {ERROR_EXPANDER_DAPI_RESET,                          "reset"},
    {ERROR_EXPANDER_DAPI_SHOW_ERROR_DATA,                "showErrorData"},
    {ERROR_EXPANDER_DAPI_CLEAR_PHY_ERROR,                "clearPHYError"},
};

// Expander Diagnostic Driver API error code definition
ErrorCodeText EXPANDERDDAPIErrorCodeText[] = {
    {ERROR_EXPANDER_DDAPI_CONSTRUCTOR,                   "Constructor"},
    {ERROR_EXPANDER_DDAPI_POPEN_FAIL,                    "popen fail"},
    {ERROR_EXPANDER_DDAPI_UNKNOWN_NAME,                  "unknown name"},
    {ERROR_EXPANDER_DDAPI_UNKNOWN_ID,                    "unknown ID"},
    {ERROR_EXPANDER_DDAPI_DIFF_PHY_SPEED,                "diff phy speed"},
    {ERROR_EXPANDER_DDAPI_UNSUPPORT_DEV_TYPE,            "unsupport dev type"},
    {ERROR_EXPANDER_DDAPI_SYS_CMD_FAIL,                  "system cmd fail"},
    {ERROR_EXPANDER_DDAPI_SES_CMD_FAIL,                  "SES cmd fail"},
    {ERROR_EXPANDER_DDAPI_SMP_CMD_FAIL,                  "SMP cmd fail"},
    {ERROR_EXPANDER_DDAPI_GET_ID_BY_NAME,                "getLogicalIDByName"},
    {ERROR_EXPANDER_DDAPI_GET_DATA_BY_ID,                "getDataByID"},
    {ERROR_EXPANDER_DDAPI_GET_ERROR_DATA_BY_ID,          "getErrorDataByID"},
   	{ERROR_EXPANDER_DDAPI_CLEAR_PHY_ERROR,               "clearPHYError"},
	{ERROR_EXPANDER_DDAPI_RESET,				         "reset"},
};

// FPGA DAPI error code text definition
ErrorCodeText FPGADAPIErrorCodeText[] = {
    { ERROR_FPGA_DAPI_CONSTRUCTOR,                      "Constructor"},
    { ERROR_FPGA_DAPI_GET_DEVICE_LIST,                  "getDeviceList"},
    { ERROR_FPGA_DAPI_READ_REG,                         "readReg"},
    { ERROR_FPGA_DAPI_WRITE_REG,                        "writeReg"},
    { ERROR_FPGA_DAPI_GET_SPAD_INFO,                    "getSPADInfo"},
    { ERROR_FPGA_DAPI_READ_SPAD,                        "readSPAD"},
    { ERROR_FPGA_DAPI_WRITE_SPAD,                       "writeSPAD"},
};

// FPGA DDAPI error code text definition
ErrorCodeText FPGADDAPIErrorCodeText[] = {
    { ERROR_FPGA_DDAPI_UNKNOWN_REGISTER,                "unknown register"},
    { ERROR_FPGA_DDAPI_UNKNOWN_BASE_ADDRESS,            "unknown base address"},
    { ERROR_FPGA_DDAPI_CONSTRUCTOR,                     "Constructor"},
    { ERROR_FPGA_DDAPI_GET_DEVICE_LIST,                 "getDeviceList"},
    { ERROR_FPGA_DDAPI_GET_REG_INFO,                    "getRegInfo"},
    { ERROR_FPGA_DDAPI_READ,                            "read"},
    { ERROR_FPGA_DDAPI_WRITE,                           "write"},
    { ERROR_FPGA_DDAPI_GET_SPAD_INFO,                   "getSPADInfo"},
    { ERROR_FPGA_DDAPI_READ_SPAD,                       "readSPAD"},
    { ERROR_FPGA_DDAPI_WRITE_SPAD,                      "writeSPAD"},
};

// NVME DAPI error code text definition
ErrorCodeText NVMEDAPIErrorCodeText[] =
{
    { ERROR_NVME_DAPI_TEST_PATTERN_COMPARE_FAIL,     "testPatternCompareFail"},
    { ERROR_NVME_DAPI_GET_INFORMATION,               "getInformation"},
    { ERROR_NVME_DAPI_DISPLAY_DATA,                  "displayData"},
    { ERROR_NVME_DAPI_GET_MEMORY_BUFFER,             "getMemoryBuffer"},
    { ERROR_NVME_DAPI_GET_SMART_LOG,                 "getSmartLog"},
    { ERROR_NVME_DAPI_GET_LOGICALID_BY_NAME,         "getLogicalIDByName"},
    { ERROR_NVME_DAPI_RELEASE_MEMORY_BUFFER,         "releaseMemoryBuffer"},
    { ERROR_NVME_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL,   "allocate backup buffer fail"},
    { ERROR_NVME_DAPI_ALLOCATE_PATTERN_BUFFER_FAIL,  "allocate patter Buffer fail"},
    { ERROR_NVME_DAPI_ALLOCATE_PCOMPARE_BUFFER_FAIL, "allocate compare buffer fail"},
    { ERROR_NVME_DAPI_CONSTRUCTOR,                   "constructor"},
    { ERROR_NVME_DAPI_GET_FW_LOG,                    "getFirmwareLog"},
    { ERROR_NVME_DAPI_ADDRESS_LINE_TEST,             "addressLineTest"},
    { ERROR_NVME_DAPI_READWRITE_TEST,                "readWriteTest"},
    { ERROR_NVME_DAPI_READWRITE_TEST_SETUP,          "readWriteTestSetup"},
    { ERROR_NVME_DAPI_READWRITE_TEST_EXECUTE,        "readWriteTestExecute"},
    { ERROR_NVME_DAPI_READWRITE_TEST_CLEAN,          "readWriteTestClean"},
    { ERROR_NVME_DAPI_READ_PERFORMANCE_TEST,         "readPerformanceTest"},
    { ERROR_NVME_DAPI_READ_PERFORMANCE_TEST_SETUP,   "readPerformanceTestSetup"},
    { ERROR_NVME_DAPI_READ_PERFORMANCE_TEST_EXECUTE, "readPerformanceTestExecute"},
    { ERROR_NVME_DAPI_READ_PERFORMANCE_TEST_CLEAN,   "readPerformanceTestClean"},
    { ERROR_NVME_DAPI_WRITE_PERFORMANCE_TEST,        "writePerformanceTest"},
    { ERROR_NVME_DAPI_WRITE_PERFORMANCE_TEST_SETUP,  "writePerformanceTestSetup"},
    { ERROR_NVME_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE,"writePerformanceTestExecute"},
    { ERROR_NVME_DAPI_WRITE_PERFORMANCE_TEST_CLEAN,  "writePerformanceTestClean"},
    { ERROR_NVME_DAPI_RESET_CTRL,                    "resetCtrl"},
    { ERROR_NVME_DAPI_FW_DOWNLOAD,                   "download firmware"},
    { ERROR_NVME_DAPI_DO_IOCTL,                      "ioctl command fail"},
    { ERROR_NVME_DAPI_GET_ERROR_COUNT,               "getErrorCount"},
    { ERROR_NVME_DAPI_GET_DEVICE_NODE_LIST,          "getNVMEDeviceNodeList"},
    { ERROR_NVME_DAPI_GET_BAR_RESOURCE,              "getNVMEBarResource"},
    { ERROR_NVME_DAPI_GET_ERROR_LOG,                 "getNVMEErrorLog"},
    { ERROR_NVME_DAPI_GET_NAMESPACE_ID,              "getNMVENamespaceID"},
    { ERROR_NVME_DAPI_IDENTITY_CONTROLLER,           "NVMEIdentityController"},
    { ERROR_NVME_DAPI_IDENTITY_NAMESPACE,            "NVMEIdentityNamespace"},
    { ERROR_NVME_DAPI_FW_ACTIVATE,                   "activate firmware"},
    { ERROR_NVME_DAPI_WRITE_ZERO,                    "writeZerotoNVMe"},
};

// NVME DDAPI error code text definition
ErrorCodeText NVMEDDAPIErrorCodeText[] = {
    { ERROR_NVME_DDAPI_READ_MEMORY,                      "Read memory fail"                },
    { ERROR_NVME_DDAPI_WRITE_MEMORY,                     "Write memory fail"               },
    { ERROR_NVME_DDAPI_OPEN_NVME_DEVICE_FAIL,            "Can not open NVME device"        },
    { ERROR_NVME_DDAPI_MMAP_FAIL,                        "NVME mmap fail"                  },
    { ERROR_NVME_DDAPI_FILE_SEEK,                        "File seek position fail"         },
    { ERROR_NVME_DDAPI_TEMPERATURE_NO_EXIST,             "NVME temp sensor not exist"      },
    { ERROR_NVME_DDAPI_EXECUTE_IOCTL,                    "NVME execute IOCTL fail"         },
    { ERROR_NVME_DDAPI_DOWNLOAD_FW,                      "downlaod firmware"               },
    { ERROR_NVME_DDAPI_SETFILEHANDLE,                    "setFileHandle"                   },
    { ERROR_NVME_DDAPI_CLOSEFILEHANDLE,                  "closeFileHandle"                 },
    { ERROR_NVME_DDAPI_GET_NVME_LIST,                    "Cannot get NVME list"            },
    { ERROR_NVME_DDAPI_READ_LINK_ERROR,                  "Read symbolic link fail"         },
    { ERROR_NVME_DDAPI_NO_NVME_CONTROLLER,               "Host has no NVME controller"     },
    { ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST,             "Get NVME device node list fail"  },
    { ERROR_NVME_DDAPI_RESET_CTRL,                       "Reset NVME Ctrl fail"            },
    { ERROR_NVME_DDAPI_RESET_SUBSYS,                     "Reset NVME Subsys fail"          },
    { ERROR_NVME_DRIVER_IOCTL_FAIL,                      "NVME ioctl fail"                 },
    { ERROR_NVME_IOCTL_INVALID_OPCODE,                   "NVME ioctl invalid OPCODE"       },
    { ERROR_NVME_IOCTL_INVALID_FIELD,                    "NVME ioctl invalid field"        },
    { ERROR_NVME_IOCTL_CMDID_CONFLICT,                   "NVME ioctl cmdid conflict"       },
    { ERROR_NVME_IOCTL_DATA_XFER_ERROR,                  "NVME ioctl data xfer error"      },
    { ERROR_NVME_IOCTL_POWER_LOSS,                       "NVME ioctl power loss"           },
    { ERROR_NVME_IOCTL_INTERNAL,                         "NVME ioctl internal error"       },
    { ERROR_NVME_IOCTL_ABORT_REQ,                        "NVME ioctl abort request"        },
    { ERROR_NVME_IOCTL_ABORT_QUEUE,                      "NVME ioctl abort queue"          },
    { ERROR_NVME_IOCTL_FUSED_FAIL,                       "NVME ioctl fused fail"           },
    { ERROR_NVME_IOCTL_FUSED_MISSING,                    "NVME ioctl fused missing"        },
    { ERROR_NVME_IOCTL_INVALID_NS,                       "NVME ioctl invalid namespace"    },
    { ERROR_NVME_IOCTL_CMD_SEQ_ERROR,                    "NVME ioctl cmd SEQ error"        },
    { ERROR_NVME_IOCTL_LBA_RANGE,                        "NVME ioctl LBA range"            },
    { ERROR_NVME_IOCTL_CAP_EXCEEDED,                     "NVME ioctl CAP exceeded"         },
    { ERROR_NVME_IOCTL_NS_NOT_READY,                     "NVME ioctl namespace not ready"  },
    { ERROR_NVME_IOCTL_CQ_INVALID,                       "NVME ioctl CQ invalid"           },
    { ERROR_NVME_IOCTL_QID_INVALID,                      "NVME ioctl QID invalid"          },
    { ERROR_NVME_IOCTL_QUEUE_SIZE,                       "NVME ioctl queue size"           },
    { ERROR_NVME_IOCTL_ABORT_LIMIT,                      "NVME ioctl abort limit"          },
    { ERROR_NVME_IOCTL_ABORT_MISSING,                    "NVME ioctl abort missing"        },
    { ERROR_NVME_IOCTL_ASYNC_LIMIT,                      "NVME ioctl async limit"          },
    { ERROR_NVME_IOCTL_FW_SLOT,                          "NVME ioctl invalid fw slot"      },
    { ERROR_NVME_IOCTL_FW_IMAGE,                         "NVME ioctl invalid fw image"     },
    { ERROR_NVME_IOCTL_FW_NEEDS_MAX_TIME_VIOLATION,      "NVME ioctl max time violation"   },
    { ERROR_NVME_IOCTL_FW_PROHIBITED,                    "NVME ioctl activat prohibited"   },
    { ERROR_NVME_IOCTL_FW_IMAGE_OVERLAP,                 "NVME ioctl image overlap"        },
    { ERROR_NVME_IOCTL_INVALID_VECTOR,                   "NVME ioctl invalid vector"       },
    { ERROR_NVME_IOCTL_INVALID_LOG_PAGE,                 "NVME ioctl invalid log page"     },
    { ERROR_NVME_IOCTL_INVALID_FORMAT,                   "NVME ioctl invalid format"       },
    { ERROR_NVME_IOCTL_BAD_ATTRIBUTES,                   "NVME ioctl bad attributes"       },
    { ERROR_NVME_IOCTL_WRITE_FAULT,                      "NVME ioctl write fault"          },
    { ERROR_NVME_IOCTL_READ_ERROR,                       "NVME ioctl read error"           },
    { ERROR_NVME_IOCTL_GUARD_CHECK,                      "NVME ioctl guard check"          },
    { ERROR_NVME_IOCTL_APPTAG_CHECK,                     "NVME ioctl APPTAG check"         },
    { ERROR_NVME_IOCTL_REFTAG_CHECK,                     "NVME ioctl REFTAG check"         },
    { ERROR_NVME_IOCTL_COMPARE_FAILED,                   "NVME ioctl compare failed"       },
    { ERROR_NVME_IOCTL_ACCESS_DENIED,                    "NVME ioctl access denied"        },
    { ERROR_NVME_INVALID_FILE_MODE,                      "NVME ioctl invalid file mode"    },
};

// Controller DDAPI error code text definition
ErrorCodeText ControllerDDAPIErrorCodeText[] = {
    { ERROR_CONTROLLER_DDAPI_CONSTRUCTOR,                "Constructor"},
    { ERROR_CONTROLLER_DDAPI_GET_LED_LIST,               "getLEDList"},
    { ERROR_CONTROLLER_DDAPI_GET_LED_STATUS,             "getLEDStatus"},
    { ERROR_CONTROLLER_DDAPI_SET_LED_STATUS,             "setLEDStatus"},
    { ERROR_CONTROLLER_DDAPI_GET_SLOT_ID,                "getSlotID"},
};

// Error code text table definition
ErrorCodeTextTable ErrorCodeTextTableCollect[] = {
    { ERRORCODE_MASK_MODULE,                ModuleErrorCodeText,        (sizeof(ModuleErrorCodeText)/sizeof(ErrorCodeText))},
    { ERRORCODE_MASK_TYPE,                  TypeErrorCodeText,          (sizeof(TypeErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_COMMON_ERROR_CODE,              CommonErrorCodeText,        (sizeof(CommonErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_MEMORY_DAPI_ERRORCODE,          MemoryDAPIErrorCodeText,    (sizeof(MemoryDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_MEMORY_DDAPI_ERRORCODE,         MemoryDDAPIErrorCodeText,   (sizeof(MemoryDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_MEMORY_DRIVER_ERRORCODE,        MemoryDriverErrorCodeText,  (sizeof(MemoryDriverErrorCodeText)/sizeof(ErrorCodeText))},

    { ERROR_SAS_DAPI_ERRORCODE,             SASDAPIErrorCodeText,       (sizeof(SASDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_SAS_DDAPI_ERRORCODE,            SASDDAPIErrorCodeText,      (sizeof(SASDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_SAS_DRIVER_ERRORCODE,           SASDriverErrorCodeText,     (sizeof(SASDriverErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_SCSI_FACTORY_ERRORCODE,         SCSIFactoryErrorCodeText,   (sizeof(SCSIFactoryErrorCodeText)/sizeof(ErrorCodeText))},

//    { ERROR_STORAGE_DAPI_ERRORCODE,         StorageDAPIErrorCodeText,   (sizeof(StorageDAPIErrorCodeText)/sizeof(ErrorCodeText))},
//    { ERROR_STORAGE_DDAPI_ERRORCODE,        StorageDDAPIErrorCodeText,  (sizeof(StorageDDAPIErrorCodeText)/sizeof(ErrorCodeText))},

    // USB
    { ERROR_USB_DAPI_ERRORCODE,             USBDAPIErrorCodeText,    (sizeof(USBDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_USB_DDAPI_ERRORCODE,            USBDDAPIErrorCodeText,   (sizeof(USBDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_USB_DRIVER_ERRORCODE,           USBDriverErrorCodeText,  (sizeof(USBDriverErrorCodeText)/sizeof(ErrorCodeText))},

    //GPIO
    { ERROR_GPIO_DAPI_ERRORCODE,        GPIODAPIErrorCodeText,    (sizeof(GPIODAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_GPIO_DDAPI_ERRORCODE,       GPIODDAPIErrorCodeText,   (sizeof(GPIODDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_GPIO_DRIVER_ERRORCODE,      GPIODriverErrorCodeText,  (sizeof(GPIODriverErrorCodeText)/sizeof(ErrorCodeText))},


    //Processor
    { ERROR_PROCESSOR_DAPI_ERRORCODE,       ProcessorDAPIErrorCodeText,    (sizeof(ProcessorDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_PROCESSOR_DDAPI_ERRORCODE,      ProcessorDDAPIErrorCodeText,   (sizeof(ProcessorDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_PROCESSOR_DRIVER_ERRORCODE,     ProcessorDriverErrorCodeText,  (sizeof(ProcessorDriverErrorCodeText)/sizeof(ErrorCodeText))},

    //Ethernet
    { ERROR_ETHERNET_DAPI_ERRORCODE,        EthernetDAPIErrorCodeText,    (sizeof(EthernetDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_ETHERNET_DDAPI_ERRORCODE,       EthernetDDAPIErrorCodeText,   (sizeof(EthernetDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_ETHERNET_DRIVER_ERRORCODE,      EthernetDriverErrorCodeText,  (sizeof(EthernetDriverErrorCodeText)/sizeof(ErrorCodeText))},

    //Storage
    { ERROR_STORAGE_DAPI_ERRORCODE,          StorageDAPIErrorCodeText,    (sizeof(StorageDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_STORAGE_DDAPI_ERRORCODE,         StorageDDAPIErrorCodeText,   (sizeof(StorageDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_STORAGE_DRIVER_ERRORCODE,        StorageDriverErrorCodeText,  (sizeof(StorageDriverErrorCodeText)/sizeof(ErrorCodeText))},

    //RTC
    { ERROR_RTC_DAPI_ERRORCODE,             RTCDAPIErrorCodeText,    (sizeof(RTCDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_RTC_DDAPI_ERRORCODE,            RTCDDAPIErrorCodeText,   (sizeof(RTCDDAPIErrorCodeText)/sizeof(ErrorCodeText))},

    //UART
    { ERROR_UART_DAPI_ERRORCODE,           UARTDAPIErrorCodeText,    (sizeof(UARTDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_UART_DDAPI_ERRORCODE,          UARTDDAPIErrorCodeText,   (sizeof(UARTDDAPIErrorCodeText)/sizeof(ErrorCodeText))},

    //PCI
    { ERROR_PCI_DAPI_ERRORCODE,           PCIDAPIErrorCodeText,    (sizeof(PCIDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_PCI_DDAPI_ERRORCODE,          PCIDDAPIErrorCodeText,    (sizeof(PCIDDAPIErrorCodeText)/sizeof(ErrorCodeText))},

    // PSU
    { ERROR_PSU_DAPI_ERRORCODE,             PSUDAPIErrorCodeText,       (sizeof(PSUDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_PSU_DDAPI_ERRORCODE,            PSUDDAPIErrorCodeText,      (sizeof(PSUDDAPIErrorCodeText)/sizeof(ErrorCodeText))},

    //LED
    { ERROR_LED_DAPI_ERRORCODE,       LEDDAPIErrorCodeText,    (sizeof(LEDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_LED_DDAPI_ERRORCODE,      LEDDDAPIErrorCodeText,   (sizeof(LEDDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_LED_DRIVER_ERRORCODE,     LEDDriverErrorCodeText,  (sizeof(LEDDriverErrorCodeText)/sizeof(ErrorCodeText))},

    //I2C
    { ERROR_I2C_DAPI_ERRORCODE,       I2CDAPIErrorCodeText,    (sizeof(I2CDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_I2C_DDAPI_ERRORCODE,      I2CDDAPIErrorCodeText,   (sizeof(I2CDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_I2C_DRIVER_ERRORCODE,     I2CDriverErrorCodeText,  (sizeof(I2CDriverErrorCodeText)/sizeof(ErrorCodeText))},

    //TEMPERATURE
    { ERROR_TEMPERATURE_DAPI_ERRORCODE,       TEMPERATUREDAPIErrorCodeText,    (sizeof(TEMPERATUREDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_TEMPERATURE_DDAPI_ERRORCODE,      TEMPERATUREDDAPIErrorCodeText,   (sizeof(TEMPERATUREDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_TEMPERATURE_DRIVER_ERRORCODE,     TEMPERATUREDriverErrorCodeText,  (sizeof(TEMPERATUREDriverErrorCodeText)/sizeof(ErrorCodeText))},

    //SPD
    { ERROR_SPD_DDAPI_ERRORCODE,            SPDDDAPIErrorCodeText,      (sizeof(SPDDDAPIErrorCodeText)/sizeof(ErrorCodeText))},

    //NVSRAM
    { ERROR_NVSRAM_DAPI_ERRORCODE,          NVSRAMDAPIErrorCodeText,       (sizeof(NVSRAMDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_NVSRAM_DDAPI_ERRORCODE,         NVSRAMDDAPIErrorCodeText,      (sizeof(NVSRAMDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_NVSRAM_DRIVER_ERRORCODE,        NVSRAMDriverErrorCodeText,     (sizeof(NVSRAMDriverErrorCodeText)/sizeof(ErrorCodeText))},

    //TPM 
    { ERROR_TPM_DAPI_ERRORCODE,          TPMDAPIErrorCodeText,       (sizeof(TPMDAPIErrorCodeText)/sizeof(ErrorCodeText)) },
    { ERROR_TPM_DDAPI_ERRORCODE,         TPMDDAPIErrorCodeText,      (sizeof(TPMDDAPIErrorCodeText)/sizeof(ErrorCodeText)) },

    //NTB 
    { ERROR_NTB_DAPI_ERRORCODE,          NTBDAPIErrorCodeText,       (sizeof(NTBDAPIErrorCodeText)/sizeof(ErrorCodeText)) },
    { ERROR_NTB_DDAPI_ERRORCODE,         NTBDDAPIErrorCodeText,      (sizeof(NTBDDAPIErrorCodeText)/sizeof(ErrorCodeText)) },
	
    //EXPANDER
    { ERROR_EXPANDER_DAPI_ERRORCODE,        EXPANDERDAPIErrorCodeText,       (sizeof(EXPANDERDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_EXPANDER_DDAPI_ERRORCODE,       EXPANDERDDAPIErrorCodeText,      (sizeof(EXPANDERDDAPIErrorCodeText)/sizeof(ErrorCodeText))},

    //FPGA
    { ERROR_FPGA_DAPI_ERRORCODE,            FPGADAPIErrorCodeText,      (sizeof(FPGADAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_FPGA_DDAPI_ERRORCODE,           FPGADDAPIErrorCodeText,      (sizeof(FPGADDAPIErrorCodeText)/sizeof(ErrorCodeText))},

    //NVMe
    { ERROR_NVME_DAPI_ERRORCODE,            NVMEDAPIErrorCodeText,       (sizeof(NVMEDAPIErrorCodeText)/sizeof(ErrorCodeText))},
    { ERROR_NVME_DDAPI_ERRORCODE,           NVMEDDAPIErrorCodeText,      (sizeof(NVMEDDAPIErrorCodeText)/sizeof(ErrorCodeText))},

    //Controller
    { ERROR_CONTROLLER_DDAPI_ERRORCODE,     ControllerDDAPIErrorCodeText,(sizeof(ControllerDDAPIErrorCodeText)/sizeof(ErrorCodeText))},
};



#endif

